$.widget('ui.datatable', {

    options: {
        hasHeader: true,
        headerHeight: 28,
        fitColumns: false,
        resizableColumns: true,
        selectable: true,
        sortable: true,
        sortCallback: null,
        sortInfo: null,
        groupBy: null,
        groupFormatter: null
    },
    _create: function () {

        this.element.addClass("ui-table");
        this._ens = _.uniqueId("table-event-"); //event namespace

        this.element.on("focus", function () {
            $(this).datatable("selectedRow", 0);
        });

        this._loadingRowsMsg = $('<div>')
            .addClass('ui-datatable-loading-rows')
            .text('Loading Rows...')
            .appendTo(this.element);

        this.refresh();

    },
    addColumn: function (config) {
        if (config) {
            if (!config.width)
                config.width = 100;

            if (!config.title)
                config.title = "&nbsp;"

            var columnEl = $("<div class='appkit-table-column'>")
                .attr("data-width", config.width)
                .html(config.title)
                .appendTo(this.element);

            if (config.id) {
                columnEl.attr("data-id", config.id);
            }

            if (config.fixed === true) {
                columnEl.attr("data-fixed", true);
            }

            if (config.stretchable !== undefined) {
                columnEl.attr("data-stretchable", config.stretchable);
            }

            if (config.sortable === false) {
                columnEl.attr("data-sortable", false);
            }

            if (config.minWidth !== undefined) {
                columnEl.attr("data-min-width", config.minWidth);
            }

            if (_.isString(config.formatter)) {
                columnEl.attr("data-formatter", config.formatter);
            }
            else if (_.isFunction(config.formatter)) {
                columnEl.data("formatter", config.formatter);
            }

            if (_.isString(config.sorter)) {
                columnEl.attr("data-sorter", config.sorter);
            }
            else if (_.isFunction(config.sorter)) {
                columnEl.data("sorter", config.sorter);
            }

            if (_.isString(config.tooltip)) {
                columnEl.attr("data-tooltip", config.tooltip);
            }
            else if (_.isFunction(config.tooltip)) {
                columnEl.data("tooltip", config.tooltip);
            }
        }
    },
    removeColumn: function (columnId) {
        if (columnId) {
            this.element.children('#' + columnId).remove();
            this.refresh();
        }
    },
    removeAllColumns: function () {
        this.element.children(".appkit-table-column").remove();
        this.refresh();
    },
    _getColumnWidth: function (idx) {
        var ccount = this._columns.length
        if (idx > -1 && idx < ccount) {
            var w = this._columns[idx].width;
            if (!_.isNumber(w)) {
                var p = parseFloat(w);
                if (this._body.is(":visible")) {
                    w = Math.floor((p / 100.0) * this._body.width() - 2 * ccount);
                }
                else {
                    w = Math.floor((p / 100.0) * this.element.width() - 2 * ccount);
                }
            }

            return w;
        }

        return 0;
    },
    adjustColumnWidths: function (skipColumnFit) {

        var self = this;

        var rowZero = this._body.find(".dummy-row-1");
        rowZero.empty();

        var ccount = this._columns.length,
            totW = 0;


        $.each(this._columns, function (idx) {

            var w = self._getColumnWidth(idx);

            self._header.find("th").eq(idx).width(w);
            totW += (w + 1);

            $("<td class='first-column'>").height(0).width(w).appendTo(rowZero);

        });


        this._header.css("left", this._body.position().left);

        if (this.options.fitColumns && skipColumnFit !== true &&
            ccount > 0 && this.element.is(":visible")) {

            /** fit the columns to fill the table width, given min-width constraints  **/

            var skippedColumnW = 0,
                lastStretch = -1;
            $.each(this._columns, function (idx, aColumn) {
                if (skipColumnFit === idx || !aColumn.stretchable || aColumn.fixed) {
                    skippedColumnW += self._getColumnWidth(idx);
                }
                else {
                    lastStretch = idx;
                }
            });

            if (totW - skippedColumnW > 2 * ccount) {

                var dw = this.element.width() - totW - 2;
                dw -= Math.max(16, $.position.scrollbarWidth());
                var p = 1 / this._columns.length;

                var changeTot = 0;
                $.each(this._columns, function (idx, aColumn) {

                    if (skipColumnFit !== idx && aColumn.stretchable && !aColumn.fixed) {

                        var w = self._getColumnWidth(idx) - 1
                        var change = p * dw;
                        w += change;

                        aColumn.width = Math.max(w, aColumn.minWidth);
                        self._header.find("th").eq(idx).width(aColumn.width);
                        rowZero.find('.first-column').eq(idx).width(aColumn.width);

                        changeTot += change;
                    }
                });
            }
        }


    },
    _addResizableColumns: function (aBOOL) {

        if (this._colresizer)
            this._colresizer.remove();

        this.element.find(".col-resize").remove();
        this._colresizing = -1;

        if (aBOOL) {
            this._colresizer = $("<div>")
                .addClass("ui-table-column-resizer")
                .appendTo(this.element).hide();

            $.each(this._columns, _.bind(function (idx, col) {

                if (!this._columns[idx].fixed) {
                    var self = this;
                    $("<div>").addClass("col-resize").height(self.options.headerHeight)
                        .prependTo(self._header.find("th").eq(idx))
                        .mousedown(function (evt) {
                            evt.preventDefault();
                            $("body").css("cursor", "ew-resize");
                            self._colstart = $(this).parent().position().left +
                            $(this).position().left + 10.0 + self._header.position().left;
                            self._colresizer.css("left", self._colstart);
                            self._colresizer.show();
                            self._colresizing = idx;


                            $(document).on("mousemove." + self._ens, function (evt) {
                                if (self._colresizing > -1) {
                                    self._colresizer.css("left", evt.pageX - self.element.offset().left);
                                }
                                else {
                                    self._doneColResizing = false;
                                }
                            });


                        }).click(function (evt) {
                            evt.stopPropagation();
                        });
                }
            }, this));
        }
    },
    _createHeader: function () {

        this._header = $("<table cellpadding='0' cellspacing='0'>")
            .addClass("ui-table-header")
            .appendTo(this.element);

        var tr = $("<tr>").appendTo(this._header);

        //get columns
        var columns = this.element.children(".appkit-table-column");

        this._columns = [];

        $.each(columns, _.bind(function (idx, col) {

            var colEl = $(col),
                w = (colEl.data('width') === undefined) ? 100 : colEl.data('width'),
                sortable = (colEl.data('sortable') === undefined) ? true : colEl.data('sortable');

            if (w === 0) {
                w = 100;
            }

            var columnLabelId = _.uniqueId('col-label');
            var h = $("<th tabindex='0'>").attr("data-col", idx)
                .attr('title', colEl.text())
                .attr('aria-labelledby', columnLabelId)
                .html($('<div>').attr('id', columnLabelId).html(colEl.html()))
                .tooltip({
                    tooltipClass: 'ui-datatable-column-tooltip',
                    position: {
                        at: "bottom center"
                    },
                    show: {
                        delay: 2000,
                        duration: 0
                    },
                    track: false
                })
                .on('focus', function () {
                    $(this).tooltip('open');
                })
                .appendTo(tr);

            if (sortable) {

                var doSort = _.bind(function (evt) {

                    this._scroller.scrollTop(0);

                    evt.preventDefault();
                    if (this._doneColResizing) {
                        this._doneColResizing = false;
                        return;
                    }

                    if (!this._data || this._data.length === 0) {
                        return;
                    }

                    if (_.isFunction(this.options.sortCallback)) {

                        if (!this.options.sortInfo)
                            this.options.sortInfo = {};

                        if (this.options.sortInfo.asc === undefined) {
                            this.options.sortInfo.asc = false;
                        } else {
                            this.options.sortInfo.asc = !this.options.sortInfo.asc;
                        }

                        this.options.sortInfo.column = this._columns[idx];

                        this.options.sortCallback(this.options.sortInfo);

                        this._doRender();

                        this.element.trigger("onSort", [this.options.sortInfo]);
                    }

                }, this);

                $('<div>').addClass('sort-indicator')
                    .css('top', (this.options.headerHeight - 24) / 2 - 1)
                    .appendTo(h);

                h.on('click', doSort)
                    .on('keydown', function (evt) {
                        if (evt.which == $.ui.keyCode.SPACE) {
                            doSort(evt);
                        }
                    })
                    .on('mousedown', function (evt) {
                    evt.preventDefault();
                });

            }

            if (idx === 0)
                h.addClass("first-column");
            if (idx == columns.length - 1)
                h.addClass("last-column");

            var formatter = colEl.data("formatter"),
                sorter = colEl.data("sorter"),
                id = colEl.data("id"),
                fixed = colEl.data("fixed"),
                stretchable = colEl.data("stretchable"),
                tooltip = colEl.data('tooltip'),
                minW = colEl.data("min-width");

            if (!id) {
                console.warn('Column %d in table %s does not have a data-id.', idx, this.element.attr('id'));
            }

            this._columns.push({
                index: idx,
                id: id,
                width: w,
                minWidth: (minW === undefined) ? 25 : minW,
                fixed: (fixed === undefined) ? false : fixed,
                stretchable: (stretchable === undefined) ? true : stretchable,
                sortable: sortable,
                formatter: formatter,
                sorter: sorter,
                tooltip: tooltip
            });
        }, this));

        //add in last column
        $("<th>").html("&nbsp;").addClass("dummy-column").appendTo(tr);

        columns.hide();

        this._addResizableColumns(this.options.resizableColumns);

        this._header.height(this.options.headerHeight);
    },
    _createBody: function () {

        var self = this;

        this._scroller = $("<div tabindex='-1'>").addClass("ui-table-scroller")
            .css("top", this.options.headerHeight)
            .on('scroll', function () {

                var st = $(this).scrollTop();

                var maxVerticalScrollHeight = $(this).get(0).scrollHeight - $(this).height();
                if (st >= maxVerticalScrollHeight) {

                    if (self._rowsRendered < self._numberOfRows()) {

                        self._loadingRowsMsg.show();

                        setTimeout(function () {
                            self._scroller.css('overflow', 'hidden'); //IE hack
                            self._renderBatchRows(250);
                            self._loadingRowsMsg.hide();
                            self._scroller.css('overflow', 'auto');

                        }, 100);
                    }

                }
            })
            .appendTo(this.element);


        this._body = $("<table cellpadding='0' cellspacing='0'>")
            .addClass("ui-table-body")
            .appendTo(this._scroller);



        this.adjustColumnWidths();

        this._scroller.css("top", this._header.height());

        this._scroller.on("keydown", function (evt) {

            if (evt.keyCode === $.ui.keyCode.UP) //select up
            {
                var selRow = self.selectedRow();
                if (selRow && selRow > 0) {
                    evt.preventDefault();
                    selRow--;
                    self.selectedRow(selRow);
                }
            }
            else if (evt.keyCode === $.ui.keyCode.DOWN) //select down
            {
                var selRow = self.selectedRow();
                if (selRow !== undefined) {
                    evt.preventDefault();
                    selRow++;
                    self.selectedRow(selRow);
                }
            }
            else if (evt.keyCode === $.ui.keyCode.SPACE) //trigger selection
            {
                self.element.trigger("selection");
            }
        });


        this._scroller.on("scroll", function () {
            self._header.css("left", self._body.position().left);
        });

        $(document).on("mouseup." + self._ens, function () {

            if (self.element.is(":visible")) {
                setTimeout(function () {
                    self._header.css("left", self._body.position().left);
                }, 0);
            }

            if (self._colresizing > -1) {

                $.each(self._columns, function (idx, col) {
                    if (idx === self._colresizing) {
                        var dw = self._colresizer.position().left - self._colstart;

                        if (_.isNumber(col.width)) {
                            col.width = Math.max(col.width + dw, col.minWidth);
                        }
                        else {
                            //column widths are percentages
                            var columnEl = self._header.find('[data-col="' + idx + '"]');
                            col.width = Math.max(columnEl.width() + dw, col.minWidth);
                        }
                    }
                });

                self._doneColResizing = true;
                self.adjustColumnWidths(true);

                $('body').css('cursor', 'default');
                $(document).off("mousemove." + self._ens);

                self._colresizer.hide();
                self._colstart = 0;
                self._colresizing = -1;
            }
        });


    },
    selectedRow: function (rowIndex) {

        if (this.options.selectable) {
            if (rowIndex !== undefined && rowIndex !== this._selectedRow) {

                var row = this._body.find(".row-" + rowIndex);

                if (row.length) {
                    this._body.find("tr").removeClass("selected");
                    row.addClass("selected");
                    var dy = row.position().top;

                    if (dy >= 0)
                        dy = Math.max(0, row.position().top + row.outerHeight() - this._scroller.outerHeight());


                    if (dy != 0)
                        this._scroller.scrollTop(this._scroller.scrollTop() + dy);

                    this._selectedRow = rowIndex;
                    this.element.trigger("selectionChange");
                }
            }

            return this._selectedRow;

        }
        return undefined;
    },
    showMessage: function (msg) {

        if (!this._msg) {
            this._msg = $("<div>").addClass("ui-table-message").appendTo(this.element);

        }
        if (msg !== undefined) {
            this._msg.html("<div class='row'><div class='content'>" + msg + "</div></div>");
        }
        this._msg.css('display', 'table');
    },
    hideMessage: function () {
        if (this._msg) {
            this._msg.remove();
            this._msg = null;
        }
    },
    _getValue: function (column, model) {
        if (this._data
            && column > -1
            && column < this._columns.length) {

            var modelID = this._columns[column].id,
                formatter = this._columns[column].formatter;
            if (this._delegate && _.isString(formatter)) {
                formatter = this._delegate[formatter];
            }

            var value = model.get(modelID);

            var final = _.isFunction(formatter) ? formatter(value, model) : value;
            return final ? final : "";
        }

        return "";
    },
    _doRender: function () {

        this._lastRowRendered = 0;
        this._lastGroupRendered = 0;
        this._rowsRendered = 0;

        if (this.options.groupBy) {

            this._groupByRender();
        }
        else {
            this._standardRender();
        }


    },
    _groupByRender: function () {

        if (this._data && this._data.data) {

            this._header.find(".sort-indicator").hide();

            if (this.options.sortInfo) {
                var sortIndicator = this._header.find(".sort-indicator").eq(this.options.sortInfo.column.index);

                if (this.options.sortInfo.asc) {
                    sortIndicator.removeClass('icon-chevron-down');
                    sortIndicator.addClass('icon-chevron-up');
                }
                else {

                    sortIndicator.addClass('icon-chevron-down');
                    sortIndicator.removeClass('icon-chevron-up');
                }

                sortIndicator.css('display', 'inline-block');
            }

            this._body.children().remove();
            this.hideMessage();
            this._body.empty();

            this._renderBatchRowsGrouped(60, true);
        }
    },
    _standardRender: function () {

        if (this._data) {
            //sort info

            this._header.find(".sort-indicator").hide();

            if (this.options.sortInfo) {
                var sortIndicator = this._header.find(".sort-indicator").eq(this.options.sortInfo.column.index);

                if (this.options.sortInfo.asc) {
                    sortIndicator.removeClass('icon-chevron-down');
                    sortIndicator.addClass('icon-chevron-up');
                }
                else {

                    sortIndicator.addClass('icon-chevron-down');
                    sortIndicator.removeClass('icon-chevron-up');
                }

                sortIndicator.css('display', 'inline-block');
            }

            this._body.children().remove();

            this.hideMessage();

            this._body.empty();

            this._renderBatchRowsStandard(60, true);

        }
    },
    _renderBatchRows: function (batch) {

        if (this.options.groupBy) {

            this._renderBatchRowsGrouped(batch, false);
        }
        else {
            this._renderBatchRowsStandard(batch, false);
        }
    },
    _renderBatchRowsGrouped: function (batch, first) {

        this.element.trigger("beforeRender");

        var colCount = this._columns.length;

        var rows = "";

        if (first) {
            rows += "<tr role='row' aria-hidden='true' class='dummy-row-1' style='height:0px;'><td class='first-column' colspan='" + colCount + 1 + "'></td></tr>";
        }

        var firstRow = this._lastRowRendered,
            lastRow = 0,
            rowsRemaining = Math.min(batch, this._flatData.length - this._rowsRendered);

        if (rowsRemaining > 0) {

            var data = this._data.data.slice(this._lastGroupRendered, this._data.data.length);
            var group = this._lastGroupRendered;

            _.each(data, _.bind(function (item) {

                if (rowsRemaining > 0) {
                    //add in group row
                    var key = _.keys(item)[0];

                    lastRow = Math.min(item[key].length, firstRow + rowsRemaining);

                    var value = item[key].slice(firstRow, lastRow);

                    if (value.length) {
                        var groupID = this._lastGroupID;

                        if (!groupID) {
                            groupID = _.uniqueId('data-group-');
                        }

                        if (firstRow === 0) {
                            var keyHTML = key;
                            if (_.isFunction(this.options.groupFormatter)) {
                                keyHTML = this.options.groupFormatter(key);
                            }

                            rows += String.format('<tr role="row" class="group-row">', groupID);
                            rows += String.format('<td colspan="{0}"><div data-group-key="{1}" data-group-id="{2}" class="icon-minus group-toggle"></div>{3}</td></tr>',
                                colCount + 1, key, groupID, keyHTML);

                        }

                        rows += this._renderArrayOfModels(value, this._rowsRendered, groupID);

                        if (lastRow < item[key].length) {
                            this._lastRowRendered = lastRow;
                            this._lastGroupID = groupID;
                        }
                        else {
                            this._lastRowRendered = 0;
                            this._lastGroupID = undefined;
                            group++;
                        }

                        rowsRemaining -= value.length;
                        this._rowsRendered += value.length;

                        this._lastGroupRendered = group;
                    }

                    firstRow = 0;
                }

            }, this));

            this._body.find('*').off();
            this._body.append(rows);

            var self = this;
            //selection
            this._body.find("tr").not('.dummy-row-1').on("click", function (evt) {
                var row = $(this).data("row-id");
                if (row !== self.selectedRow()) {
                    self.selectedRow(row);
                    self.element.trigger("selection");
                }
            }).on('mouseover', function () {
                $(this).addClass('ui-state-hover');
            }).on('mouseout', function () {
                $(this).removeClass('ui-state-hover');
            });

            //group toggle

            if (this._data.expandState) {
                _.each(this._data.data, _.bind(function (item) {
                    var key = _.keys(item)[0];

                    if (this._data.expandState[key]) {
                        if (this._data.expandState[key][this._data.currentPage] === false) {
                            var toggleEl = this._body.find('div[data-group-key="' + key + '"]');
                            toggleEl.removeClass('icon-minus');
                            toggleEl.addClass('icon-plus');
                            var groupID = toggleEl.data('group-id');
                            this._body.find('tr[data-group-id="' + groupID + '"]').hide();
                        }
                    }
                }, this));

            }

            this._body.find('.group-toggle')
                .button({text: false});


            this._body.find('.group-row').on('mousedown', function (evt) {
                evt.preventDefault();
            }).on('click', function () {

                var toggle = $(this).find('.group-toggle');
                var groupID = toggle.data('group-id');

                if (toggle.hasClass('icon-minus')) { //group is expanded, collapse

                    self._body.find('tr[data-group-id="' + groupID + '"]').hide();

                    toggle.removeClass('icon-minus');
                    toggle.addClass('icon-plus');

                    self.element.trigger('onGroupCollapsed', toggle.data('group-key'));
                }
                else { //group is collapsed, expand

                    self._body.find('tr[data-group-id="' + groupID + '"]').show();

                    toggle.removeClass('icon-plus');
                    toggle.addClass('icon-minus');

                    self.element.trigger('onGroupExpanded', toggle.data('group-key'));
                }
            });

            this.adjustColumnWidths(true);
            this._renderTooltips();

            this.element.trigger("afterRender");
        }


    },
    _renderBatchRowsStandard: function (batch, first) {

        var colCount = this._columns.length;

        var rows = "";

        this.element.trigger("beforeRender");

        if (first) {
            rows += "<tr role='row' aria-hidden='true' class='dummy-row-1' style='height:0px;'><td class='first-column' colspan='" + colCount + 1 + "'></td></tr>";
        }

        var firstRow = this._lastRowRendered;
        this._lastRowRendered = Math.min(this._data.length, firstRow + batch);

        var toRender = this._data.slice(firstRow, this._lastRowRendered);


        if (toRender.length) {

            rows += this._renderArrayOfModels(toRender, firstRow);

            this._rowsRendered += toRender.length;

            this._body.find('*').off();
            this._body.append(rows);


            //selection
            var self = this;
            this._body.find("tr").not('.dummy-row-1').on("click", function (evt) {
                var row = $(this).data("row-id");
                if (row !== self.selectedRow()) {
                    self.selectedRow(row);
                    self.element.trigger("selection");
                }
            }).on('mouseover', function () {
                $(this).addClass('ui-state-hover');
            }).on('mouseout', function () {
                $(this).removeClass('ui-state-hover');
            });

            this.adjustColumnWidths(true);

            this._renderTooltips();

            this.element.trigger("afterRender");
        }

    },
    _renderTooltips: function () {

        var count = this._columns.length,
            i = 0;

        for (; i < count; i++) {
            var tt = this._columns[i].tooltip,
                colid = this._columns[i].id;

            if (tt) {
                var self = this;
                if (_.isFunction(tt)) {
                    this._body.find('td[data-col-id="' + colid + '"]').each(function (row, el) {

                        var model = self._getModelAtRow(row);
                        var content = tt(row, model);


                        if (content) {

                            if ($(el).tooltip('instance')) {
                                $(el).tooltip('destroy');
                            }

                            $(el).tooltip({
                                content: content,
                                show: {
                                    delay: 1100,
                                    duration: 0,
                                    easing: false,
                                    effect: null
                                },
                                hide: false,
                                items: 'td',
                                position: {my: "center top", at: "center bottom+5"}
                            });
                        }
                    });
                }
                else {
                    this._body.find('td[data-col-id="' + colid + '"]').each(function (row, rowEl) {

                        if ($(rowEl).tooltip('instance')) {
                            $(rowEl).tooltip('destroy');
                        }

                        $(rowEl).tooltip({
                            content: tt,
                            show: {
                                delay: 1100,
                                duration: 0,
                                easing: false,
                                effect: null
                            },
                            hide: false,
                            items: 'td',
                            position: {my: "center top", at: "center bottom+5"}
                        });
                    });
                }
            }
        }

    },
    _renderArrayOfModels: function (anArray, startIndex, groupID) {

        var colCount = this._columns.length;

        var idx = startIndex,
            count = anArray.length;

        var rows = "";

        var N = this._numberOfRows();

        _.each(anArray, _.bind(function (model) {

            if (groupID) {
                if (idx % 2 === 0) {

                    if (idx === N - 1)
                        rows += String.format('<tr tabindex="0" role="row" class="last-row" data-group-id="{0}">', groupID);
                    else
                        rows += String.format('<tr tabindex="0" role="row" data-group-id="{0}">', groupID);
                }
                else {
                    if (idx === N - 1)
                        rows += String.format('<tr tabindex="0"  role="row" class="row-odd last-row" data-group-id="{0}">', groupID);
                    else
                        rows += String.format('<tr tabindex="0" role="row" class="row-odd" data-group-id="{0}">', groupID);
                }
            }
            else {

                if (idx % 2 === 0) {
                    if (idx === N - 1)
                        rows += ('<tr tabindex="0" role="row" class="last-row">');

                    else
                        rows += ('<tr tabindex="0" role="row">');
                }
                else {
                    if (idx === N - 1)
                        rows += ('<tr tabindex="0" role="row" class="row-odd last-row" >');
                    else
                        rows += ('<tr tabindex="0" role="row" class="row-odd">');
                }
            }

            for (var i = 0; i < colCount + 1; i++) {

                if (i < colCount) {
                    var inner = this._getValue(i, model);

                    if (i === 0) {
                        rows += String.format('<td role="gridcell" class="first-column" data-col-id="{0}" data-model-id="{1}">{2}</td>', this._columns[i].id, model.cid, inner);
                    }
                    else if (i === colCount - 1) {
                        rows += String.format('<td role="gridcell" class="last-column"  data-col-id="{0}" data-model-id="{1}">{2}</td>', this._columns[i].id, model.cid, inner);
                    }
                    else {
                        rows += String.format('<td role="gridcell" data-col-id="{0}" data-model-id="{1}">{2}</td>', this._columns[i].id, model.cid, inner);
                    }
                }
                else {
                    //empty column
                    rows += String.format('<td role="gridcell" aria-hidden="true" class="dummy-column" data-row-id="{0}">&nbsp;</td>', idx);
                }
            }
            rows += "</tr>";
            idx++;

        }, this));

        return rows;
    },
    sizer: function () {
        this.adjustColumnWidths();

    },
    _getModelAtRow: function (row) {
        if (!this._data)
            return;

        if (this.options.groupBy) {

            if (!this._flatData || this._flatData.length === 0) {
                return;
            }

            return this._flatData[row];
        }

        return this._data[row];
    },
    setData: function (anArray, controller) {

        if (!anArray)
            return;

        this._data = anArray;

        this._flatData = [];

        if (this.options.groupBy && this._data.data) {

            var i = 0,
                count = this._data.data.length;

            for (; i < count; i++) {
                var obj = this._data.data[i];
                var vals = _.values(obj);
                if (vals.length > 0) {
                    this._flatData = _.union(this._flatData, vals[0]);
                }
            }
        }

        this._delegate = controller;
    },
    render: function (anArray, controller) {
        this._doRender();
    },
    refresh: function () {

        if (this._scroller)
            this._scroller.remove();

        if (this._header)
            this._header.remove();

        var namespace = this.element.attr("id");
        $(document).off("mouseup." + namespace)
            .off("mousemove." + namespace)
            .off("click." + namespace);

        this._createHeader();
        this._createBody();

        this.adjustColumnWidths();

        this._selectedRow = null;

    },
    _destroy: function () {

        $(document).off("mouseup." + this._ens).off("mousemove." + this._ens);

        this._super();

    },
    _numberOfRows: function () {

        if (this.options.groupBy) {

            return this._flatData.length;

        }
        else {
            return this._data.length;
        }

    },
    _setOption: function (key, value) {
        if (key === "hasHeader") {
            if (value == false) {
                this._header.hide();
                this._scroller.css("top", -1);
            }

            if (value === true) {
                this._header.show();
                this._scroller.css("top", this._header.height());
            }
        }

        if (key === "selectable") {
            if (value) {
                this.element.attr("tabindex", 0);
                this._scroller.attr("tabindex", -1);
            } else {
                this.element.removeAttr("tabindex");
                this._scroller.removeAttr("tabindex");
            }
        }

        if (key === "headerHeight") {
            this._header.height(value);
            this._header.find(".col-resize").height(value);
            this._scroller.css("top", this._header.height());
        }

        if (key === "fitColumns") {

            this.options.fitColumns = value;

            if (value) {
                this.adjustColumnWidths();
            }
        }

        if (key === "resizableColumns") {
            this._addResizableColumns(value);
        }

        this._super(key, value);
    }

});


