//= require ./Control.js
//= require ../jquery/combobox.js


AppKit.ComboBox = AppKit.Control.extend({
    classID: 'appkit-combo',
    options: {
        placeholder: '',
        radio: false,
        tabindex: 0,
        menu: undefined
    },
    initialize: function (options) {
        this.$el.combobox();

        this._super(options);

        var self = this;

        this.input().bind('focus change keydown paste', function () {
            AppKit.Form.clearValidationWarnings();
        }).on('change keydown paste', function () {
            setTimeout(function () {
                self.$el.trigger("valueChanged", [self.get('value')]);
            }, 0);
        }).on('mousedown', function (evt) {
            if (!self.get('enabled')) {
                evt.preventDefault();
                evt.stopPropagation();
            }
        });

        this.on('selection', function () {
            self.set('value', self.get('selectedValue'));
            self.input().focus();
        });

    },
    events: {
        selection: 'fire',
        focus: function () {
            this.input().focus();
        }
    },
    configureMenu: function (config) {
        if (_.isObject(config)) {
            var m = new AppKit.Menu(config);
            this.$el.combobox('setMenu', m.$el);
        }
    },
    input: function () {
        return this.$el.find('input');
    },
    selectedIndex: function (idx) {

        if (idx !== undefined) {
            this.$el.combobox('selected', idx, true);
        }

        return this.$el.combobox('selected');
    },
    selectedValue: function (aString) {
        if (aString !== undefined) {
            this.$el.combobox('selectedValue', aString, true);
        }

        return this.$el.combobox('selectedValue');
    },
    placeholder: function (aString) {
        if (aString !== undefined) {
            this.$el.combobox('option', 'placeholder', aString);
        }
        return this.$el.combobox('option', 'placeholder');
    },
    editable: function (aBool) {
        if (aBool === false) {
            this.input().attr('readonly', 'true');
        }
        else if (aBool == true) {
            this.input().removeAttr('readonly');

        }

        return this.input().hasAttr('readonly');
    },
    enabled: function (aBool) {
        if (aBool == true) {
            this.$el.combobox('option', 'enabled', true);
            this.$el.attr('tabindex', 0);
        }
        else if (aBool == false) {
            this.$el.combobox('option', 'enabled', false);
            this.$el.attr('tabindex', -1);
        }
        return !this.$el.hasClass('disabled');
    },
    radio: function (aBOOL) {
        if (aBOOL !== undefined)
            this.$el.combobox('option', 'radio', aBOOL);

        return this.$el.combobox('option', 'radio');
    },
    clearValidationWarning: function () {
        if (this.input().hasClass('validation-error')) {
            this.input().removeClass('validation-error');
            this.input().tooltip('destroy');
            this.input().removeAttr('title');
        }
    },
    validate: function (aValidator) {
        if (!_.isFunction(aValidator))
            return true;

        var message = aValidator(this.value());

        if (message) {
            if (this.input().hasClass('validation-error'))
                this.input().tooltip('destroy');

            this.input().tooltip({
                content: message,
                items: 'input',
                show: {
                    duration: 0,
                    easing: false,
                    effect: null
                },
                position: {
                    my: "left center",
                    at: "right+10 center",
                    collision: "flipfit",
                    using: function (position, feedback) {
                        $(this).css(position);
                        $("<div>")
                            .addClass("arrow")
                            .addClass(feedback.vertical)
                            .addClass(feedback.horizontal)
                            .appendTo(this);
                    }
                },
                tooltipClass: "right"
            });

            this.input().addClass('validation-error');
            this.input().tooltip('open');


            return false;
        }

        return true;
    },
    value: function (aValue) {
        if (aValue !== undefined) {
            this.$el.combobox('value', aValue);
            this.clearValidationWarning();
        }

        return this.$el.combobox('value');
    }
});