//= require ./Control.js
//= require ../jquery/datatable.js

AppKit.TableView = AppKit.Control.extend({
    classID: 'appkit-table',
    options: {
        tabindex: null,
        controller: null,
        source: null,
        header: true,
        headerHeight: 28,
        fitColumns: false,
        resizableColumns: true,
        selectable: false,
        sortable: true,
        filter: undefined,
        groupBy: undefined,
        groupFormatter: undefined,
        groupSorter: undefined
    },
    initialize: function (options) {

        var self = this;
        this.$el.datatable({
            sortCallback: function (sortInfo) {
                self.sortByColumn(sortInfo);
            }
        });

        this._super(options);
    },
    destroy: function () {

        if (this._collection) {
            this._collection.off('reset remove add', this._update, this);
        }

        this._collection = null;

        this._super();
    },
    addColumn: function (config) {

        if (this.$el.datatable('instance')) {
            this.$el.datatable('addColumn', config);
            this.$el.datatable('refresh');
        }


    },
    removeColumn: function (columnId) {
        if (columnId) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable("removeColumn", columnId);
            }
        }
    },
    removeAllColumns: function () {
        if (this.$el.datatable('instance')) {
            this.$el.datatable("removeAllColumns");
        }
    },
    delegate: function (aDelegate) {

        if (aDelegate !== undefined) {
            this._delegate = AppKit.get(aDelegate);
        }
        return this._delegate;
    },
    source: function (aCollection) {

        if (_.isString(aCollection)) {

            if (this._collection) {
                this._collection.off('reset remove add', this._update, this);
            }

            this._collection = AppKit.get(aCollection);
            if (this._collection) {
                this._collection.on('reset remove add', this._update, this);
            }
        }
        else if (aCollection instanceof Backbone.Collection) {

            if (this._collection) {
                this._collection.off('reset remove add', this._update, this);
            }

            this._collection = aCollection;
            if (this._collection) {
                this._collection.on('reset remove add', this._update, this);
            }
        }

        return this._collection;
    },
    filter: function (aFilterFunction) {
        if (aFilterFunction !== undefined) {
            this._filter = aFilterFunction;
            if (this._cachedData !== null) {
                this._update();
            }
        }

        return this._filter;
    },
    groupBy: function (aString) {

        if (aString !== undefined) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'groupBy', aString);
                this._update();
            }
        }
        if (this.$el.datatable('instance')) {
            return this.$el.datatable('option', 'groupBy');
        }
    },
    groupFormatter: function (aFunction) {

        if (aFunction !== undefined) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'groupFormatter', aFunction);
                this._update();
            }
        }
        if (this.$el.datatable('instance')) {
            return this.$el.datatable('option', 'groupFormatter');
        }

    },
    groupSorter: function (aFunction) {

        if (aFunction !== undefined) {
            this._groupSorter = aFunction;
            this._update();
            this.render();
        }

        return this._groupSorter;
    },
    header: function (aBOOL) {
        if (aBOOL !== undefined) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'hasHeader', aBOOL);
            }
        }
        if (this.$el.datatable('instance')) {
            this.$el.datatable('option', 'hasHeader');
        }
    },
    headerHeight: function (aNumber) {
        if (aNumber !== undefined) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'headerHeight', aNumber);
            }
        }
        if (this.$el.datatable('instance')) {
            return this.$el.datatable('option', 'headerHeight');
        }
        return 0;
    },
    fitColumns: function (aBOOL) {
        if (aBOOL !== undefined) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'fitColumns', aBOOL);
            }
        }
        if (this.$el.datatable('instance')) {
            return this.$el.datatable('option', 'fitColumns');
        }
        return false;
    },
    resizableColumns: function (aBOOL) {
        if (aBOOL !== undefined) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'resizableColumns', aBOOL);
            }
        }
        if (this.$el.datatable('instance')) {
            return this.$el.datatable('option', 'resizableColumns');
        }
        return false;
    },
    selectable: function (aBOOL) {
        if (aBOOL !== undefined) {

            if (aBOOL) {
                this.$el.attr('tabindex', this.enabled() ? 0 : -1);
            } else {
                this.$el.removeAttr('tabindex');
            }
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'selectable', aBOOL);
            }
        }
        if (this.$el.datatable('instance')) {
            return this.$el.datatable('option', 'selectable');
        }
        return false;
    },
    sortable: function (aBOOL) {
        if (aBOOL !== undefined) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'sortable', aBOOL);
            }
        }
        if (this.$el.datatable('instance')) {
            return this.$el.datatable('option', 'sortable');
        }

        return false;
    },
    sortByColumn: function (config) {

        if (this.get('sortable')) {

            if (!this._cachedData) {
                this._cachedData = this.tableData();
            }

            this.$el.datatable('option', 'sortInfo', _.clone(config));

            var self = this;

            //apply sorting info to array
            var sortInfo = this.$el.datatable('option', 'sortInfo');
            if (sortInfo) {

                if (this.get('groupBy') && this._cachedData.data) {
                    //sort grouped data
                    _.each(this._cachedData.data, function (item) {

                        //add in group row
                        var key = _.keys(item)[0],
                            value = item[key];

                        value.sort(function (model1, model2) {
                            var modelID = sortInfo.column.id;
                            var val1 = model1.get(modelID),
                                val2 = model2.get(modelID);

                            if (!val1)
                                return 1;

                            if (!val2)
                                return -1;


                            var ret = 0;

                            var sorter = sortInfo.column.sorter;

                            if (_.isString(sorter)) {
                                if (self._delegate) {
                                    sorter = self._delegate[sorter];
                                }
                            }


                            if (_.isFunction(sorter)) {
                                ret = sorter(val1, val2);

                                //maintain a stable sort, sort by model id
                                if (ret == 0) {
                                    return model1.cid >= model2.cid ? 1 : -1;
                                }

                                return sortInfo.asc ? ret : -1.0 * ret;
                            }

                            //maintain a stable sort, sort by model id
                            if (val1 == val2) {
                                return model1.cid <= model2.cid ? 1 : -1;
                            }

                            ret = (val1 <= val2) ? -1 : 1;
                            return sortInfo.asc ? ret : -1.0 * ret;
                        });

                    });
                }
                else {
                    this._cachedData.sort(function (model1, model2) {

                        var modelID = sortInfo.column.id;
                        var val1 = model1.get(modelID),
                            val2 = model2.get(modelID);

                        if (!val1)
                            return 1;

                        if (!val2)
                            return -1;

                        var ret = 0;

                        var sorter = sortInfo.column.sorter;

                        if (_.isString(sorter)) {
                            if (self._delegate) {
                                sorter = self._delegate[sorter];
                            }
                        }

                        if (_.isFunction(sorter)) {
                            ret = sorter(val1, val2);

                            //maintain a stable sort, sort by model id
                            if (ret == 0) {
                                return model1.cid >= model2.cid ? 1 : -1;
                            }

                            return sortInfo.asc ? ret : -1.0 * ret;
                        }

                        //maintain a stable sort, sort by model id
                        if (val1 == val2) {
                            return model1.cid >= model2.cid ? 1 : -1;
                        }

                        ret = (val1 <= val2) ? -1 : 1;
                        return sortInfo.asc ? ret : -1.0 * ret;

                    });
                }

                this._initData();
            }

        }

    },
    showMessage: function (msg) {
        if (this.$el.datatable('instance')) {
            this.$el.datatable('showMessage', msg);
        }
    },
    hideMessage: function () {
        if (this.$el.datatable('instance')) {
            this.$el.datatable('hideMessage');
        }

    },
    sizer: function () {
        if (this.$el.datatable('instance')) {
            this.$el.datatable('sizer');
        }
    },
    render: function () {
        if (this.$el.datatable('instance')) {
            this.$el.datatable('render');
        }
    },
    rowCount: function () {
        return this.tableData().length;
    },
    _update: function () {
        this._cachedData = null;
        this._initData();

        if (this._cachedData.length === 0) {
            if (this.$el.datatable('instance')) {
                this.$el.datatable('option', 'sortInfo', undefined);
                this.render();
            }
        }
    },
    tableData: function () {

        if (!this._collection) {
            this._cachedData = [];
        }

        if (!this._cachedData) {
            //get the data as an array (apply filter)

            if (_.isFunction(this._filter)) {
                this._cachedData = this._collection.filter(this._filter);
            }
            else if (_.isString(this._filter)) {
                if (this._delegate && _.isFunction(this._delegate[this._filter])) {
                    var filter = this._delegate[this._filter];
                    this._cachedData = this._collection.filter(filter);
                }
            }
            else {
                this._cachedData = this._collection.toArray();
            }


            //group data
            var groupBy = this.get('groupBy')
            if (groupBy) {
                this._cachedData.sort(_.bind(function (model1, model2) {
                    var val1 = model1.get(groupBy),
                        val2 = model2.get(groupBy);

                    if (_.isFunction(this.get('groupSorter'))) {
                        var f = this.get('groupSorter');
                        return f(val1, val2);
                    }

                    return (val1 >= val2) ? -1 : 1;
                }, this));

                var groups = _.groupBy(this._cachedData, _.bind(function (model) {
                    return model.get(this.get('groupBy'));
                }, this));

                this._cachedData = {length: this._cachedData.length, data: []};

                _.each(groups, _.bind(function (value, aKey) {
                    var val = {};
                    val[aKey] = value;
                    this._cachedData.data.push(val);
                }, this));
            }
        }


        return this._cachedData;
    },
    _initData: function () {

        var data = this.tableData();
        if (this.$el.datatable('instance')) {
            this.$el.datatable('setData', data, this._delegate);
        }

        return data;
    }
});