/**
 *
 * controller for patient search
 * dialog and form
 *
 * **/

JLV.PatientSearch = {};

JLV.PatientSearch.Controller = {
    init: function () {
        this.identifierInput.$el.mask('999-99-9999');

        this.sponsorSSNInput.$el.mask('999-99-9999');
        this.telephoneInput.$el.mask('(999)999-9999')

        this.identifierInput.on('valueChanged', _.bind(function (evt, value) {

            this.sponsorSSNInput.set('value', '');

            value = value.replace(/-|_/g, '');
            if (value.length > 0) {
                this.sponsorSSNInput.set('enabled', false);
            }
            else {
                this.sponsorSSNInput.set('enabled', true);
            }
        }, this));

        this.identifierInput.on('blur', _.bind(function (evt) {
            if (this.identifierInput.get('value').replace(/-/g, '').length === 0)
                this.sponsorSSNInput.set('enabled', true);
        }, this));

        this.sponsorSSNInput.on('valueChanged', _.bind(function (evt, value) {
            value = value.replace(/-|_/g, '');

            this.identifierInput.set('value', '');

            if (value.length > 0) {
                this.identifierInput.set('enabled', false);
            }
            else {
                this.identifierInput.set('enabled', true);
            }
        }, this));

        this.sponsorSSNInput.on('blur', _.bind(function (evt) {

            if (this.sponsorSSNInput.get('value').replace(/-/g, '').length === 0)
                this.identifierInput.set('enabled', true);

        }, this));

    },
    onDialogOpen: function (aDialog) {
        aDialog.center();

        if (this.tabView) {
            this.tabView.set('activeTab', 0);
        }

        /* update recently viewed */
        var self = this;
        JLV.PatientSearch.RecentlyViewed.fetch({
            reset: true,
            success: function (collection) {
                if (collection.length === 0) {
                    return self.recentlyViewedTable.showMessage('No Recently Viewed Patients');
                }
                else {
                    self.recentlyViewedTable.render();
                }
            },
            error: function () {
                self.recentlyViewedTable.showMessage('No Recently Viewed Patients');
            }
        });

        setTimeout(_.bind(function () {
            this.edipiInput.$el.focus();
        }, this), 0);



    },
    onDialogClose: function () {

        //reset form
        this.dialog.$el.find('input').val('');

        this.sponsorSSNInput.set('enabled', true);
        this.identifierInput.set('enabled', true);

        //hide advanced
        $('#AdvancedSearchOptions').hide();
        $('#ShowAdvancedBtn').text('<< Show Advanced Search Options');
        this.dialog.set('height', 580);
        $('#PatientSearchTabView').height(255);

        //clear search results
        JLV.PatientSearch.Results.reset();
    },
    /**
     * validates the search form for required fields
     * @returns boolean true if valid form, false otherwise
     */
    validateSearchForm: function () {

        //check if edipi is used
        var edipi = this.edipiInput.validate(function (value) {

            if (value.trim().length > 0) { //edipi has value
                // EDIPI is all digits (0-9) and has 10 digits
                if (!(/^\+?([0-9]\d*)$/.test(value) && value.length == 10)) {
                    return 'DoD ID must be 10 digits.';
                }
            }
        });

        //using edipi
        if (this.edipiInput.get('value').trim().length > 0) {
            return edipi;
        }

        //check if lastName or DOB is used

        var self = this;
        var isValid = false;

        //check if SSN is used.
        if (this.identifierInput.get('value').replace(/-|_/g, '').trim().length > 0 ||
            this.sponsorSSNInput.get('value').replace(/-|_/g, '').trim().length > 0) {

            if (this.lastNameInput.get('value').length > 0) //validate last name
            {
                isValid = this.lastNameInput.validate(function (value) {

                    if (!value || value.length === 0)
                        return 'When searching by SSN, last name or DOB is required.';

                });

            }
            else { //validate DOB

                if (this.dobInput.$el.is(':visible')) {
                    isValid = this.dobInput.validate(function (value) {

                        if (!value || value.replace(/\//g, '').trim().length === 0)
                            return 'When searching by SSN, last name or DOB is required.';

                        if (!AppKit.DatePicker.dateFromString(value, self.dobInput.get('format'))) {
                            return "Invalid date";
                        }
                    });
                }


                if (!isValid) {
                    isValid = this.lastNameInput.validate(function (value) {
                        if (!value || value.length === 0)
                            return 'When searching by SSN, last name or DOB is required.';

                    });
                }
            }

        }
        //SSN not used check if lastname is used,
        else if (this.lastNameInput.get('value').trim().length > 0) {

            //validate DOB, lastname

            isValid = self.firstNameInput.validate(function (value) {
                if (!value || value.length === 0)
                    return 'Required Field';
            });

            if (self.dobInput.$el.is(':visible')) {
                isValid = self.dobInput.validate(function (value) {
                    if (!value || value.replace(/\//g, '').trim().length === 0)
                        return 'Required Field';


                    if (!AppKit.DatePicker.dateFromString(value, self.dobInput.get('format'))) {
                        return "Invalid date";
                    }

                }) && isValid;
            }

            if (self.genderInput.$el.is(':visible')) {
                isValid = self.genderInput.validate(function (value) {
                    if (!value || value.length === 0)
                        return 'Required Field';
                }) && isValid;

            }

        }
        else {
            isValid = this.identifierInput.validate(function (value) {
                value = value.replace(/-|_/g, '');
                if (!value || value.trim().length === 0)
                    return 'Either SSN or DoD ID are required.';
            });

            isValid = this.edipiInput.validate(function (value) {
                if (!value || value.trim().length === 0)
                    return 'Either SSN or DoD ID are required.';
            }) && isValid;
        }

        if (!isValid)
            return false;

        //validate misc fields

        /* if (this.telephoneInput.$el.is(':visible')) {
            isValid = this.telephoneInput.validate(function (value) {

                value = value.replace(/(|)|-|_/g, '');
         if (value && value.length > 0 && value.length < 10) {
                    return "Please enter a 10 digit telephone number."
                }
            });
        }

        if (this.zipInput.$el.is(':visible')) {
            isValid = this.zipInput.validate(function (value) {

                if (value && value.length > 0 && value.length < 5) {
                    return "Please enter a valid 5 digit zip."
                }
            }) && isValid;
         } */


        return isValid;

    },
    /**
     *  action when the search form is submitted
     */
    onSearch: function () {


        //select patient search tab
        if (this.tabView) {
            this.tabView.set('activeTab', 1);
        }

        if (this.validateSearchForm()) {

            this.searchTable.showMessage('<div style="width:115px;margin:auto;"><div class="jlv-table-data-loading" style="float:left;width:22px;"></div><label style="display:inline-block;">Searching...</label></div>')

            var edipi = this.edipiInput.get('value');

            var data;

            if (edipi.trim().length > 0) //search with edipi
            {
                data = {
                    queryPatId: edipi.trim(),
                    queryPatIdType: 'EDIPI',
                    queryInquiryType: 'PR' // PDWS Person Retrieve
//                    queryFirstName: '',
//                    queryLastName: ''
                };

            }
            else { //search with name & ssn

                var searchType = "PS"; // PDWS Person Search
                var val = this.identifierInput.get('value').replace(/-/g, '');
                if (val.length === 0) {
                    val = this.sponsorSSNInput.get('value').replace(/-/g, '');
                    if (val.length > 0)
                        searchType = "FS" // PDWS Family Search
                }

                data = {
                    queryPatId: val,
                    queryPatIdType: 'SSN',
                    queryFirstName: _.escape(this.firstNameInput.get('value')),
                    queryLastName: _.escape(this.lastNameInput.get('value')),
                    queryDoB: _.escape(this.dobInput.get('value').replace(/\//g, '')),
                    queryGender: _.escape(this.genderInput.get('value')),
                    queryMI: _.escape(this.middleInitialInput.get('value')),
                    queryAddress: _.escape(this.addressInput.get('value')),
                    queryCity: _.escape(this.cityInput.get('value')),
                    queryState: _.escape(this.stateInput.get('value')),
                    queryZip: _.escape(this.zipInput.get('value')),
                    queryTel: _.escape(this.telephoneInput.get('value').replace(/\(|\)|\-/g, '')),
                    queryInquiryType: searchType
                };
            }


            var self = this;

            JLV.PatientSearch.Search(data, function (collection) {
                if (collection.length === 0) {
                    return self.searchTable.showMessage('No Results');
                }
                else {
                    self.searchTable.render();
                }
            });



        }
    },
    toggleAdvancedOptions: function () {
        var options = $('#AdvancedSearchOptions');
        if (options.is(':visible')) {
            options.hide();
            this.advSearchBtn.set('label', '<< Show Advanced Search Options');
            this.dialog.set('height', 580);
            $('#PatientSearchTabView').height(255);
        }
        else {
            options.show();
            this.advSearchBtn.set('label', '>> Hide Advanced Search Options');
            this.dialog.set('height', 620);
            $('#PatientSearchTabView').height(190);

        }

        this.dialog.center();
    },
    // patient name column formatter in patient search table
    patientNameFormatter: function (value, model, index) {
        return _.template('<a href="#" class="patient-name-search" data-index="{{index}}" data-patient-id="{{edipi}}">{{name}}</a>')(
            _.extend({index: index}, model.toJSON())
        );
    },
    closeSponsorResultsDialog: function () {
        this.sponsorResultsDialog.close();
    }
};

AppKit.launch(function () {
    JLV.PatientSearch.Controller.init();
});

/**
 *  Collection that stores patient search results
 */
JLV.PatientSearch.PatientCollection = Backbone.Collection.extend({
    model: Backbone.Model.extend({
        parse: function (data) {
            if (data.name) {
                data.name = data.name.replace(/,/g, ', ');
            }

            return data;
        }

    })
});

JLV.PatientSearch.Results = new JLV.PatientSearch.PatientCollection();

/**
 *  Collection that stores recently viewed patients
 */

JLV.PatientSearch.RecentlyViewedPatientCollection = JLV.PatientSearch.PatientCollection.extend({
    url: 'patientSearch/getRecentlyViewedPatients.json'
});

JLV.PatientSearch.RecentlyViewed = new JLV.PatientSearch.RecentlyViewedPatientCollection();


/** collection that stores sponsor dependent results patients **/

JLV.PatientSearch.SponsorResults = new JLV.PatientSearch.PatientCollection();


/**
 * Searches for patient with input data
 * @param data - JS object with attributes
 *          {
 *               queryPatId:  ,
 *               queryPatIdType:  ,
 *               queryFirstName:  ,
 *               queryLastName:
 *            };
 * @param callback - function to execute when search is complete
 * @constructor
 */

JLV.PatientSearch.Search = function (data, callback) {

    $('#PatientSearchTable').removeClass('jlv-sponsor-search-table');

    if (data.queryInquiryType === 'FS') //family search
    {
        $('#PatientSearchTable').addClass('jlv-sponsor-search-table');
    }

    JLV.PatientSearch.Results.fetch({
        data: data,
        type: 'POST',
        reset: true,
        url: 'patientSearch/list.json',
        success: callback,
        error: function (model, response) {
            console.error(response.responseText);
            AppKit.AlertDialog.alert('Patient Search Error',
                'An error occurred while trying to search for this patient. Please try again.',
                function () {
                    callback(model);
                }
            );
        }
    });
};

JLV.PatientSearch.FamilySearch = function (sponsorModel, callback) {

    var data = {
        queryPatId: sponsorModel.get('edipi'),
        queryPatIdType: 'EDIPI',
        queryInquiryType: 'FR'
    };

    JLV.PatientSearch.SponsorResults.fetch({
        data: data,
        type: 'POST',
        reset: true,
        url: 'patientSearch/list.json',
        success: callback,
        error: function (model, response) {
            console.error(response.responseText);
            AppKit.AlertDialog.alert('Patient Search Error',
                "An error occurred while trying to search for this sponsor's dependents. Please try again.",
                function () {
                    callback(model);
                }
            );
        }
    });

};


JLV.ActivePatientModel = null;


AppKit.launch(function () {

    var patientSearchTable = $('#PatientSearchTable').widget();
    var recentPatientsTable = $('#RecentlyViewedPatientTable').widget();
    var sponsorResultsTable = $("#SponsorResultsTable").widget();

    /**
     * after rendering the patient search table, add patient select handlers
     * */
    patientSearchTable.on('afterRender', function () {

        $('.patient-name-search').on('click', function () {

            var patientID = "" + $(this).data('patient-id');
            if (patientID) {
                var patientModels = patientSearchTable.get('source').where({edipi: patientID});
                if (patientModels.length) {

                    var patientModel = patientModels[0];

                    if ($('#PatientSearchTable').hasClass('jlv-sponsor-search-table')) {//show family

                        var d = $('#SponsorResultsDialog').widget();
                        $('#SponorResultsSponsorInfo').empty();

                        $('<tr>').html('<th>Name</th><th>SSN</th><th>Date of Birth</th><th>Gender</th>')
                            .appendTo($('#SponorResultsSponsorInfo'));
                        $('<tr>').html(_.template('<td>{{name}}</td><td>{{fmpssn}}</td><td>{{dob}}</td><td>{{gender}}</td>')(patientModel.toJSON()))
                            .appendTo($('#SponorResultsSponsorInfo'));

                        sponsorResultsTable.showMessage('<div style="width:115px;margin:auto;"><div class="jlv-table-data-loading" style="float:left;width:22px;"></div><label style="display:inline-block;">Searching...</label></div>')

                        d.center();
                        d.open();

                        JLV.PatientSearch.FamilySearch(patientModel, function (collection) {
                            if (collection.length === 0) {
                                return sponsorResultsTable.showMessage('No Results');
                            }
                            else {
                                sponsorResultsTable.render();
                            }
                        });

                    }
                    else {
                        JLV.SelectPatientFromPatientSearch(patientModel);
                        $('#PatientSearchDialog').widget().close();
                    }
                }

            }

        });
    });

    /**
     * after rendering the recently view patients table, add patient select handlers
     * */
    recentPatientsTable.on('afterRender', function () {

        $('.patient-name-search').on('click', function () {

            var patientID = "" + $(this).data('patient-id');
            if (patientID) {
                var patientModels = recentPatientsTable.get('source').where({edipi: patientID});
                if (patientModels.length)
                    JLV.SelectPatient(patientModels[0]);
            }
            $('#PatientSearchDialog').widget().close();
        });
    });

    /**
     * after rendering the sponsor results table, add patient select handlers
     * */
    sponsorResultsTable.on('afterRender', function () {

        $('.patient-name-search').on('click', function () {

            var patientID = "" + $(this).data('patient-id');
            if (patientID) {
                var patientModels = sponsorResultsTable.get('source').where({edipi: patientID});
                if (patientModels.length) {
                    JLV.SelectPatient(patientModels[0]);

                    $('#SponsorResultsDialog').widget().close();
                    $('#PatientSearchDialog').widget().close();
                }
            }


        });
    });


    //restrict keys on form input fields
    $('#PatientSearchEDIPIInput, #PatientSearchZipInput').keypress(function (evt) {
        if (!String.fromCharCode(evt.charCode).match(/[0-9]/g)
                && evt.keyCode != 8     // BACKSPACE-Delete previous character
                && evt.keyCode != 9     // TAB-Tab to next form field
                && evt.keyCode != 13    // ENTER-Initiate search button
                && evt.keyCode != 35    // END-Move cursor to end of field
                && evt.keyCode != 36    // HOME-Move cursor to beggining of field
                && evt.keyCode != 37    // LEFTARROW-Move cursor 1 character left
                && evt.keyCode != 39    // RIGHTARROW-Move cursor 1 character right
                && evt.keyCode != 46) { // DELETE-Delete next character
            return false;
        }
    });

    //restrict selected fields to only allow alphabet characters
    $('#PatientSearchLastNameInput, #PatientSearchFirstNameInput').keypress(function (evt) {
        if (!String.fromCharCode(evt.charCode).match(/[^0-9]/g)) {
            return false;
        }
    });

});

/**
 * validate SSN
 * @param value
 * @returns {boolean}
 */
function isValidSSN(value) {
    var re = /^([0-6]\d{2}|7[0-6]\d|77[0-2])([ \-]?)(\d{2})\2(\d{4})$/;
    if (!re.test(value)) {
        return false;
    }
    var temp = value;
    if (value.indexOf('-') != -1) {
        temp = (value.split('-')).join('');
    }
    if (value.indexOf(' ') != -1) {
        temp = (value.split(' ')).join('');
    }
    /*if (temp.substring(0, 3) == '000') {
     return false;
     }
     if (temp.substring(3, 5) == '00') {
     return false;
     }
     if (temp.substring(5, 9) == '0000') {
     return false;
     }*/
    return true;
}

