JLV.DetailsWindow = AppKit.Dialog.extend({
    className: "jlv-detail-dialog",
    options: {
        closable: true,
        maximizable: true,
        minimizable: true,
        width: 600,
        height: 400,
        url: null,
        modal: false,
        queryParams: null,
        print: true,
        clipboard: true,
        vlerDomain: false
    },
    initialize: function (options) {

        var self = this;
        var tools = $('<div>').addClass('jlv-window-tools');
        this._printBtn = $('<div title="Print" tabindex="0" class="jlv-window-toolbutton">')
            .append('<div class="icon-print">')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doPrint)) {
                    self.doPrint();
                }
            }).appendTo(tools);

        this._clipboardBtn = $('<div title="Copy to clipboard" tabindex="0" class="jlv-window-toolbutton">')
            .append('<div class="icon-clipboard2">')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doClipboard)) {
                    self.doClipboard();
                }
            }).appendTo(tools);

        if (JLV.Session.dodVLER) {

            this._vlerLoadBtn = $('<div title="Force VLER Data Refresh - This can take a while" tabindex="0" class="jlv-window-toolbutton vler">')
                .append('<div class="icon-refresh-alt">')
                .button()
                .on('click', function () {
                    //self.vlerLoadMessage("Community partner data retrieval in progress...");

                    AppKit.AlertDialog.yesNo('Confirm VLER Refresh',
                        'A forced refresh of community partner data may take a few minutes. Click Yes to continue. Click No to cancel.',
                        function (yes) {
                            if (yes) {
                                JLV.Session.forceRefresh = true;
                                if (_.isFunction(JLV.LoadVlerData)) {
                                    JLV.LoadVlerData();
                                }
                            }
                        }
                    );
                }).appendTo(tools);
        }

        this._super(options);

        this.$el.on('dialogclose', function () {
            $(this).remove(); //destroy on close
        });

        tools.appendTo(this.$el.parent().find('.ui-dialog-titlebar-tools'));

        JLV.Desktop.registerWindow(this);

        JLV.NotificationCenter.addObserverForNotification({
            name: 'closeNotification',
            observer: this,
            action: 'close'
        });


        this.loadDetails();
    },
    loadDetails: function () {
        var url = this.get('url'),
            query = this.get('queryParams');

        if (url && query) {
            $('<div>')
                .addClass('jlv-loader').html(
                '<div class="jlv-loader-image">' +
                '<div class="jlv-table-data-loading" alt="loading indicator"></div></div>')
                .appendTo(this.$el);

            var self = this;
            $.post(url, query, function (response) {
                if (_.isFunction(self.renderer)) {
                    self.$el.html('<div style="padding:10px;">' + self.renderer(response) + '</div>');
                }

            }).fail(function (response) {
                if (_.isFunction(self.fail)) {
                    self.$el.html('<div style="padding:10px;">' + self.fail(response) + '</div>');
                }
                else {
                    self.$el.html('<div style="padding:10px;">An error has occurred.</div>');
                }
            });
        }
        else {
            if (_.isFunction(this.renderer))
                this.$el.html('<div style="padding:10px;">' + this.renderer() + '</div>');
        }
    },
    doPrint: function () {

        var dataHTML = '<html><head><title>{{title}}</title>\n{{cssLink}}</head>';
        dataHTML += '<body><div style="text-align: center;font-weight: bold;">For Official Use Only</div>';

        if (JLV.common.isDoDProvider(JLV.ActiveProvider)) {
            dataHTML += '<div><b>Patient Name: </b>{{name}} &nbsp;<b>DOB: </b>{{dob}} &nbsp;<b>Gender: </b>{{gender}} &nbsp;<b>DoD ID: </b>{{edipi}}</div>';
        }
        else {
            dataHTML += '<div><b>Patient Name: </b>{{name}} &nbsp;<b>DOB: </b>{{dob}} &nbsp;<b>Gender: </b>{{gender}} &nbsp;<b>SSN: </b>{{fmpssn}}</div>';
        }

        dataHTML += '<div>{{bodyHTML}}</div></body></html>';

        try {
            var printHTML = _.template(dataHTML)(_.extend({
                title: "Patient: " + JLV.ActivePatientModel.get('name'),
                cssLink: '<link rel="stylesheet" type="text/css" href="assets/view/print.css">',
                bodyHTML: this.$el.html()
            }, JLV.ActivePatientModel.toJSON()));

            var printWindow = window.open("", "Print");
            printWindow.document.write(printHTML);
            printWindow.document.close();
            setTimeout(function () {
                printWindow.print();
                printWindow.close();
            }, 200);
        } catch (e) {

        }
    },
    doClipboard: function () {
        JLV.common.copyWindowHTML(this.$el.html());
    },
    print: function (aBool) {
        if (!JLV.common.OUTSIDE_PROVIDER_FLAG) {
            if (aBool === true) {
                this._printBtn.show();
            }
            else if (aBool === false) {
                this._printBtn.hide();
            }
        }
        else {
            // Do not show print for outside providers (VACV)
            this._printBtn.hide();
        }

        return this._printBtn.is(':visible');
    },
    clipboard: function (aBool) {
        if (!JLV.common.OUTSIDE_PROVIDER_FLAG) {
            if (aBool === true) {
                this._clipboardBtn.show();
            }
            else if (aBool === false) {
                this._clipboardBtn.hide();
            }
        }
        else {
            // Do not show clipboard for outside providers (VACV)
            this._clipboardBtn.hide();
        }

        return this._clipboardBtn.is(':visible');
    },
    vlerDomain: function (aBool) {

        if (this._vlerLoadBtn) {
            if (aBool === true) {
                this._vlerLoadBtn.show();
            }
            else if (aBool === false) {
                this._vlerLoadBtn.hide();
            }

            return this._vlerLoadBtn.is(':visible');
        }

        return false;

    }
});



