//= require ./WidgetWindow.js

jQuery(function () {


    JLV.WidgetWindow.FilterPanel = AppKit.View.extend({
        className: 'jlv-more-filter-panel',
        template: $('#WidgetMoreFilterTemplate').template(),
        window: null,
        filters: [],
        initialize: function (options) {

            this._super(options);

            var self = this;
            this.subview('.jlv-more-filter-close').on('click', function () {
                if (self.window) {
                    self.window.toggleFilterPanel();
                }
            }).on('keydown', function (evt) {
                if (evt.which === $.ui.keyCode.SPACE) {
                    if (self.window) {
                        self.window.toggleFilterPanel();
                        setTimeout(function () {
                            self.window.$el.find('.filter').focus();
                        }, 0);
                    }
                }
            });

            this.updateFilters();
        },
        clearFilters: function (skip) {

            var combo = null;
            if (this.filters.length > 0 && skip != 0) {
                combo = this.subview('.first-combo');
                combo.set('selectedIndex', 0);
                combo.set('value', '');

            }

            if (this.filters.length > 1 && skip != 1) {
                combo = this.subview('.second-combo');
                combo.set('selectedIndex', 0);
                combo.set('value', '');
            }

            if (this.filters.length > 2 && skip != 2) {
                combo = this.subview('.third-combo');
                combo.set('selectedIndex', 0);
                combo.set('value', '');
            }

            if (this.filters.length > 3 && skip != 3) {
                combo = this.subview('.fourth-combo');
                combo.set('selectedIndex', 0);
                combo.set('value', '');
            }
        },
        updateFilters: function () {

            if (this.filters.length > 0) {

                this.$el.find('.first-filter').css('display', 'inline-block');
                this.$el.find('.first-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.first-combo'), 0);
                this.$el.find('.first-filter-label').show().html('Filter by ' + _.text(this.filters[0].title) + ':');
            }
            else {
                this.$el.find('.first-filter-label').hide()
                this.$el.find('.first-filter').hide();
            }

            if (this.filters.length > 1) {
                this.$el.find('.second-filter').css('display', 'inline-block');
                this.$el.find('.second-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.second-combo'), 1);
                this.$el.find('.second-filter-label').html('Filter by ' + _.text(this.filters[1].title) + ':');
            }
            else {
                this.$el.find('.second-filter-label').hide()
                this.$el.find('.second-filter').hide();
            }

            if (this.filters.length > 2) {
                this.$el.find('.third-filter').css('display', 'inline-block');
                this.$el.find('.third-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.third-combo'), 2);
                this.$el.find('.third-filter-label').html('Filter by ' + _.text(this.filters[2].title) + ':');
            }
            else {
                this.$el.find('.third-filter-label').hide();
                this.$el.find('.third-filter').hide();
            }

            if (this.filters.length > 3) {
                this.$el.find('.fourth-filter').css('display', 'inline-block');
                this.$el.find('.fourth-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.fourth-combo'), 3);
                this.$el.find('.fourth-filter-label').html('Filter by ' + _.text(this.filters[3].title) + ':');
            }
            else {
                this.$el.find('.fourth-filter-label').hide();
                this.$el.find('.fourth-filter').hide();
            }
        },
        _updateFilterAtIndex: function (combo, idx) {

            if (combo && idx < this.filters.length) {

                combo.set('placeholder', 'Select ' + _.text(this.filters[idx].title));

                combo.$el.attr('data-column', this.filters[idx].id);
                combo.input().attr('aria-label', 'Select ' + _.text(this.filters[idx].title));

                var values = _.uniq(this.data.pluck(this.filters[idx].id)).sort();

                var count = values.length,
                    i = 0,
                    menuItems = [{'data-label': 'All', 'data-value': 0}];

                for (; i < count; i++) {
                    var val = values[i],
                        formatter = this.filters[idx].formatter;

                    val = _.text(_.isFunction(formatter) ? formatter(val) : val);

                    if (val && val.trim().length > 0) {
                        menuItems.push({
                            'data-label': val
                        });
                    }

                }

                var selected = Math.max(0, combo.get('selectedIndex'));

                combo.configureMenu({
                    items: menuItems,
                    className: 'jlv-filter-menu',
                    radio: true
                });

                combo.set('selectedIndex', selected);

                var self = this;
                combo.set('action', function () {
                    self.clearFilters(idx);
                    var val = this.get('selectedValue');
                    this.set('value', (val ? val : "All"));
                    self.window.$el.trigger('beforeFilter', [val]);
                    self.applyFilters();
                    self.window.$el.trigger('afterFilter', [val]);
                });
            }
        },
        filterBy: function (filterInfo) {

            var $combo = this.$el.find('[data-column="' + filterInfo.id + '"]');
            if ($combo) {
                var combo = $combo.widget();
                if (combo) {
                    this.clearFilters();
                    combo.set('selectedValue', filterInfo.value);
                    combo.set('value', filterInfo.value);
                    this.window.$el.trigger('beforeFilter', [filterInfo.value]);
                    this.applyFilters();
                    this.window.$el.trigger('afterFilter', [filterInfo.value]);
                }
            }
        },
        applyFilters: function () {

            if (this.window) {

                var datagrid = this.window.get('datagrid');
                var self = this;
                datagrid.set('filter', function (model) {
                    var keep = true;
                    if (self.filters.length > 0) {
                        var firstCombo = self.subview('.first-combo'),
                            val = firstCombo.get('selectedValue');

                        if (val === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[0].formatter,
                                testVal = model.get(self.filters[0].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (testVal === val);

                        }


                    }
                    if (self.filters.length > 1) {
                        var secondCombo = self.subview('.second-combo'),
                            val = secondCombo.get('selectedValue');

                        if (val === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[1].formatter,
                                testVal = model.get(self.filters[1].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (testVal === val);
                        }
                    }
                    if (self.filters.length > 2) {
                        var thirdCombo = self.subview('.third-combo'),
                            val = thirdCombo.get('selectedValue');

                        if (val === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[2].formatter,
                                testVal = model.get(self.filters[2].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (testVal === val);
                        }
                    }

                    if (self.filters.length > 3) {
                        var fourthCombo = self.subview('.fourth-combo'),
                            val = fourthCombo.get('selectedValue');

                        if (val === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[3].formatter,
                                testVal = model.get(self.filters[3].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (testVal === val);
                        }
                    }

                    return keep;
                });

                this.window.renderData();
            }
        }
    });


    JLV.WidgetWindow.DateFilterPanel = JLV.WidgetWindow.FilterPanel.extend({
        className: 'jlv-more-date-filter-panel',
        template: $('#WidgetMoreDateFilterTemplate').template(),
        window: null,
        startDate: null,
        endDate: null,
        filters: [],
        initialize: function (options) {

            this._super(options);

            var self = this;
            this.subview('.jlv-more-date-filter-apply').on('click', function () {
                if (self.window) {
                    var data = self.window.get('data');
                    if (data) {

                        data.startDate = moment(self.startDate);
                        data.endDate = moment(self.endDate);

                        var startDay = data.startDate.diff(moment(), 'days');
                        if (startDay > 0) {//fix diff rounding error
                            startDay++;
                        }

                        var endDay = data.endDate.diff(moment(), 'days');
                        if (endDay > 0) {
                            endDay++;
                        }

                        self.window.widget.set('startDay', startDay);
                        self.window.widget.set('endDay', endDay);
                        self.window.loadData();

                        if (self.window.widget.savePortalUI) {
                            self.window.widget.savePortalUI();
                        }
                    }
                }
            });

            this.$el.find('.preset').on('click', function () {
                self.applyDatePreset($(this).data('value'));

            });

            this._startDateInput = this.subview('.jlv-start-date-input');
            this._endDateInput = this.subview('.jlv-end-date-input');

            if (this._startDateInput) {
                this._startDateInput.set('date', this.startDate);
                this._startDateInput.on('dateSet', function () {
                    self.startDate = this.get('date');
                    self._endDateInput.set('minDate', self.startDate);
                });

                self._startDateInput.set('enabled', !JLV.common.OUTSIDE_PROVIDER_FLAG);
            }

            if (this._endDateInput) {
                this._endDateInput.set('date', this.endDate);
                this._endDateInput.on('dateSet', function () {
                    self.endDate = this.get('date');
                    self._startDateInput.set('maxDate', self.endDate);
                });
            }



        },
        applyDatePreset: function (value) {

            if (this.window) {
                var data = this.window.get('data');
                if (data) {
                    data.endDate = moment();
                    if (value === '1w') {
                        data.startDate = moment().subtract(7, 'days');
                    }
                    else if (value === '2w') {
                        data.startDate = moment().subtract(14, 'days');
                    }
                    else if (value === '1m') {
                        data.startDate = moment().subtract(1, 'months');
                    }
                    else if (value === '3m') {
                        data.startDate = moment().subtract(3, 'months');
                    }
                    else if (value === '6m') {
                        data.startDate = moment().subtract(6, 'months');
                    }
                    else if (value === '1y') {
                        data.startDate = moment().subtract(1, 'years');
                    }
                    else if (value === '2y') {
                        data.startDate = moment().subtract(2, 'years');
                    }
                    else if (value === '3y') {
                        data.startDate = moment().subtract(3, 'years');
                    }

                    this.window.widget.set('startDay', data.startDate.diff(moment(), 'days'));
                    this.window.widget.set('endDay', data.endDate.diff(moment(), 'days'));

                    this._startDateInput.set('date', data.startDate.toDate());
                    this._endDateInput.set('date', data.endDate.toDate());
                    this.window.loadData();
                }
            }
        },
        applyFilters: function () {

            if (this.window) {
                var data = this.window.widget.get('data');

                this._startDateInput.set('date', data.startDate.toDate());
                this._endDateInput.set('date', data.endDate.toDate());
            }

            this._super();

        }
    });
});