//= require lib/AppKit.js
//= require ../JLV.js
//= require ../desktop/WindowManager.js

JLV.WidgetWindow = AppKit.Dialog.extend({
    classID: 'jlv-widget-window',
    normCodeSystems: [],
    widget: null,
    options: {
        name: "",
        closable: true,
        dateFilter: false,
        vlerDomain: false,
        filter: false,
        print: false,
        fitColumns: true,
        refresh: false,
        status: false,
        clipboard: false,
        width: 800,
        height: 560,
        minWidth: 800,
        maximizable: true,
        minimizable: true,
        columns: undefined,
        filters: [],
        data: null,
        defaultSort: null,
        defaultFilter: null,
        gridHeaderHeight: 24,
        groupBy: undefined,
        groupFormatter: undefined,
        groupSorter: undefined,
        displayMode: 'paged'
    },
    initialize: function (options) {

        var self = this;
        this._tools = $('<div>').addClass('jlv-window-tools');

        this._refreshBtn = $('<div title="Refresh" tabindex="0" class="jlv-window-toolbutton refresh">')
            .append('<div class="icon-refresh">')
            .button()
            .appendTo(this._tools)
            .on('click', function (evt) {
                self.loadData();
            });

        this._printBtn = $('<div title="Print" tabindex="0" class="jlv-window-toolbutton print">')
            .append('<div class="icon-print">')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doPrint)) {
                    self.doPrint();
                }
            }).appendTo(this._tools);

        this._clipboardBtn = $('<div title="Copy to clipboard" tabindex="0" class="jlv-window-toolbutton clipboard">')
            .append('<div class="icon-clipboard2">')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doClipboard)) {
                    self.doClipboard();
                }
            }).appendTo(this._tools);

        this._interfaceStatusBtn = $('<div title="Interface Status" tabindex="0" class="jlv-window-toolbutton status">')
            .append('<div class="icon-question-circle-small">')
            .button()
            .on('click', function () {
                if (self.widget) {
                    self.widget.showInterfaceStatus();
                }
            }).appendTo(this._tools);

        if (JLV.Session.dodVLER) {
            this._vlerLoadBtn = $('<div title="Force VLER Data Refresh - This can take a while" tabindex="0" class="jlv-window-toolbutton vler">')
                .append('<div class="icon-refresh-alt">')
                .button()
                .on('click', function () {
                    //self.vlerLoadMessage("Community partner data retrieval in progress...");

                    AppKit.AlertDialog.yesNo('Confirm VLER Refresh',
                        'A forced refresh of community partner data may take a few minutes. Click Yes to continue. Click No to cancel.',
                        function (yes) {
                            if (yes) {
                                JLV.Session.forceRefresh = true;
                                if (_.isFunction(JLV.LoadVlerData)) {
                                    JLV.LoadVlerData();
                                }
                            }
                        }
                    );
                }).appendTo(this._tools);
        }

        this._filterBtn = $('<div title="Filter" tabindex="0" class="jlv-window-toolbutton filter">')
            .append('<div class="icon-filter">')
            .button()
            .on('click', function () {
                self.toggleFilterPanel();
            }).appendTo(this._tools);

        this._addBtn = $('<div title="Add" tabindex="0" class="jlv-window-toolbutton add">')
            .append('<div class="icon-plus">')
            .button()
            .appendTo(this._tools).hide();


        this._body = $('<div role="presentation">').addClass('jlv-window-body').appendTo(this.$el);
        this._footer = $('<div role="presentation">').addClass('jlv-window-footer').appendTo(this.$el);


        if (this.options.columns || options.columns) {
            var tablePager = new AppKit.Pager();

            this._datagrid = new AppKit.PagedTableView({
                headerHeight: this.options.gridHeaderHeight,
                controller: this,
                pager: tablePager,
                fitColumns: true
            });

            this._datagrid.$el.addClass('jlv-window-table').appendTo(this._body);

            tablePager.$el.appendTo(this._footer);

            tablePager.on('selection', function () {
                self._updatePageInfoDisplay();
            });

            this._showAllBtn = new AppKit.Button({
                className: 'jlv-show-all-btn',
                label: 'Show All',
                action: function () {

                    if (self.get('displayMode') === 'all') {
                        self.set('displayMode', 'paged');
                    }
                    else {
                        self.set('displayMode', 'all');
                    }
                }
            });

            this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {

                if (this.widget) {
                    if (_.findIndex(this.widget.options.columns, {
                            id: sortInfo.column.id
                        }) > -1) {

                        this.widget._defaultSort = {
                            asc: sortInfo.asc,
                            id: sortInfo.column.id
                        };

                        if (_.isFunction(this.widget.savePortalUI)) {
                            this.widget.savePortalUI();
                        }
                    }

                }

                this._defaultSort = {
                    asc: sortInfo.asc,
                    id: sortInfo.column.id
                };

            }, this));

            this._footer.append(this._showAllBtn.$el);

            this._tablePageInfo = $('<span role="presentation">')
                .addClass('jlv-table-page-info')
                .appendTo(this._footer)
                .hide();

        }

        this._super(_.extend({dialogClass: 'jlv-widget-window'}, options));

        this._tools.appendTo(this.$el.parent().find('.ui-dialog-titlebar-tools'));

        if (this._datagrid) {
            this._datagrid.sizer();
            this.$el.on('finishresize maximized restored', function () {
                self._datagrid.sizer();
            });
        }

        JLV.Desktop.registerWindow(this);

        JLV.NotificationCenter.addObserverForNotification({
            name: 'closeNotification',
            observer: this,
            action: 'close'
        });


        if (this.widget) {
            if (_.isFunction(this.widget.itemClicked)) {
                this._datagrid.on('afterRender.click', function () {

                    this.$el.find('.jlv-clickable-link').on('click', function () {
                        var modelId = $(this).data('model-id'),
                            model = self._datagrid.get('source').get(modelId);

                        self.widget.itemClicked(model, $(this));
                    });
                });
            }

            this.$el.addClass(this.widget.get('widgetId'));
        }

        this.toggleFilterPanel();

        this.set('title', this.get('name'));


    },
    /**
     * Update the pages displayed information in the widget footer
     */
    _updatePageInfoDisplay: function () {

        if (this._datagrid) {
            var range = this._datagrid.displayedRange(),
                count = this._datagrid.get('rowCount');
            if (range && count) {
                this._tablePageInfo.html(_.template('Displaying {{start}}-{{end}} of {{total}}')({
                    start: range.start + 1,
                    end: range.end,
                    total: count
                })).show();
            }
            else {
                this._tablePageInfo.hide();
            }
        }

        this._body.css({
            bottom: this._footer.outerHeight()
        });
    },
    doClipboard: function () {
        if (this._datagrid && this.get('displayMode') !== 'all') {
            var clone = this._datagrid.$el.clone();
            clone.find('.appkit-table-column, .ui-datatable-loading-rows, .dummy-row-1, .dummy-column').remove();
            JLV.common.copyGridPanelHTML(clone.html());
        }

    },
    loadData: function () {
        if (this.widget && this._datagrid) {

            this.widget.loadData(_.bind(function () {
                if (this._displayMode === 'all') {
                    this._datagrid.set('recordsPerPage', this.get('data').size());
                }

                this.renderData();
                JLV.VLERDataAvailable();
            }, this));

            this._datagrid.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');


            this._updatePageInfoDisplay();
        }
    },
    vlerLoadMessage: function (mesg){
        if (!this._vlerLoadPanel) {

            var data = this.get('data');

            if (data) {

                this._vlerLoadPanel = new JLV.WidgetWindow.VlerLoadPanel({
                    window: this
                });

                if (this._vlerLoadPanel) {
                    this._vlerLoadPanel.$el.insertBefore(this._body);
                    this._vlerLoadPanel.$el.hide();
                }
            }
        }

        if (this._vlerLoadPanel) {
            var $loading = this.$el.find('.jlv-vler-loading');

            $loading.html(mesg);
            $loading.removeClass('jlv-vler-loading-img');

            if (this._vlerLoadPanel.get('hidden')) {
                this._vlerLoadPanel.set('hidden', false);
            }

            this.adjustHeight();
        }
    },
    displayMode: function (mode) {

        if (mode !== undefined) {

            this._displayMode = mode;

            if (this._displayMode === 'all') {

                this._datagrid.set('recordsPerPage', this.get('data').size());
                this.renderData(true);
                this._updatePageInfoDisplay();
                this._showAllBtn.set('label', 'Show Paged');

                this._clipboardBtn.tooltip({
                    content: 'Clipboard is disabled when displaying the table in "Show All" mode. To enable the clipboard, please set the table to "Show Paged" mode.'
                });

                this._clipboardBtn.css({opacity: 0.5});

                this._printBtn.tooltip({
                    content: 'Printing is disabled when displaying the table in "Show All" mode. To enable printing, please set the table to "Show Paged" mode.'
                });

                this._printBtn.css({opacity: 0.5});

            }
            else if (this._datagrid && this._displayMode === 'paged') {

                this._datagrid.set('recordsPerPage', 25);
                this.renderData(true);
                this._updatePageInfoDisplay();
                this._showAllBtn.set('label', 'Show All');


                this._clipboardBtn.css({opacity: 1.0});
                if (this._clipboardBtn.tooltip('instance'))
                    this._clipboardBtn.tooltip('destroy');

                this._printBtn.css({opacity: 1.0});
                if (this._printBtn.tooltip('instance'))
                    this._printBtn.tooltip('destroy');

            }
        }


        return this._displayMode;
    },
    hideVlerLoadMessage: function (){
        if (this._vlerLoadPanel) {


            this._vlerLoadPanel.set('hidden', true);
            this.adjustHeight();
        }
    },
    renderData: function () {

        if (this._datagrid) {
            if (this._datagrid.get('rowCount') === 0) {
                this._datagrid.showMessage("No Data");
            }
            else if (this._defaultSort !== null) {
                var column = _.findWhere(this.options.columns, {id: this._defaultSort.id});
                if (column) {
                    column.index = _.indexOf(this.options.columns, column);

                    var sortInfo = {
                        asc: this._defaultSort.asc,
                        column: column
                    };

                    this._datagrid.sortByColumn(sortInfo);
                    this._datagrid.render();

                    this._updatePageInfoDisplay();

                }
                else {
                    this._datagrid.render();
                }

            }
            else {
                this._datagrid.render();
            }

            this._updatePageInfoDisplay();

            //update the window title to reflect the number of records
            this.set('title', this.get('name') + ' (' + this._datagrid.get('rowCount') + ')');


            this._dataRendered = true;

        }
    },
    /** data setter & getter */
    data: function (aBBCollection) {

        if (this._datagrid) {
            if (aBBCollection !== undefined) {

                this._dataRendered = false;
                if (typeof(aBBCollection) === 'function') {
                    this._datagrid.set('source', aBBCollection ? new aBBCollection() : null);
                }
                else {
                    this._datagrid.set('source', aBBCollection);

                    if (this.get('visible')) {
                        this.renderData();
                    }

                    if (this._filterPanel) {
                        this._filterPanel.updateFilters();
                        this._filterBtn.applyFilters();
                    }
                }

                var data = this._datagrid.get('source');
                data.bind('reset', _.bind(function () {
                    if (this._filterPanel) {
                        this._filterPanel.updateFilters();
                    }
                }, this));
            }

            return this._datagrid.get('source');
        }
    },
    datagrid: function () {
        return this._datagrid;
    },
    /**
     * Columns setter/getter
     * @param anArray - an Array of column config objects
     * @returns anArray of column configurations
     */
    columns: function (anArray) {

        if (anArray !== undefined && this._datagrid) {

            var colcount = anArray.length,
                i = 0;

            this._datagrid.removeAllColumns();

            for (; i < colcount; i++) {

                var colInfo = anArray[i];
                this._datagrid.addColumn(colInfo);
            }

            this._columns = anArray;
        }

        return this._columns;
    },
    /**
     * widget window clipboard tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    clipboard: function (aBOOL) {
        if (!JLV.common.OUTSIDE_PROVIDER_FLAG) {
            if (aBOOL === true) {
                this._clipboardBtn.show();
            }
            else if (aBOOL === false) {
                this._clipboardBtn.hide();
            }
        }
        else {
            // Do not show clipboard for outside providers (VACV)
            this._clipboardBtn.hide();
        }

        return this._clipboardBtn.is(':visible');
    },
    /**
     * widget window filter tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    filter: function (aBOOL) {
        if (aBOOL === true) {
            this._filterBtn.show();
        }
        else if (aBOOL === false) {
            this._filterBtn.hide();
        }

        return this._filterBtn.is(':visible');

    },
    /**
     * widget date filter tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    vlerDomain: function (aBOOL) {

        if (this._vlerLoadBtn) {
            if (aBOOL === true) {
                this._vlerLoadBtn.show();
            }
            else if (aBOOL === false) {
                this._vlerLoadBtn.hide();
            }

            return this._vlerLoadBtn.is(':visible');
        }

        return false;

    },
    /**
     * widget window print tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    print: function (aBOOL) {
        if (!JLV.common.OUTSIDE_PROVIDER_FLAG) {
            if (aBOOL === true) {
                this._printBtn.show();
            }
            else if (aBOOL === false) {
                this._printBtn.hide();
            }
        }
        else {
            // Do not show print for outside providers (VACV)
            this._printBtn.hide();
        }

        return this._printBtn.is(':visible');

    },
    /**
     * widget window refresh tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    refresh: function (aBOOL) {
        if (aBOOL === true) {
            this._refreshBtn.show();
        }
        else if (aBOOL === false) {
            this._refreshBtn.hide();
        }

        return this._refreshBtn.is(':visible');
    },
    /**
     * widget window status tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    status: function (aBOOL) {

        if (aBOOL === true) {
            this._interfaceStatusBtn.show();
        }
        else if (aBOOL === false) {
            this._interfaceStatusBtn.hide();
        }
        return this._interfaceStatusBtn.is(':visible');
    },
    /**
     * widget window plus/add tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    plus: function (aBOOL) {
        if (aBOOL === true) {
            this._addBtn.show();
        }
        else if (aBOOL === false) {
            this._addBtn.hide();
        }

        return this._addBtn.is(':visible');
    },
    fitColumns: function (aBOOL) {
        if (this._datagrid) {
            if (aBOOL !== undefined) {
                this._datagrid.set('fitColumns', aBOOL);
            }

            return this._datagrid.get('fitColumns');
        }

        return false;

    },
    groupBy: function (aString) {
        if (this._datagrid)
            return this._datagrid.set('groupBy', aString);
    },
    groupFormatter: function (aFunction) {
        if (this._datagrid)
            return this._datagrid.set('groupFormatter', aFunction);
    },
    groupSorter: function (aFunction) {
        if (this._datagrid)
            return this._datagrid.set('groupSorter', aFunction);
    },
    gridHeaderHeight: function (aNumber) {
        if (this._datagrid) {
            if (_.isNumber(aNumber)) {
                this._datagrid.set('headerHeight', aNumber);
            }

            return this._datagrid.get('headerHeight');
        }
    },
    defaultSort: function (sort) {
        if (sort !== undefined) {
            if (sort && this._defaultSort) {
                if (sort.id !== this._defaultSort.id) {
                    this._dataRendered = false
                }
            }
            this._defaultSort = sort;
        }

        return this._defaultSort;
    },
    visible: function (aBOOL) {

        this._super(aBOOL);

        if (aBOOL === true) {

            if (this._datagrid)
                this._datagrid.showMessage("Loading...");

            setTimeout(_.bind(function () {

                if (!this._dataRendered) {
                    this.renderData();
                }

                if (this._filterPanel) {

                    if (this.get('defaultFilter')) {
                        this._filterPanel.filterBy(this.get('defaultFilter'));
                    }
                    else {
                        this._filterPanel.applyFilters();
                    }
                }
                else {
                    this.renderData();
                }

                if (this._datagrid)
                    this._datagrid.sizer();

                this._body.css({
                    bottom: this._footer.outerHeight()
                });

            }, this), 0);

            if (this._filterPanel) {
                this._filterPanel.clearFilters();
            }


        }
    },
    adjustHeight: function(){
        var newHeight = 0; //this._header.outerHeight();
        if(this._vlerLoadPanel)
            newHeight += this._vlerLoadPanel.el.offsetHeight

        this._body.css({
            top: newHeight,
            bottom: this._footer.outerHeight()
        });
    },

    //cleans up the widget
    destroy: function () {

        JLV.NotificationCenter.removeObserverForNotificationName(this, 'closeNotification');

        this.close();

        $('#WindowDock').widget().removeWindow(this);

        setTimeout(_.bind(function () {
            this.$el.remove();
        }, this), 0);
    },


    toggleFilterPanel: function () {
        if (this.get('filter')) {
            if (!this._filterPanel) {

                var filterIds = this.get('filters');
                var filters = _.toArray(_.pick(_.indexBy(this.get('columns'), 'id'), filterIds));

                var data = this.get('data');

                if (data) {

                    if (this.get('dateFilter')) {
                        if (data.startDate !== undefined && data.endDate !== undefined) {

                            this._filterPanel = new JLV.WidgetWindow.DateFilterPanel({
                                window: this,
                                startDate: data.startDate.toDate(),
                                endDate: data.endDate.toDate(),
                                filters: filters,
                                data: data
                            });
                        }
                    }
                    else {

                        this._filterPanel = new JLV.WidgetWindow.FilterPanel({
                            window: this,
                            filters: filters,
                            data: data
                        });
                    }

                    if (this._filterPanel) {
                        this._filterPanel.$el.prependTo(this._body);
                        this._filterPanel.$el.hide();
                    }
                }
            }

            if (this._filterPanel) {
                if (this._filterPanel.get('hidden')) {
                    this._filterPanel.set('hidden', false);
                    if (this.get('filter')) {
                        if (this._datagrid)
                            this._datagrid.$el.css('top', this._filterPanel.el.offsetTop + this._filterPanel.el.offsetHeight);
                    }
                    else {
                        if (this._datagrid){
                            if (this._datagrid.el.offsetTop - this._filterPanel.el.offsetHeight < 0) {
                                this._datagrid.$el.css('top', 0);
                            } else {
                                this._datagrid.$el.css('top', this._filterPanel.el.offsetTop - this._filterPanel.el.offsetHeight);
                            }
                        }

                    }
                }
                else {
                    this._filterPanel.set('hidden', true);
                    if (this._datagrid)
                        this._datagrid.$el.css('top', 0);

                }
            }

        }

    }
});

$(function () {
    JLV.WidgetWindow.VlerLoadPanel = AppKit.View.extend({
        className: 'jlv-vler-loading-panel',
        template: $('#WidgetVlerLoadingTemplate').template(),
        window: null,
        initialize: function (options) {
            this._super(options);

            var self = this;
            this.subview('.jlv-vler-loading').on('click', function () {
                if (self.window) {
                    //self.window.hideVlerLoadMessage();
                }
            });

        }
    });
});
