//= require ./PatientWidget.js
//= require ./windows/patientdemographicsWidgetWindow.js
//= require ./windows/PatientInsuranceWidgetWindow.js


JLV.PatientDemographics = AppKit.View.extend({
    className: 'jlv-patient-demographics',
    siteData: [],
    vlerSiteData: [],
    vlerTabPanel: null,
    initialize: function (options) {
        this._tabview = new AppKit.TabView();
        this.$el.append(this._tabview.$el);

        this._super(options);


        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFetchedNotification,
            action: _.bind(function () {
                this.vlerTabPanel.empty();
                $(this._generateVlerLoadingContent()).appendTo(this.vlerTabPanel);
            }, this)
        });


        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataReadyNotification,
            action: _.bind(function () {
                if (this.vlerTabPanel)
                    this.vlerTabPanel.find('.jlv-vler-demographics-loading-label').remove();
            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {
                this.vlerTabPanel.empty();
                $(this._generateVlerLoadingContent()).appendTo(this.vlerTabPanel);
                //vler data is ready, neet to refetch demographics data
                JLV.PatientDemographics.Fetch(JLV.ActivePatientModel, _.bind(function (data) {
                    JLV.ActivePatientModel.set('demographics', data);
                    this.render(true);
                }, this));

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                if (this.vlerTabPanel) {
                    this.vlerTabPanel.empty();
                    $(this._generateVlerErrorContent()).appendTo(this.vlerTabPanel);
                }
            }, this)
        });

    },
    _renderDemographicsDataToElement: function ($el, data, noInsurance) {

        //remove any old content
        $el.find('.jlv-patient-demographics-content').remove();
        $(this._generatePatientDemographicsContent(data)).appendTo($el);

        data.standardizedRaceCode = 'None';
        data.standardizedRaceDisplay = 'None';

        if (data.codes) {
            for (var j in data.codes) {
                if (data.codes[j].system) {
                    if (data.codes[j].system.toUpperCase().indexOf("CDC") >= 0) {
                        data.standardizedRaceCode = data.codes[j].code;
                        data.standardizedRaceDisplay = data.codes[j].display;
                        break;
                    }
                }
            }
        }

        if (data.codeGray) {

            var warnBtn = $('<div class="appkit-button jlv-demographicsAlert">')
                .html("Alert")
                .data('demographics', data)
                .appendTo($el.find('.alert-btn'))
                .awaken('appkit-button');

            warnBtn.on('click', function () {
                var ddata = $(this).data('demographics');
                try {
                    $.post('patientDemographics/auditAlerts',
                        {
                            patToken: JLV.ActivePatientModel.get('patientToken'),
                            recordSiteMoniker: ddata.site.moniker,
                            recordSiteAgency: ddata.site.agency
                        });
                }
                catch (err) {
                }

                AppKit.AlertDialog.alert(JLV.common.formatMonikerDisplay(ddata.site) + " Alerts",
                    '<pre>' + ddata.codeGrayMessage + '</pre>'
                );
            });

            this.vistaAlerts.push({
                site: data.site,
                msg: data.codeGrayMessage
            });

        }

        $el.find('.jlv-patient-race-display').tooltip({
            content: _.template('<b>National Standard CDC Race </b>' +
            '<div>Standardized Name: {{standardizedRaceDisplay}}<br/>' +
            'Standardized Code: {{standardizedRaceCode}}</div>')(data),
            show: {
                delay: 600,
                duration: 0,
                easing: false,
                effect: null
            },
            hide: false,
            items: 'div',
            position: {my: "center top", at: "center bottom+5"}
        });

        if (!noInsurance) {
            var showInsurance = _.bind(function () {
                if (!this.insuranceWindow) {
                    this.insuranceWindow = new JLV.PatientInsuranceWidgetWindow({
                        patient: JLV.ActivePatientModel,
                        defaultSort: JLV.PatientDemographics.InsuraceSortInfo
                    });

                    this.insuranceWindow.on('sortChange', function (evt, sortInfo) {
                        JLV.PatientDemographics.InsuraceSortInfo = sortInfo;
                        JLV.NotificationCenter.postNotificationName(JLV.Desktop.SavePatientPortalUINotification);
                    });
                }

                this.insuranceWindow.center();
                this.insuranceWindow.open();


            }, this);

            $el.find('.jlv-insurance-link')
                .click(showInsurance)
                .keypress(function (evt) {
                    if (evt.which === $.ui.keyCode.ENTER)
                        showInsurance();
                })
                .keydown(function (evt) {
                    if (evt.keyCode === $.ui.keyCode.TAB) {
                        if (!evt.shiftKey) {
                            var next = $el.nextInDOM('.jlv-widget');
                            evt.preventDefault();
                            if (next.length) {
                                if (!next.is(":visible")) {
                                    $('#AddWidgetButton').focus();
                                } else {
                                    next.focus();
                                }
                            }
                            else {
                                $('#AddWidgetButton').focus();
                            }
                        }
                    }
                })
                .mousedown(function (evt) {
                    evt.preventDefault();
                });
        }

    },
    render: function (vlerLoading) {

        this._tabview.removeAllTabs(true);

        if (this.get('patient')) {

            var demographics = this.get('patient').get('demographics'),
                count = demographics.length,
                i = 0;


            this.dodAdapterSourceReport = [];
            this.dodAdapterStatusRec = [];
            this.vlerSiteData = [];
            this.siteData = [];
            this.vistaAlerts = [];

            if (this.demographicsDetailsWindow) {
                this.demographicsDetailsWindow.$el.remove();
                this.demographicsDetailsWindow = null;
            }


            for (; i < count; i++) {

                var data = demographics[i],
                    moniker = data.site.moniker,
                    name = data.site.name,
                    agency = data.site.agency;

                //skip DoD for outside providers
                if (JLV.common.OUTSIDE_PROVIDER_FLAG &&
                    (agency === JLV.common.AGENCY_DOD || agency === JLV.common.AGENCY_DOD_VLER)) {


                    continue;
                }

                if (moniker === JLV.common.DODADAPTER_STATUS_REPORT_FLAG) {
                    this.dodAdapterSourceReport.push(data.name);
                    this.dodAdapterStatusRec.push(data);

                }
                else if (agency === JLV.common.AGENCY_DOD_VLER) {
                    data.guiId = _.uniqueId('gui-id-');
                    this.vlerSiteData.push(data);
                }
                else if (name !== 'VLER') {

                    this.siteData.push(data);

                    var title = JLV.common.formatSiteMonikerDisplay(data.site);

                    if (data.name) {
                        data.name = data.name.replace(/,/g, ', ');
                    }

                    var tabID = _.uniqueId('tab');

                    var $el = $('<div>').attr('id', tabID).appendTo(this._tabview.$el);

                    var interfaceBtn = $('<div class="appkit-button jlv-demographics-interfaceStatus-button" aria-label="interface status button">')
                        .html("?")
                        .appendTo($el)
                        .awaken('appkit-button');

                    interfaceBtn.widget().set('action', _.bind(function () {

                        this.showInterfaceStatus();

                    }, this));

                    this._renderDemographicsDataToElement($el, data);

                    this._tabview.addTab(title, tabID, false);

                    var showDetails = _.bind(function () {
                        if (!this.demographicsDetailsWindow) {
                            this.demographicsDetailsWindow = new JLV.PatientDemographicsWidgetWindow({
                                data: this.siteData,
                                demographicsWidget: this
                            });
                        }

                        this.demographicsDetailsWindow.center();
                        this.demographicsDetailsWindow.open();

                    }, this);

                    $el.find('.jlv-patient-name')
                        .click(showDetails)
                        .mousedown(function (evt) {
                            evt.preventDefault();
                        });

                    $el.find('.jlv-patient-name')
                        .parent()
                        .keypress(function (evt) {
                            if (evt.which === $.ui.keyCode.ENTER)
                                showDetails();
                        });

                    if (!vlerLoading)
                        this._displayErrorAlerts(data);

                }
            }

            if (JLV.Session.dodVLER) {

                var vlerSite = {
                    agency: "VLER",
                    name: "VLER",
                    moniker: "DoD Community Partners"
                };


                var tabID = _.uniqueId('tab-vler-');

                if (this.vlerTabPanel) {
                    this.vlerTabPanel.remove();
                }

                this.vlerTabPanel = $('<div>').attr('id', tabID).appendTo(this._tabview.$el);

                var interfaceBtn = $('<div class="appkit-button jlv-demographics-interfaceStatus-button">')
                    .html("?")
                    .appendTo(this.vlerTabPanel)
                    .awaken('appkit-button');

                interfaceBtn.widget().set('action', _.bind(function () {

                    this.showInterfaceStatus();

                }, this));

                this._tabview.addTab(JLV.common.formatSiteMonikerDisplay(vlerSite)
                    , tabID, false);

                if (JLV.Session.dodVLERReady) {

                    //remove any old content
                    this.vlerTabPanel.find('.jlv-patient-demographics-content').remove();
                    $(this._generateVlerContent(this.vlerSiteData)).appendTo(this.vlerTabPanel);

                    var self = this;
                    $('.jlv-vler-demographics-narrative a').on('click', function (evt) {

                        var guiId = $(this).data('gui-id');
                        //console.log(guiId)
                        //console.log(self.vlerSiteData)
                        var dataIndex = _.findIndex(self.vlerSiteData, {
                            guiId: guiId
                        });

                        // console.log(dataIndex);

                        var vlerData = null;

                        if (dataIndex > -1)
                            vlerData = self.vlerSiteData[dataIndex];


                        if (vlerData) {

                            var vlerDemographicDetail = new JLV.PatientDemographics.VlerDemographicsDetail({
                                title: "Demographics - " + vlerData.site.name
                            });

                            self._renderDemographicsDataToElement(vlerDemographicDetail.$el, vlerData);


                            vlerDemographicDetail.center();
                            vlerDemographicDetail.open();

                        }

                    }).mousedown(function (evt) {
                        evt.preventDefault();
                    });

                }
                else {
                    $(this._generateVlerLoadingContent()).appendTo(this.vlerTabPanel);
                }
            }

            if (count > 0) {
                this._tabview.selectTabAtIndex(0);
            }

            if (this.vistaAlerts.length > 0 && !vlerLoading) {

                var count = this.vistaAlerts.length,
                    i = 0;

                var msg = '<div style="margin-left:15px;">';

                for (; i < count; i++) {
                    msg += ('<table cellpadding="5" style="border:1px solid;margin:5px;"><tr><td>Site</td><td>Alert Message</td></tr><td>' +
                    JLV.common.formatSiteDisplay(this.vistaAlerts[i].site) + '</td><td>' +
                    this.vistaAlerts[i].msg + '</td></tr></table>');
                }

                msg += '</div>';

                AppKit.AlertDialog.alert("Vista Alerts", msg, null, {height: 350});
            }

        }

        //adjust height of content
        var minH = 0;

        $('.jlv-patient-demographics-content').each(function (idx, el) {
            minH = Math.max(minH, $(el).height());
        });

        $('.jlv-patient-demographics').height(Math.max(minH + 70, 270));
        $('.jlv-patient-demographics-details .ui-tabs-panel, .jlv-patient-demographics .ui-tabs-panel')
            .height(Math.max(minH, 200));
    },

    destroy: function () {

        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataFetchedNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataReadyNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataRefreshConfirmNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataFailNotification);


        if (this.insuranceWindow) {
            this.insuranceWindow.destroy();
        }

        if (this._interfaceStatusWindow)
            this._interfaceStatusWindow.close();

        setTimeout(_.bind(function () {
            this.$el.remove();
        }, this), 0);

    },

    sizer: function () {
        this._tabview.refresh();
    },
    _generateVlerContent: function (vlerDataArray) {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DOD',
                name: 'VLER',
                moniker: 'DoD Community Partners'
            },
            name: name
        };

        var templateString = '<div class="jlv-patient-demographics-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) + '</div>' +
            '';

        var count = vlerDataArray.length,
            i = 0;

        templateString += '<div class="jlv-vler-demographics-narratives">';
        for (; i < count; i++) {

            var vlerData = vlerDataArray[i];
            templateString += _.template('<div class="jlv-vler-demographics-narrative"><a data-gui-id="{{guiId}}" title="DoD VLER: {{site.moniker}}" href="#">{{site.name}} </a></div>')(vlerData)

        }

        templateString += "</div>";

        return _.template(templateString)(data);

    },
    _generateVlerLoadingContent: function () {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DOD',
                name: 'VLER',
                moniker: 'DoD Community Partners'
            },
            name: name
        };

        return _.template('<div class="jlv-demographics-panel-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
        JLV.common.formatSiteIconDisplay(data.site) + '</div><div class="table jlv-vler-demographics-loading-label"><div class="tr"><div class="td jlv-vler-demographics-label">Community partner data retrieval in progress...</div><div class="td"><div style="width:22px;" class="jlv-table-data-loading"></div></div></div></div>')(data);
    },
    _generateVlerErrorContent: function () {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DOD',
                name: 'VLER',
                moniker: 'DoD Community Partners'
            },
            name: name
        };

        return _.template('<div class="jlv-demographics-panel-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
        JLV.common.formatSiteIconDisplay(data.site) + '</div><div class="jlv-vler-demographics-label">Connection unavailable. Community partner data is not available to display.</div>')(data);
    },
    _generateVlerDetailDemographicsContent: function (data) {
        try {
            return _.template('<div class="jlv-patient-demographics-content">' +
            '<div class="jlv-demographics-panel-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) +
            '</div><div style="display: inline-block;" class="alert-btn"></div>' +
            '<br/>' +
            '<div role="presentation" style="min-width:300px;margin-top:15px;" class="table"><div role="presentation" class="tr"><div role="presentation" class="td" style="vertical-align:top;"><div style="display:inline-block;">' +
            '<div role="presentation">{{-address1}}</div>' +
            '<div role="presentation">{{-address2}}</div>' +
            '<div role="presentation">{{-city}}, {{state}}</div>' +
            '<div role="presentation">{{-zipCode}}</div>' +
            '<br/><div role="presentation" style="max-width:200px;" class="table">' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Language:</div><div role="presentation" class="td jlv-demographic-item-value">{{-languages}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Religious Affiliation:</div><div role="presentation" class="td jlv-demographic-item-value">{{-religion}}</div></div>' +
            '</div>' +
            '<div class="jlv-insurance-link" tabindex="0">Insurance</div><br/><br/>' +
            '<div class="jlv-demographic-item">Next of Kin:</div>' +
            '{[ _.each(nextOfKin, function(kin) { ]}' +
            '{{-kin.name}} <br/>' +
            'Relationship: {{-kin.relationship}} <br/>' +
            '{{-kin.street1}} {{-kin.street2}} {{-kin.street3}}<br/>' +
            '{{-kin.city}}, {{-kin.stateProvince}} {{-kin.postalCode}}<br/>' +
                //'Phone: {{-kin.telecoms}}<br/>' +
            'Phone: ' +
            '{[ _.each(kin.telecoms, function(telecom) { ]}' +
            '{{-telecom.usageType}} {{-telecom.value}} <br/>' +
            '{[ }) ]}' +
            '{[ }) ]}' +
            '</div></div>' +
            '<div role="presentation" class="td" style="padding-left:10px;vertical-align:top;"><div role="presentation" class="table"><div role="presentation" class="tr">' +
            '<div role="presentation" class="td jlv-demographic-item">Patient ID:</div><div role="presentation" style="font-weight:bold;" class="td jlv-demographic-item-value">{{-edipi}}</div>' +
            '</div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Age:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-age}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">DOB:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;" >{{-dob}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Race:</div><div role="presentation" class="td jlv-demographic-item-value jlv-patient-race-display" style="font-weight:bold;">&nbsp;{{-race}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">(h):</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-phone1}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">(w):</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-phone2}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Gender:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-gender}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Marital Status:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-maritalStatus}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Source:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-sourceProtocol}}</div></div>' +
            '<div role="presentation" class="tr" style="height:10px;"></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Emergency Contact:</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td">' +
            '{[ _.each(emergencyContact, function(contact) { ]}' +
            '{{-contact.name}} <br/>' +
            'Relationship: {{-contact.relationship}} <br/>' +
            '{{-contact.street1}} {{-contact.street2}} {{-contact.street3}}<br/>' +
            '{{-contact.city}}, {{-contact.stateProvince}} {{-contact.postalCode}}<br/>' +
            'Phone: ' +
            '{[ _.each(contact.telecoms, function(telecom) { ]}' +
            '{{-telecom.usageType}} {{-telecom.value}} <br/>' +
            '{[ }) ]}' +
            '{[ }) ]}' +
            '</div></div>' +
            '</div></div></div><br/>' +
            '</div>' +
            '</div>')(data);
        } catch (e) {
        }

    },
    _generatePatientDemographicsContent: function (data) {

        //compute age
        var dob;

        if (data.site.agency === JLV.common.AGENCY_DOD) {
            dob = moment(data.dob, 'YYYYMMDD');
        }
        else {
            dob = moment(data.dob, 'DD MMM YYYY');
        }

        var today = moment();
        data.age = today.diff(dob, 'years');

        if (data.site && data.site.agency === JLV.common.AGENCY_DOD_VLER)
            return this._generateVlerDetailDemographicsContent(data);

        try {
            return _.template('<div class="jlv-patient-demographics-content" role="presentation">' +
            '<div class="jlv-demographics-panel-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) +
            '</div><div style="display: inline-block;" class="alert-btn"></div>' +
            '<br/>' +
            '<div style="min-width:320px;margin-top:15px;" class="table" role="presentation"><div role="presentation" class="tr"><div role="presentation" class="td" style="vertical-align:top;"><div role="presentation" style="display:inline-block;">' +
            '<div role="presentation">{{-address1}}</div>' +
            '<div role="presentation">{{-address2}}</div>' +
            '<div role="presentation">{{-city}}, {{state}}</div>' +
            '<div role="presentation">{{-zipCode}}</div>' +
            '<br/>' +
            '{[ if(site.agency === JLV.common.AGENCY_DOD) { ]}' +
            '<div class="table" role="presentation" style="max-width:250px;">' +
            '<div class="tr" role="presentation"><div role="presentation" class="td jlv-demographic-item">Rank:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-rank}}</div></div></div>' +
            '<div class="tr" role="presentation"><div role="presentation" class="td jlv-demographic-item">FMP/SSN:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-fmp}}</div></div></div>' +
            '<div class="tr" role="presentation"><div role="presentation" class="td jlv-demographic-item">Sponsor Name:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-sponsorName}}</div></div></div></div>' +
            '{[ } ]}' +
            '<br/><div role="presentation" class="table" style="max-width:250px;">' +
            '{[ if(site.agency === JLV.common.AGENCY_VA) { ]}' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">SC Percent:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-percentServiceConnected}}%</div></div></div>' +
            '{[ } ]}' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">PCM Name:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-pcm}}</div></div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">PCM Date:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-pcmDate}}</div></div></div>' +
            '{[ if(site.agency === JLV.common.AGENCY_VA) { ]}' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">PCM Team:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-pcTeam}}</div></div></div>' +
            '{[ } else { ]}' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">PCM Clinic:</div><div role="presentation" class="td jlv-demographic-item-value"><div>{{-pcmClinic}}</div></div></div>' +
            '{[ } ]}' +
            '</div>' +
            '</div></div>' +

            '<div role="presentation" style="padding-left:10px;vertical-align:top;"><div role="presentation" class="table"><div  role="presentation" class="tr">' +
            '{[ if(site.agency === JLV.common.AGENCY_DOD) { ]}' +
            '<div role="presentation" class="td jlv-demographic-item">DoD ID:</div><div role="presentation" style="font-weight:bold;" class="td jlv-demographic-item-value">{{-edipi}}</div>' +
            '{[ } else { ]}' +
            '<div role="presentation" class="td jlv-demographic-item">SSN:</div><div role="presentation" style="font-weight:bold;" class="td jlv-demographic-item-value">{{-fmpssn}}</div>' +
            '{[ } ]}' +
            '</div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Age:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-age}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">DOB:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;" >{{-dob}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Race:</div><div role="presentation" class="td jlv-demographic-item-value jlv-patient-race-display" style="font-weight:bold;">&nbsp;{{-race}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">(h):</div><div role="presentation"  class="td jlv-demographic-item-value" style="font-weight:bold;">{{-phone1}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">(w):</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-phone2}}</div></div>' +
            '<div role="presentation" class="tr"><div role="presentation" class="td jlv-demographic-item">Gender:</div><div role="presentation" class="td jlv-demographic-item-value" style="font-weight:bold;">{{-gender}}</div></div>' +
            '</div></div></div>' +
            '</div>' +
            '<div class="jlv-insurance-link" tabindex="0">Insurance</div>' +
            '</div>')(data);
        } catch (e) {
        }
    },
    showInterfaceStatus: function () {

        if (this._interfaceStatusWindow) {
            this._interfaceStatusWindow.close();
        }


        this._interfaceStatusWindow = new JLV.DetailsWindow({
                title: 'Interface Status ~ Patient Demographics',
                width: 600,
                height: 500,
                maximizable: true,
                minimizable: true,
                resizable: true,
                modal: false,
                className: 'jlv-interface-status'
            });

        var sourceStatus = JLV.common.DODADAPTER_STATUS_NO_REPORT_MSG;

        if (this.dodAdapterSourceReport.length) {
            sourceStatus = '';

            for (var i = 0; i < this.dodAdapterSourceReport.length; i++) {
                sourceStatus += this.dodAdapterSourceReport[i] + '</br></br>';
            }
        }

        this._interfaceStatusWindow.$el.html('<div class="jlv-dodadapter-report">' + sourceStatus + '</div>');

        this._interfaceStatusWindow.center();
        this._interfaceStatusWindow.open();
    },
    _displayErrorAlerts: function (data) {
        if (data.name == "" || data.name.toLowerCase().search("connection unavailable") != -1) {
            if (data.site.agency == "DOD") {
                if (!JLV.common.OUTSIDE_PROVIDER_FLAG) {
                    AppKit.AlertDialog.alert(JLV.common.formatMonikerDisplay(data.site) + "&nbsp;Unavailable",
                        "AHLTA data service is unavailable: DoD data may not display.");
                }

            } else {

                AppKit.AlertDialog.alert(JLV.common.formatMonikerDisplay(data.site) + "&nbsp;Unavailable",
                    "An error occurred while connecting to " + data.site.moniker + ".");

            }
        }
    }
});

//used to save/load the last sessions sort info
JLV.PatientDemographics.InsuraceSortInfo = {
    id: 'planName',
    asc: true
};


JLV.PatientDemographics.Fetch = function (data, callback) {

    jQuery.post('patientDemographics/list.json',
        {
            patToken: data.get('patientToken'),
            includeVLER: JLV.Session.dodVLERReady == true
        },
        function (responseData) {
            if (data.get('fmpssn') === JLV.ActivePatientModel.get('fmpssn')) { //make sure data callback is for active patient
                if (_.isFunction(callback))
                    callback(responseData);
            }
        });
};

JLV.PatientDemographics.VlerDemographicsDetail = JLV.DetailsWindow.extend({
    className: 'jlv-vler-demographics-details',
    options: {
        width: 460,
        height: 400,
        vlerDomain: true
    },
    loadDetails: function () {
        //do nothing
    }
});