//= require PatientWidget.js
//= require ./windows/PatientProblemListWidgetWindow.js

JLV.ProblemListCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'description',
    url: 'patientProblemList/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['onsetDate', 'lastUpdated'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            if (_.isString(data.status)) {
                data.status = data.status.toUpperCase();
            }

            return this._super(data);
        }
    }),
    doFetch: function (patient, callback) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                includeVLER: JLV.Session.dodVLERReady == true,
                status: 'A'
            },
            reset: true,
            type: 'POST',
            success: callback
        });
    }
});


JLV.PatientProblemList = JLV.PatientWidget.extend({
    options: {
        widgetId: 'problemlist_widget',
        name: 'Problem List',
        status: true,
        vlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientProblemListWidgetWindow',
        columns: [
            {
                title: 'ICD',
                width: 52,
                stretchable: false,
                id: 'icdCode'
            },
            {
                title: 'Onset',
                width: 70,
                stretchable: false,
                id: 'onsetDate',
                formatter: function (aDate, model) {
                    return model.get('dateTime');
                }
            },
            {
                title: 'Description',
                minWidth: 80,
                id: 'description',
                formatter: function (value, model) {
                    if (model && (model.get('recordDetailsToken') || model.get('narrativeId'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br>' +
                                'of Medicine - Clinical Terms<br>' +
                                '(SNOMED CT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br>' +
                        'of Medicine - Clinical Terms<br>' +
                        '(SNOMED CT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'Updated',
                width: 74,
                stretchable: false,
                id: 'lastUpdated',
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Status',
                width: 60,
                stretchable: false,
                id: 'status'
            },
            {
                title: 'Site',
                width: 100,
                stretchable: false,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'lastUpdated',
            asc: false
        },
        data: JLV.ProblemListCollection
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.set('visible', true);
        }
        else if (model.get('recordDetailsToken')) {
            var _isValidStr = function (str) {
                return str != null && str.length > 0;
            };

            var dialog = new JLV.DetailsWindow({
                title: String.format("Problem Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('description')),
                url: 'patientProblemList/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                renderer: function (response) {

                    if (!response.sessionCachedRecord) {
                        return '<pre>' + response.detailText + '</pre>';
                    }
                    else {
                        var html = _.template([
                            '<div class="problemdetailstitle">{{description}}</div>',
                            '<table align="center" class="problemdetails">',
                            '<tr>',
                            '<td align="right">Onset Date:</td>',
                            '<td align="left">{{onsetDate}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Status:</td>',
                            '<td align="left">{{status}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Provider:</td>',
                            '<td align="left">{{responsibleProvider}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Clinic:</td>',
                            '<td align="left">{{hospitalLocation}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Recorded Date:</td>',
                            '<td align="left">{{recordedDate}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Recorded By:</td>',
                            '<td align="left">{{recordedBy}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Entered Date:</td>',
                            '<td align="left">{{enteredDate}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Entered By:</td>',
                            '<td align="left">{{enteredBy}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Modified Date:</td>',
                            '<td align="left">{{lastUpdated}}</td>',
                            '</tr>',
                            '</table>',
                            '<div class="problemdetailstitle">Comments</div>'
                        ].join(''))(response);

                        for (var note in response.notes) {
                            if (_isValidStr(response.notes[note].noteEnteredBy)) {
                                html += _.template([
                                    '<table align="center" class="problemdetails">',
                                    '<tr>',
                                    '<td align="left">{{noteText}}</td>',
                                    '</tr>',
                                    '<tr>',
                                    '<td align="left">Entered By: {{noteEnteredBy}} on {{noteDate}}</td>',
                                    '</tr>',
                                    '</table>',
                                ].join(''))(response.notes[note]);
                            }
                            else {
                                html += _.template([
                                    '<table align="center" class="problemdetails">',
                                    '<tr>',
                                    '<td align="left">{{noteText}}</td>',
                                    '</tr>',
                                    '</table>',
                                ].join(''))(response.notes[note]);
                            }
                        }

                        return html;
                    }
                }
            });

            dialog.center();
            dialog.open();
        }
    },
    initialize: function (options) {
        this._super(options);

        if (this._datagrid) {
            this._datagrid.set('filter', function (model) {
                if (_.isString(model.get('status')))
                    return (model.get('status').toUpperCase() === "ACTIVE");

                return false;
            });
        }
    }

});

JLV.registerJLVWidget('problemlist_widget', JLV.PatientProblemList);