//= require ./PatientWidget.js
//= require ./windows/PatientProceduresWidgetWindow.js

JLV.PatientProceduresCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'location',
    url: 'patientProcedures/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['date'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("CPT") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientProcedures = JLV.PatientWidget.extend({
    options: {
        widgetId: 'procedures_widget',
        name: 'Procedures',
        gridHeaderHeight: 30,
        dateFilter: true,
        status: true,
        vlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientProceduresWidgetWindow',
        columns: [
            {
                title: 'Date/Time',
                id: 'date',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                },
                sortable: true,
                width: 110
            },
            {
                width: 100,
                title: 'Location',
                sortable: true,
                id: 'location'
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Procedure <br/> Description',
                sortable: true,
                id: 'description',
                formatter: function (value, model) {
                    if (model && (model.get('recordDetailsToken') || model.get('narrativeId'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Current Procedural Terminology<br>' +
                                '(CPT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Current Procedural Terminology<br>' +
                        '(CPT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'date',
            asc: false
        },
        data: JLV.PatientProceduresCollection
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.set('visible', true);
        }

    }
});


JLV.registerJLVWidget('procedures_widget', JLV.PatientProcedures);