//= require ./PatientWidget.js
//= require ./windows/PatientProgressNotesWidgetWindow.js

JLV.PatientProgressNotesCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'noteTitle',
    url: 'patientProgressNotes/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['noteDate'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("NOTES") === 0 ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientProgressNotes = JLV.PatientWidget.extend({
    options: {
        widgetId: 'outpatientnotes_widget',
        name: 'Progress Notes',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientProgressNotesWidgetWindow',
        columns: [
            {
                title: 'Date',
                id: 'noteDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Document Type/Title',
                id: 'noteTitle',
                width: 180,
                sortable: true,
                formatter: function (value, model) {
                    if (!model || value === 'Connection unavailable.' || !model.get('recordDetailsToken')) {
                        return value;
                    }

                    var noteTitle = value;
                    if (!noteTitle || noteTitle.length < 1) {
                        noteTitle = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                    }

                    if (model.get('sensitive')) {
                        noteTitle = noteTitle.replace(/\n/g, '<br />');
                    }

                    return JLV.common.linkFormatter(noteTitle, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes<br>' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'noteDate',
            asc: false
        },
        data: JLV.PatientProgressNotesCollection
    },
    itemClicked: function (model) {

        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            }

            var self = this;

            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function(agree, auditText) {
                if (agree) {
                    self._handleDetails(model, auditText);
                }
            });

        } else {

            this._handleDetails(model);
        }
    },
    _handleDetails: function (model, auditText) {
        var auditUrl;

        if (model.get('noteAsBinaryStream')) {
            if (auditText) {
                auditUrl = String.format("patientProgressNotes/getNoteDataStream?patToken={0}&recToken={1}&auditText={2}",
                    JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'), auditText);
            } else {
                auditUrl = String.format("patientProgressNotes/getNoteDataStream?patToken={0}&recToken={1}",
                    JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));
            }

            var noteURL = auditUrl;

            JLV.common.openAuxBrowserWindow(noteURL, 'JLV_PROGRESS_NOTE');
        }

        if (model.get('noteAsPlainText')) {
            auditUrl = 'patientProgressNotes/details.json';
            if (auditText) {

            }
            var dialog = new JLV.DetailsWindow({
                title: String.format("Progress Note Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('noteTitle')),
                url: auditUrl,
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                renderer: function (response) {
                    return '<pre>' + response.noteText + response.images + '</pre>';
                }
            });

            dialog.center();
            dialog.set('visible', true);

        }
    }

});


JLV.registerJLVWidget('outpatientnotes_widget', JLV.PatientProgressNotes);