//= require ./PatientWidget.js
//= require ./windows/PatientRadiologyExamsWidgetWindow.js

JLV.PatientRadiologyExamsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'study',
    url: 'patientRadiologyExams/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['examDate', 'amendedDate'],
        parse: function (data) {

            if (data.codes) {
                var loincSet = false;
                var cptSet = false;
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0 && !loincSet) {
                            data.normCodeLoinc = data.codes[i].code;
                            data.normDisplayLoinc = data.codes[i].display;
                            loincSet = true;
                        }

                        if (data.codes[i].system.toUpperCase().indexOf("CPT") >= 0 && !cptSet) {
                            data.normCodeCpt = data.codes[i].code;
                            data.normDisplayCpt = data.codes[i].display;
                            cptSet = true;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientRadiologyExams = JLV.PatientWidget.extend({
    options: {
        widgetId: 'radiologyexams_widget',
        name: 'Radiology Exams',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientRadiologyExamsWidgetWindow',
        startDay: -365,
        endDay: 0,
        columns: [
            {
                title: 'Date',
                id: 'examDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Exam',
                id: 'study',
                width: 100,
                sortable: true,
                formatter: function (value, model) {
                    var formattedString = value;

                    if (model && model.get('recordDetailsToken')) {
                        formattedString = JLV.common.linkFormatter(value, model);
                    }

                    if (model && model.get('abnormal')) {
                        if (model.get('abnormal').toLowerCase().indexOf('abnormal') > -1
                            || model.get('abnormal').toLowerCase() !== "normal")
                        {
                            formattedString = '<div class="jlv-alert-row">' + formattedString + '</div>';
                        }
                    }
                    return formattedString;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Type: ' + normDisplayLoinc + '<br>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br>' +
                                'Standardized Type: ' + normDisplayCpt + '<br>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 60,
                sortable: false,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {

                        if (!JLV.common.ALLOW_MEDWEB_PLUGIN || !model.get('medWebAvailable')) {
                            return '';
                        }

                        var medWebURL = String.format("patientMedWeb?patToken={0}&recToken={1}",
                            JLV.ActivePatientModel.get('patientToken'), model.get('medWebToken'));

                        var medWebCmd = String.format("JLV.common.openAuxBrowserWindow(\"{0}\", \"JLV_MEDWEB\");return false;", medWebURL);

                        return String.format('<span class="medweb-link" title="View in MedWeb" onclick=\'{0}\' alt="Link View in Medweb"><img src="JLV/assets/camera_icon.png"  alt=\'Camera Icon\'/></span>', medWebCmd);
                    }

                    return '';

                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'examDate',
            asc: false
        },
        data: JLV.PatientRadiologyExamsCollection
    },
    initialize: function (options) {
        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.vlerLoadMessage(JLV.common.DOD_VLER_NOT_AVAILABLE_MSG);
                if (this._moreWindow) {
                    this._moreWindow.vlerLoadMessage(JLV.common.DOD_VLER_NOT_AVAILABLE_MSG);
                }

            }, this)
        });

        if (this._datagrid) {
            //make rows with abnormal results pink
            this._datagrid.on('afterRender', function () {

                var $row = this.$el.find('.jlv-alert-row').parents('tr');
                $row.css(
                    {
                        'backgroundColor': '#FFC0CB',
                        'color': '#000000'
                    }
                );
                $row.find('a').css('color', '#000');
            });
        }
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {
            var dialog = new JLV.DetailsWindow({
                width: 780,
                height: 540,
                title: String.format("Radiology Report ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('study')),
                url: 'patientRadiologyExams/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken'),
                    radPriority: model.get('priority')
                },
                renderer: function (response) {

                    if (response.plainTextReport) {
                        var html = "";
                        html += _.template(
                            '<table class="radiology-report" width="100%" border="0">' +
                            '<tr>' +
                            '<td class="rad-header rad-td">Priority:</td>' +
                            '<td class="rad-td">{{priority}}</td>' +
                            '<td>&nbsp;</td>' +
                            '<td>&nbsp;</td>' +
                            '<td>&nbsp;</td>' +
                            '</tr>' +
                            '</table>'
                        )(response);

                        html += '<pre class="plain-text">' + response.reportText + '</pre>';

                        return html;
                    }
                    else {

                        response.amendedText = model.get('amendedText');
                        if (model.get('amendedDate')) {
                            response.amendedDate = moment(model.get('amendedDate')).format('M/D/YY');
                        }
                        else {
                            response.amendedDate = "";
                        }

                        var html = "";
                        if (JLV.ActiveProvider.agency !== JLV.common.AGENCY_DOD) {
                            html += _.template(
                                '<table class="radiology-report" width="100%" border="0">' +
                                '<tr>' +
                                '<td class="rad-header">Priority:</td>' +
                                '<td>{{priority}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Number:</td>' +
                                '<td>{{examId}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Procedure:</td>' +
                                '<td>{{procedure}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Date:</td>' +
                                '<td>{{examDate}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Result Code:</td>' +
                                '<td>{{resultCode}} {{abnormal}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Report Status:</td>' +
                                '<td>{{reportStatus}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Reason for Order:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{orderReason}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Impression:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{impressionText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Report:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{reportText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '{[ if(site.agency !== JLV.common.AGENCY_DOD) { ]}' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Transcription D/T:</span>' +
                                '&nbsp;' +
                                '<span>{{transcribeDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Interpreted by:</span>' +
                                '&nbsp;' +
                                '<span>{{interpretingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Approved by:</span>' +
                                '&nbsp;' +
                                '<span>{{approvingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Date:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Text:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedText}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '{[ } ]}' +
                                '</table>'
                            )(response);
                        } else {
                            html += _.template(
                                '<table class="radiology-report" width="100%" border="0">' +
                                '<tr>' +
                                '<td class="rad-header"></td>' +
                                '<td></td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Number:</td>' +
                                '<td>{{examId}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Procedure:</td>' +
                                '<td>{{procedure}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Date:</td>' +
                                '<td>{{examDate}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Result Code:</td>' +
                                '<td>{{resultCode}} {{abnormal}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Report Status:</td>' +
                                '<td>{{reportStatus}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Reason for Order:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{orderReason}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Impression:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{impressionText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Report:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{reportText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '{[ if(site.agency !== JLV.common.AGENCY_DOD) { ]}' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Transcription D/T:</span>' +
                                '&nbsp;' +
                                '<span>{{transcribeDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Interpreted by:</span>' +
                                '&nbsp;' +
                                '<span>{{interpretingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Approved by:</span>' +
                                '&nbsp;' +
                                '<span>{{approvingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Date:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Text:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedText}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '{[ } ]}' +
                                '</table>'
                            )(response);
                        }


                        return html;

                    }

                }
            });

            dialog.center();
            dialog.open();
        }

    }

});


JLV.registerJLVWidget('radiologyexams_widget', JLV.PatientRadiologyExams);