//= require ./PatientWidget.js
//= require ./windows/PatientVitalsWidgetWindow.js

JLV.PatientVitalsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'type',
    url: 'patientVitals/list.json',
    model: JLV.PatientDataModel.extend({
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("VITALS") >= 0 ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            if (data.dateTaken) {
                data.dateTaken = new Date(data.dateTaken);
                if (!data.dateTaken.isValid()) {
                    data.dateTaken = null;
                }
                else {
                    data.dateTaken.setHours(0, 0, 0, 0);
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientVitals = JLV.PatientWidget.extend({
    options: {
        widgetId: 'vitals_widget',
        name: 'Vitals',
        dateFilter: true,
        status: true,
        vlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        groupBy: 'dateTaken',
        groupFormatter: function (value) {
            return 'Date Taken: &nbsp;' + JLV.common.dateFormatter(new Date(value));
        },
        moreWindowClass: 'JLV.PatientVitalsWidgetWindow',
        startDay: -365,
        endDay: 0,
        columns: [
            {
                title: 'Date Taken',
                id: 'dateTaken',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 170
            },
            {
                title: 'Type',
                id: 'type',
                width: 160,
                sortable: true,
                formatter: function (value, model) {
                    if (model && (model.get('site').agency === JLV.common.AGENCY_DOD || model.get('site').agency === JLV.common.AGENCY_DOD_VLER)) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    return value;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes<br>' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Result',
                id: 'result',
                width: 100,
                sortable: true
            },
            {
                title: 'Units',
                id: 'units',
                width: 90,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                sortable: true,
                sorter: JLV.common.siteSorter,
                width: 90,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'dateTaken',
            asc: false
        },
        data: JLV.PatientVitalsCollection
    },
    initialize: function (options) {
        this._super(options);

        var defaultSort = this.get('defaultSort');
        if (defaultSort.id === 'dateTaken') {
            if (defaultSort.asc) {
                this.set('groupSorter', function (val1, val2) {
                    return (val1 >= val2) ? 1 : -1;
                });
            }
            else {
                this.set('groupSorter', function (val1, val2) {
                    return (val1 <= val2) ? 1 : -1;
                });
            }
        }

        this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {
            if (sortInfo.column.id === 'dateTaken') {
                if (sortInfo.asc) {
                    this.set('groupSorter', function (val1, val2) {
                        return (val1 >= val2) ? 1 : -1;
                    });
                }
                else {
                    this.set('groupSorter', function (val1, val2) {
                        return (val1 <= val2) ? 1 : -1;
                    });
                }
            }
        }, this));
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                width: 800,
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.open();
        }
        else {

            var dialog = new JLV.DetailsWindow({
                width: 600,
                title: String.format("Vitals ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('type')),
                renderer: function () {
                    try {
                        var data = _.clone(model.toJSON());
                        data.dateTaken = moment(data.dateTaken).format("M/D/YY");

                        try {
                            if (data.type.toLowerCase().indexOf('pain') >= 0) {
                                return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table">' +
                                    '<tr><td>Date Taken:</td><td>{{-dateTaken}}</td></tr>' +
                                    '<tr><td>Result:</td><td>{{-result}}</td></tr>' +
                                        //'<tr><td>Result Comments:</td><td>{{-resultComments}}</td></tr>' +
                                    '<tr><td>Comments:</td><td>{{-comments}}</td></tr>' +
                                    '</table>'
                                )(data);
                            } else {
                                return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table">' +
                                    '<tr><td>Date Taken:</td><td>{{-dateTaken}}</td></tr>' +
                                    '<tr><td>Result:</td><td>{{-result}}</td></tr>' +
                                        //'<tr><td>Result Comments:</td><td>{{-resultComments}}</td></tr>' +
                                    '<tr><td>Comments:</td><td>{{-comments}}</td></tr>' +
                                    '<tr><td>Modifiers:</td><td>' +
                                    '{[ if(vitalModifiers.length > 0) { ]}' +
                                    '<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><td>Modifier Type</td><td>Value</td>' +
                                    '{[ _.each(vitalModifiers, function(modifier) { ]}' +
                                    '<tr><td>{{-modifier.type}}</td><td>{{-modifier.value}}</td></tr>' +
                                    '{[ }) ]}' +
                                    '</table>' +
                                    '{[ } ]}' +
                                    '</td></tr>' +
                                    '</table>'
                                )(data);
                            }
                        } catch (e) {
                            console.info('template error');
                        }

                    } catch (e) {
                    }
                }
            });

            dialog.center();
            dialog.open();


        }


    }

});


JLV.registerJLVWidget('vitals_widget', JLV.PatientVitals);