//= require lib/excanvas.js
//= require lib/jquery.flot.js
//= require lib/jquery.flot.time.js
//= require lib/jquery.flot.navigate.js
//= require lib/jquery.flot.axislabels.js

JLV.LabResultsGraphWindow = JLV.DetailsWindow.extend({
    className: 'jlv-graph-window',
    options: {
        height: 600,
        width: 800,
        print: false,
        clipboard: false,
        resizable: false,
        maximizable: false,
        data: null,
        units: '',
        type: ''
    },
    initialize: function (options) {

        this._plot = $('<div>').addClass('jlv-detail-graph').appendTo(this.$el);

        this.$el.append($('<div style="margin:15px;">Use mouse wheel to zoom, click and drag to pan.</div>'));

        this._super(options);
    },
    loadDetails: function () {

        var data = this.get('data'),
            count = data.length,
            i = 0;

        var series1 = [];

        for (; i < count; i++) {
            var model = data[i];
            series1.push([JLV.common.createUTCDate(model.get('resultDate')), parseFloat(model.get('result'))]);
        }

        this._plot.plot([series1], {
            series: {
                points: {show: true, radius: 4},
                lines: {show: true, lineWidth: 3}
            },
            legend: {
                show: true
            },
            colors: ["#426ab3", "#d09b2c"],
            grid: {
                hoverable: true,
                clickable: true
            },
            zoom: {
                interactive: true
            },
            pan: {
                interactive: true
            },
            axisLabels: {
                show: true
            },
            xaxis: {mode: "time", timeformat: "%b %d, %y"},
            yaxes: [{
                position: 'left',
                axisLabel: this.get('units')
            }]
        });

        var self = this;
        var showGraphTooltip = function (x, y, contents) {
            $('<div id="' + self.id + '_tooltip">' + contents + '</div>').css({
                position: 'absolute',
                display: 'none',
                'text-align': 'center',
                top: y - 80,
                left: x - 50,
                zIndex: 10000,
                color: '#444',
                border: '1px solid #fdd',
                padding: '2px',
                'background-color': '#fee'
            }).appendTo('body').fadeIn(200);
        };


        var previousPoint = null;
        this._plot.bind("plothover", function (event, pos, item) {

            if (item) {
                if (previousPoint !== item.dataIndex) {

                    previousPoint = item.dataIndex;

                    $("#" + self.id + "_tooltip").remove();

                    var x = item.datapoint[0],
                        y = item.datapoint[1];

                    var records = self.get('data');
                    var idx = item.dataIndex;
                    var model = records[idx];
                    var result = model.get('result');

                    //flot graph lib always displays timestamps according to UTC
                    if (parseFloat(result) === y && JLV.common.createUTCDate(model.get('resultDate')).getTime() === x) {

                        showGraphTooltip(item.pageX, item.pageY,
                            JLV.common.formatSiteIconDisplay(model.get('site')) + '&nbsp;' + JLV.common.formatSiteDisplay(model.get('site')) +
                            "<br/>" +
                            model.get('result') + " " + model.get('units') +
                            "<br/>" +
                            JLV.common.dateFormatter(model.get('resultDate')));
                    }


                }
            }
            else {
                $("#" + self.id + "_tooltip").remove();
                previousPoint = null;
            }

        });


        var summary = '<table class="jlv-filtered-summary" style="margin:15px;">' +
            '<tr><td><b>Type:</b></td><td>' + this.get('type') + '</td>' +
            '<tr><td><b>Units:</b></td><td>' + this.get('units') + '</td>' +
            '</table>';


        this.$el.append($(summary));
    }

});