//= require ./PatientWidgetWindow.js
//= require ./PatientAppointmentsWidgetWindow.js

JLV.PatientEncountersWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Encounters',
        width: 950,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['clinic', 'taxonomy'],
        columns: [
            {
                title: 'Date',
                id: 'apptDate',
                sortable: true,
                width: 100,
                formatter: function (value, model) {

                    if (model && model.get('clinic') === 'Connection unavailable.') {
                        return JLV.common.dateFormatter(value);
                    }
                    if (model) {

                        if (model.get('complexDataUrl') || model.get('noteText') || model.get('vaNote'))
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model) + ' ' + JLV.common.linkFormatter("Note", model);
                        else
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model) + ' ' + "Note"



                    }
                    return value;
                }
            },
            {
                title: 'Clinic',
                id: 'clinic',
                width: 90,
                sortable: true

            },
            {
                width: 75,
                title: 'Status',
                sortable: true,
                id: 'status'
            },
            {
                width: 90,
                title: 'Type',
                sortable: true,
                id: 'apptType'

            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 90,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Provider Specialty',
                id: 'taxonomy',
                width: 160,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Reason',
                width: 100,
                sortable: true,
                id: 'reason'
            },
            {
                title: 'Diagnosis',
                width: 100,
                id: 'diagnosisList',
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br>' +
                                'of Medicine - Clinical Terms<br>' +
                                '(SNOMED CT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br>' +
                        'of Medicine - Clinical Terms<br>' +
                        '(SNOMED CT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized Diagnosis',
                width: 180,
                id: 'normDisplay',
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br>' +
                                'of Medicine - Clinical Terms<br>' +
                                '(SNOMED CT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br>' +
                        'of Medicine - Clinical Terms<br>' +
                        '(SNOMED CT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'appointmentDate',
            asc: false
        }

    }
});