//= require ./PatientWidgetWindow.js

JLV.PatientInpatientMedicationsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Inpatient Medications',
        width: 800,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['drugName', 'normDisplay', 'taxonomy'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Last Fill',
                id: 'lastFilledDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 80
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 130,
                sortable: true,
                formatter: function (value, model) {
                    return JLV.common.linkFormatter(value, model, null, "report");
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br>' +
                        'RxNorm Concept Unique Identifiers</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized<br/>Drug Name',
                id: 'normDisplay',
                width: 130,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br>' +
                        'RxNorm Concept Unique Identifiers</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 100,
                sortable: true
            },
            {
                width: 150,
                title: 'Ordering HCP',
                sortable: true,
                id: 'providerName',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 180,
                title: 'Ordering<br/>HCP Specialty',
                sortable: true,
                id: 'taxonomy',
                tooltip: function (row, model) {
                    try {
                        if (model) {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: {{-taxonomy}}<br/>' +
                                'NUCC Taxonomy Code: {{-taxonomyCode}}'
                            )(model.toJSON());
                        }
                    } catch (e) {
                        console.info('template error');
                    }
                }
            },
            {
                title: 'Start Date',
                id: 'startDate',
                width: 100,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Stop Date',
                id: 'stopDate',
                width: 100,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Schedule',
                id: 'sig',
                sortable: true,
                width: 100
            },
            {
                width: 60,
                title: 'Quantity',
                sortable: true,
                id: 'quantity'
            },
            {
                title: 'Site',
                id: 'site',
                width: 50,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'lastFilledDate',
            asc: false
        },
        data: JLV.InpatientMedicationsCollection
    }
});