//= require ../../DetailsWindow.js

JLV.PatientLabReportWindow = JLV.DetailsWindow.extend({
    className: 'labreport',
    labResult: null,
    options: {
        width: 700
    },
    _isValidStr: function (str) {
        return str != null && str.length > 0;
    },
    renderLabReport: function () {

        var data = this.labResult;

        //send audit details call
        $.post('patientLabs/auditDetails', {
            patToken: this.patientToken,
            recordItemId: data.accession,
            recordSiteMoniker: data.site ? data.site.moniker : '',
            recordSiteAgency: data.site ? data.site.agency : ''
        });

        var html = '<div style="padding:5px;">';
        html += _.template([
            '<table width="100%" border="0">',
            '<tr>',
            '<td><span class="report-header">Lab Test:</span>&nbsp;{{testName}}</td>',
            '<td><span class="report-header">Specimen Source:</span>&nbsp;{{specimen}}</td>',
            '</tr>',
            '<tr>',
            '<td><span class="report-header">Standardized Lab Test:</span>&nbsp;',
            '{[ if(typeof normDisplay != "undefined") { ]}',
                '{{normDisplay}}  Code: {{normCode}}',
            '{[ } ]}',
            '</td>',
            '<td><span class="report-header">Performing Lab:</span>&nbsp;{{performingLabName}}</td>',
            '</tr>',
            '<tr>',
            '<td><span class="report-header">Accession Num:</span>&nbsp;{{accession}}</td>',
            '<td><span class="report-header">Order Date:</span>&nbsp;{{orderDate}}</td>',
            '</tr>'].join(''))(data);

        if (this._isValidStr(data.resultStatus)) {
            html += _.template([
                '<tr>',
                '<td><span class="report-header">Status:</span>&nbsp;{{resultStatus}}</td>',
                '<td colspan="3"><span class="report-header">Certification Date:</span>&nbsp;{{resultDate}}</td>',
                '</tr>'
            ].join(''))(data);
        } else {
            html += _.template([
                '<tr>',
                '<td><span class="report-header">Status:</span>&nbsp;</td>',
                '<td colspan="3"><span class="report-header">Certification Date:</span>&nbsp;{{resultDate}}</td>',
                '</tr>'
            ].join(''))(data);
        }

        if (this._isValidStr(data.orderComment)) {
            html += _.template([
                '<tr>',
                '<td><span class="report-header">Order Comment:</span>&nbsp;<pre class="plain-text comment" width="100%">{{orderComment}}</pre></td>',
                '</tr>'
            ].join(''))(data);
        } else {
            html += _.template([
                '<tr>',
                '<td><span class="report-header">Order Comment:</span>&nbsp;<pre class="plain-text comment" width="100%"></pre></td>',
                '</tr>'
            ].join(''))(data);
        }

        if (this._isValidStr(data.comment)) {
            html += _.template([
                '<tr>',
                '<td colspan="4"><span class="report-header">Result Comment:</span>&nbsp;<pre class="plain-text comment" width="100%">{{comment}}</pre></td>',
                '</tr>'
            ].join(''))(data);
        } else {
            html += _.template([
                '<tr>',
                '<td colspan="4"><span class="report-header">Result Comment:</span>&nbsp;<pre class="plain-text comment" width="100%"></pre></td>',
                '</tr>'
            ].join(''))(data);
        }

        if (this._isValidStr(data.result)) {
            html += _.template([
                '<tr>',
                '<td colspan="4" valign="top" class="result-markup"><span class="report-header">Result:</span>&nbsp;<pre class="plain-text">{{result}}</pre></td>',
                '</tr>'
            ].join(''))(data);
        } else {
            html += _.template([
                '<tr>',
                '<td colspan="4" valign="top" class="result-markup"><span class="report-header">Result:</span>&nbsp;<pre class="plain-text"></pre></td>',
                '</tr>'
            ].join(''))(data);
        }
        html += '</table></div>';


        this.$el.html(html);

    },
    loadDetails: function () {

        if (!this.labResult) {
            this.$el.html("No lab results found.");
            return;
        }

        $('<div>')
            .addClass('jlv-loader').html(
            '<div class="jlv-loader-image">' +
            '<div class="jlv-table-data-loading" alt="loading indicator"></div></div>')
            .appendTo(this.$el);

        //$.post('patientLabs/report', {
        //        patToken: this.patientToken,
        //        recToken: this.labResults[0].recordDetailsToken
        //    },
        //    function (response) {
        //        self.labResults[0].result = response.reportText;
        //        self.renderLabReport();
        //    }).fail(function () {
        //        self.labResults[0].result = "There was an error retrieve details.";
        //        self.renderLabReport();
        //    });
        this.renderLabReport();
    }
});


JLV.PatientLabResultsWindow = JLV.DetailsWindow.extend({
    className: 'labresults',
    lab: null,
    options: {
        width: 900,
        height: 500
    },
    loadDetails: function () {

        if (this.labOrder && this.labOrder.get('labResults')) {
            this.labResults = this.labOrder.get('labResults')
        }


        if (!this.labResults || this.labResults.length < 1) {
            this.$el.html("No lab results found.");
            return;
        }

        var data = this.labResults[0];

        if(data.collectionDate){
            data.collectionDate = moment(data.collectionDate).format('MMM DD, YYYY HH:mm')
        }
        if (this.labOrder) {
            data.collectionDate = moment(this.labOrder.get('collectionDate')).format('MMM DD, YYYY hh:mm A');
            data.orderId = this.labOrder.get('orderId');
            //data.comment = this.labOrder.get('comment');
        }
        //else {
        //    data.collectionDate = "";
        //    data.comment = "";
        //}

        //send audit details call
        $.post('patientLabs/auditDetails', {
            patToken: this.patientToken,
            recordItemId: data.accession,
            recordSiteMoniker: data.site ? data.site.moniker : '',
            recordSiteAgency: data.site ? data.site.agency : ''
        });

        var html = '<div style="padding:5px;">';
        html += _.template([
            '<table>',
            '<thead>',
            '<tr><td class="results-header">Order Date:</td><td>{{orderDate}}</td></tr>',
            '<tr><td class="results-header">Collection Date:</td><td>{{collectionDate}}</td></tr>',
            '<tr><td class="results-header">Order Number:</td><td>{{orderId}}</td></tr>',
            '<tr><td class="results-header">Order Comments:</td><td>{{orderComment}}</td></tr>',
            '<tr><td class="results-header">Accession:</td><td>{{accession}}</td></tr>',
            '</thead>',
            '</table><br/>',
            '<table width="100%">',
            '<thead>',
            '<tr>',
            '<th scope="col" class="table-header">Lab Test</th>',
            '<th scope="col" class="table-header">Standardized Lab Test</th>',
            '<th scope="col" class="table-header">Specimen Source</th>',
            '<th scope="col" class="table-header">Result</th>',
            '<th scope="col" class="table-header">Result Date</th>',
            '<th scope="col" class="table-header">Units</th>',
            '<th scope="col" class="table-header">Interpretation</th>',
            //'<th scope="col" class="table-header">Quantity</th>',
            '<th scope="col" class="table-header">Reference Range</th>',
            '<th scope="col" class="table-header">Certification Date</th>',
            '<th scope="col" class="table-header">Performing Lab</th>',
            '<th scope="col" class="table-header">Comment</th>',
            '</tr>',
            '</thead>',
            '<tbody>'
        ].join(''))(data);


        var count = this.labResults.length,
            i = 0;

        for (; i < count; i++) {

            var resultData = this.labResults[i];

            var test = resultData.printName;

            if (resultData.testName && resultData.testName.length > 0) {
                test = resultData.testName;
            }

            var bgcolor = "white";
            var abnFlag = false;
            if (resultData.interpretation && resultData.interpretation.length > 0) {
                bgcolor = "#FFCCCC";
                if (resultData.interpretation.toLowerCase().indexOf("critical") > -1 || resultData.interpretation.indexOf("*") > 0) {
                    bgcolor = "#901100";
                    abnFlag = true;
                }
            }
            var certifyinit = "";
            var certifydt = "\nCertified: " + resultData.resultDate;

            if (resultData.verifiedBy && resultData.verifiedBy.length > 0) {
                certifyinit = ":" + resultData.verifiedBy;
            }

            var certify = certifydt + certifyinit;

            resultData.test = test;
            resultData.bgcolor = bgcolor;
            resultData.abnFlag = abnFlag;
            resultData.certify = certify;

            resultData.resultDate = moment(resultData.resultDate).format("M/D/YY");

            if (resultData.abnFlag) {
                html += _.template([
                    '<tr bgcolor="{{bgcolor}}">',
                    '<td style="color:white;">',
                    '{{test}}',
                    '</td>',
                    '<td style="color:white;">',
                    '{[ if(typeof normDisplay != "undefined") { ]}',
                    '{{normDisplay}}  <br/>Standardized Code: {{normCode}}',
                    '{[ } ]}',
                    '</td>',
                    '<td style="color:white;">',
                    '{{specimen}}',
                    '</td>',
                    '<td style="color:white;">',
                    '{{result}}',
                    '</td>',
                    '<td style="color:white;">',
                    '{{resultDate}}',
                    '</td>',
                    '<td style="color:white;">',
                    '{{units}}',
                    '</td>',
                    '<td style="color:white;">',
                    '{{interpretation}}',
                    '</td>',
                    //'<td style="color:white;">',
                    //'{{quantity}}',
                    //'</td>',
                    '<td style="color:white;">',
                    '{{referenceRange}}',
                    '</td>',
                    '<td style="color:white;">',
                    '{{certify}}' +
                    '</td>',
                    '<td style="color:white;">',
                    '{{performingLabName}}' +
                    '</td>',
                    '<td style="color:white;">',
                    '{{comment}}' +
                    '</td>',
                    '</tr>'
                ].join(''))(resultData);
            } else {
                html += _.template([
                    '<tr bgcolor="{{bgcolor}}">',
                    '<td style="color:black;">',
                    '{{test}}',
                    '</td>',
                    '<td style="color:black;">',
                    '{[ if(typeof normDisplay != "undefined") { ]}',
                    '{{normDisplay}}  <br/>Standardized Code: {{normCode}}',
                    '{[ } ]}',
                    '</td>',
                    '<td style="color:black;">',
                    '{{specimen}}',
                    '</td>',
                    '<td style="color:black;">',
                    '{{result}}',
                    '</td>',
                    '<td style="color:black;">',
                    '{{resultDate}}',
                    '</td>',
                    '<td style="color:black;">',
                    '{{units}}',
                    '</td>',
                    '<td style="color:black;">',
                    '{{interpretation}}',
                    '</td>',
                    //'<td style="color:black;">',
                    //'{{quantity}}',
                    //'</td>',
                    '<td style="color:black;">',
                    '{{referenceRange}}',
                    '</td>',
                    '<td style="color:black;">',
                    '{{certify}}' +
                    '</td>',
                    '<td style="color:black;">',
                    '{{performingLabName}}' +
                    '</td>',
                    '<td style="color:black;">',
                    '{{comment}}' +
                    '</td>',
                    '</tr>'].join(''))(resultData);
            }
            }

        html += '</tbody></table>';
        html += '<table><tr><td style="padding:10px;"><span>L=Low | H=High | *=Critical | R=Resist | S=Susc | MS=Mod Susc | I=Intermed | []=Uncert <br>/A=Amended | Comments=  (O)rder, (I)nterpretations, (R)esult</span></td></tr></table>';
        html += '</div>';

        this.$el.html(html);


    }
});