//= require ./PatientWidgetWindow.js
//= require ./LabResultsGraphWindow.js

JLV.PatientLabResultsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Lab Results',
        width: 1000,
        minWidth: 800,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        gridHeaderHeight: 30,
        fitColumns: false,
        filters: ['testName', 'normDisplay', 'accessionType'],
        columns: [
            {
                title: 'Date',
                id: 'collectionDate',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                },
                sortable: true,
                width: 120
            },
            {
                title: 'Order Number',
                id: 'orderId',
                sortable: true,
                width: 80
            },
            {
                title: 'Status',
                id: 'resultStatus',
                sortable: true,
                width: 80
            },
            {
                width: 120,
                title: 'Ordering HCP',
                sortable: true,
                id: 'providerName',
                tooltip: function (row, model) {

                    if (model) {
                        try {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description:' +
                                '{[ if (obj.taxonomy) { ]}' +
                                '{{-taxonomy}}' +
                                '{[ } ]}' +
                                '<br/>NUCC Taxonomy Code: ' +
                                '{[ if (obj.taxonomyCode) { ]}' +
                                '{{-taxonomyCode}}' +
                                '{[ } ]}' +
                                '<br/>Requesting Location: {{-requestingLocation}}' //+
                                //'<b>Provider Address: </b>{{-provider.street1}} {{-provider.street2}} {{-provider.street3}}' +
                                //'{{-provider.city}}, {{-provider.stateProvince}} {{-provider.postalCode}} <br/>' +
                                // '<b>Provider Phone: </b> {{-provider.phone}}'
                            )(model.toJSON());
                        } catch (e) {
                            console.info('template error');
                        }
                    }
                }
            },
            {
                width: 120,
                title: 'Ordering HCP <br/> Specialty',
                sortable: true,
                id: 'taxonomy',
                tooltip: function (row, model) {

                    if (model) {
                        try {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description:' +
                                '{[ if (obj.taxonomy) { ]}' +
                                '{{-taxonomy}}' +
                                '{[ } ]}' +
                                '<br/>NUCC Taxonomy Code: ' +
                                '{[ if (obj.taxonomyCode) { ]}' +
                                '{{-taxonomyCode}}' +
                                '{[ } ]}' +
                                '<br/>Requesting Location: {{-requestingLocation}}' //+
                                //'<b>Provider Address: </b>{{-provider.street1}} {{-provider.street2}} {{-provider.street3}}' +
                                //'{{-provider.city}}, {{-provider.stateProvince}} {{-provider.postalCode}} <br/>' +
                                // '<b>Provider Phone: </b> {{-provider.phone}}'
                            )(model.toJSON());
                        } catch (e) {
                            console.info('template error');
                        }
                    }
                }
            },
            {
                title: 'Lab Test',
                id: 'testName',
                width: 110,
                sortable: true,
                formatter: function (value, model) {

                    if (model && model.get('interpretation')) {
                        if(model.get('interpretation').toLowerCase().indexOf("critical") > -1
                            || model.get('interpretation').toLowerCase().indexOf("*") > -1){
                            return '<div class="jlv-high-alert-row">' + JLV.common.linkFormatter(value, model) + '</div>';
                        }
                        return '<div class="jlv-alert-row">' + JLV.common.linkFormatter(value, model) + '</div>';
                    }

                    return JLV.common.linkFormatter(value, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes<br>' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized Lab<br/> Test',
                width: 150,
                sortable: true,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes<br>' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 50,
                title: 'Type',
                sortable: true,
                id: 'accessionType'
            },
            {
                width: 70,
                title: 'Specimen<br/>Source',
                sortable: true,
                id: 'specimen'
            },
            {
                title: 'Result',
                id: 'result',
                width: 60,
                sortable: true,
                formatter: function (value, model) {
                    if( model.get('accessionType') == "MICRO"
                        || model.get('accessionType') == "MI"
                        || model.get('accessionType') == "AP"){
                        return JLV.common.linkFormatter("result", model);
                    } else if (!isNaN(parseFloat(value)) && model.get('units')) {
                        return value + " " + model.get('units')
                    }

                    return value;
                }
            },
            /*{
                title: 'Abnormal <br/> Indicators',
                id: 'abnormalIndicators',
                width: 120,
                sortable: true
             },*/
            {
                title: 'Interpretation',
                id: 'interpretation',
                width: 120,
                sortable: true
            },
            {
                width: 60,
                title: 'Units',
                sortable: true,
                id: 'units'
            },
            {
                width: 70,
                title: 'Ref Range',
                sortable: true,
                id: 'referenceRange'
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 50,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'resultDate',
            asc: false
        }
    },
    toggleFilterPanel: function () {
        if (!this._filterPanel) {

            var filterIds = this.get('filters');
            var filters = _.toArray(_.pick(_.indexBy(this.get('columns'), 'id'), filterIds));

            var data = this.get('data');

            if (data) {
                if (data.startDate !== undefined && data.endDate !== undefined) {
                    this._filterPanel = new JLV.PatientLabResultsWidgetWindow.FilterPanel({
                        window: this,
                        startDate: data.startDate.toDate(),
                        endDate: data.endDate.toDate(),
                        filters: filters,
                        data: data
                    });

                    this._filterPanel.$el.prependTo(this._body);
                    this._filterPanel.$el.hide();
                }
            }
        }

        this._super();
    },
    initialize: function (options) {
        this._super(options);

        if (this._datagrid) {
            //make rows with abnormal results pink
            this._datagrid.on('afterRender', function () {

                var $row = this.$el.find('.jlv-alert-row').parents('tr');

                $row.css(
                    {
                        'backgroundColor': '#FFC0CB',
                        'color': '#000000'
                    }
                );

                $row.find('a').css('color', '#000');

                var $cRow = this.$el.find('.jlv-high-alert-row').parents('tr');
                $cRow.css(
                    {
                        'backgroundColor': '#901100',
                        'color': '#FFFFFF'
                    }
                );
                $cRow.find('a').css('color', '#FFF');
            });
        }


    }
});


jQuery(function () {

    JLV.PatientLabResultsWidgetWindow.FilterPanel = JLV.WidgetWindow.DateFilterPanel.extend({
        initialize: function (options) {

            this._super(options);

            //add table button
            this._tableButton = new AppKit.Button({
                className: 'jlv-filter-field-button',
                label: "Show Table",
                enabled: false,
                action: _.bind(function () {
                    //generate table html
                    if (this.window) {

                        var lab = this.subview('.first-combo').get('value');
                        var stdLab = this.subview('.second-combo').get('value');

                        var data = this.window.get('datagrid').tableData();
                        var i = 0,
                            count = data.length;

                        var html = '<div></div><table class="jlv-filtered-table-view"><tr><th>Date</th>' +
                            '<th>Test</th>' +
                            '<th>Specimen Source</th>' +
                            '<th>Result</th>' +
                            '<th>Units</th>' +
                            '<th>Interpretation</th>' +
                            //'<th>Quantity</th>' +
                            '<th>Reference Range</th>' +
                            '<th>Site</th>' +
                            '</tr>';

                        var itemCount = 0;
                        var units = '';
                        for (; i < count; i++) {
                            var model = data[i];
                            var interpretation = model.get('interpretation');

                            if (interpretation && interpretation.length) {
                                html += '<tr style="background:#ffcdcb;">';
                            }
                            else {
                                html += '<tr>';
                            }

                            html += '<td>' + JLV.common.dateFormatter(model.get('resultDate')) + '</td>';
                            html += '<td>' + model.get('testName') + '</td>';
                            html += '<td>' + model.get('specimen') + '</td>';
                            html += '<td>' + model.get('result') + '</td>';
                            html += '<td>' + model.get('units') + '</td>';
                            html += '<td>' + model.get('interpretation') + '</td>';
                            //html += '<td>' + model.get('quantity') + '</td>';
                            html += '<td>' + model.get('referenceRange') + '</td>';
                            html += '<td>' + JLV.common.formatMonikerDisplay(model.get('site')) + '</td>';
                            html += '</tr>';

                            itemCount++;
                        }

                        html += '</table>';

                        //key
                        html += "<table style='font-size:11px;'><tr><td style='padding:10px;'><span>L=Low | H=High | *=Critical | R=Resist | S=Susc | MS=Mod Susc | I=Intermed | []=Uncert <br>/A=Amended | Comments=  (O)rder, (I)nterpretations, (R)esult</span></td></tr></table></div>";


                        var tableDialog = new JLV.DetailsWindow({
                            title: String.format("Lab Results - {0} ({1})", lab.length ? lab : stdLab, itemCount),
                            width: 850,
                            height: 500,
                            renderer: function () {
                                return html;
                            }
                        });

                        tableDialog.center();
                        tableDialog.open();
                    }

                }, this)
            });

            //add graph button
            this._graphButton = new AppKit.Button({
                className: 'jlv-filter-field-button',
                label: "Show Graph",
                enabled: false,
                action: _.bind(function () {

                    var lab = this.subview('.first-combo').get('value');
                    var dataArray = this.window.get('datagrid').tableData();

                    if (dataArray.length) {
                        var graphDialog = new JLV.LabResultsGraphWindow({
                            title: String.format("Lab Results Graph - {0}", lab),
                            data: dataArray,
                            units: dataArray[0].get('units'),
                            type: lab
                        });

                        graphDialog.center();
                        graphDialog.open();
                    }


                }, this)
            });

            $("<div>").addClass('field-filter-buttons')
                .append(this._graphButton.$el)
                .append(this._tableButton.$el)
                .appendTo(this.$el);


        },
        clearFilters: function (skip) {

            if (this._graphButton) {
                this._graphButton.set('enabled', false);
            }

            if (this._tableButton) {
                this._tableButton.set('enabled', false);
            }

            this._super(skip);
        },
        applyFilters: function () {
            this._super();

            var firstCombo = this.subview('.first-combo');

            if (firstCombo.get('selectedIndex') > 0) {
                this._tableButton.set('enabled', true);

                if (this.window) {
                    var dataArray = this.window.get('datagrid').tableData();
                    if (dataArray.length) {
                        var model = dataArray[0];
                        var result = model.get('result');

                        this._graphButton.set('enabled', !_.isNaN(parseInt(result, 10)));
                    }
                }
            }
            else {
                this._tableButton.set('enabled', false);
                this._graphButton.set('enabled', false);
            }

            var secondCombo = this.subview('.second-combo');

            if (secondCombo.get('selectedIndex') > 0) {
                this._tableButton.set('enabled', true);
            }
            else if (firstCombo.get('selectedIndex') === 0) {
                this._tableButton.set('enabled', false);
            }
        }
    });
});