//= require ./PatientWidgetWindow.js

JLV.PatientOutpatientMedicationsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Outpatient Medications',
        width: 1024,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['drugName', 'normDisplay', 'taxonomy'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Last Fill',
                id: 'lastFilledDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 130,
                sortable: true,
                formatter: function (value, model) {
                    return JLV.common.linkFormatter(value, model, null, "report");
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br>' +
                        'RxNorm Concept Unique Identifiers</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized <br/>Drug Name',
                id: 'normDisplay',
                width: 120,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br>' +
                        'RxNorm Concept Unique Identifiers</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 100,
                title: 'Sig',
                sortable: true,
                id: 'sig'
            },

            {
                width: 100,
                title: 'Quantity',
                sortable: true,
                id: 'quantity'
            },
            {
                width: 100,
                title: 'Days Supply',
                sortable: true,
                id: 'daysSupply'
            },
            {
                width: 100,
                title: 'Refills',
                sortable: true,
                id: 'refills',
                formatter: function (value, model) {
                    if (model && model.get('site')) {
                        var site = model.get('site');
                        if (site.agency === JLV.common.AGENCY_VA) {
                            if (model && model.get('prescriptionFills')
                                && model.get('prescriptionFills').length
                                && model.get('lastFilledDate')) {
                                return JLV.common.linkFormatter(value, model);
                            }
                        }
                    }

                    return value;
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 100,
                sortable: true

            },
            {
                width: 120,
                title: 'Ordering HCP',
                sortable: true,
                id: 'providerName',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 120,
                title: 'Ordering HCP <br/> Specialty',
                sortable: true,
                id: 'taxonomy',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Expires',
                id: 'stopDate',
                width: 100,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                width: 50,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'lastFilledDate',
            asc: false
        }
    },
    initialize: function (options) {
        this._super(options);

        this.set('copyActiveMeds', true);
    },
    copyActiveMeds: function (aBool) {
        // Do not show/create copy active Rxs to clipboard button for outside providers (VACV)
        if (!JLV.common.OUTSIDE_PROVIDER_FLAG) {
            if (aBool === true) {

                if (!this._copyActiveMedsBtn) {
                    var self = this;
                    this._copyActiveMedsBtn = $('<div title="Copy Active Rxs to clipboard" tabindex="0" class="jlv-window-toolbutton">')
                        .append('<div class="icon-med-drugs">')
                        .button()
                        .insertBefore(self.$el.parent().find('.clipboard'))
                        .on('click', function (evt) {

                            if (self.widget) {
                                self.widget._doCopyActiveMeds();
                            }
                        });
                }

                this._copyActiveMedsBtn.show();
            }
            else if (aBool === false) {
                this._copyActiveMedsBtn.hide();
            }

            return this._copyActiveMedsBtn.is(':visible');
        }
    }
});