//= require ./PatientWidgetWindow.js

JLV.PatientRadiologyExamsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Radiology Exams',
        width: 1000,
        gridHeaderHeight: 30,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['study', 'normDisplayLoinc'],
        columns: [
            {
                title: 'Date',
                id: 'examDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Exam<br/>Number',
                id: 'examId',
                sortable: true,
                width: 70
            },
            {
                title: 'Exam',
                id: 'study',
                width: 100,
                sortable: true,
                formatter: function (value, model) {
                    var formattedString = value;

                    if (model && model.get('recordDetailsToken')) {
                        formattedString = JLV.common.linkFormatter(value, model);
                    }

                    if (model && model.get('abnormal')) {
                        if (model.get('abnormal').toLowerCase().indexOf('abnormal') > -1
                            || model.get('abnormal').toLowerCase() !== "normal")
                        {
                            formattedString = '<div class="jlv-alert-row">' + formattedString + '</div>';
                        }
                    }
                    return formattedString;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Type: ' + normDisplayLoinc + '<br>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br>' +
                                'Standardized Type: ' + normDisplayCpt + '<br>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 120,
                title: 'Standardized <br/> Radiology Exam',
                sortable: true,
                id: 'normDisplayLoinc',
                formatter: function (value, model) {
                    if (model && model.get('normCodeLoinc')) {
                        return value;
                    }
                    return '';
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Type: ' + normDisplayLoinc + '<br>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br>' +
                                'Standardized Type: ' + normDisplayCpt + '<br>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 120,
                title: 'CPT Description',
                id: 'normDisplayCpt',
                sortable: true,
                formatter: function (value, model) {
                    if (model && model.get('normCodeCpt')) {
                        return value;
                    }
                    return '';
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Type: ' + normDisplayLoinc + '<br>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br>' +
                                'Standardized Type: ' + normDisplayCpt + '<br>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }

            },
            {
                id: 'status',
                width: 100,
                title: 'Status',
                sortable: true
            },
            {
                title: 'Results',
                id: 'abnormal',
                sortable: true,
                width: 100
            },
            {
                title: 'Image',
                id: 'image',
                width: 60,
                sortable: false,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {

                        if (!JLV.common.ALLOW_MEDWEB_PLUGIN || !model.get('medWebAvailable')) {
                            return '';
                        }

                        var medWebURL = String.format("patientMedWeb?patToken={0}&recToken={1}",
                            JLV.ActivePatientModel.get('patientToken'), model.get('medWebToken'));

                        var medWebCmd = String.format("JLV.common.openAuxBrowserWindow(\"{0}\", \"JLV_MEDWEB\");return false;", medWebURL);

                        return String.format('<span class="medweb-link" title="View in MedWeb" onclick=\'{0}\' alt="Link View in Medweb"><img src="JLV/assets/camera_icon.png"  alt=\'Camera Icon\'/></span>', medWebCmd);
                    }

                    return '';

                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'examDate',
            asc: false
        }
    },
    initialize: function (options) {
        this._super(options);

        if (this._datagrid) {
            //make rows with abnormal results pink
            this._datagrid.on('afterRender', function () {

                var $row = this.$el.find('.jlv-alert-row').parents('tr');

                $row.css(
                    {
                        'backgroundColor': '#FFC0CB',
                        'color': '#000000'
                    }
                );

                $row.find('a').css('color', '#000');
            });
        }
    }
});