//= require ../Widget.js

JLV.ProviderWidget = JLV.Widget.extend({
    savePortalUI: function () {
        JLV.NotificationCenter.postNotificationName(JLV.Desktop.SaveProviderPortalUINotification);
    },
    /**
     *
     * Fetch the data from the server using the Collection
     *
     **/
    loadData: function (callback) {

        var data = this.get('data');

        if (!data && this._datagrid) {
            console.warn(this.get('name') + ' widget data not set.');
        }
        if (data && this._datagrid) {

            if (!_.isFunction(data.doFetch)) {
                this._datagrid.render();
                return;
            }

            data.reset();

            this._super();

            var self = this;
            data.doFetch(function () {

                    if (self._datagrid.get('rowCount') === 0) {
                        self._datagrid.showMessage("No Data");
                    }
                    else if (self._defaultSort !== null) {

                        var column = _.findWhere(self.options.columns, {id: self_.defaultSort.id});
                        if (column) {
                            column.index = _.indexOf(self.options.columns, column);
                            var sortInfo = {
                                asc: self._defaultSort.asc,
                                column: column
                            };

                            self._datagrid.sortByColumn(sortInfo);
                            self._datagrid.render();

                            if (self.get('moreWindowClass')) {
                                self._moreBtn.show();
                            }


                        }
                        else {
                            self._datagrid.render();

                            if (self.get('moreWindowClass')) {
                                self._moreBtn.show();
                            }
                        }

                    }
                    else {
                        self._datagrid.render();

                        if (self.get('moreWindowClass')) {
                            self._moreBtn.show();
                        }


                    }
                    //update the widget title to reflect the number of records
                    self.set('title', self.get('name') + ' (' + self._datagrid.get('rowCount') + ')');

                    /* render the more window in the background,
                     delay so  widgets on page load smooth
                     */
                    if (!this._moreWindow) {
                        setTimeout(function () {
                            if (!self._moreWindow) {
                                if (self.get('moreWindowClass')) {
                                    self._createMoreWindow();
                                }
                            }
                        }, 10000);
                    }

                    if (_.isFunction(callback)) {
                        callback();
                    }

                    self._updatePageInfoDisplay();

                },
                /* handle error */
                function (collection, err) {

                    console.info(err)

                    self._datagrid.showMessage(_.template(
                            '<div class="server-error-msg" style="color:red;cursor:pointer;text-decoration:underline;">{{statusText}} ({{status}})</div>')(err)
                    );

                    self._datagrid.$el.find('.server-error-msg').click(function () {
                        var alertDialog = new AppKit.AlertDialog({
                            title: err.statusText,
                            message: err.responseText,
                            icon: 'icon-frown-o',
                            cancelButton: false,
                            height: 500
                        });

                        alertDialog.center();
                        alertDialog.set('visible', true);
                    });

                });
        }
    },
    doPrint: function () {

        if (this._datagrid) {
            //fit columns to 775 width
            this._datagrid.$el.width(775);
            this._datagrid.sizer();

            try {

                var printHTML = _.template('<html><head><title>{{title}}</title>\n{{fontLink}}\n{{cssLink1}}\n{{cssLink2}}</head><body>{{tableHTML}}</body></html>')({
                    title: this.get('name') + " Print out - ",
                    fontLink: '<link rel="stylesheet" type="text/css" href="assets/fonts/opensans/style.css">',
                    cssLink1: '<link rel="stylesheet" type="text/css" href="assets/AppKit/datatable.css">',
                    cssLink2: '<link rel="stylesheet" type="text/css" href="assets/view/print.css">',
                    tableHTML: this._datagrid.$el.html()
                });

                var printWindow = window.open("", "Print");
                printWindow.document.write(printHTML);
                printWindow.document.close();
                setTimeout(function () {
                    printWindow.print();
                    printWindow.close();
                }, 200);

            }
            catch (e) {

            }


            this._datagrid.$el.width('100%');
            this._datagrid.sizer();


        }

    }
});