//= require ProviderWidget.js


JLV.ProviderReferences = JLV.ProviderWidget.extend({
    initialize: function (options) {

        if (!JLV.Session.prov_ref) {
            JLV.Session.prov_ref = [
                {linkDesc: 'CDC', linkUrl: 'http://www.cdc.gov/'},
                {linkDesc: 'FDA', linkUrl: 'http://www.fda.gov'},
                {linkDesc: 'Medscape', linkUrl: 'http://emedicine.medscape.com'},
                {linkDesc: 'webMD.com', linkUrl: 'http://www.webmd.com'},
                {linkDesc: 'DermAtlas', linkUrl: 'http://dermatlas.med.jhmi.edu/derm/'}
            ];
        }

        var self = this;
        JLV.NotificationCenter.addObserverForNotification({
            name: 'JLV.ProviderReferences.Update',
            action: function () {
                self.set('_data', new Backbone.Collection(JLV.Session.prov_ref));
                self.loadData();
            }
        });


        this._super(options);

        this.set('_data', new Backbone.Collection(JLV.Session.prov_ref));
    },
    options: {
        widgetId: 'prov_references_widget',
        name: 'References',
        plus: true,
        dateFilter: false,
        status: false,
        clipboard: true,
        print: true,
        refresh: false,
        columns: [
            {
                title: "Link",
                width: 55,
                sortable: true,
                id: 'linkDesc',
                formatter: function (value, model) {
                    return String.format(
                        '<b><a id="{1}" class="referenceLink" href="{1}" target="_blank" title="{1}">{0}</a></b>',
                        value, model.get('linkUrl'));
                }
            }
        ]
    },
    onAdd: function () {
        var addReferencesDialog = $('#AddReferencesDialog').widget();
        addReferencesDialog.center();
        addReferencesDialog.set('visible', true);
    }
});


JLV.registerJLVWidget('prov_references_widget', JLV.ProviderReferences);

JLV.ProviderReferences.AddReferencesController = {
    onDialogOpen: function () {
        //reset
        this.ReferenceNameInput.set('value', '');
        this.ReferenceURLInput.set('value', '');

        setTimeout(_.bind(function () {
            this.ReferenceNameInput.$el.focus();
        }, this), 0);
    },
    onAdd: function () {
        var name = _.escape(this.ReferenceNameInput.get('value'));
        if (name && name.trim().length > 0) {
            var url = _.escape(this.ReferenceURLInput.get('value'));

            if (url.substring(0, 7) !== 'http://') {
                url = 'http://' + url;
            }


            JLV.Session.prov_ref.push({
                linkDesc: name,
                linkUrl: url
            });

            JLV.NotificationCenter.postNotificationName('JLV.ProviderReferences.Update', this);
            JLV.NotificationCenter.postNotificationName('saveProviderPortalUI');

            $('#AddReferencesDialog').widget().set('visible', false);

        }
        else {
            AppKit.AlertDialog.alert('Form Error', 'Missing reference name.');
        }


    },
    onCancel: function (sender) {
        $('#AddReferencesDialog').widget().set('visible', false);
    }
};