package jlv

import gov.va.med.enterprise.bean.ResponseGUIPatientQuery
import gov.va.med.foundations.utilities.EncryptUtil
import gov.va.med.foundations.utilities.VistaKernelHash
import gov.va.med.groovy.user.AppUser
import gov.va.med.guibeans.GUISite
import gov.va.med.guibeans.GUIUser
import gov.va.med.jmeadows.webservice.IehrUserProfile
import gov.va.med.jmeadows.webservice.PdwsQueryBean
import org.apache.commons.lang3.StringUtils

class AHLTALoginController {
    def authUserDao
    def userDao
    def tokenUtils
    def patientDao
    def auditDao
    def appConfig

    def index = {
        if (request.method != "POST") {
            return false
        }

        String userEdipi = params["userEDIPI"]
        String userName = params["userName"]
        String userNcid = params["userNCID"]
        String patientEdipi = params["patientEDIPI"]
        String resourceUserId = params["resourceUserID"]
        String resourcePassword = params["resourcePassword"]
        String facilityNcid = params["facilityNCID"]
        String loginAgency = "DOD"
        IehrUserProfile profile

        session.profileName = userName
        session.ahltaLoginFlag = 'false'

        if (StringUtils.isEmpty(userEdipi)) {
            userEdipi = "N" + userNcid;
        }

        if (appConfig.getAuditAhltaLoginAttempt()) {
            StringBuilder auditAttempt = new StringBuilder()
            auditAttempt.append("AHLTALoginAttempt")
            auditAttempt.append("^userEDIPI=" + userEdipi)
            auditAttempt.append("^userName=" + userName)
            auditAttempt.append("^userNCID=" + userNcid)
            auditAttempt.append("^patientEDIPI=" + patientEdipi)
            auditAttempt.append("^resourceUserID=" + resourceUserId)
            auditAttempt.append("^facilityNCID=" + facilityNcid)
            auditDao.auditDetailsQuery(auditAttempt.toString(), null, null, null, null, null)
        }

        // Verify user is whitelisted in AUTH_USER table
//        try {
//            AuthUserInfo authUserInfo = new AuthUserInfo();
//            authUserInfo.setCardId(userEdipi)
//            authUserInfo.setOrganization(loginAgency)

//            authUserInfo = authUserDao.getAuthUser(authUserInfo)

//            if (!authUserInfo) {
//                log.error("Unauthorized user")
//                response.sendError(403, "UserAccessDenied")
//                return false
//            }
//        }
//        catch (DaoException e) {
//            log.error("Failed to verify user authorization: " + e.getMessage())
//            response.sendError(403, "UserAccessDenied")
//            return false
//        }

        // Load user profile into session
        try {
            profile = userDao.getIehrUserProfile(userEdipi, loginAgency);

            // Profile exists in database
            if (StringUtils.isNotEmpty(profile.getUserID())) {
                if (profile.getFlags() == null) {
                    profile.setFlags("")
                }
                if (profile.getLocID() == null) {
                    profile.setLocID("")
                }
                if (profile.getCfg() == null) {
                    profile.setCfg("")
                }
            }
            // Profile was not in database
            else {
                profile.setUserID("")
                profile.setSmartCardID(userEdipi)
                profile.setSmartCardAgency(loginAgency)
                profile.setFlags("")
                profile.setLocID(facilityNcid);
                profile.setCfg("")
            }

            session.userProfile = profile;
        }
        catch (Exception e) {
            log.error("Failed to retrieve user profile: " + e.getMessage())
            response.sendError(403, "AHLTALoginError-Profile") // Failed to load user profile
            return false
        }

        // Get list of active sites that have endpoints from database
        if (!session.sites) {
            try {
                def sites = userDao.getSites()

                if (sites) {
                    Iterator<GUISite> i = sites.iterator()
                    GUISite site

                    while (i.hasNext()) {
                        site = i.next()

                        if (!site.getEndpoints() || site.getEndpoints().size() == 0 || site.getStatus() != "active") {
                            i.remove()
                        }
                    }

                    session.sites = sites
                }
            }
            catch (Exception e) {
                log.error("Failed to retrieve sites list: " + e.getMessage())
                response.sendError(403, "AHLTALoginError-Sites") // Failed to load active sites
                return false
            }
        }

        // Create GUISite
        GUISite hostSite = new GUISite()
        hostSite.setAgency(loginAgency)
        hostSite.setDmisId(facilityNcid)
        hostSite.setMoniker("AHLTA")
        hostSite.setName("AHLTA")
        hostSite.setSiteCode(facilityNcid)
        hostSite.setStatus("active")

        // Create GUIUser
        GUIUser user = new GUIUser()
        if (userName.indexOf(",") < 0) {
            userName += ",JLV"
        }
        user.setHostSite(hostSite)
        user.setAgency(loginAgency)
        user.setName(userName.toUpperCase())
        user.setUserIen(userNcid)
        user.setEncryptedAvCode(VistaKernelHash.encrypt(resourceUserId + ";" + resourcePassword, false));
        user.setEncryptedFederatedUid(EncryptUtil.encrypt("123456789"))

        AppUser appUser = new AppUser(user)

        try {
            if (userDao.setIehrUserProfile(profile)) {
                if (StringUtils.isBlank(profile.getUserID())) {
                    profile = userDao.getIehrUserProfile(profile.getSmartCardID(), profile.getSmartCardAgency());
                }
            }
        }
        catch (Exception e) {
            log.error("Failed to save user profile: " + e.getMessage());
            response.sendError(403, "AHLTALoginError-ProfileSave")
            return false;
        }

        appUser.setSecurityKey(tokenUtils.generateSecurityKey())
        appUser.setUserId(profile.getUserID())
        appUser.setCfg(profile.getCfg())

        session.appUser = appUser
        session.userProfile = profile

        try {
            PdwsQueryBean qb = new PdwsQueryBean();
            qb.setPatientId(patientEdipi);
            qb.setPatientIdType("EDIPI");
            qb.setInquiryType("PR");

            ResponseGUIPatientQuery response = patientDao.lookupPatientPDWS(session.appUser, qb, session.appUser.securityKey)

            // PDWS search by EDIPI returns either 0 (no match) or 1 (match) result
            if (response && response.getGuiPatients() && response.getGuiPatients().size > 0) {
                session.ahltaLoginPatient = response.getGuiPatients().get(0)
                session.ahltaLoginFlag = 'true'

                // Successful login audit
                StringBuilder auditSuccess = new StringBuilder()
                auditSuccess.append("AHLTALoginSuccess")
                auditSuccess.append("^userEDIPI=" + userEdipi)
                auditSuccess.append("^userName=" + userName)
                auditSuccess.append("^userNCID=" + userNcid)
                auditSuccess.append("^patientEDIPI=" + patientEdipi)
                auditSuccess.append("^resourceUserID=" + resourceUserId)
                auditSuccess.append("^facilityNCID=" + facilityNcid)
                auditDao.auditDetailsQuery(auditSuccess.toString(), null, null, null, null, null)

                redirect(controller: 'app')
                return true
            } else {
                log.error("Could not retrieve patient from PDWS")
                response.sendError(403, "AHLTALoginError-PDWSRetrieve")
                return false
            }
        }
        catch (Exception e) {
            log.error("Failed retrieve patient from PDWS: " + e.getMessage())
            response.sendError(403, "AHLTALoginError-PDWSRetrieve")
            return false
        }
    }
}
