/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUIAdmission
import gov.va.med.vhahon.common.Utils
import org.apache.commons.lang.StringUtils

class PatientAdmissionsController extends BaseController {
    def admissionDao

    static allowedMethods = [list: 'POST', details: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        }

        dateRangeCheck();
    }

    def list = {

        def guiAdmissions = admissionDao.getPatientAdmissions(session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        //generate record details token
        if (guiAdmissions != null) {
            for (def guiAdm : guiAdmissions) {
                if (StringUtils.isNotEmpty(guiAdm.getAdmissionDate())) {
                    def queryMap = [(HttpKeys.ADMISSION_DATE): guiAdm.getAdmissionDate().toString()]

                    guiAdm.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    "patientAdmissions",
                                    queryMap,
                                    guiAdm.getSite(), guiAdm.getSourceProtocol()))
                }
            }

            if (session.appUser.vlerSiteStatus != null && session.appUser.vlerSiteStatus.length() > 0) {
                GUIAdmission statusRecord = new GUIAdmission();
                statusRecord.setWard(session.appUser.vlerSiteStatus);
                statusRecord.setDiagnosisText(session.appUser.vlerSiteStatus);
                statusRecord.setSite(getStatusReportSite());
                guiAdmissions.add(statusRecord);
            }
        }

        sendResponse(guiAdmissions)
    }

    def details = {

        def guiAdmissionDetails = admissionDao.getPatientAdmissionDetails(
                session.appUser,
                request.patient,
                request.recordQuery[HttpKeys.ADMISSION_DATE],
                request.recordQuery[HttpKeys.RECORD_SITE_CODE],
                Utils.getCal(params[HttpKeys.START_DATE]),
                Utils.getCal(params[HttpKeys.END_DATE]))

        sendResponse(guiAdmissionDetails)
    }
}
