/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUIProgressNote
import gov.va.med.vhahon.common.Utils
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.lang.StringUtils

class PatientDischargeSummariesController extends PatientNoteDetailBaseController {
    def noteDao
    def auditDao

    static allowedMethods = [list: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        } else dateRangeCheck();
    }

    def list = {

        def guiInpatientNotes = noteDao.getPatientDischargeSummaries(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        if (guiInpatientNotes != null) {
            for (def guiInpatientNote : guiInpatientNotes) {
                if (StringUtils.isNotEmpty(guiInpatientNote.getNoteId())) {
                    def queryMap = [(HttpKeys.NOTE_ID)            : guiInpatientNote.getNoteId(),
                                    (HttpKeys.RECORD_DATE)        : guiInpatientNote.getNoteDate(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiInpatientNote.site.moniker,
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiInpatientNote.site.agency]

                    guiInpatientNote.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiInpatientNote.getSite(),
                                    guiInpatientNote.getSourceProtocol(),
                                    guiInpatientNote.isSessionCachedRecord()))

                    if (guiInpatientNote.isSessionCachedRecord()) {
                        //cache inpatient progress note in session
                        def progressNoteDetail = BeanUtils.cloneBean(guiInpatientNote)

                        session.appUser.inpatientNotesMap.put(
                                getCachedRecordKey(guiInpatientNote.getRecordDetailsToken()), progressNoteDetail)

                        //don't send note text with note listings
                        guiInpatientNote.setNoteText("")
                        guiInpatientNote.setImages("")
                    }
                }
            }
        }

        sendResponse(guiInpatientNotes)
    }

    def details = {

        //discharge summary notes are all cached
        def guiNote = null

        //audit details query
        auditDao.auditDetailsQuery("DischargeSummary", session.appUser, request.patient,
                request.recordQuery[HttpKeys.RECORD_DATE],
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

        guiNote = session.appUser.inpatientNotesMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        if (guiNote.status == "CDA" && guiNote.sourceProtocol == appConfig.getProtocolDODAdapter()) {
            makeCIS_CDAText(guiNote.noteText)
            return true
        }

        sendResponse(guiNote)
    }

    def getDischargeNoteDataStream = {

        //retrieve cached note
        GUIProgressNote guiNote = session.appUser.inpatientNotesMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        def auditQueryName = "GetDischargeNoteDataStream"

        if ((guiNote == null) || !guiNote.noteAsBinaryStream) {
            response.sendError 500
            return false;
        }

        //stream share/essentris CDA notes
        if (guiNote.getSite().siteCode.startsWith("2.16.840.1.113883.3.42.10002.100001.8")) {
            auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                    request.recordQuery[HttpKeys.RECORD_DATE],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            makeCIS_CDAText(guiNote.getNoteText())
            return true;
        }
        //fetch dod note image
        else {

            //if note attachment is null, send an error message

            String errorMsg = "An error occurred while attempting to retrieve discharge summary note. - JLV";

            response.setContentType("text");
            response.setContentLength(errorMsg.length());
            response.getWriter() << errorMsg;
            response.getWriter().flush();

            response.sendError 500
            return false;
        }
    }
}

