package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.dao.DaoException
import gov.va.med.guibeans.GUIPatientEncounter
import gov.va.med.guibeans.GUIProgressNote
import gov.va.med.jmeadows.webservice.NoteImage
import gov.va.med.vhahon.common.Utils
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.lang.StringUtils

class PatientEncountersController extends BaseController {

    def appointmentDao
    def auditDao
    def noteDao
    def sensitiveRecordDao

    static allowedMethods = [list: 'POST', auditDetails: 'POST', sensitiveEncounter: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        dateRangeCheck();
    }

    def list = {

        def guiEncounters = appointmentDao.getPatientEncounters(session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        String ipAddress = request.getHeader("X-FORWARDED-FOR");
        if (ipAddress == null) {
            ipAddress = request.getRemoteAddr();
        }

        if (guiEncounters != null) {
            for (def guiEncounter : guiEncounters) {
                if (StringUtils.isNotEmpty(guiEncounter.getEncounterId())) {
                    String noteId = "";
                    if (guiEncounter.getDocuments() != null
                            && guiEncounter.getDocuments().get(0).getId() != null) {
                        noteId = guiEncounter.getDocuments().get(0).getId();
                        guiEncounter.setVaNote(true);
                    }

                    def queryMap = [(HttpKeys.ENCOUNTER_ID)       : guiEncounter.getEncounterId(),
                                    (HttpKeys.NOTE_ID)            : noteId,
                                    (HttpKeys.RECORD_DATE)        : guiEncounter.getApptDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiEncounter.getSite().getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiEncounter.getSite().getMoniker()]

                    guiEncounter.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiEncounter.getSite(),
                                    guiEncounter.getSourceProtocol(),
                                    guiEncounter.isSessionCachedRecord()))

                    guiEncounter.setGuiIpAddress(ipAddress)

                    if (guiEncounter.isSessionCachedRecord()) {
                        //cache encounter report in session
                        def encounterDetail = BeanUtils.cloneBean(guiEncounter)

                        session.appUser.encounterMap.put(
                                getCachedRecordKey(guiEncounter.getRecordDetailsToken()), encounterDetail)

                        //don't send report text with note listings
                        //guiEncounter.setReport("")
                    }

                    // Don't pass out any sensitive data as part of the progress notes list
                    if (guiEncounter.isSensitive()) {
                        guiEncounter.setClinic("** Sensitive **");
                        guiEncounter.setReason("** Sensitive **");
                        guiEncounter.setDiagnosis(null);
                        guiEncounter.setProcedures(null);
                        guiEncounter.setEncounterNumber("");
                        guiEncounter.setEncounterId("");
                        if (guiEncounter.getComplexDataUrl() != null && !guiEncounter.getComplexDataUrl().isEmpty()) {
                            guiEncounter.setComplexDataUrl("** Sensitive **");
                        } else {
                            guiEncounter.setComplexDataUrl("");
                        }
                        guiEncounter.setReasonNarrative("");
                        guiEncounter.setDischargeDisposition("");
                        guiEncounter.setDocuments(null);
                        guiEncounter.setReasonCode("");
                        guiEncounter.setCodes(null);
                    }
                }
            }
        }

        if (session.appUser.vlerSiteStatus != null && session.appUser.vlerSiteStatus.length() > 0) {
            GUIPatientEncounter statusRecord = new GUIPatientEncounter();
            statusRecord.setClinic(session.appUser.vlerSiteStatus);
            statusRecord.setSite(getStatusReportSite());
            guiEncounters.add(statusRecord);
        }

        sendResponse(guiEncounters)
    }

    def auditDetails = {
        def recordSiteAgency = params[HttpKeys.RECORD_SITE_AGENCY]
        def recordSiteMoniker = params[HttpKeys.RECORD_SITE_MONIKER]
        def recordDate = params[HttpKeys.RECORD_DATE]

        auditDao.auditDetailsQuery("encounterDetail", session.appUser, request.patient, recordDate, recordSiteMoniker, recordSiteAgency)

        sendResponse([success: "true"])
    }

    def getNoteDataStream = {

        //retrieve cached note
        GUIPatientEncounter guiEncounter = session.appUser.encounterMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        def auditQueryName = "GetEncounterDataStream"

        if ((guiEncounter == null) || !guiEncounter.noteAsBinaryStream) {
            response.sendError 500
            return false;
        }

        if (guiEncounter.isSensitive()) {
            String auditText = params[HttpKeys.AUDIT_TEXT]
            // audit with break the glass
            auditText = "btgPatientEncounters^" + auditText;

            sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText, session.scId, session.scEmail)
        }

        //stream share/essentris CDA notes
        if (guiEncounter.getSite().siteCode.startsWith("2.16.840.1.113883.3.42.10002.100001.8")) {
            auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                    request.recordQuery[HttpKeys.RECORD_DATE],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            makeCIS_CDAText(guiEncounter.getReport())
            return true;
        }
        //fetch dod note image
        else {
            String complexDataUrl = guiEncounter.getComplexDataUrl();

            NoteImage noteImage = null;

            if (complexDataUrl != null) {

                auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                        "complexDataUrl:" + complexDataUrl,
                        request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                        request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

                try {
                    noteImage = noteDao.getBHIENoteImage(session.appUser, complexDataUrl);
                }
                catch (DaoException e) {
                    log.error();
                }

                response.setHeader("Pragma", "public");
                response.setHeader("Cache-Control", "maxage=1");
                //response.setHeader("Expires", "0");

                if (noteImage != null && noteImage.getNoteBytes() != null) {
                    response.setContentType(noteImage.getContentType());
                    response.setHeader("Content-Disposition", noteImage.getContentDisposition());

//                    if (noteImage.getContentType() != null && noteImage.getContentType().endsWith("/rtf") &&
//                            noteImage.getNoteBytes() != null) {
//                        String rtfStr = new String(noteImage.getNoteBytes());
//                        StringBuilder sb = new StringBuilder();
//
//                        boolean imua = true;
//                        //remove all fonttbl
//                        while (imua) {
//                            int startIndex = rtfStr.indexOf("{\\fonttbl");
//                            if (startIndex < 0) {
//                                sb.append(rtfStr);
//                                //out.print(rtfStr);
//                                imua = false;
//                            } else {
//                                int endIndex = rtfStr.indexOf("}}", startIndex);
//                                sb.append(rtfStr.substring(0, startIndex));
//                                //out.print(rtfStr.substring(0, startIndex));
//                                rtfStr = rtfStr.substring(endIndex + 2, rtfStr.length());
//                            }
//                        }
//                        //response.setContentLength(sb.toString().getBytes(Charset.forName("UTF-8")).length);
//                        //out.print(sb.toString());
//                        response.getOutputStream().write(sb.toString().getBytes(Charset.forName("UTF-8")));
//                    } else {
//                        //response.setContentLength(noteImage.getNoteBytes().length);
                    response.getOutputStream().write(noteImage.getNoteBytes());
//                    }
                    response.getOutputStream().flush();
                    response.getOutputStream().close();
                    return true;
                }
            }

            //if note attachment is null, send an error message

            String errorMsg = "An error occurred while attempting to retrieve note.";

            response.setContentType("text");
            response.setContentLength(errorMsg.length());
            response.getWriter() << errorMsg;
            response.getWriter().flush();

            response.sendError 500
            return false;
        }
    }

    def details = {

        GUIProgressNote guiNote = null

        if (!request.isSessionCachedRecord) {
            guiNote = noteDao.getProgressNote(
                    session.appUser,
                    request.patient,
                    request.recordQuery[HttpKeys.NOTE_ID],
                    request.recordQuery[HttpKeys.RECORD_SITE_CODE]);

            sendResponse(guiNote)
            return true;
        }

        //retrieve cached note
        guiNote = session.appUser.outpatientNotesMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        if (guiNote.sensitive) {
            String auditText = params[HttpKeys.AUDIT_TEXT]
            // audit with break the glass
            auditText = "btgPatientEncounters^" + auditText;

            sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText, session.scId, session.scEmail)
        }

        auditDao.auditDetailsQuery("ProgressNoteDetail", session.appUser, request.patient,
                request.recordQuery[HttpKeys.RECORD_DATE],
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

        sendResponse(guiNote)
    }

    def sensitiveEncounter = {
        GUIPatientEncounter guiEncounter = null

        //retrieve cached encounter
        guiEncounter = session.appUser.encounterMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))


        String auditText = params[HttpKeys.AUDIT_TEXT]
        // audit with break the glass
        auditText = "btgPatientEncounters^" + auditText;

        sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText, session.scId, session.scEmail)

        sendResponse(guiEncounter)
    }

}
