package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.*
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.lang.StringUtils

class PatientFormsController extends PatientNoteDetailBaseController {

    def questionnaireDao
    def auditDao

    static allowedMethods = [list: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }
    }

    def list = {

        def guiForms = questionnaireDao.getPatientForms(session.appUser, request.patient)

        if (guiForms != null) {
            for (def guiForm : guiForms) {
                if (StringUtils.isNotEmpty(guiForm.formId)) {
                    def queryMap = [(HttpKeys.FORM_ID)            : guiForm.getFormId(),
                                    (HttpKeys.RECORD_DATE)        : guiForm.getFormDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiForm.site.getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiForm.site.getMoniker()]

                    guiForm.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiForm.getSite(),
                                    guiForm.getSourceProtocol(),
                                    guiForm.isSessionCachedRecord()))

                    if (guiForm.isSessionCachedRecord()) {
                        //cache consult report in session
                        def formDetail = BeanUtils.cloneBean(guiForm)

                        session.appUser.formMap.put(
                                getCachedRecordKey(guiForm.getRecordDetailsToken()), formDetail)

                        //don't send report text with note listings
                        guiForm.setDeploymentForm(null);
                        guiForm.setQuestionnaireBean(null);
                    }
                }
            }
        }

        sendResponse(guiForms)

    }

    def details = {

        GUIPatientForm guiForm = null
        GUIQuestionnaire result = null

        if (request.isSessionCachedRecord) {

            //audit details query

            auditDao.auditDetailsQuery("PatQuestionnaireDetails", session.appUser, request.patient,
                    request.recordQuery[HttpKeys.RECORD_DATE],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            guiForm = session.appUser.formMap.get(
                    getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))
        }

        if (guiForm != null && guiForm.getQuestionnaireBean() == null) {
            result = new GUIQuestionnaire();
            result.setName("This entry is empty.");
        } else {
            result = guiForm.getQuestionnaireBean();
        }


        if (result != null) {
            StringBuilder report = new StringBuilder();


            report.append("<div id='questionnaireTableWrapper'><table class='questionnaireTable'><tr>");
            report.append("<td>Document Title:</td><td>" + result.name + "</td></tr>");
            report.append("<td>Entered By:</td><td>" + result.enteredBy.name + "</td></tr>");
            report.append("<td>Response Date:</td><td>" + guiForm.formDate + "</td></tr>");
            report.append("<td>Version:</td><td>" + result.version + "</td></tr></table></div>");

            report.append("Comments: " + result.comments + "<br><br>");

            String selected = "";
            List<GUIQuestionnaireDetails> detailsList = result.getResponses();
            if (detailsList != null) {
                report.append("<table width='100%' border='1' class='questionnaire-table questionnaire-details'>")
                for (GUIQuestionnaireDetails detail : detailsList) {
                    List<GUIQuestionnaireDetailChoices> choicesList = detail.getChoices();
                    List<GUIQuestionnaireDetailAnswers> answersList = detail.getAnswers();
                    List<GUIQuestionnaireDetailComments> commentsList = detail.getComments();
                    int choicesCount = 0;
                    int answersCount = 0;
                    int commentsCount = 0;

                    if (choicesList != null) {
                        choicesCount = choicesList.size();
                    }

                    if (answersList != null) {
                        answersCount = answersList.size();
                    }

                    if (commentsList != null) {
                        commentsCount = commentsList.size();
                    }

                    // Sequence column
                    report.append("<tr>")
                    report.append("<td valign='top' align='middle'>" + detail.getSequence() + ".</td>")

                    // Yes/No, True/Falst, Multiple choice types
                    report.append("<td>")
                    report.append("<table width='100%' class='questionnaire-table questionnaire-choices'>")
                    if (choicesCount > 0) {
                        // Question
                        report.append("<tr>")
                        report.append("<td colspan='" + choicesCount + "'>" + detail.getQuestion() + "</td>");
                        report.append("</tr>")
                        // Choices & answers
                        if (choicesCount > 0) {
                            report.append("<tr>")
                            for (GUIQuestionnaireDetailChoices choice : choicesList) {
                                selected = "";
                                if (choicesCount > 1) {
                                    if (choice.isSelected()) {
                                        selected = "&#x2612;";
                                    } else {
                                        selected = '&#x2610;';
                                    }
                                }
                                selected = selected + " " + choice.getChoiceText();
                                report.append("<td width='" + Math.floor(100 / choicesCount).toInteger() + "%'>" + selected + "</td>");
                            }
                            report.append("</tr>")
                        }
                        if (commentsCount > 0) {
                            for (GUIQuestionnaireDetailComments comment : commentsList) {
                                // Comments by
                                report.append("<tr class='questionnaire-highlighted'>")
                                report.append("<td colspan='" + choicesCount + "'>Comment entered by " + comment.getCommentBy() + " @ " + comment.getDateEntered() + "</td>")
                                report.append("</tr>")
                                // Comment
                                report.append("<tr class='questionnaire-highlighted'>")
                                report.append("<td colspan='" + choicesCount + "'>" + comment.getNoteText() + "</td>")
                                report.append("</tr>")
                            }
                        }
                    }
                    report.append("</table>")
                    report.append("</td>")
                    report.append("</tr>")
                }
                report.append("</table>")
            }

            result.setReportText(report.toString());
        }

        sendResponse(result)
    }

    def getNoteDataStream = {

        //retrieve cached note
        GUIPatientForm guiForm = session.appUser.formMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        def auditQueryName = "PatDeploymentFormDetails"

        if ((guiForm == null) || !guiForm.noteAsBinaryStream) {
            response.sendError 500
            return false;
        }

        //stream CDA data
        if (guiForm.getDeploymentForm() != null && guiForm.getDeploymentForm().getNoteText() != null) {
            auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                    request.recordQuery[HttpKeys.RECORD_DATE],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            makeCIS_CDATextAndHeader(guiForm.getDeploymentForm().getNoteText());
            return true;
        }

    }
}
