/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUILabOrder
import gov.va.med.guibeans.GUILabResult
import gov.va.med.vhahon.common.Utils
import org.apache.commons.beanutils.BeanUtils

class PatientLabsController extends BaseController {
    def labDao
    def auditDao
    def sensitiveRecordDao

    static allowedMethods = [list: 'POST', auditDetails: 'POST', report: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'histogram' || actionName == 'report') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        } else dateRangeCheck();
    }

    def list = {


        def guiLabs = labDao.getPatientLabs(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        //generate record details token
        if (guiLabs != null) {
            for (def guiLab : guiLabs) {
                //non-micro and non-AP VA labs require secondary report text call
                if (!guiLab.subscript.equalsIgnoreCase("MI") &&
                        !guiLab.subscript.equalsIgnoreCase("AP") &&
                        guiLab.site.agency.equalsIgnoreCase("VA") &&
                        (guiLab.getLabResults() != null) && (guiLab.getLabResults().size() > 0) &&
                        !guiLab.isSensitive()) {
                    def queryMap = null;

                    queryMap = [
                            (HttpKeys.LAB_RESULT_COLLECTED_DATE): guiLab.getCollectionDate(),
                            (HttpKeys.LAB_ACCESSION_NUMBER)     : guiLab.labResults[0].getAccession()
                    ]
                    guiLab.labResults[0].setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiLab.getSite(), guiLab.getSourceProtocol()))

                }

                if (guiLab.isSensitive()) {
                    //cache the sensitive lab order
                    def queryMap = [(HttpKeys.LAB_ORDER_ID)       : guiLab.getOrderId(),
                                    (HttpKeys.LAB_RESULT_COLLECTED_DATE) : guiLab.getCollectionDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiLab.getSite().getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiLab.getSite().getMoniker()]
                    guiLab.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiLab.getSite(),
                                    guiLab.getSourceProtocol(),
                                    true
                            )
                    )

                    def sensitiveLabOrder = BeanUtils.cloneBean(guiLab)

                    session.appUser.sensitiveLabOrdersMap.put(
                            getCachedRecordKey(guiLab.getRecordDetailsToken()),
                            sensitiveLabOrder
                    )

                    guiLab.setTestName("** Sensitive **");
                    guiLab.setComment("");
                    guiLab.setSample("");
                    guiLab.setStatus("");
                    guiLab.setFreeTextReport(false);
                    guiLab.setCollectionSample("");
                    guiLab.setLabOrderStatus("");
                    guiLab.setLabResults(null);
                    guiLab.setSubscript("");
                    guiLab.setUrgency("");
                }
            }
        }

        if (session.appUser.vlerSiteStatus != null && session.appUser.vlerSiteStatus.length() > 0) {
            GUILabResult statusRecord = new GUILabResult();
            statusRecord.setTestName(session.appUser.vlerSiteStatus);
            statusRecord.setSite(getStatusReportSite());
            guiLabs.add(statusRecord);
        }

        sendResponse(guiLabs)
    }

    def report = {

        def guiLabReport = labDao.getPatientLabReport(
                session.appUser,
                request.patient,
                request.recordQuery[HttpKeys.LAB_RESULT_COLLECTED_DATE],
                request.recordQuery[HttpKeys.LAB_ACCESSION_NUMBER],
                request.recordQuery[HttpKeys.RECORD_SITE_CODE])

        sendResponse(guiLabReport)
    }

    def auditDetails = {
        def recordSiteAgency = params[HttpKeys.RECORD_SITE_AGENCY]
        def recordSiteMoniker = params[HttpKeys.RECORD_SITE_MONIKER]
        def recordItemId = params[HttpKeys.RECORD_ITEM_ID]

        auditDao.auditDetailsQuery("labResults", session.appUser, request.patient, recordItemId, recordSiteMoniker, recordSiteAgency)

        sendResponse([success: "true"])
    }

    def results = {

        String ipAddress = request.getHeader("X-FORWARDED-FOR");
        if (ipAddress == null) {
            ipAddress = request.getRemoteAddr();
        }

        def guiResults = labDao.getPatientLabResults(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        //generate record details token
        if (guiResults != null) {
            for (def guiResult : guiResults) {
                if (guiResult.site.agency.equalsIgnoreCase("VA")
                        && !guiResult.subscript.equalsIgnoreCase("CH")
                        && !guiResult.subscript.equalsIgnoreCase("MI")
                        && !guiResult.subscript.equalsIgnoreCase("AP")) {
                    //non-chem and non-micro va labs require secondary report text call
                    def queryMap = [
                            (HttpKeys.LAB_RESULT_COLLECTED_DATE): guiResult.getResultDate(),
                            (HttpKeys.LAB_ACCESSION_NUMBER)     : guiResult.getAccession()
                    ]
                    guiResult.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiResult.getSite(), guiResult.getSourceProtocol()))
                }

                guiResult.setGuiIpAddress(ipAddress);

                // if the result is sensitive cache it and then mask it
                if (guiResult.isSensitive()) {
                    def queryMap = [(HttpKeys.LAB_TEST_ID)       : guiResult.getTestId(),
                                    (HttpKeys.LAB_RESULT_COLLECTED_DATE) : guiResult.getResultDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiResult.getSite().getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiResult.getSite().getMoniker()]

                    guiResult.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiResult.getSite(),
                                    guiResult.getSourceProtocol(), true))

                    def sensitiveLabResult = BeanUtils.cloneBean(guiResult)

                    session.appUser.sensitiveLabResultsMap.put(
                            getCachedRecordKey(guiResult.getRecordDetailsToken()), sensitiveLabResult)

                    guiResult.setTestName("** Sensitive **")
                    guiResult.setResult("")
                    guiResult.setCodes(null);
                    guiResult.setReferenceRange("");
                    guiResult.setUnits("");
                    guiResult.setComment("");
                    guiResult.setInterpretation("");
                    guiResult.setSpecimen("");
                    guiResult.setAccessionType("");
                    guiResult.setPrintName("");
                    guiResult.setSubscript("");
                }
            }
        }

        if (session.appUser.vlerSiteStatus != null && session.appUser.vlerSiteStatus.length() > 0) {
            GUILabResult statusRecord = new GUILabResult();
            statusRecord.setTestName(session.appUser.vlerSiteStatus);
            statusRecord.setSite(getStatusReportSite());
            guiResults.add(statusRecord);
        }

        sendResponse(guiResults)
    }

    def sensitiveResult = {
        GUILabResult guiResult = null

        // retrieve cached result
        guiResult = session.appUser.sensitiveLabResultsMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        String auditText = params[HttpKeys.AUDIT_TEXT]
        // audit with break the glass
        auditText = "btgLabResult^" + auditText;

        sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText, session.scId, session.scEmail)

        sendResponse(guiResult)
    }

    def sensitiveOrder = {
        GUILabOrder guiOrder = null

        // retrieve cached result
        guiOrder = session.appUser.sensitiveLabOrdersMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        String auditText = params[HttpKeys.AUDIT_TEXT]
        // audit with break the glass
        auditText = "btgLabOrder^" + auditText;

        sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText, session.scId, session.scEmail)

        sendResponse(guiOrder)
    }

}
