/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.dao.MedicationDao
import gov.va.med.guibeans.GUIMedication
import gov.va.med.vhahon.common.Utils
import org.apache.commons.lang.StringUtils

class PatientOutpatientMedicationsController extends BaseController {
    def auditDao
    def medicationDao

    static allowedMethods = [list: 'POST', auditRxFillsQuery: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        dateRangeCheck();
    }

    def list = {

        def guiMeds = medicationDao.getPatientMedications(
                session.appUser,
                request.patient,
                MedicationDao.MEDICATION_STATUS_OUTPATIENT,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        //generate record details token
        if (guiMeds != null) {
            for (def guiMed : guiMeds) {
                if (appConfig.getProtocolRPC() == guiMed.getSourceProtocol() &&
                        StringUtils.isNotEmpty(guiMed.getMedicationId())) {
                    def queryMap = [(HttpKeys.MEDICATION_ID): guiMed.getMedicationId()]
                    guiMed.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiMed.getSite(),
                                    guiMed.getSourceProtocol()))
                }
            }
        }

        if (session.appUser.vlerSiteStatus != null && session.appUser.vlerSiteStatus.length() > 0) {
            GUIMedication statusRecord = new GUIMedication();
            statusRecord.setDrugName(session.appUser.vlerSiteStatus);
            statusRecord.setSite(getStatusReportSite());
            guiMeds.add(statusRecord);
        }

        sendResponse(guiMeds)
    }

    def auditRxFillsQuery = {
        def recordSiteAgency = params[HttpKeys.RECORD_SITE_AGENCY]
        def recordSiteMoniker = params[HttpKeys.RECORD_SITE_MONIKER]
        def recordDate = params[HttpKeys.RECORD_DATE]

        auditDao.auditDetailsQuery("prescriptionFills", session.appUser, request.patient, recordDate, recordSiteMoniker, recordSiteAgency)

        sendResponse([success: "true"])
    }

    def details = {
        def guiMedDetail = medicationDao.getMedicationDetails(
                session.appUser,
                request.patient,
                request.recordQuery[HttpKeys.MEDICATION_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_CODE]);

        sendResponse(guiMedDetail)
    }
}
