/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUIProblem
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.NarrativeV4
import org.apache.commons.lang.StringUtils

class PatientProblemListController extends BaseController {

    def problemDao
    def auditDao

    static allowedMethods = [list: 'POST', details: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        }
    }

    def list = {

        def includeVLER = false;

        if(params["includeVLER"] == "true"){
            includeVLER = true;
        }

        HashSet narrativeIds = new HashSet();

        def guiProblems = problemDao.getPatientProblemList(session.appUser,
                request.patient, params.status, includeVLER )

        if (guiProblems != null) {
            for (def guiProb : guiProblems) {

                if (guiProb.getNarrativeId() != null && !"".equalsIgnoreCase(guiProb.getNarrativeId())) {
                    narrativeIds.add(guiProb.getNarrativeId());
                    if(guiProb.getSite() != null && guiProb.getSite().getName() != null
                            && "VLER".equalsIgnoreCase(guiProb.getSite().getName())
                            && guiProb.getSite().getSiteCode() != null){
                        String siteName = session.appUser.partnersMap.get(guiProb.getSite().getSiteCode());
                        if(siteName != null){
                            guiProb.getSite().setName(siteName);
                        }
                        guiProb.getSite().setAgency("VLER");
                    }
                }

                //no record details for chcs problems
                if (StringUtils.isNotEmpty(guiProb.getProblemId())) {
                    def queryMap = [(HttpKeys.PROBLEM_ID)         : guiProb.getProblemId(),
                                    (HttpKeys.RECORD_DATE)        : guiProb.getLastUpdated(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiProb.site.moniker,
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiProb.site.agency]

                    guiProb.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiProb.getSite(),
                                    guiProb.getSourceProtocol(),
                                    guiProb.isSessionCachedRecord()))

                    if (guiProb.isSessionCachedRecord()) {
                        //cache problem in session
                        session.appUser.problemListMap.put(
                                getCachedRecordKey(guiProb.getRecordDetailsToken()), guiProb)
                    }
                }
            }
        }

        if (includeVLER && session.appUser.narrativesMap != null
                && !session.appUser.narrativesMap.isEmpty()) {
            Iterator narrativeIter = session.appUser.narrativesMap.entrySet().iterator();

            //loop through all narratives, check the allergy ones to see if they are referenced in this call
            //if they are not referenced, then create a record for them so the user is able to view this narrative
            while (narrativeIter.hasNext()) {
                Map.Entry narrative = (Map.Entry) narrativeIter.next();
                if (narrative.getKey().toString().contains("11450-4")) {
                    //check to see if record using this narrative exists
                    if (!narrativeIds.contains(narrative.getKey().toString())) {
                        NarrativeV4 narrativeItem = session.appUser.narrativesMap.get(narrative.getKey().toString());
                        String siteName = "VLER";
                        String siteMoniker = "VLER Partner";
                        if (narrativeItem != null)
                            if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                    && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                                siteName = session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier());
                            }
                        if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                            siteMoniker = session.appUser.partnersMap.get(narrativeItem.getNarrativeAssigningAuthorityIdentifier());
                        }
                        GUISite narrativeSite = new GUISite();
                        narrativeSite.setAgency("VLER");
                        narrativeSite.setName(siteName);
                        narrativeSite.setMoniker(siteMoniker);
                        narrativeSite.setSiteCode("VLER");

                        GUIProblem narrativeRecord = new GUIProblem();
                        narrativeRecord.setNarrativeId(narrative.getKey().toString());
                        narrativeRecord.setSite(narrativeSite);
                        narrativeRecord.setDescription("Narrative");
                        guiProblems.add(narrativeRecord);

                        narrativeIds.add(narrative.getKey().toString());
                    }

                }
            }

        }
        sendResponse(guiProblems)
    }

    def details = {

        def guiProblemDetail = null

        if (request.isSessionCachedRecord) {
            //audit details query

            auditDao.auditDetailsQuery("ProblemDetail", session.appUser, request.patient,
                    request.recordQuery[HttpKeys.RECORD_DATE],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            guiProblemDetail = session.appUser.problemListMap.get(
                    getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))
        } else {
            guiProblemDetail = problemDao.getProblemDetails(
                    session.appUser,
                    request.patient,
                    request.recordQuery[HttpKeys.PROBLEM_ID],
                    request.recordQuery[HttpKeys.RECORD_SITE_CODE])
        }


        sendResponse(guiProblemDetail)
    }
}
