/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUIKeyValuePair
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.Patient

class PatientSelectController extends BaseController {

    def patientDao

    def tokenUtils

    static allowedMethods = [list: 'POST']

    def beforeInterceptor = {

        if (params[HttpKeys.PATIENT_TOKEN] == null) {
            response.sendError 500
            return false;
        }
        session.appUser.narrativesMap.clear();
        session.appUser.partnersMap.clear();
    }

    def auditVARestrictedAccess = {
        def selectedPatientMap = session.appUser.selectedPatientMap;
        def patientToken = params[HttpKeys.PATIENT_TOKEN];
        def auditText = params["auditTxt"];
        def selectedPatient = null;

        if (selectedPatientMap != null) {
            selectedPatient = selectedPatientMap[patientToken];
        }

        if (selectedPatient != null) {
            patientDao.auditVARestrictedAccess(session.appUser, selectedPatient, auditText);
        }

        sendResponse(["success": true])
    }

    def selectPatientForVASensitive = {
        def selectedPatientMap = session.appUser.selectedPatientMap;
        def patientToken = params[HttpKeys.PATIENT_TOKEN];
        def selectedPatient = null;

        if (selectedPatientMap != null) {
            selectedPatient = selectedPatientMap[patientToken];
        }

        if (selectedPatient != null) {
            patientDao.selectPatientForVASensitive(session.appUser, selectedPatient);
        }

        sendResponse(["success": true])
    }

    def selectAssignedPatient = {
        def patientToken = params[HttpKeys.PATIENT_TOKEN]
        def assignedPatientId = params['assignedPatId']

        def provider = session.appUser
        def selectedPatientMap = session.appUser.selectedPatientMap

        def patientParams = tokenUtils.decryptTokenToMap(session.appUser.securityKey, patientToken)

        def paramPatId = patientParams[HttpKeys.PATIENT_ID]
        def paramIdType = patientParams[HttpKeys.ID_TYPE]
        def paramPatSiteCode = patientParams[HttpKeys.PATIENT_SITE_CODE]

        provider.setProviderFlag(assignedPatientId)
        def responseGUIPatientSelect = patientDao.selectPatient(provider, null, paramPatId, paramIdType, paramPatSiteCode)
        def selectedPatient = responseGUIPatientSelect.getGUIPatient();

// TODO: TEMP - REMOVING SITES THAT HAVE NO ENDPOINTS
        if (selectedPatient != null && selectedPatient.getVistaSites() != null) {
            GUISite tmpSite
            GUIKeyValuePair tmpKvp
            for (Iterator<GUISite> siteIt = selectedPatient.getVistaSites().iterator(); siteIt.hasNext();) {
                tmpSite = siteIt.next()
                if (tmpSite.getEndpoints() == null || tmpSite.getEndpoints().isEmpty()) {
                    if (selectedPatient.getPatientIens() != null) {
                        for (Iterator<GUIKeyValuePair> kvpIt = selectedPatient.getPatientIens().iterator(); kvpIt.hasNext();) {
                            tmpKvp = kvpIt.next()
                            if (tmpKvp.getKey() == tmpSite.getSiteCode()) {
                                log.debug("!!! Removed Site/IEN pair: " + tmpKvp.getKey() + "/" + tmpKvp.getValue())
                                kvpIt.remove()
                            }
                        }
                    }
                    log.debug("!!! Removed Site: " + tmpSite.getSiteCode())
                    siteIt.remove();
                }
            }
        }

        if (selectedPatient == null) {
            log.error("selected patient is null")
            response.sendError 500
            return false
        }

        selectedPatientMap.put(patientToken, selectedPatient)
        log.debug("Patient select success, mapping patient to patientToken: ${patientToken}")

        String ipAddress = request.getHeader("X-FORWARDED-FOR");
        if (ipAddress == null) {
            ipAddress = request.getRemoteAddr();
        }

        // Retrieve context management item values to be sent to the vault
        String cmName = "";
        String cmEdipi = "";
        String cmIcn = "";
        String cmVaIen = "";
        String cmVAIenSiteId = "";
        if (selectedPatient != null) {
            cmName = selectedPatient.getName();
            cmEdipi = selectedPatient.getEDIPI();
            cmIcn = selectedPatient.getICN();

            if (selectedPatient.getPatientIens() != null && provider != null &&
                    provider.getHostSite() != null) {
                def providerSiteId = provider.getHostSite().getSiteCode();

                for (GUIKeyValuePair kvp in selectedPatient.getPatientIens()) {
                    if (kvp.key == providerSiteId) {
                        cmVaIen = kvp.value;
                        cmVAIenSiteId = kvp.key;
                    }
                }
            }
        }

        sendResponse([
                "patSelected"       : true,
                "resultScript"      : '',
                "isSuccess"         : responseGUIPatientSelect.isSuccess(),
                "isVARestricted"    : responseGUIPatientSelect.isVARestricted(),
                "isAllowVAAccess"   : responseGUIPatientSelect.isAllowVAAccess(),
                "errorMsg"          : responseGUIPatientSelect.getErrorMsg(),
                "isSensitive"       : selectedPatient.isSensitive(),
                "providerName"      : provider.getName(),
                "providerSite"      : provider.getHostSite().getSiteCode(),
                "ipAddress"         : ipAddress,
                "patientName"       : cmName,
                "patientEdipi"      : cmEdipi,
                "patientIcn"        : cmIcn,
                "patientVaIen"      : cmVaIen,
                "patientVaIenSiteId": cmVAIenSiteId
        ])
    }

    def select = {

        def patientToken = params[HttpKeys.PATIENT_TOKEN]
        def cmNotify = params[HttpKeys.CONTEXT_NOTIFY];
        def patName = params[HttpKeys.PATIENT_NAME];
        def patEdipi = params["patEdipi"];
        def patSsn = params["patSsn"];
        def patSponsorSsn = params["patSponsorSsn"];
        def patGender = params["patGender"];
        def patDob = params["patDob"];
        def patSensitive = params["patSensitive"];

        def provider = session.appUser
        def selectedPatientMap = session.appUser.selectedPatientMap

        if (log.debugEnabled) {
            log.debug("Decrypted patient token: ${tokenUtils.decryptTokenToString(session.appUser.securityKey, patientToken)}");
        }

        def patientParams = tokenUtils.decryptTokenToMap(session.appUser.securityKey, patientToken)

        log.debug("Patient params from token: ${patientParams}")

        def paramPatId = patientParams[HttpKeys.PATIENT_ID]
        def paramIdType = patientParams[HttpKeys.ID_TYPE]
        def paramPatSiteCode = patientParams[HttpKeys.PATIENT_SITE_CODE]

        // Selected patient's info to be stored in the user's recently viewed patients table
        def patient = new Patient();
        patient.setName(patName);
        patient.setEDIPI(patEdipi);
        patient.setSSN(patSsn);
        patient.setFMP(patSponsorSsn);
        patient.setGender(patGender);
        patient.setDob(patDob);
        patient.setSensitive(Boolean.parseBoolean(patSensitive));

        def responseGUIPatientSelect = patientDao.selectPatient(provider, patient, paramPatId, paramIdType, paramPatSiteCode)

        session.externalIDs = responseGUIPatientSelect.getExternalIDs();

        def selectedPatient = responseGUIPatientSelect.getGUIPatient();

        String resultScript = ""

        if (selectedPatient == null) {
            log.error("selected patient is null")
            response.sendError 500
            return false
        }

        selectedPatientMap.put(patientToken, selectedPatient)
        log.debug("Patient select success, mapping patient to patientToken: ${patientToken}")

        String ipAddress = request.getHeader("X-FORWARDED-FOR");
        if (ipAddress == null) {
            ipAddress = request.getRemoteAddr();
        }

        // Retrieve context management item values to be sent to the vault
        String cmName = "";
        String cmEdipi = "";
        String cmIcn = "";
        String cmVaIen = "";
        String cmVAIenSiteId = "";
        if (selectedPatient != null) {
            cmName = selectedPatient.getName();
            cmEdipi = selectedPatient.getEDIPI();
            cmIcn = selectedPatient.getICN();

            if (selectedPatient.getPatientIens() != null && provider != null &&
                    provider.getHostSite() != null) {
                def providerSiteId = provider.getHostSite().getSiteCode();

                for (GUIKeyValuePair kvp in selectedPatient.getPatientIens()) {
                    if (kvp.key == providerSiteId) {
                        cmVaIen = kvp.value;
                        cmVAIenSiteId = kvp.key;
                    }
                }
            }
        }

        sendResponse([
                "patSelected"       : true,
                "resultScript"      : resultScript,
                "isSuccess"         : responseGUIPatientSelect.isSuccess(),
                "isVARestricted"    : responseGUIPatientSelect.isVARestricted(),
                "isAllowVAAccess"   : responseGUIPatientSelect.isAllowVAAccess(),
                "errorMsg"          : responseGUIPatientSelect.getErrorMsg(),
                "isSensitive"       : selectedPatient.isSensitive(),
                "providerName"      : provider.getName(),
                "providerSite"      : provider.getHostSite().getSiteCode(),
                "ipAddress"         : ipAddress,
                "patientName"       : cmName,
                "patientEdipi"      : cmEdipi,
                "patientIcn"        : cmIcn,
                "patientVaIen"      : cmVaIen,
                "patientVaIenSiteId": cmVAIenSiteId
        ])
    }
}
