/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUISite
import gov.va.med.guibeans.GUIVitals
import gov.va.med.jmeadows.webservice.NarrativeV4
import gov.va.med.vhahon.common.Utils

class PatientVitalsController extends BaseController {
    def vitalsDao

    static allowedMethods = [list: 'POST', histogram: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'histogram') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        } else dateRangeCheck();
    }

    def list = {

        def includeVLER = false;

        if(params["includeVLER"] == "true"){
            includeVLER = true;
        }

        HashSet narrativeIds = new HashSet();

        def guiVitals = vitalsDao.getPatientVitals(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]), includeVLER)

        for (GUIVitals guiVital : guiVitals) {

            if (guiVital.getNarrativeId() != null && !"".equalsIgnoreCase(guiVital.getNarrativeId())) {
                narrativeIds.add(guiVital.getNarrativeId());
                if(guiVital.getSite() != null && guiVital.getSite().getName() != null
                        && "VLER".equalsIgnoreCase(guiVital.getSite().getName())
                        && guiVital.getSite().getSiteCode() != null){
                    String siteName = session.appUser.partnersMap.get(guiVital.getSite().getSiteCode());
                    if(siteName != null){
                        guiVital.getSite().setName(siteName);
                    }
                    guiVital.getSite().setAgency("VLER");

                }
            }

            def queryMap = [(HttpKeys.VITAL_TYPE)      : guiVital.getType(),
                            (HttpKeys.VITAL_UNITS_CODE): guiVital.getUnitsCode(),
                            (HttpKeys.START_DATE)      : request[HttpKeys.START_DATE],
                            (HttpKeys.END_DATE)        : request[HttpKeys.END_DATE]]
            guiVital.setHistogramRecordToken(
                    generateRecordDetailsToken(
                            params[HttpKeys.PATIENT_TOKEN],
                            controllerName,
                            queryMap,
                            guiVital.getSite(), guiVital.getSourceProtocol()))
        }

        if (includeVLER && session.appUser.narrativesMap != null
                && !session.appUser.narrativesMap.isEmpty()) {
            Iterator narrativeIter = session.appUser.narrativesMap.entrySet().iterator();

            //loop through all narratives, check the allergy ones to see if they are referenced in this call
            //if they are not referenced, then create a record for them so the user is able to view this narrative
            while (narrativeIter.hasNext()) {
                Map.Entry narrative = (Map.Entry) narrativeIter.next();
                if (narrative.getKey().toString().contains("8716-3")) {
                    //check to see if record using this narrative exists
                    if (!narrativeIds.contains(narrative.getKey().toString())) {
                        NarrativeV4 narrativeItem = session.appUser.narrativesMap.get(narrative.getKey().toString());
                        String siteName = "VLER";
                        String siteMoniker = "VLER Partner";
                        if (narrativeItem != null)
                            if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                    && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                                siteName = session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier());
                            }
                        if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                            siteMoniker = session.appUser.partnersMap.get(narrativeItem.getNarrativeAssigningAuthorityIdentifier());
                        }
                        GUISite narrativeSite = new GUISite();
                        narrativeSite.setAgency("VLER");
                        narrativeSite.setName(siteName);
                        narrativeSite.setMoniker(siteMoniker);
                        narrativeSite.setSiteCode("VLER");

                        GUIVitals narrativeRecord = new GUIVitals();
                        narrativeRecord.setNarrativeId(narrative.getKey().toString());
                        narrativeRecord.setSite(narrativeSite);
                        narrativeRecord.setType("Narrative");
                        guiVitals.add(narrativeRecord);

                        narrativeIds.add(narrative.getKey().toString());
                    }

                }
            }

        }

        sendResponse(guiVitals)
    }

    def histogram = {

        def guiHistogramResponse = null;
        guiHistogramResponse = vitalsDao.getVitalsGraphData(
                session.appUser,
                request.patient,
                request.recordQuery[HttpKeys.VITAL_TYPE],
                request.recordQuery[HttpKeys.VITAL_UNITS_CODE],
                Utils.getCal(request.recordQuery[HttpKeys.START_DATE]),
                Utils.getCal(request.recordQuery[HttpKeys.END_DATE]))

        sendResponse(guiHistogramResponse)
    }
}
