/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.common;

import org.apache.commons.codec.EncoderException;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.net.URLCodec;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.SecureRandom;

/**
 *
 *
 */
@Component
public class AESCryptoUtil {

    @Autowired
    private AppConfig appConfig;

    private SecureRandom random = new SecureRandom();
    private byte[] iv = random.generateSeed(16);

    private static final String CIPHER_TRANSFORMATION = "AES/CBC/PKCS5Padding";
    private static final String ALGORITHM = "AES";

    private static final Logger logger = Logger.getLogger(AESCryptoUtil.class.getName());

    public byte[] encrypt(byte[] securityKey, String message) {
        try {
            SecretKeySpec skeySpec = new SecretKeySpec(securityKey, ALGORITHM);
            Cipher cipher = Cipher.getInstance(CIPHER_TRANSFORMATION);
            cipher.init(Cipher.ENCRYPT_MODE, skeySpec, new IvParameterSpec(iv));
            return cipher.doFinal(message.getBytes("UTF-8"));

        } catch (Exception e) {
            throw new RuntimeException("AES encryption failed", e);
        }
    }

    public byte[] decrypt(byte[] securityKey, byte[] encrypted) {
        try {
            SecretKeySpec skeySpec = new SecretKeySpec(securityKey, ALGORITHM);
            Cipher cipher = Cipher.getInstance(CIPHER_TRANSFORMATION);
            cipher.init(Cipher.DECRYPT_MODE, skeySpec, new IvParameterSpec(iv));
            return cipher.doFinal(encrypted);
        } catch (Exception e) {
            throw new RuntimeException("AES decryption failed", e);
        }
    }

    public byte[] generateSecurityKey() {
        KeyGenerator generator;
        try {
            generator = KeyGenerator.getInstance(ALGORITHM);
            generator.init(128);
            return generator.generateKey().getEncoded();

        } catch (Exception e) {
            throw new RuntimeException("AES generateSecurityKey failure", e);
        }
    }

    public boolean isEncryptedTokenDebug() {
        return appConfig.isEncryptedTokenDebug();
    }

    public String generateEncryptedQuery(byte[] securityKey, String queryParams) {
        if (isEncryptedTokenDebug()) {
            try {
                URLCodec urlCodec = new URLCodec();
                return urlCodec.encode(queryParams);
            } catch (EncoderException ex) {
                logger.error(ex.getMessage(), ex);
                return null;
            }
        } else {
            byte[] encryptedBytes = encrypt(securityKey, queryParams);
            return Base64.encodeBase64URLSafeString(encryptedBytes);
        }

    }
}
