package gov.va.med.common;

import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.JMeadowsQuery;
import gov.va.med.jmeadows.webservice.Patient;
import gov.va.med.jmeadows.webservice.User;
import gov.va.med.vhahon.common.Utils;
import org.dozer.DozerBeanMapper;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.Calendar;

public class JMeadowsQueryBuilder {
    private Integer _active;
    private Calendar _endDate;
    private String _itemId;
    private Integer _max;
    private GUIPatient _patient;
    private String _recordSiteCode;
    private String _sortBy;
    private Calendar _startDate;
    private String _status;
    private GUIUser _user;
    private boolean _includeVLER;
    private boolean _forceRefresh;

    private DozerBeanMapper dozerBeanMapper;
    private AppConfig appConfig;

    public JMeadowsQueryBuilder(AppConfig appConfig, DozerBeanMapper dozerBeanMapper) {
        this.appConfig = appConfig;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    public JMeadowsQueryBuilder active(Integer active) {
        this._active = active;
        return this;
    }

    public JMeadowsQueryBuilder endDate(Calendar endDate) {
        this._endDate = endDate;
        return this;
    }

    public JMeadowsQueryBuilder itemId(String itemId) {
        this._itemId = itemId;
        return this;
    }

    public JMeadowsQueryBuilder max(Integer max) {
        this._max = max;
        return this;
    }

    public JMeadowsQueryBuilder patient(GUIPatient patient) {
        this._patient = patient;
        return this;
    }

    public JMeadowsQueryBuilder recordSiteCode(String recordSiteCode) {
        this._recordSiteCode = recordSiteCode;
        return this;
    }

    public JMeadowsQueryBuilder sortBy(String sortBy) {
        this._sortBy = sortBy;
        return this;
    }

    public JMeadowsQueryBuilder startDate(Calendar startDate) {
        this._startDate = startDate;
        return this;
    }

    public JMeadowsQueryBuilder status(String status) {
        this._status = status;
        return this;
    }

    public JMeadowsQueryBuilder user(GUIUser user) {
        this._user = user;
        return this;
    }

    public JMeadowsQueryBuilder includeVLER(boolean includeVLER){
        this._includeVLER = includeVLER;
        return this;
    }

    public JMeadowsQueryBuilder forceRefresh(boolean forceRefresh){
        this._forceRefresh = forceRefresh;
        return this;
    }

    public JMeadowsQuery build() {
        JMeadowsQuery queryBean = new JMeadowsQuery();
        queryBean.setActive(_active);
        queryBean.setItemId(_itemId);
        queryBean.setStatus(_status);
        queryBean.setSortBy(_sortBy);
        queryBean.setRecordSiteCode(_recordSiteCode);
        queryBean.setMax(_max);
        queryBean.setIncludeVLER(_includeVLER);
        queryBean.setForceRefresh(_forceRefresh);
        queryBean.setRequestingApp(appConfig.getAppName());

        if (_user != null)
            queryBean.setUser(dozerBeanMapper.map(_user, User.class));

        if (_patient != null)
            queryBean.setPatient(dozerBeanMapper.map(_patient, Patient.class));

        XMLGregorianCalendar startXmlDate = null;
        XMLGregorianCalendar endXmlDate = null;

        if (_startDate != null)
            startXmlDate = Utils.toXMLGregorianCalendar(Utils.pushDateToStartOfDay(_startDate.getTime()));

        if (_endDate != null)
            endXmlDate = Utils.toXMLGregorianCalendar(Utils.pushDateToEndOfDay(_endDate.getTime()));

        queryBean.setStartDate(startXmlDate);
        queryBean.setEndDate(endXmlDate);

        return queryBean;
    }
}
