package gov.va.med.dao;


import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.AssignedPatient;
import gov.va.med.jmeadows.webservice.Consult;
import gov.va.med.jmeadows.webservice.OutsideProvider;
import gov.va.med.jmeadows.webservice.Patient;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

@Component
public class CommunityDao extends DaoBase {

    public void updateOutsideProvider(GUIUser user, GUIOutsideProvider outsideProvider) {

        try {
            getServiceHandler().updateOutsideProvider(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .build(),
                    dozerBeanMapper.map(outsideProvider, OutsideProvider.class)
            );
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public boolean doesUsernameExist(GUIUser user, String username) {

        try {
            OutsideProvider provider = getServiceHandler().getOutsideProviderExist(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .build(),
                    username
            );

            return provider != null;

        } catch (Exception e) {
            throw new DaoException(e);
        }
    }


    public List<GUIPatient> lookupPatient(GUIUser user, String searchTerm) {

        try {

            Patient[] patients = getServiceHandler().lookupPatient(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .recordSiteCode(user.getHostSite().getSiteCode())
                            .itemId(searchTerm)
                            .build()
            );

            List<GUIPatient> guiList = new ArrayList<GUIPatient>();

            for (int i = 0; i < patients.length; i++) {
                guiList.add(dozerBeanMapper.map(patients[i], GUIPatient.class));
            }


            return guiList;

        } catch (Exception e) {
            throw new DaoException(e);

        }

    }


    public List<GUIOutsideProvider> lookupOutsideProvider(GUIUser user, String searchName) {
        try {

            List<OutsideProvider> lst = getServiceHandler().getOutsideProviderList(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .build(),
                    searchName);

            List<GUIOutsideProvider> guiList = new ArrayList<GUIOutsideProvider>();

            for (OutsideProvider outsideProvider : lst) {
                guiList.add(dozerBeanMapper.map(outsideProvider, GUIOutsideProvider.class));
            }

            return guiList;


        } catch (Exception e) {
            throw new DaoException(e);

        }
    }

    public void addOutsideProvider(GUIUser user, GUIOutsideProvider outsideProvider) {
        try {
            getServiceHandler().addOutsideProvider(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .build(),
                    dozerBeanMapper.map(outsideProvider, OutsideProvider.class)
            );
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }


    public void deleteOutsideProvider(GUIUser user, String outsideProviderId) {
        try {
            getServiceHandler().deleteOutsideProvider(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .build(),
                    outsideProviderId);
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public void addAssignedPatient(GUIUser user, Calendar startDate, GUIAssignedPatient assignedPatient) {

        try {
            getServiceHandler().addAssignedPatient(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .status("ACTIVE")
                            .startDate(startDate)
                            .build(),
                    dozerBeanMapper.map(assignedPatient, AssignedPatient.class)
            );
        } catch (Exception e) {
            throw new DaoException(e);
        }

    }

    public void deleteAssignedPatient(GUIUser user, String assignedPatientId) {
        try {

            System.out.println(assignedPatientId);
            getServiceHandler().deleteAssignedPatient(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .build(),
                    assignedPatientId
            );
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public List<GUIAssignedPatient> getAssignedPatientList(GUIUser user, String providerId, byte[] encryptSecurityKey) {

        try {

            List<AssignedPatient> lst = getServiceHandler().getAssignedPatientList(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .build(),
                    providerId);

            List<GUIAssignedPatient> guiList = new ArrayList<GUIAssignedPatient>();
            GUIAssignedPatient guiAP;

            for (AssignedPatient assignedPatient : lst) {
                guiAP = dozerBeanMapper.map(assignedPatient, GUIAssignedPatient.class);
                guiAP.setPatientToken(tokenUtils.generatePatientToken(encryptSecurityKey,
                        appConfig.getIdTypeVAIen(), guiAP.getIen(), guiAP.getSiteId()));
                guiList.add(guiAP);

            }

            return guiList;


        } catch (Exception e) {
            throw new DaoException(e);

        }
    }

    public List<GUIConsult> getPatientActiveConsultRequest(GUIUser user, GUIPatient patient) {

        try {

            List<Consult> lst = getServiceHandler().getPatientActiveConsultRequest(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .build()

            );

            List<GUIConsult> guiConsults = new ArrayList<GUIConsult>();
            String requestDate;
            StringBuilder consultId = new StringBuilder();
            StringBuilder outCode = new StringBuilder();
            String outDescription;
            String outStatus;
            String consultReport;
            String service;
            String newConsultId;
            String authNum;

            int i = 0;
            for (Consult consult : lst) {
                requestDate = (consult.getRequestDate() != null) ? Utils.formatDisplayDate(consult.getRequestDate()) : "";
                consultReport = (consult.getReport() != null) ? consult.getReport() : "";
                outDescription = (consult.getProcedureConsult() != null) ? consult.getProcedureConsult() : "";
                service = (consult.getService() != null) ? consult.getService() : "";

                consultId.setLength(0);
                outCode.setLength(0);
                if (consult.getService() != null) {
                    if (StringUtils.isBlank(service)) {
                        outCode.append("click to view consult");
                    } else {
                        outCode.append(consult.getService());
                    }
                }
                outStatus = (consult.getStatus() != null) ? consult.getStatus() : "";
                newConsultId = (consult.getId() != null) ? consult.getId() : "";

                authNum = Utils.getPiece(consult.getId(), "^", 2);
                if (StringUtils.isNotEmpty(authNum)) {
                    outCode.append("  #");
                    outCode.append(authNum);
                }

                GUIConsult guiConsult = new GUIConsult();
                //if rpc or dod adapter protocol then flag record for caching
                if (appConfig.getProtocolDODAdapter().equals(consult.getSourceProtocol())) {
                    guiConsult.setSessionCachedRecord(true);
                    consultId.append(appConfig.getCachedTag());
                    consultId.append(i);
                } else {
                    consultId.append(newConsultId);
                }

                guiConsult.setConsultId(consultId.toString());
                guiConsult.setOutCode(outCode.toString());
                guiConsult.setOutDescription(outDescription);
                guiConsult.setOutStatus(outStatus);
                guiConsult.setRequestDate(requestDate);
                guiConsult.setReport(consultReport);
                guiConsult.setService(service);
                guiConsult.setComplexDataUrl(consult.getComplexDataUrl());

                if (consult.getProvider() != null) {

                    guiConsult.setProvider(dozerBeanMapper.map(consult.getProvider(), GUIProvider.class));
                }


                //if (consultReport.contains("CISURL"))
                if (appConfig.getProtocolDODAdapter().equalsIgnoreCase(consult.getSourceProtocol()) &&
                        StringUtils.isNotBlank(guiConsult.getComplexDataUrl())) {
                    guiConsult.setNoteAsBinaryStream(true);
                }
                addSiteDataToGUIBean(guiConsult, consult);
                guiConsults.add(guiConsult);

                i++;
            }

            return guiConsults;





        } catch (Exception e) {
            throw new DaoException(e);
        }

    }



}
