package gov.va.med.dao;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import javax.xml.soap.SOAPException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

@Component
public class HeartbeatDao extends DaoBase {
    private static final Logger logger = Logger.getLogger(HeartbeatDao.class.getName());

    public List<String> getServicesStatus() {
        List<String> serviceStatus = new ArrayList<String>();

        try {
            String serviceErrors = getServiceHandler().getServiceErrors();

            if (!StringUtils.isBlank(serviceErrors)) {
                String token;
                StringTokenizer tokenizer = new StringTokenizer(serviceErrors, "^");
                StringTokenizer subTokenizer;

                while (tokenizer.hasMoreTokens()) {
                    token = tokenizer.nextToken();

                    if ("PDWSService".equalsIgnoreCase(token)) {
                        serviceStatus.add("Patient Lookup service is unavailable: Unable to search for a patient.");
                    } else if ("MVIService".equalsIgnoreCase(token)) {
                        serviceStatus.add("VA Patient Identity service is unavailable: VA data may not display.");
                    } else if ("BHIERelayService".equalsIgnoreCase(token)) {
                        serviceStatus.add("AHLTA data service is unavailable: DoD data may not display.");
                    } else if ("VistaDataService".equalsIgnoreCase(token)) {
                        serviceStatus.add("VistA data service is unavailable: VA data may not display.");
                    } else if (token.contains("ShareService")) {
                        // Must have 3 tokens ("ShareService", <siteCode>, <siteName>)
                        subTokenizer = new StringTokenizer(token, ":");
                        if (subTokenizer.countTokens() == 3) {
                            // Skip to <siteName> token
                            subTokenizer.nextToken();
                            subTokenizer.nextToken();

                            serviceStatus.add(subTokenizer.nextToken() + " login service is unavailable: Users may be unable to login to this site.");
                        }
                    } else if ("SnareworksService".equalsIgnoreCase(token)) {
                        serviceStatus.add("Snareworks login service is unavailable. Users may be unable to login to this site.");
                    }
                }
            }
        } catch (SOAPException e) {
            logger.error(e.getMessage(), e);
            serviceStatus = null;
        }

        return serviceStatus;
    }
}
