/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.*;
import gov.va.med.vhahon.common.Utils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

@Component
public class MedicationDao extends DaoBase {
    public static final int MEDICATION_STATUS_OUTPATIENT = 0;
    public static final int MEDICATION_STATUS_INPATIENT = 1;

    private static final Logger logger = Logger.getLogger(MedicationDao.class.getName());

    public List<GUIMedication> getPatientMedications(GUIUser user, GUIPatient patient, int medStatus, Calendar startDate, Calendar endDate) {
        try {
            String queryMedStatus = "I";
            if (medStatus == MEDICATION_STATUS_OUTPATIENT) {
                queryMedStatus = "O";
            }

            JMeadowsQuery qb = jMeadowsQueryBuilderFactory.getBuilder()
                    .user(user)
                    .patient(patient)
                    .startDate(startDate)
                    .endDate(endDate)
                    .build();

            qb.setStatus(queryMedStatus);
            qb.setActive(2);

            Medication[] meds = getServiceHandler().getPatientMedications(qb);

            List<GUIMedication> guiMeds = new ArrayList<GUIMedication>();

            for (int i = 0; i < meds.length; i++) {
                Medication med = meds[i];

                GUIMedication guiMed = dozerBeanMapper.map(med, GUIMedication.class);

                String orderDate = "No Date";
                String lastFilledDate = "No Date";
                String fillExpirationDate = "No Date";
                String dispenseDate = "No Date";
                String drugName = "";
                String refills = "";
                String sig = "";
                String quantity = "";
                String daysSupply = "";
                String stopDate = "";
                String startMedDate = "";
                String pharmacy = "";
                String comment = "";
                String medicationId = "";
                String status = "";

                if (med.getOrderDate() != null) {
                    orderDate = Utils.formatDisplayDate(med.getOrderDate());
                }

                if (med.getLastFilledDate() != null) {
                    lastFilledDate = Utils.formatDisplayDate(med.getLastFilledDate());
                }

                if (med.getFillExpirationDate() != null) {
                    fillExpirationDate = Utils.formatDisplayDate(med.getFillExpirationDate());
                }

                if (med.getDrugName() != null) {
                    drugName = med.getDrugName();
                }

                if (med.getRefills() != null) {
                    refills = med.getRefills();
                }

                if (med.getSigCode() != null) {
                    sig = med.getSigCode();
                }


                if (med.getQuantity() != null) {
                    quantity = med.getQuantity();
                }

                if (med.getDaysSupply() != null) {
                    daysSupply = med.getDaysSupply();
                }

                if (med.getStopDate() != null) {
                    stopDate = Utils.formatDisplayDate(med.getStopDate());
                }

                if (med.getStartDate() != null) {
                    startMedDate = Utils.formatDisplayDate(med.getStartDate());
                }

                if (med.getLastDispensingPharmacy() != null) {
                    pharmacy = med.getLastDispensingPharmacy();
                }

                if (med.getComment() != null) {
                    comment = med.getComment();
                }

                if (med.getMedId() != null) {
                    medicationId = med.getMedId();
                }

                if (med.getStatus() != null) {
                    status = med.getStatus();
                }
                guiMed.setLastFilledDate(lastFilledDate);
                guiMed.setOrderDate(orderDate);
                guiMed.setFillExpirationDate(fillExpirationDate);
                guiMed.setDrugName(drugName);
                guiMed.setRefills(refills);
                guiMed.setSig(sig);
                if (med.getOrderingProvider() != null) {
                    guiMed.setOrderingProvider(dozerBeanMapper.map(med.getOrderingProvider(), GUIProvider.class));
                }
                guiMed.setQuantity(quantity);
                guiMed.setDaysSupply(daysSupply);
                guiMed.setStopDate(stopDate);
                guiMed.setStartDate(startMedDate);
                guiMed.setPharmacy(pharmacy);
                guiMed.setComment(comment);
                guiMed.setMedicationId(medicationId);
                guiMed.setStatus(status);

                if (med.getPrescriptionFills() != null && !med.getPrescriptionFills().isEmpty()) {
                    guiMed.getPrescriptionFills().addAll(mapGUIRxFills(med));

                    if (appConfig.getProtocolDODAdapter().equals(med.getSourceProtocol())) {
                        if (med.getPrescriptionFills().get(0).getDispenseDate() != null) {
                            dispenseDate = Utils.formatDisplayDate(med.getPrescriptionFills().get(0).getDispenseDate());
                        }
                    }
                }
                guiMed.setDispenseDate(dispenseDate);
                if (appConfig.getProtocolDODAdapter().equals(med.getSourceProtocol())) {
                    if (med.getCodes() != null && !med.getCodes().isEmpty()) {
                        for (Code code : med.getCodes()) {
                            if ("NDC".equalsIgnoreCase(code.getSystem())) {
                                guiMed.setNdc(code.getCode());
                                break;
                            }
                        }
                    }
                }
                addSiteDataToGUIBean(guiMed, med);

                List<GUICode> guiCodes = new ArrayList<GUICode>();
                for (Code code : med.getCodes()) {
                    guiCodes.add(dozerBeanMapper.map(code, GUICode.class));
                }
                guiMed.setCodes(guiCodes);

                guiMeds.add(guiMed);
            }

            return guiMeds;

        } catch (RuntimeException e) {
            throw new DaoException(e);
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public GUIMedicationDetail getMedicationDetails(GUIUser user, GUIPatient patient, String medicationId,
                                                    String recordSiteCode) {
        try {
            FreeTextReport freeTextReport = getServiceHandler().getMedicationDetails(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .itemId(medicationId)
                            .recordSiteCode(recordSiteCode)
                            .build());
            GUIMedicationDetail medDetail = new GUIMedicationDetail();
            String reportText = null;
            if (freeTextReport != null) reportText = freeTextReport.getReportText();
            medDetail.setMedDetail(reportText);
            if (freeTextReport != null)
                addSiteDataToGUIBean(medDetail, freeTextReport);

            return medDetail;
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    private List<GUIPrescriptionFill> mapGUIRxFills(Medication rx) {
        List<PrescriptionFill> rxFills = rx.getPrescriptionFills();

        String drugName = "";
        String quantity = "";
        ;
        String daysSupply = "";
        ;
        String expirationDate = "";
        ;
        String comment = "";
        String orderDate = "";
        String lastFilledDate = "";
        String dispensingPharmacy = "";

        if (rx.getDrugName() != null) {
            drugName = rx.getDrugName();
        }

        if (rx.getQuantity() != null) {
            quantity = rx.getQuantity();
        }

        if (rx.getDaysSupply() != null) {
            daysSupply = rx.getDaysSupply();
        }

        if (rx.getStopDate() != null) {
            expirationDate = Utils.formatDisplayDate(rx.getStopDate());
        }

        if (rx.getComment() != null) {
            comment = rx.getComment();
        }

        if (rx.getOrderDate() != null) {
            orderDate = Utils.formatDisplayDate(rx.getOrderDate());
        }

        if (rx.getLastFilledDate() != null) {
            lastFilledDate = Utils.formatDisplayDate(rx.getLastFilledDate());
        }

        if (rx.getLastDispensingPharmacy() != null) {
            dispensingPharmacy = rx.getLastDispensingPharmacy();
        }

        List<GUIPrescriptionFill> guiRxFills = new ArrayList<GUIPrescriptionFill>();

        if (rxFills != null && rxFills.size() > 0) {
            for (PrescriptionFill rxFill : rxFills) {
                String rxFillFillOrderDate = "";
                String rxFillDispensingPharmacy = "";
                String rxFillQuantity = "";
                String rxFillRouting = "";
                String rxFillReleaseDate = "";
                String rxFillDaysSupply = "";

                if (rxFill.getDispenseDate() != null) {
                    rxFillFillOrderDate = Utils.formatDisplayDateTime(rxFill.getDispenseDate());
                }

                if (rxFillFillOrderDate.length() < 2) rxFillFillOrderDate = "No Date";

                if (rxFill.getDispensingPharmacy() != null) {
                    rxFillDispensingPharmacy = rxFill.getDispensingPharmacy();
                }

                if (rxFill.getDispensingQuantity() != null) {
                    rxFillQuantity = rxFill.getDispensingQuantity();
                }

                if (rxFill.getDaysSupply() != null) {
                    rxFillDaysSupply = rxFill.getDaysSupply();
                }

                if (rxFill.getReleaseDate() != null) {
                    rxFillReleaseDate = Utils.formatDisplayDateTime(rxFill.getReleaseDate());
                }

                if (rxFill.getRouting() != null) {
                    rxFillRouting = rxFill.getRouting();
                }

                guiRxFills.add(generateGUIRXFill(rxFillFillOrderDate, orderDate, drugName, rxFillQuantity,
                        rxFillDaysSupply, expirationDate, comment, rxFillDispensingPharmacy, rxFillReleaseDate, rxFillRouting, rxFill.isPartial()));
            }
        } else {
            guiRxFills.add(generateGUIRXFill(lastFilledDate, orderDate, drugName, quantity,
                    daysSupply, expirationDate, comment, dispensingPharmacy, null, null, false));
        }

        return guiRxFills;
    }

    private GUIPrescriptionFill generateGUIRXFill(String lastFilledDate, String orderDate, String drugName, String quantity,
                                                  String daysSupply, String expirationDate, String comment,
                                                  String dispensingPharmacy, String fillReleaseDate, String fillRouting, boolean partial) {
        GUIPrescriptionFill guiRxFill = new GUIPrescriptionFill();

        guiRxFill.setDrugName(drugName);
        guiRxFill.setComment(comment);
        guiRxFill.setDaysSupply(daysSupply);
        guiRxFill.setDispensingPharmacy(dispensingPharmacy);
        guiRxFill.setLastFilledDate(lastFilledDate);
        guiRxFill.setOrderDate(orderDate);
        guiRxFill.setQuantity(quantity);
        guiRxFill.setExpirationDate(expirationDate);
        guiRxFill.setReleaseDate(fillReleaseDate);
        guiRxFill.setRouting(fillRouting);
        guiRxFill.setPartial(partial);

        return guiRxFill;
    }
}
