/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.rpc;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

/**
 *
 *
 */
public class DdrLister extends DdrQuery {
    private String file = "";
    private String iens = "";
    private String[] requestedFields;       // Holds user's requested order
    private LinkedHashMap<String, String> requestedFieldsTbl;   // Needed to fetch by FileMan field #, and to hold if External or not
    private List<String> ienLst;               // Needed to hold record IENs across methods.
    private String flags = "";
    private String max = "";
    private String from = "";
    private String part = "";
    private String xref = "";
    private String screen = "";
    private String id = "";
    private String options = "";
    private String moreFrom = "";
    private String moreIens = "";

    //public boolean more = false;

    public DdrLister(RPCConnection cxn) {
        super(cxn);
    }

    protected String buildRequest() throws RPCException {
        if ("".equals(file)) {
            throw new RPCException("Must have a file!");
        }
        VistaQuery vqry = vistaQueryFactory.getVistaQuery("DDR LISTER");
        DictionaryHashList paramLst = new DictionaryHashList();
        paramLst.add("\"FILE\"", file);
        if (!"".equals(iens)) {
            paramLst.add("\"IENS\"", iens);
        }
        if (requestedFields.length > 0) {
            paramLst.add("\"FIELDS\"", getFieldsArg());
        }
        if (!"".equals(flags)) {
            paramLst.add("\"FLAGS\"", flags);
        }
        if (!"".equals(max)) {
            paramLst.add("\"MAX\"", max);
        }
        if (!"".equals(from)) {
            paramLst.add("\"FROM\"", from);
        }
        if (!"".equals(part)) {
            paramLst.add("\"PART\"", part);
        }
        if (!"".equals(xref)) {
            paramLst.add("\"XREF\"", xref);
        }
        if (!"".equals(screen)) {
            paramLst.add("\"SCREEN\"", screen);
        }
        if (!"".equals(id)) {
            paramLst.add("\"ID\"", id);
        }
        if (!"".equals(options)) {
            paramLst.add("\"OPTIONS\"", options);
        }
        if (!"".equals(moreFrom)) {
            paramLst.add("\"FROM\"", moreFrom);
        }
        if (!"".equals(moreIens)) {
            paramLst.add("\"IENS\"", moreIens);
        }
        vqry.addParameter(VistaQuery.LIST, paramLst);
        return vqry.buildMessage();
    }

    public String[] execute() throws RPCException {
        String request = buildRequest();
        String response = this.execute(request);
        return buildResult(response.replace('|', '^'));
    }

    public void setIenNameQuery(String fileNum) {
        this.setFile(fileNum);
        this.setFields(".01");
        this.setFlags("IP");
        this.setXref("#");
    }
    
//    public static DdrLister buildIenNameQuery(RPCConnection cxn, String fileNum) {
//        DdrLister query = new DdrLister(cxn);
//        query.setFile(fileNum);
//        query.setFields(".01");
//        query.setFlags("IP");
//        query.setXref("#");
//        return query;
//    }

//    public static DdrLister buildFileQuery(RPCConnection cxn, VistaFile file) {
//        DdrLister query = new DdrLister(cxn);
//        query.setFile(file.getFileNum());
//        query.setFields(file.getFieldString());
//        query.setFlags("IP");
//        query.setXref("#");
//        return query;
//    }

    public void setFileQuery(VistaFile file) {
        this.setFile(file.getFileNum());
        this.setFields(file.getFieldString());
        this.setFlags("IP");
        this.setXref("#");
    }
    
    private String getFieldsArg() {
        StringBuilder resultSB = new StringBuilder("@");
        for (int i = 0; i < requestedFields.length; i++) {
            if ("@".equals(requestedFields[i])) {
                continue;
            }
            resultSB.append(';');
            resultSB.append(requestedFields[i]);
        }
        return resultSB.toString();
    }

    private void setMoreParams(String line) throws RPCException {
        String[] flds = StringUtils.split(line, StringUtils.CARET);
        if (!"MORE".equals(flds[0])) {
            StringBuilder exceptionMsg = new StringBuilder();
            exceptionMsg.append("Invalid return data: expected 'MORE', got ");
            exceptionMsg.append(flds[0]);
            throw new RPCException(exceptionMsg.toString());
        }
        moreFrom = flds[1];
        moreIens = flds[2];
        //more = true;
    }

    public String[] buildResult(String rtn) throws RPCException {
        String[] rtc;
        String[] lines = StringUtils.split(rtn, StringUtils.CRLF);
        lines = StringUtils.trimArray(lines);
        int lnum = 0;

        if (lines[lnum].equals(VistaConstants.MISC)) {
            if (!lines[++lnum].startsWith("MORE")) {
                StringBuilder exceptionMsg = new StringBuilder();
                exceptionMsg.append("Error packing LISTER return; expected 'MORE...', got ");
                exceptionMsg.append(lines[lnum]);
                throw new RPCException(exceptionMsg.toString());
            }
            setMoreParams(lines[lnum]);
        }

        if (this.flags.indexOf("P") == -1) {
            rtc = packResult(lines);
        } else {
            rtc = parsePackedResult(lines);
        }
        return rtc;
    }

    private String[] parsePackedResult(String[] lines) throws RPCException {
        int idx = StringUtils.getIdx(lines, VistaConstants.BEGIN_ERRS, 0);
        if (idx != -1) {
            throw new RPCException(getErrMsg(lines, idx));
        }

        idx = StringUtils.getIdx(lines, VistaConstants.BEGIN_DATA, 0);
        if (idx == -1) {
            StringBuilder exceptionMsg = new StringBuilder();
            exceptionMsg.append("Error parsing packed result: expected ");
            exceptionMsg.append(VistaConstants.BEGIN_DATA);
            exceptionMsg.append(", found none.");
            throw new RPCException(exceptionMsg.toString());
        }

        ArrayList<String> lst = new ArrayList<String>();
        idx++;
        while (idx < lines.length && !lines[idx].equals(VistaConstants.END_DATA)) {
            lst.add(lines[idx++]);
        }
        return lst.toArray(new String[0]);
    }

    private String[] packResult(String[] lines) throws RPCException {
        String[] rtc;
        int idx = 0;

        if (lines[idx].equals(VistaConstants.UNPACKED_NO_RESULTS)) {
            rtc = new String[0];
        } else {
            LinkedHashMap<String, LinkedHashMap<String, DdrField>> result = new LinkedHashMap<String, LinkedHashMap<String, DdrField>>();
    
            idx = StringUtils.getIdx(lines, VistaConstants.BEGIN_ERRS, 0);
            if (idx != -1) {
                throw new RPCException(getErrMsg(lines, idx));
            }
    
            idx = StringUtils.getIdx(lines, VistaConstants.BEGIN_DATA, 0);
            if (idx != -1) {
                ienLst = new ArrayList<String>();
                if (!lines[++idx].equals(VistaConstants.BEGIN_IENS)) {
                    throw new RPCException("Incorrectly formatted return data");
                }
                idx++;
                while (!lines[idx].equals(VistaConstants.END_IENS)) {
                    ienLst.add(lines[idx++]);
                }
    
                idx++;
                if (!lines[idx].equals(VistaConstants.BEGIN_IDVALS)) {
                    throw new RPCException("Incorrectly formatted return data");
                }
                String[] flds = StringUtils.split(lines[++idx], StringUtils.SEMICOLON);
    
                int recIdx = 0;
                idx++;
                while (!lines[idx].equals(VistaConstants.END_IDVALS)) {
                    // the last field in flds is the field count, not a field
                    LinkedHashMap<String, DdrField> rec = new LinkedHashMap<String, DdrField>();
                    for (int fldIdx = 0; fldIdx < flds.length - 1; fldIdx++) {
                        DdrField fld = new DdrField();
                        fld.setFmNumber(flds[fldIdx]);
                        String requestedOptions = requestedFieldsTbl.get(fld.getFmNumber());
                        fld.setHasExternal(requestedOptions.indexOf('E') != -1);
                        if (fld.isHasExternal()) {
                            fld.setExternalVal(lines[idx++]);
                        }
                        if (requestedOptions.indexOf('I') != -1) {
                            fld.setVal(lines[idx++]);
                        }
                        rec.put(fld.getFmNumber(), fld);
                    }
                    result.put(ienLst.get(recIdx++), rec);
                }
                // at this point line should be VistaConstants.END_DATA
                // unless more functionality is added.
            }
            rtc = toStringArray(result);
        }
        return rtc;
    }

    private String getErrMsg(String[] lines, int idx) {
        StringBuilder msgSB = new StringBuilder(lines[idx + 3]);
        int endIdx = StringUtils.getIdx(lines, VistaConstants.END_ERRS, 0);
        for (int i = idx + 4; i < endIdx; i++) {
            if (msgSB.charAt(msgSB.length() - 1) != '.') {
                msgSB.append(". ");
            }
            msgSB.append(lines[i]);
        }
        return msgSB.toString();
    }

    private String[] toStringArray(LinkedHashMap hashedRex) {
        ArrayList<String> lst = new ArrayList<String>();
        StringBuilder sbld = new StringBuilder();
        int ienLstSize = ienLst.size();
        for (int recnum = 0; recnum < ienLstSize; recnum++) {
            sbld.setLength(0);
            sbld.append(ienLst.get(recnum));
            LinkedHashMap hashedFlds = (LinkedHashMap) hashedRex.get(ienLst.get(recnum));
            for (int fldnum = 0; fldnum < requestedFields.length; fldnum++) {
                String fmNum = requestedFields[fldnum];
                boolean external = false;
                if (fmNum.indexOf('E') != -1) {
                    fmNum = StringUtils.CSubstring(fmNum, 0, fmNum.length() - 1);
                    external = true;
                }
                DdrField fld = (DdrField) hashedFlds.get(fmNum);
                if (external) {
                    sbld.append('^');
                    sbld.append(fld.getExternalVal());
                } else {
                    sbld.append('^');
                    sbld.append(fld.getVal());
                }
            }
            lst.add(sbld.toString());
        }
        return lst.toArray(new String[0]);
    }

    public String getFile() {
        return file;
    }

    public void setFile(String file) {
        this.file = file;
    }

    public String getFlags() {
        return flags;
    }

    public void setFlags(String flags) {
        this.flags = flags;
    }

    public String getFrom() {
        return from;
    }

    public void setFrom(String from) {
        this.from = from;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getMax() {
        return max;
    }

    public void setMax(String max) {
        this.max = max;
    }

    public String getMoreFrom() {
        return moreFrom;
    }

    public void setMoreFrom(String moreFrom) {
        this.moreFrom = moreFrom;
    }

    public String getMoreIens() {
        return moreIens;
    }

    public void setMoreIens(String moreIens) {
        this.moreIens = moreIens;
    }

    public String getOptions() {
        return options;
    }

    public void setOptions(String options) {
        this.options = options;
    }

    public String getPart() {
        return part;
    }

    public void setPart(String part) {
        this.part = part;
    }

    public String getScreen() {
        return screen;
    }

    public void setScreen(String screen) {
        this.screen = screen;
    }

    public String getXref() {
        return xref;
    }

    public void setXref(String xref) {
        this.xref = xref;
    }

    public void setFields(String value) {
        String s = value;
        requestedFields = StringUtils.split(s, StringUtils.SEMICOLON);
        requestedFieldsTbl = new LinkedHashMap<String, String>(requestedFields.length);
        StringBuilder fieldOption = new StringBuilder();
        for (int i = 0; i < requestedFields.length; i++) {
            if ("".equals(requestedFields[i])) {
                continue;
            }
            String fldnum = requestedFields[i];
            String option = "I";
            if (fldnum.indexOf('E') != -1) {
                fldnum = StringUtils.CSubstring(fldnum, 0, fldnum.length() - 1);
                option = "E";
            }
            if (requestedFieldsTbl.containsKey(fldnum)) {
                fieldOption.setLength(0);
                fieldOption.append(requestedFieldsTbl.get(fldnum));
                fieldOption.append(option);
                requestedFieldsTbl.put(fldnum, fieldOption.toString());
            } else {
                requestedFieldsTbl.put(fldnum, option);
            }
        }
    }

    public String getIens() {
        return iens;
    }

    public void setIens(String iens) {
        this.iens = iens;
    }
}
