/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.rpc;

/**
 *
 *
 */
public class StringUtils {
    public static final String CRLF = "\r\n";
    public static final String CARET = "^";
    public static final String STICK = "|";
    public static final String COLON = ":";
    public static final String SEMICOLON = ";";
    public static final String COMMA = ",";
    public static final String PERIOD = ".";
    public static final String SLASH = "/";
    public static final String SPACE = " ";
    public static final String EQUALS = "=";
    public static final String AMPERSAND = "&";
    public static final String ATSIGN = "@";

    public static String[] split(String str, char delimiter) {
        return split(str, "" + delimiter);
    }

    public static String[] split(String str, String delimiter) {
        return str.split(escapeRegExMetaChars(delimiter));
    }

    private static String escapeRegExMetaChars(String str) {
        StringBuilder sbld = new StringBuilder();
        int strLength = str.length();
        for (int i = 0; i < strLength; i++) {
            if (isRegExMetaChar(str.charAt(i))) {
                sbld.append("\\");
                sbld.append(str.charAt(i));
            } else {
                sbld.append(str.charAt(i));
            }
        }
        return sbld.toString();
    }

    private static boolean isRegExMetaChar(char chr) {
        boolean rtc;
        switch(chr) {
            case '^': 
                rtc = true;
                break;
            case '.': 
                rtc = true;
                break;
            case '$': 
                rtc = true;
                break;
            case '|': 
                rtc = true;
                break;
            case '*': 
                rtc = true;
                break;
            case '+': 
                rtc = true;
                break;
            case '?': 
                rtc = true;
                break;
            case '-': 
                rtc = true;
                break;
            default: 
                rtc = false;
        }
        return rtc;
    }

    public static String[] trimArray(String[] str) {
        int pos;
        for (pos = str.length - 1; pos >= 0; pos--) {
            if (!"".equals(str[pos])) {
                break;
            }
        }
        String[] result = new String[pos + 1];
        int len = pos + 1;
        for (int j = 0; j < len; j++) {
            result[j] = str[j];
        }

        return result;
    }

    public static boolean isNumeric(String str) {
        boolean rtc = true;
        if ("".equals(str)) {
            rtc = false;
        } else {
            int strLength = str.length();
            for (int i = 0; i < strLength; i++) {
                if (str.charAt(i) < '0' || str.charAt(i) > '9') {
                    rtc = false;
                    break;
                }
            }
        }
        return rtc;
    }

    public static boolean isAlpha(String str) {
        boolean rtc = true;
        if ("".equals(str)) {
            rtc = false;
        } else {
            int strLength = str.length();
            for (int i = 0; i < strLength; i++) {
                if (!isAlphaChar(str.charAt(i))) {
                    rtc = false;
                    break;
                }
            }
        }
        return rtc;
    }

    public static boolean isNumericChar(char chr) {
        return (chr >= '0' && chr <= '9');
    }

    public static boolean isAlphaChar(char chr) {
        return ((chr >= 'A' && chr <= 'Z') || (chr >= 'a' && chr <= 'z'));
    }

    public static boolean isAlphaNumericChar(char chr) {
        return (isAlphaChar(chr) || isNumericChar(chr));
    }

    public static boolean isWhiteSpace(char chr) {
        return (chr == ' ' || chr == '\t' || chr == '\n' || chr == '\r');
    }

    public static String removeNonNumericChars(String str) {
        String rtc;
        if (str == null) {
            rtc = null;
        } else {
            StringBuilder rtnSB = new StringBuilder("");
            int strLength = str.length();
            for (int i = 0; i < strLength; i++) {
                if (str.charAt(i) >= '0' && str.charAt(i) <= '9') {
                    rtnSB.append(str.charAt(i));
                }
            }
            rtc = rtnSB.toString();
        }
        return rtc;
    }

    public static String piece(String str, String delimiter, int pieceNum) {
        String rtc;
        String[] flds = split(str, delimiter);
        if (pieceNum > flds.length) {
            rtc = null;
        } else {
            rtc = flds[pieceNum - 1];
        }
        return rtc;
    }

    public static String piece(String str, int pieceNum) {
        return piece(str, "^", pieceNum);
    }

    public static boolean isEmpty(String str) {
        return str == null || str.length() == 0;
    }

    public static String LPack(String str, int ndigits) {
        int lth = str.length();
        String sLth = Integer.toString(lth);
        int width = sLth.length();
        if (ndigits < width) {
            throw new IllegalArgumentException("Too few digits");
        }
        StringBuilder resultSB = new StringBuilder("000000000");
        resultSB.append(Integer.toString(lth));
        StringBuilder packed = new StringBuilder();
        packed.append(resultSB.substring(resultSB.length() - ndigits));
        packed.append(str);
        return packed.toString();
    }

    public static String SPack(String str) {
        int lth = str.length();
        if (lth > 255) {
            throw new IllegalArgumentException("Parameter exceeds 255 chars");
        }
        StringBuilder sbld = new StringBuilder();
        sbld.append((char) lth);
        sbld.append(str);
        return sbld.toString();
    }

    public static String strPack(String sre, int len) {
        StringBuilder resultSB = new StringBuilder();
        resultSB.append(sre.length());
        while (resultSB.length() < len) {
            resultSB.insert(0, '0');
        }

        resultSB.append(sre);

        return resultSB.toString();
    }

    public static int getFirstWhiteSpaceAfter(String str, int idx) {
        int pos = idx;
        while (pos >= 0 && !isWhiteSpace(str.charAt(pos))) {
            pos--;
        }
        return pos;
    }

    public static String filteredString(String str) {
        StringBuilder resultSB = new StringBuilder("");
        int strLength = str.length();
        for (int i = 0; i < strLength; i++) {
            int chr = asciiAt(str, i);
            if (chr == 9) {
                resultSB.append("        ");
            } else if ((chr >= 32 && chr <= 127) || (chr >= 161 && chr <= 255)) {
                resultSB.append(str.charAt(i));
            } else if (chr >= 128 && chr <= 159) {
                resultSB.append("?");
            } else if (chr == 10 || chr == 13 || chr == 160) {
                resultSB.append(" ");
            }
        }
        return resultSB.toString();
    }

    public static int asciiAt(String str, int idx) {
        char chr = str.charAt(idx);
        return (int) chr;
    }

    public static String prependChars(String str, char chr, int sLen) {
        StringBuilder sbld = new StringBuilder();
        for (int i = 0; i < sLen; i++) {
            sbld.append(chr);
        }
        String chars = sbld.toString();
        sbld.setLength(0);
        sbld.append(CSubstring(chars, 0, sLen - str.length()));
        sbld.append(str);
        return sbld.toString();
    }

    public static int getIdx(String[] lines, String target, int startingIdx) {
        int rtc = -1;
        for (int i = startingIdx; i < lines.length; i++) {
            if (lines[i].startsWith(target)) {
                rtc = i;
                break;
            }
        }
        return rtc;
    }

    public static String trimTrailingZeroes(String str) {
        int pos = str.length() - 1;
        while (str.charAt(pos) == '0' && pos > 0) {
            pos--;
        }
        return CSubstring(str, 0, pos + 1);
    }

    public static String reverse(String str) {
        StringBuilder result = new StringBuilder("");
        for (int i = str.length() - 1; i >= 0; i--) {
            result.append(str.charAt(i));
        }

        return result.toString();
    }

    /**
     * Return a string with all invalid XML 1.0 characters
     * (per W3C standard: http://www.w3.org/TR/REC-xml/#charsets) removed
     */
    public static String stripInvalidXmlCharacters(String str) {
        String rtc;
        if (str == null || "".equals(str)) {
            rtc = "";
        } else {
            StringBuilder sbld = new StringBuilder();
            int strLength = str.length();
            for (int i = 0; i < strLength; i++) {
                char current = str.charAt(i);
                if (current == 0x9 || current == 0xA || current == 0xD
                    || ((current >= 0x20) && (current <= 0xD7FF))
                    || ((current >= 0xE000) && (current <= 0xFFFD))
                    || ((current >= 0x10000) && (current <= 0x10FFFF))) {
                    sbld.append(current);
                }
            }
            rtc = sbld.toString();
        }
        return rtc;
    }

    /**
     * Returns a new string that is a substring of str.
     * Function is a C# porting utility.
     * @param str Source string
     * @param beginIndex The beginning index, inclusive.
     * @param length The number of characters in the substring. 
     * @return a new string that is a substring of str.
     */
    public static String CSubstring(String str, int beginIndex, int length) {
        return str.substring(beginIndex, beginIndex + length);
    }
}
