/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.rpc;

import java.lang.reflect.Field;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 *
 *
 */
public class VistaFile {
    private String fileName;
    private String fileNum;
    private String global;
    private String mdoName;
    private DictionaryHashList fields;

    public DictionaryHashList getFields() {
        return fields;
    }

    public void setFields(DictionaryHashList fields) {
        this.fields = fields;
    }

    public String getFileName() {
        return fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public String getFileNum() {
        return fileNum;
    }

    public void setFileNum(String fileNum) {
        this.fileNum = fileNum;
    }

    public String getGlobal() {
        return global;
    }

    public void setGlobal(String global) {
        this.global = global;
    }

    public String getMdoName() {
        return mdoName;
    }

    public void setMdoName(String mdoName) {
        this.mdoName = mdoName;
    }

    public String getFieldString() {
        StringBuilder resultSB = new StringBuilder("");
        int fieldsSize = fields.size();
        for (int i = 0; i < fieldsSize; i++) {
            KeyValuePair kvp = fields.get(i);
            resultSB.append(((VistaField) kvp.getValue()).getVistaNum() + ';');
        }
        if (!"".equals(resultSB.toString())) {
            resultSB = new StringBuilder(StringUtils.CSubstring(resultSB.toString(), 0, resultSB.length() - 1));
        }
        return resultSB.toString();
    }

    public LinkedHashMap<String, Object> toMdo(RPCConnection cxn,
                                               String[] response, String mdoName) throws RPCException {
        LinkedHashMap<String, Object> rtc;
        if (response == null || response.length == 0) {
            rtc = null;
        } else {
    
            try {
                LinkedHashMap<String, Object> result = new LinkedHashMap<String, Object>(response.length);
                for (int lineIdx = 0; lineIdx < response.length; lineIdx++) {
                    //Need to instantiate the mdo here
                    Object theMdo = Class.forName(mdoName, true, Thread.currentThread().getContextClassLoader()).newInstance();
                    //Object theMdo = Activator.CreateInstance(Type.GetType(mdoName), new object[] { });
                    //Type theClass = theMdo.GetType();
                    Class theClass = theMdo.getClass();
                    String[] flds = StringUtils.split(response[lineIdx], StringUtils.CARET);
                    for (int fldIdx = 0; fldIdx < flds.length; fldIdx++) {
                        VistaField vfld = (VistaField) (fields.get(fldIdx)).getValue();
    
                        Field theField = theClass.getField(vfld.getMdoName());
    
                        //FieldInfo theField = theClass.GetField(vf.MdoName,BindingFlags.NonPublic | BindingFlags.Instance);
    
                        if ("string".equals(vfld.getMdoType())) {
                            if (vfld.getMapping() != null && "decode".equals(vfld.getMapping().getType()))
                            {
                                if (vfld.getMapping().getDecodeMap().containsKey(flds[fldIdx])) {
                                    theField.set(theMdo,  vfld.getMapping().getDecodeMap().get(flds[fldIdx]));
                                } else {
                                    theField.set(theMdo, flds[fldIdx]);
                                }
                            } else {
                                theField.set(theMdo, flds[fldIdx]);
                            }
                            //TODO: shouldnt this be getMdoType? decode below won't be true....
//                        } else if ("kvp".equals(vfld.getMapping().getType())) {
                        } else if ("kvp".equals(vfld.getMdoType())) {
                            String key = flds[fldIdx];
                            String value = "";
                            if (vfld.getMapping() != null && "decode".equals(vfld.getMapping().getType())) {
                                value = vfld.getMapping().getDecodeMap().get(key);
                            } else {
                                Map<String, String> lookupTbl = cxn.getLookupTable(vfld.getMapping().getVistaFileNum());
                                if (lookupTbl.containsKey(key)) {
                                    value = lookupTbl.get(key);
                                }
                            }
                            theField.set(theMdo, new KeyValuePair<String, String>(key, value));
                            //KeyValuePair<string, string> kvp = new KeyValuePair<string, string>(key, value);
                        }
                    }
                    result.put(flds[0], theMdo);
                }
                rtc = result;
            } catch (ClassNotFoundException cnfe) {
                throw new RPCException(cnfe);
            } catch (NoSuchFieldException nsfe) {
                throw new RPCException(nsfe);
            } catch (InstantiationException ie) {
                throw new RPCException(ie);
            } catch (IllegalAccessException iac) {
                throw new RPCException(iac);
            }
        }
        return rtc;
    }
}
