package gov.va.med.vds.dataservicehandler;

import gov.hhs.fha.nhinc.common.nhinccommonentity.RespondingGatewayCrossGatewayQueryRequestType;
import gov.hhs.fha.nhinc.entitydocquery.EntityDocQuery;
import gov.hhs.fha.nhinc.entitydocquery.EntityDocQueryPortType;
import gov.va.med.vds.common.NHINHandlerException;
import gov.va.med.vds.vistabeans.Author;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.VLERDocument;
import gov.va.med.vhahon.common.Utils;

import java.math.BigInteger;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;

import oasis.names.tc.ebxml_regrep.xsd.query._3.AdhocQueryRequest;
import oasis.names.tc.ebxml_regrep.xsd.query._3.AdhocQueryResponse;
import oasis.names.tc.ebxml_regrep.xsd.query._3.ResponseOptionType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.AdhocQueryType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.ClassificationType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.ExternalIdentifierType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.ExtrinsicObjectType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.IdentifiableType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.ObjectRefType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.RegistryObjectListType;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.SlotType1;
import oasis.names.tc.ebxml_regrep.xsd.rim._3.ValueListType;
import oasis.names.tc.ebxml_regrep.xsd.rs._3.RegistryError;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created with IntelliJ IDEA.
 * User: melcolm
 * Date: 5/21/14
 * Time: 1:57 PM
 * To change this template use File | Settings | File Templates.
 */
public class AdapterDocQueryHandler extends AdapterHandler {

    private static final Logger LOGGER = LoggerFactory.getLogger(AdapterDocQueryHandler.class);
    private EntityDocQuery docQueryService;

    public AdapterDocQueryHandler(String url) {
        super();

        serviceURL = url;
        serviceName = "AdapterDocQuery";
        serviceNamespace = "http://adapterdocquery.nhinc.fha.hhs.gov";
        qServiceName = new QName(serviceNamespace, serviceName);
    }

    private EntityDocQueryPortType getAdapterDocQueryPort() throws MalformedURLException, ConnectException {
        if (docQueryService == null) {
            createService();
        }

        EntityDocQueryPortType port = null;

//        if (includeMTOMFeature)
//        {
//            port = docQueryService.getEntityDocQueryPortSoap11(new MTOMFeature());
//        }
//        else
        port = docQueryService.getEntityDocQueryPortSoap11();

        //set port endpoint address
        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);

        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        if ("test".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()) || "development".equalsIgnoreCase(APP_CONFIG.getAppEnvironment())) {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    private void createService() throws NHINHandlerException {
        if (docQueryService == null) {
            synchronized (this) {
                try {
                    docQueryService = new EntityDocQuery();
                } catch (Exception e) {
                    throw new NHINHandlerException(e);
                }
            }
        }
    }

    public List<VLERDocument> documentQuery(QueryBean qbean) throws NHINHandlerException {
        List<VLERDocument> output = new ArrayList<VLERDocument>();
        try {

            RespondingGatewayCrossGatewayQueryRequestType qrt = generateDocumentQuery(qbean);
            AdhocQueryResponse aqr = getAdapterDocQueryPort().respondingGatewayCrossGatewayQuery(qrt);
            RegistryObjectListType rolt = aqr.getRegistryObjectList();

            ObjectRefType objectRef;
            ExtrinsicObjectType extrinsicObject;

            //checking to see if results are empty, and errors exist
            if ((aqr.getTotalResultCount() != null)
                    && (aqr.getTotalResultCount().intValue() == 0)
                    && (aqr.getRegistryErrorList() != null)) {
                for (RegistryError re : aqr.getRegistryErrorList().getRegistryError()) {
                    if (re == null) {
                        String name = "There was an error querying for documents";
                        String source = "Connection Unavailable";
                        String details = "The server returned an empty/invalid response";
                        LOGGER.error("VLER Document Query Error" + " :: " + "The server returned an empty/invalid response.");

                        output.add(generateErrorDoc(name, source, details));
                    } else {
                        output.add(generateErrorDoc(re.getErrorCode(), "Error: " + re.getCodeContext(), re.getValue()));
                        LOGGER.error(re.getErrorCode() + " :: " + re.getValue());
                    }
                }

            } else if (aqr.getTotalResultCount() == null
                    && aqr.getStatus() != null
                    && aqr.getStatus().indexOf("Success") > 0) {
                String name = "There are no documents available for this patient.";
                String source = "None";
                String details = "The server returned an empty response; most likely no patient correlation.";

                LOGGER.error("VLER Document Query Error" + " :: " + "The server returned an empty response; most likely no patient correlation.");
                output.add(generateErrorDoc(name, source, details));
            } else if (aqr.getTotalResultCount() == null) {
                String name = "There was an error querying for documents.";
                String source = "Connection Unavailable";
                String details = "The server returned an empty/invalid response";

                LOGGER.error("VLER Document Query Error" + " :: " + "The server returned an empty/invalid response.");
                output.add(generateErrorDoc(name, source, details));
            }


            if (rolt != null) {

                List<JAXBElement<? extends IdentifiableType>> jbList = rolt.getIdentifiable();

                for (JAXBElement<? extends IdentifiableType> jaxb : jbList) {
                    if (jaxb.getValue() instanceof ObjectRefType) {
                        objectRef = (ObjectRefType) jaxb.getValue();

                        if (objectRef != null
                            && (objectRef.getHome() == null || objectRef.getHome().isEmpty())) {
                            objectRef.setHome("test");
                        }
                    }
                    if (jaxb.getValue() instanceof ExtrinsicObjectType) {
                        extrinsicObject = (ExtrinsicObjectType) jaxb.getValue();

                        if (extrinsicObject != null
                            && (extrinsicObject.getHome() != null && !extrinsicObject.getHome().isEmpty())) {

                            //extrinsicObject.setHome("TEST");

                            VLERDocument vDoc = new VLERDocument();
                            vDoc.setDocumentUniqueId("");
                            vDoc.setHomeCommunityId(extrinsicObject.getHome());
                            vDoc.setRepositoryUniqueId("");
                            vDoc.setName(extrinsicObject.getName().getLocalizedString().get(0).getValue());

                            for (SlotType1 st1 : extrinsicObject.getSlot()) {
                                if (SOURCE_PATIENT_ID.equalsIgnoreCase(st1.getName())) {
                                    vDoc.setSourcePatientId(st1.getValueList().getValue().get(0));
                                } else if (REPOSITORY_UNIQUE_ID.equalsIgnoreCase(st1.getName())) {
                                    vDoc.setRepositoryUniqueId(st1.getValueList().getValue().get(0));
                                } else if (CREATION_TIME.equalsIgnoreCase(st1.getName())) {
                                    vDoc.setCreationTime(Utils.getCal(st1.getValueList().getValue().get(0)));
                                }
                            }

                            for (ExternalIdentifierType it : extrinsicObject.getExternalIdentifier()) {
                                if (DOCUMENT_UNIQUE_ID.equalsIgnoreCase(it.getName().getLocalizedString().get(0).getValue())) {
                                    vDoc.setDocumentUniqueId(it.getValue());
                                }
                            }

                            List<Author> authorList = new ArrayList<Author>();
                            for (ClassificationType ct : extrinsicObject.getClassification()) {
                                //author info
                                if (AUTHOR_INFORMATION.equals(ct.getClassificationScheme())) {
                                    Author docAuth = new Author();
                                    for (SlotType1 st : ct.getSlot()) {
                                        if (!st.getValueList().getValue().isEmpty()) {

                                            if (AUTHOR_PERSON.equalsIgnoreCase(st.getName())) {
                                                docAuth.setName(st.getValueList().getValue().get(0));
                                            } else if (AUTHOR_INSTITUTION.equalsIgnoreCase(st.getName())) {
                                                docAuth.setInstitution(st.getValueList().getValue().get(0));
                                            }

                                        }
                                    }
                                    authorList.add(docAuth);

                                } 
//                                else if (CLASS_CODE.equals(ct.getClassificationScheme())) {
//
//                                }
                            }
                            vDoc.setAuthorList(authorList);

                            output.add(vDoc);
                        }
                    }
                }
            }
//
// List<JAXBElement<? extends IdentifiableType >> jbe = rolt.getIdentifiable();

            //print doc query response
            if ("test".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()) || "development".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()))
            {
                JAXBContext jaxbContext = JAXBContext.newInstance(AdhocQueryResponse.class.getPackage().getName());
                Marshaller jaxbMarshaller = jaxbContext.createMarshaller();

                // for getting nice formatted output
                jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
                jaxbMarshaller.marshal(aqr, System.out);
            }
        } catch (RuntimeException e) {
            if (e.getCause() instanceof ConnectException) {
                output.add(generateErrorDoc("Connectivity Error", "Connection Unavailable", e.getLocalizedMessage()));
                LOGGER.error("VLER Document Query Error" + " :: " + e.getMessage());
//                return output;
            } else {
                throw new NHINHandlerException(e);
            }
        } catch (Exception e) {
            LOGGER.error("Error performing VLER Document Query: " + e.getMessage());
            throw new NHINHandlerException(e);
        }

        return output;
    }

    private VLERDocument generateErrorDoc(String name, String source, String detail) {
        VLERDocument vDoc = new VLERDocument();
        vDoc.setName("Error: " + name);
        vDoc.setHomeCommunityId(detail);
        Author author = new Author();
        author.setInstitution(source);
        List<Author> authorList = new ArrayList<Author>();
        authorList.add(author);
        vDoc.setAuthorList(authorList);

        return vDoc;
    }

    private RespondingGatewayCrossGatewayQueryRequestType generateDocumentQuery(QueryBean qbean) {
        RespondingGatewayCrossGatewayQueryRequestType qrt = new RespondingGatewayCrossGatewayQueryRequestType();

        AdhocQueryRequest aqr = new AdhocQueryRequest();

        try {

            aqr.setId("JLV_" + System.currentTimeMillis());
            aqr.setStartIndex(BigInteger.valueOf(0));
            aqr.setMaxResults(BigInteger.valueOf(-1));

            //set pat ICN
            SlotType1 stPatICN = new SlotType1();
            ValueListType vlt = new ValueListType();
            vlt.getValue().add("'" + qbean.getPatient().getICN() + PATIENT_ICN_STRING + "'");
            stPatICN.setName(SLOT_PATIENT_ID);
            stPatICN.setValueList(vlt);

            //set entry class code
            SlotType1 stClassCode = new SlotType1();
            stClassCode.setName(SLOT_CLASS_CODE);
            ValueListType vaList = new ValueListType();
            vaList.getValue().add(C32_CLASS_CODE);
            stClassCode.setValueList(vaList);

            //set entry status
            SlotType1 stEntryStatus = new SlotType1();
            stEntryStatus.setName(SLOT_STATUS);
            ValueListType vListType = new ValueListType();
            vListType.getValue().add(SLOT_STATUS_STRING);
            stEntryStatus.setValueList(vListType);

    //        //set format code
    //        SlotType1 stFormatCode = new SlotType1();
    //        stFormatCode.setName(SLOT_FORMAT_CODE);
    //        ValueListType vListTypeCode = new ValueListType();
    //        vListTypeCode.getValue().add("('urn:ihe:pcc:xphr:2007')");
    //        stFormatCode.setValueList(vListTypeCode);

            ResponseOptionType rot = new ResponseOptionType();
            rot.setReturnType(RETURN_TYPE);
            rot.setReturnComposedObjects(true);
            aqr.setResponseOption(rot);

            AdhocQueryType aqt = new AdhocQueryType();
            aqt.setHome(VA_HOME_ID);
            aqt.getSlot().add(stClassCode);
            aqt.getSlot().add(stPatICN);
            aqt.getSlot().add(stEntryStatus);
    //        aqt.getSlot().add(stFormatCode);
            aqt.setId(FIND_DOCUMENTS);
            aqr.setAdhocQuery(aqt);

            qrt.setAdhocQueryRequest(aqr);
            qrt.setAssertion(generateAssertion(qbean));

//print doc query request
            // create JAXB context and initializing Marshaller
//            JAXBContext jaxbContext = JAXBContext.newInstance(RespondingGatewayCrossGatewayQueryRequestType.class.getPackage().getName());
//            Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
//
//            // for getting nice formatted output
//            jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
//            jaxbMarshaller.marshal(new JAXBElement<RespondingGatewayCrossGatewayQueryRequestType>(new QName("uri","RespondingGateway_CrossGatewayQueryRequest"), RespondingGatewayCrossGatewayQueryRequestType.class, qrt), System.out);

            //System.out.print(qrt);
        } catch (Exception e) {
            throw new NHINHandlerException(e);
        }
        return qrt;
    }
}
