/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.patient.dao.rpc.AllergyRPC;
import gov.va.med.vds.vistabeans.Allergy;
import gov.va.med.vds.vistabeans.AllergyDetail;
import gov.va.med.vds.vistabeans.QueryBean;

import java.util.Calendar;

/**
 *
 *
 */
public class AllergyDao extends DaoBase {
    public AllergyDao(RPCConnection cxn) {
        super(cxn);
    }

    /**
     * Retrieves list of patient allergies.
     *
     * @param queryBean QueryBean.
     * @return List of patient allergies.
     * @throws RPCException if there was an error..
     */
    public Allergy[] getPatientAllergies(QueryBean queryBean) throws RPCException {
        AllergyRPC rpc = new AllergyRPC(cxn);
//        return rpc.getAllergies();

        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -200);
        return rpc.getAllergiesVPR(startDate, endDate);
    }

    /**
     * Retrieves patient allergy details.
     *
     * @param allergyIen Internal entry number of the patient allergy file.
     * @return AllergyDetail containing allergy report text.
     * @throws RPCException if there was an error..
     */
    public AllergyDetail getAllergyDetail(String allergyIen) throws RPCException {
        AllergyRPC rpc = new AllergyRPC(cxn);
        return rpc.getAllergyDetail(allergyIen);
    }
}
