/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.patient.dao.rpc.PatientRPC;
import gov.va.med.vds.patient.dataservice.PatientDataService;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.PatientDemographics;
import gov.va.med.vds.vistabeans.PatientDemographicsDetail;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;

import com.intersys.objects.CacheException;

/**
 *
 *
 */
public class PatientDao extends DaoBase {

    public PatientDao(RPCConnection cxn) {
        super(cxn);
    }

    public Patient[] lookupPatient(QueryBean queryBean) throws CacheException, RPCException {
        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.match(queryBean.getItemId());
    }

    public Patient selectPatient() throws CacheException, RPCException {
        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.select(cxn.getPatientIen());
    }

    public Boolean issueSensitivePatientBulletin() throws RPCException {
        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.issueSensitivePatientBulletin(cxn.getPatientIen());
    }

    public PatientDemographics getPatientDemographics(QueryBean queryBean) throws CacheException, RPCException {
        //SB: no longer using cache demographics routine due cache server load issues

        PatientDemographics demo = null;
        PatientRPC rpc = new PatientRPC(cxn);
        try {
            demo = rpc.getPatientDemographicsVPR(queryBean.getUser());
        } catch (RPCException e) {
            if ("No such patient".equalsIgnoreCase(e.getMessage())) {
                demo = createUnknownPatientBean(cxn.getSite(), PROTOCOL_RPC/*, AGENCY_VA*/);
            } else {
                throw e;
            }
        }

        return demo;
    }

    private PatientDemographics createUnknownPatientBean(Site site,
                                                         String protocol/*, String agency*/) {
        PatientDemographics demo = new PatientDemographics();
        demo.setName(PatientDataService.UNKNOWN_PATIENT_MSG);
        demo.setAddress1("");
        demo.setAddress2("");
        demo.setAge("");
        demo.setCity("");
        demo.setClinic("");
        demo.setDob("");
        demo.setGender("");
//        demo.setPatientId("");
        demo.setPrimaryProvider("");
        demo.setPhone1("");
        demo.setPhone2("");
        demo.setSSN("");
        demo.setState("");
        demo.setZipCode("");
        demo.setCodeGreen("0");
        demo.setSite(site);
        demo.setSourceProtocol(protocol);

        return demo;
    }

    public PatientDemographicsDetail getPatientDemographicsDetail(QueryBean queryBean) throws CacheException, RPCException {

        PatientRPC rpc = new PatientRPC(cxn);
        return rpc.getPatientDemographicsDetail();
    }

    /**
     * Determines if a patient is flagged as sensitive in VistA.
     *
     * @param queryBean Required queryBean params: User, patient
     * @return true if a patient is flagged as a sensitive in VistA.
     * @throws gov.va.med.vds.base.dataservice.VDSException
     *          if an error has occurred.
     */
    public Boolean isSensitivePatient(QueryBean queryBean) throws RPCException {
        PatientRPC patientRPC = new PatientRPC(cxn);
        return patientRPC.isSensitivePatient();
    }
}
