/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.RPCUtils;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.Timestamp;
import gov.va.med.rpc.VistaQuery;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.RPCUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.Code;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.Medication;
import gov.va.med.vds.vistabeans.PrescriptionFill;
import gov.va.med.vds.vistabeans.Provider;
import gov.va.med.vhahon.common.Utils;

import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.*;


/**
 *
 *
 */
public class MedicationsRPC extends BaseRPC {

    private static final Logger LOGGER = LoggerFactory.getLogger(MedicationsRPC.class);

    private static final String INPATIENT_SORT = "I";
    private static final String OUTPATIENT_SORT = "O";
//    private static final String ALL_SORT = "A";

    public MedicationsRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Medication[] getOutpatientMedications(Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(startDate, endDate, OUTPATIENT_SORT);
    }

    //SB: Missing refills
    public Medication[] getOutpatientMedications(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(patientIen, startDate, endDate, OUTPATIENT_SORT);
    }

    public Medication[] getInpatientMedications(Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(startDate, endDate, INPATIENT_SORT);
    }

    public Medication[] getInpatientMedications(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        return getMedicationsVPR(patientIen, startDate, endDate, INPATIENT_SORT);
    }


    public Medication[] getNonVAMeds(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        Medication[] rtc;
        String request = buildGetNonVAMedsRequest(patientIen);
        String response = (String) cxn.query(request);
        Medication[] meds = toNonVAMeds(response/*, patientIen*/);
        if (meds == null) {
            rtc = new Medication[0];
        } else {
            rtc = filterMedsByDate(meds, startDate, endDate);
        }
        return rtc;
    }

    private Medication[] filterMedsByDate(Medication[] meds, Calendar startDate, Calendar endDate) {
        ArrayList<Medication> medsFiltered = new ArrayList<Medication>();

        //filter by date range
        for (Medication med : meds) {
            Calendar medDate = med.getOrderDate();

            if (medDate.compareTo(startDate) >= 0 && medDate.compareTo(endDate) <= 0) {
                medsFiltered.add(med);
            }
        }
        Medication[] rtc;
        if (medsFiltered.isEmpty()) {
            rtc = new Medication[0];
        } else {
            rtc = medsFiltered.toArray(new Medication[0]);

        }
        return rtc;
    }

    private String buildGetNonVAMedsRequest(String patientIen) {
        return RPCUtils.buildReportTextRequest(vistaQueryFactory, patientIen, "", "", 0, "OR_RXN:HERBAL/OTC/NON-VA MEDS~NVA;ORDV06A;0;");
    }

    private Medication[] toNonVAMeds(String response/*, String patientIen*/) {
        Medication[] rtc;
        if ("".equals(response)) {
            rtc = null;
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<Medication> list = new ArrayList<Medication>();
            Medication rec = null;
            StringBuilder sig = new StringBuilder();
//            String dose = "";
            StringBuilder comment = new StringBuilder();
            for (int i = 0; i < lines.length; i++) {
                if ("".equals(lines[i])) {
                    continue;
                }
                String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
                int fldnum = Integer.parseInt(flds[0]);
                switch (fldnum) {
                    case 1:
                        if (rec != null) {
                            rec.setComment(comment.toString());
                            rec.setSigCode(sig.toString());
                            list.add(rec);
                        }
                        rec = new Medication();
                        //rec.IsOutpatient = true;
                        //rec.IsNonVA = true;
                        //rec.Type = "NV";
                        rec.setMedType("O");
                        sig.setLength(0);
//                        dose = "";
                        comment.setLength(0);
                        fillCommonDataBeanValues(rec);
                        break;
                    case 2:
                        if (flds.length == 2 && rec != null) {
                            rec.setDrugName(flds[1]);
                        }
                        break;
                    case 3:
                        if (flds.length == 2 && rec != null) {
                            rec.setActive(flds[1]);
                        }
                            //rec.Status = flds[1];
                        break;
                    case 4:
//                        if (flds.length == 2) {
//                            dose = flds[1];
//                            //rec.Dose = flds[1];
//                        }
                        break;
                    case 5:
                        if (flds.length == 2 && rec != null) {
                            rec.setOrderDate(Utils.getCal(Timestamp.toUtcFromRdv(flds[1])));
                        }
                            //rec.DateDocumented = VistaTimestamp.toUtcFromRdv(flds[1]);
                        break;
                    case 6:
                        if (flds.length == 2 && rec != null) {
                            Provider orderingProvider = new Provider();
                            orderingProvider.setName(flds[1]);
                            rec.setOrderingProvider(orderingProvider);
                        }
                            //rec.Documentor = new Author("",flds[1],"");
                        break;
                    case 7:
                        if (flds.length == 2 && rec != null) {
                            rec.setStopDate(Utils.getCal(Timestamp.toUtcFromRdv(flds[1])));
                        }
                            //rec.StopDate = flds[1];
                        break;
                    case 8:
                        if (flds.length == 2) {
                            sig.append(flds[1]);
                            sig.append('\n');
                        }
                        break;
                    case 10:
                        if (flds.length == 2) {
                            comment.append(flds[1]);
                            comment.append('\n');
                        }
                        break;
                    default:
                        LOGGER.error("Invalid length");
                        break;
                }
            }
            if (rec != null) {
                if (comment.length() > 0) {
                    rec.setComment(comment.toString());
                }
                if (sig.length() > 0) {
                    rec.setSigCode(sig.toString());
                }
                list.add(rec);
            }
            rtc = (Medication[]) list.toArray(new Medication[0]);
        }
        return rtc;
    }

    /**
     * Returns report text for the given medID using ORWPS DETAIL using the patientIen associated with the connection
     *
     * @param medID Medication Identifier that dynamically points to the proper pharmacy file depending on the type of medication.
     * @return Report Formatted Data
     * @throws RPCException
     */
    public FreeTextReport getMedicationDetail(String medID) throws RPCException {
        return getMedicationDetail(cxn.getPatientIen(), medID);
    }

    /**
     * Returns report text for the given patientIen and medID using ORWPS DETAIL.
     *
     * @param patientIen Patient data file number
     * @param medID      Medication Identifier that dynamically points to the proper pharmacy file depending on the type of medication.
     * @return Report Formatted Data
     * @throws RPCException
     */
    public FreeTextReport getMedicationDetail(String patientIen, String medID) throws RPCException {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPS DETAIL");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, medID);
        String request = vquery.buildMessage();

        String reportText = (String) cxn.query(request);
        FreeTextReport freeTextReport = new FreeTextReport();
        freeTextReport.setReportText(reportText);
        fillCommonDataBeanValues(freeTextReport);

        return freeTextReport;
    }

    /**
     * Uses the VPR RPC with "meds" as the type.
     *
     * @param startDate The start of the date range for the list of problems
     * @param endDate   The end of the date range for the list of problems
     * @return The list of medications for the specified date range
     * @throws RPCException
     */
    public Medication[] getMedicationsVPR(Calendar startDate, Calendar endDate, String inptOrOutpt) throws RPCException {
        return getMedicationsVPR(cxn.getPatientIen(), startDate, endDate, inptOrOutpt);
    }

    /**
     * Uses the VPR RPC with "meds" as the type.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of problems
     * @param endDate    The end of the date range for the list of problems
     * @return The list of medications for the specified date range
     * @throws RPCException
     */
    public Medication[] getMedicationsVPR(String patientIen, Calendar startDate, Calendar endDate, String inptOrOutpt) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.MEDS);
        String response = (String) cxn.query(request);

        Medication[] meds = parseMedsFromVPR(/*patientIen, */response, inptOrOutpt);
        Medication[] dateFilteredMeds;
        if (meds != null && meds.length > 0) {
            dateFilteredMeds = filterMedsByDate(meds, startDate, endDate);
        } else {
            dateFilteredMeds = meds;
        }


        return dateFilteredMeds;
    }

    private Medication[] parseMedsFromVPR(/*String patientIen, */String response, String inptOrOutpt) throws RPCException {
        Medication[] rtc;
        if ("".equals(response)) {
            rtc = new Medication[0];
        } else {
    
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            List<Medication> medsList = new ArrayList<Medication>();
    
            final String XPATH_MED = "//med";
    
    
            Document doc = null;
            XPathExpression exprMed = null;
            try {
                doc = parseXMLDocument(sbld.toString());
    
                XPathFactory xPathfactory = XPathFactory.newInstance();
                XPath xpath = xPathfactory.newXPath();
                exprMed = xpath.compile(XPATH_MED);
    
    
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList medsNodeList = null;
            try {
                medsNodeList = (NodeList) exprMed.evaluate(doc, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }
    
            List<VPRMedsParser> medsParserList = new ArrayList<VPRMedsParser>();
            int medsNodeListLength = medsNodeList.getLength();
            for (int medNum = 0; medNum < medsNodeListLength; medNum++) {
                Node medNode = medsNodeList.item(medNum);
                medNode.getParentNode().removeChild(medNode);
    
                medsParserList.add(new VPRMedsParser(medNode, inptOrOutpt));
            }
    
            ExecutorService executorService = Executors.newFixedThreadPool(5);
            List<Future<Medication>> futures = null;
            try {
                futures = executorService.invokeAll(medsParserList, 2, TimeUnit.MINUTES);
    
                if (futures == null) {
                    throw new RPCException("Failed to parse medications results : accession node parser futures list is null.");
                }
    
            } catch (InterruptedException e) {
                logErrorMessage(e);
                throw new RPCException(e);
            } finally {
                executorService.shutdown();
            }
    
            Iterator<Future<Medication>> futuresItr = futures.iterator();
    
            while (futuresItr.hasNext()) {
                try {
                    Future<Medication> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Medication Data");
                    } else {
                        Medication med = future.get();
    
                        //exclude null results
                        if (med != null) {
                            medsList.add(med);
                        }
                    }
    
                } catch (InterruptedException e) {
                    logErrorMessage(e);
                } catch (ExecutionException e) {
                    logErrorMessage(e);
                }
            }
    
            rtc = medsList.toArray(new Medication[0]);
        }
        return rtc;
    }

    private void logErrorMessage(Exception except) {
        LOGGER.error(except.getMessage(), except);
    }

    private class VPRMedsParser implements Callable<Medication> {
        String inptOrOutpt;
        private Node medNode;

        public VPRMedsParser(Node medNode, String inptOrOutpt) {
            this.medNode = medNode;
            this.inptOrOutpt = inptOrOutpt;
        }

        public Medication call() throws RPCException {
            Medication rtc;
            final String XPATH_DAYS_SUPPLY = "./daysSupply/@value";                 // O
            final String XPATH_FILLS_REMAINING = "./fillsRemaining/@value";         // O
            final String XPATH_LAST_FILLED = "./lastFilled/@value";                 // O
            final String XPATH_MED_ID = "./medID/@value";
            final String XPATH_NAME = "./name/@value";
            final String XPATH_ORDER_ID = "./orderID/@value";
            final String XPATH_ORDERED = "./ordered/@value";

            final String XPATH_ORDERING_PROVIDER = "./orderingProvider";
            final String XPATH_CURRENT_PROVIDER = "./currentProvider";

            final String XPATH_CODE = "./@code";
            final String XPATH_NAME_ATTRIB = "./@name";
            final String XPATH_TAXONOMY_CODE = "./@taxonomyCode";
            final String XPATH_PROVIDER_TYPE = "./@providerType";
            final String XPATH_CLASSIFICATION = "./@classification";
            final String XPATH_SPECIALIZATION = "./@specialization";
            final String XPATH_SERVICE = "./@service";
            final String XPATH_OFFICE_PHONE = "./@officePhone";
            final String XPATH_FAX = "./@fax";
            final String XPATH_ANALOG_PAGER = "./@analogPager";
            final String XPATH_EMAIL = "./@email";

            final String XPATH_PHARMACIST_CODE = "./pharmacist/@code";              // O, I, V
            final String XPATH_PHARMACIST_NAME = "./pharmacist/@name";
            final String XPATH_PRESCRIPTION = "./prescription/@value";              // O
            final String XPATH_PT_INSTRUCTIONS = "./ptInstructions";                // O
            final String XPATH_QUANTITY = "./quantity/@value";                      // O
            final String XPATH_SIG = "./sig";                                       // O, N, I
            final String XPATH_RATE = "./rate/@value";                                       // V
            final String XPATH_START = "./start/@value";
            final String XPATH_STOP = "./stop/@value";                     // O "Prescription", N "OTC"
            final String XPATH_VA_STATUS = "./vaStatus/@value";                     // "ACTIVE, DISCONTINUED, EXPIRED"
            final String XPATH_VA_TYPE = "./vaType/@value";                         // "O, N, I, V"

            final String XPATH_DOSES = "./doses";
            final String XPATH_DOSE = "./dose";
            final String XPATH_DOSE_ROUTE = "./@route";
            final String XPATH_DOSE_SCHEDULE = "./@schedule";
            final String XPATH_FILLS = "./fills";
            final String XPATH_FILL = "./fill";
            final String XPATH_FILL_FILL_DATE = "./@fillDate";
            final String XPATH_FILL_FILL_ROUTING = "./@fillRouting";
            final String XPATH_FILL_RELEASE_DATE = "./@releaseDate";
            final String XPATH_FILL_FILL_QUANTITY = "./@fillQuantity";
            final String XPATH_FILL_FILL_DAYS_SUPPLY = "./@fillDaysSupply";
            final String XPATH_FILL_PARTIAL = "./@partial";

            final String XPATH_PRODUCTS = "./products";
            final String XPATH_PRODUCT = "./product";
            final String XPATH_PRODUCT_NAME = "./@name";
            final String XPATH_PRODUCT_VA_PRODUCT_NAME = "./vaProduct/@name";
            final String XPATH_PRODUCT_VA_PRODUCT_VUID = "./vaProduct/@vuid";


            XPathExpression exprDaysSupply = null;
            XPathExpression exprFillsRemaining = null;
            XPathExpression exprLastFilled = null;
            XPathExpression exprMedId = null;
            XPathExpression exprName = null;
            XPathExpression exprOrderId = null;
            XPathExpression exprOrdered = null;
            XPathExpression exprNameAttrib = null;
            XPathExpression exprOrderingProvider = null;
            XPathExpression exprCurrentProvider = null;
            XPathExpression exprCode = null;
            XPathExpression exprTaxonomyCode = null;
            XPathExpression exprProviderType = null;
            XPathExpression exprClassification = null;
            XPathExpression exprOfficePhone = null;
            XPathExpression exprFax = null;
            XPathExpression exprAnalogPager = null;
            XPathExpression exprEmail = null;
            XPathExpression exprSpecialization = null;
            XPathExpression exprService = null;
            XPathExpression exprPharmacistCode = null;
            XPathExpression exprPharmacistName = null;
            XPathExpression exprPrescription = null;
            XPathExpression exprPtInstructions = null;
            XPathExpression exprQuantity = null;
            XPathExpression exprSig = null;
            XPathExpression exprRate = null;
            XPathExpression exprStart = null;
            XPathExpression exprStop = null;
            XPathExpression exprVaStatus = null;
            XPathExpression exprVaType = null;
            XPathExpression exprDose = null;
            XPathExpression exprDoses = null;
            XPathExpression exprDoseRoute = null;
            XPathExpression exprDoseSchedule = null;
            XPathExpression exprFills = null;
            XPathExpression exprFill = null;
            XPathExpression exprFillFillDate = null;
            XPathExpression exprFillFillRouting = null;
            XPathExpression exprFillReleaseDate = null;
            XPathExpression exprFillFillDaysSupply = null;
            XPathExpression exprFillPartial = null;
            XPathExpression exprFillFillQuantity = null;
            XPathExpression exprProducts = null;
            XPathExpression exprProduct = null;
            XPathExpression exprProductName = null;
            XPathExpression exprProductVaProductName = null;
            XPathExpression exprProductVaProductVuid = null;
            XPathFactory xPathfactory = XPathFactory.newInstance();
            XPath xpath = xPathfactory.newXPath();

            try {
                exprDaysSupply = xpath.compile(XPATH_DAYS_SUPPLY);
                exprFillsRemaining = xpath.compile(XPATH_FILLS_REMAINING);
                exprLastFilled = xpath.compile(XPATH_LAST_FILLED);
                exprMedId = xpath.compile(XPATH_MED_ID);
                exprName = xpath.compile(XPATH_NAME);
                exprOrderId = xpath.compile(XPATH_ORDER_ID);
                exprOrdered = xpath.compile(XPATH_ORDERED);
                exprOrderingProvider = xpath.compile(XPATH_ORDERING_PROVIDER);
                exprCurrentProvider = xpath.compile(XPATH_CURRENT_PROVIDER);
                exprNameAttrib = xpath.compile(XPATH_NAME_ATTRIB);
                exprCode = xpath.compile(XPATH_CODE);
                exprTaxonomyCode = xpath.compile(XPATH_TAXONOMY_CODE);
                exprProviderType = xpath.compile(XPATH_PROVIDER_TYPE);
                exprClassification = xpath.compile(XPATH_CLASSIFICATION);
                exprSpecialization = xpath.compile(XPATH_SPECIALIZATION);
                exprService = xpath.compile(XPATH_SERVICE);
                exprOfficePhone = xpath.compile(XPATH_OFFICE_PHONE);
                exprAnalogPager = xpath.compile(XPATH_ANALOG_PAGER);
                exprFax = xpath.compile(XPATH_FAX);
                exprEmail = xpath.compile(XPATH_EMAIL);

                exprPharmacistCode = xpath.compile(XPATH_PHARMACIST_CODE);
                exprPharmacistName = xpath.compile(XPATH_PHARMACIST_NAME);
                exprPrescription = xpath.compile(XPATH_PRESCRIPTION);
                exprPtInstructions = xpath.compile(XPATH_PT_INSTRUCTIONS);
                exprQuantity = xpath.compile(XPATH_QUANTITY);
                exprSig = xpath.compile(XPATH_SIG);
                exprRate = xpath.compile(XPATH_RATE);
                exprStart = xpath.compile(XPATH_START);
                exprStop = xpath.compile(XPATH_STOP);
                exprVaStatus = xpath.compile(XPATH_VA_STATUS);
                exprVaType = xpath.compile(XPATH_VA_TYPE);

                exprDoses = xpath.compile(XPATH_DOSES);
                exprDose = xpath.compile(XPATH_DOSE);
                exprDoseRoute = xpath.compile(XPATH_DOSE_ROUTE);
                exprDoseSchedule = xpath.compile(XPATH_DOSE_SCHEDULE);
                exprFills = xpath.compile(XPATH_FILLS);
                exprFill = xpath.compile(XPATH_FILL);
                exprFillFillDate = xpath.compile(XPATH_FILL_FILL_DATE);
                exprFillFillQuantity = xpath.compile(XPATH_FILL_FILL_QUANTITY);
                exprFillFillRouting = xpath.compile(XPATH_FILL_FILL_ROUTING);
                exprFillReleaseDate = xpath.compile(XPATH_FILL_RELEASE_DATE);
                exprFillFillDaysSupply = xpath.compile(XPATH_FILL_FILL_DAYS_SUPPLY);
                exprFillPartial = xpath.compile(XPATH_FILL_PARTIAL);

                exprProducts = xpath.compile(XPATH_PRODUCTS);
                exprProduct = xpath.compile(XPATH_PRODUCT);
                exprProductName = xpath.compile(XPATH_PRODUCT_NAME);
                exprProductVaProductName = xpath.compile(XPATH_PRODUCT_VA_PRODUCT_NAME);
                exprProductVaProductVuid = xpath.compile(XPATH_PRODUCT_VA_PRODUCT_VUID);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }

            String vaType = null;
            try {
                vaType = (String) exprVaType.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }

            // skip parsing this node if type doesn't match
             // inpatient covers all non-outpatient and non-nonVA
            if (INPATIENT_SORT.equals(inptOrOutpt) && ("O".equals(vaType) || "N".equals(vaType))) {
                rtc = null;
            } else if (OUTPATIENT_SORT.equals(inptOrOutpt) && (!"O".equals(vaType) && !"N".equals(vaType))) {                // outpatient is only O or N
                rtc = null;
            } else {
    

            Medication medication = new Medication();
            fillCommonDataBeanValues(medication);

            String daysSupply = null;
            try {
                daysSupply = (String) exprDaysSupply.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String fillsRemaining = null;
            try {
                fillsRemaining = (String) exprFillsRemaining.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String lastFilled = null;
            try {
                lastFilled = (String) exprLastFilled.evaluate(medNode, XPathConstants.STRING);
                // Zero date returned from VPR call. Converting it to empty string so no date is displayed
                if ("0".equalsIgnoreCase(lastFilled)) {
                    lastFilled = "";
                }
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String medId = null;
            try {
                medId = (String) exprMedId.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String name = null;
            try {
                name = (String) exprName.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String orderId = null;
            try {
                orderId = (String) exprOrderId.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String ordered = null;
            try {
                ordered = (String) exprOrdered.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }

            Node orderingProviderNode = RPCUtil.evaluateNodeToNode(medNode, exprOrderingProvider, LOGGER);
            if (orderingProviderNode != null) {
                String orderingProviderName = RPCUtil.evaluateNodeToString(orderingProviderNode, exprNameAttrib, LOGGER);
                String orderingProviderCode = RPCUtil.evaluateNodeToString(orderingProviderNode, exprCode, LOGGER);
                String orderingProviderTaxonomyCode = RPCUtil.evaluateNodeToString(orderingProviderNode, exprTaxonomyCode, LOGGER);
                String orderingProviderProviderType = RPCUtil.evaluateNodeToString(orderingProviderNode, exprProviderType, LOGGER);
                String orderingProviderClassification = RPCUtil.evaluateNodeToString(orderingProviderNode, exprClassification, LOGGER);
                String orderingProviderSpecialization = RPCUtil.evaluateNodeToString(orderingProviderNode, exprSpecialization, LOGGER);
                String orderingProviderService = RPCUtil.evaluateNodeToString(orderingProviderNode, exprService, LOGGER);
                String orderingProviderOfficePhone = RPCUtil.evaluateNodeToString(orderingProviderNode, exprOfficePhone, LOGGER);
                String orderingProviderAnalogPager = RPCUtil.evaluateNodeToString(orderingProviderNode, exprAnalogPager, LOGGER);
                String orderingProviderFax = RPCUtil.evaluateNodeToString(orderingProviderNode, exprFax, LOGGER);
                String orderingProviderEmail = RPCUtil.evaluateNodeToString(orderingProviderNode, exprEmail, LOGGER);

                Provider orderingProvider = new Provider();
                orderingProvider.setName(orderingProviderName);
                orderingProvider.setCode(orderingProviderCode);
                orderingProvider.setTaxonomyCode(orderingProviderTaxonomyCode);
                orderingProvider.setProviderType(orderingProviderProviderType);
                orderingProvider.setClassification(orderingProviderClassification);
                orderingProvider.setSpecialization(orderingProviderSpecialization);
                orderingProvider.setService(orderingProviderService);
                orderingProvider.setPhone(orderingProviderOfficePhone);
                orderingProvider.setAnalogPager(orderingProviderAnalogPager);
                orderingProvider.setFax(orderingProviderFax);
                orderingProvider.setEmail(orderingProviderEmail);
                medication.setOrderingProvider(orderingProvider);
            }

            Node currentProviderNode = RPCUtil.evaluateNodeToNode(medNode, exprCurrentProvider, LOGGER);
            if (currentProviderNode != null) {
                String currentProviderName = RPCUtil.evaluateNodeToString(currentProviderNode, exprNameAttrib, LOGGER);
                String currentProviderCode = RPCUtil.evaluateNodeToString(currentProviderNode, exprCode, LOGGER);
                String currentProviderTaxonomyCode = RPCUtil.evaluateNodeToString(currentProviderNode, exprTaxonomyCode, LOGGER);
                String currentProviderProviderType = RPCUtil.evaluateNodeToString(currentProviderNode, exprProviderType, LOGGER);
                String currentProviderClassification = RPCUtil.evaluateNodeToString(currentProviderNode, exprClassification, LOGGER);
                String currentProviderSpecialization = RPCUtil.evaluateNodeToString(currentProviderNode, exprSpecialization, LOGGER);
                String currentProviderService = RPCUtil.evaluateNodeToString(currentProviderNode, exprService, LOGGER);
                String currentProviderOfficePhone = RPCUtil.evaluateNodeToString(currentProviderNode, exprOfficePhone, LOGGER);
                String currentProviderAnalogPager = RPCUtil.evaluateNodeToString(currentProviderNode, exprAnalogPager, LOGGER);
                String currentProviderFax = RPCUtil.evaluateNodeToString(currentProviderNode, exprFax, LOGGER);
                String currentProviderEmail = RPCUtil.evaluateNodeToString(currentProviderNode, exprEmail, LOGGER);

                Provider currentProvider = new Provider();
                currentProvider.setName(currentProviderName);
                currentProvider.setCode(currentProviderCode);
                currentProvider.setTaxonomyCode(currentProviderTaxonomyCode);
                currentProvider.setProviderType(currentProviderProviderType);
                currentProvider.setClassification(currentProviderClassification);
                currentProvider.setSpecialization(currentProviderSpecialization);
                currentProvider.setService(currentProviderService);
                currentProvider.setPhone(currentProviderOfficePhone);
                currentProvider.setAnalogPager(currentProviderAnalogPager);
                currentProvider.setFax(currentProviderFax);
                currentProvider.setEmail(currentProviderEmail);
                medication.setCurrentProvider(currentProvider);
            }

            String pharmacistCode = null;
            try {
                pharmacistCode = (String) exprPharmacistCode.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String pharmacistName = null;
            try {
                pharmacistName = (String) exprPharmacistName.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String prescription = null;
            try {
                prescription = (String) exprPrescription.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String quantity = null;
            try {
                quantity = (String) exprQuantity.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String start = null;
            try {
                start = (String) exprStart.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String stop = null;
            try {
                stop = (String) exprStop.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String vaStatus = null;
            try {
                vaStatus = (String) exprVaStatus.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String ptInstructions = null;
            try {
                ptInstructions = (String) exprPtInstructions.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            String sig = null;
            try {
                sig = (String) exprSig.evaluate(medNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }

            Node fillsNode = null;
            try {
                fillsNode = (Node) exprFills.evaluate(medNode, XPathConstants.NODE);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }

            if (fillsNode != null) {
                NodeList fillNodeList = null;
                try {
                    fillNodeList = (NodeList) exprFill.evaluate(fillsNode, XPathConstants.NODESET);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                if (fillNodeList != null) {
                    List<PrescriptionFill> rxFillList = new ArrayList<PrescriptionFill>();
                    int fillNodeListLength = fillNodeList.getLength();
                    for (int fillNum = 0; fillNum < fillNodeListLength; fillNum++) {
                        Node fillNode = fillNodeList.item(fillNum);
                        fillNode.getParentNode().removeChild(fillNode);

                        PrescriptionFill rxFill = new PrescriptionFill();
                        fillCommonDataBeanValues(rxFill);

                        String fillDateString = null;
                        try {
                            fillDateString = (String) exprFillFillDate.evaluate(fillNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        Calendar fillDate = null;
                        if (fillDateString != null) {
                            fillDate = Timestamp.toDateTime(fillDateString);
                        }
                        String fillQuantity = null;
                        try {
                            fillQuantity = (String) exprFillFillQuantity.evaluate(fillNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        String fillReleaseDateString = null;
                        try {
                            fillReleaseDateString = (String) exprFillReleaseDate.evaluate(fillNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        Calendar fillReleaseDate = null;
                        if (fillReleaseDateString != null) {
                            fillReleaseDate = Timestamp.toDateTime(fillReleaseDateString);
                        }
                        String fillRouting = null;
                        try {
                            fillRouting = (String) exprFillFillRouting.evaluate(fillNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        String fillDaysSupply = null;
                        try {
                            fillDaysSupply = (String) exprFillFillDaysSupply.evaluate(fillNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        String fillPartial = null;
                        try {
                            fillPartial = (String) exprFillPartial.evaluate(fillNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }

                        rxFill.setDispenseDate(fillDate);
                        rxFill.setDispensingQuantity(fillQuantity);
                        rxFill.setReleaseDate(fillReleaseDate);
                        rxFill.setDaysSupply(fillDaysSupply);
                        rxFill.setRouting(fillRouting);
                        if (fillPartial != null && "1".equals(fillPartial)) {
                            rxFill.setPartial(true);
                        }

                        rxFillList.add(rxFill);
                    }

                    if (!rxFillList.isEmpty()) {
                        medication.setPrescriptionFills(rxFillList.toArray(new PrescriptionFill[0]));
                    }
                }
            }

            Node productsNode = null;
            try {
                productsNode = (Node) exprProducts.evaluate(medNode, XPathConstants.NODE);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
            }
            if (productsNode != null) {
                NodeList productNodeList = null;
                try {
                    productNodeList = (NodeList) exprProduct.evaluate(productsNode, XPathConstants.NODESET);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }
                if (productNodeList != null) {
                    StringBuilder productNameSB = new StringBuilder("");
                    String productVaProductName = null;
                    String productVaProductVuid = null;
                    List<Code> codeList = new ArrayList<Code>();
                    int productNodeListLength = productNodeList.getLength();
                    for (int productNum = 0; productNum < productNodeListLength; productNum++) {
                        Node productNode = productNodeList.item(productNum);
                        productNode.getParentNode().removeChild(productNode);

                        if (productNameSB.length() > 0) {
                                productNameSB.append("; ");
                        }                       
			try {
                            productNameSB.append((String) exprProductName.evaluate(productNode, XPathConstants.STRING));
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            productVaProductName = (String) exprProductVaProductName.evaluate(productNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            productVaProductVuid = (String) exprProductVaProductVuid.evaluate(productNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        if (!StringUtils.isEmpty(productVaProductVuid)) {
                            Code code = new Code();
                            code.setSystem("VUID");
                            code.setCode(productVaProductVuid);
                            if (!StringUtils.isEmpty(productVaProductName)) {
                                code.setDisplay(productVaProductName);
                            }
                            codeList.add(code);
                            productVaProductVuid = null;
                        }
                    }
                    medication.setDrugName(productNameSB.toString());
                    medication.setCodes(codeList.toArray(new Code[0]));
                }
            }

            Calendar lastFilledDate = null;
            Calendar orderedDate;
            Calendar stopDate;
            Calendar startDate;

            if (!StringUtils.isEmpty(stop)) {
                stopDate = Timestamp.toDateTime(stop);
                medication.setStopDate(stopDate);
            }
            if (!StringUtils.isEmpty(start)) {
                startDate = Timestamp.toDateTime(start);
                medication.setStartDate(startDate);
            }

            medication.setMedId(medId);
            medication.setStatus(vaStatus);
            medication.setMedType(vaType);

            medication.setSigCode(sig);
            medication.setOrderId(orderId);
            medication.setLastDispensingPharmacy(pharmacistName);
            medication.setPharmacyId(pharmacistCode);

            if (StringUtils.isEmpty(medication.getDrugName())) {
                medication.setDrugName(name);
            }

            if (!StringUtils.isEmpty(ordered)) {
                orderedDate = Timestamp.toDateTime(ordered);
                medication.setOrderDate(orderedDate);
            }

            if ("O".equalsIgnoreCase(vaType) || "N".equalsIgnoreCase(vaType)) {
                medication.setRefills(fillsRemaining);
                medication.setDaysSupply(daysSupply);

                if (!StringUtils.isEmpty(lastFilled)) {
                    lastFilledDate = Timestamp.toDateTime(lastFilled);
                    medication.setLastFilledDate(lastFilledDate);
                }
                medication.setRxNumber(prescription);
                medication.setComment(ptInstructions);
                medication.setQuantity(quantity);

                if ("N".equalsIgnoreCase(vaType)) {
                    StringBuilder nonVADrugName = new StringBuilder(name);
                    nonVADrugName.insert(0, "NON-VA ");
                    medication.setDrugName(nonVADrugName.toString());
                }
            } else {
                // type is "I" or "V"

// set lastFilledDate first with most recent fill relesedDate, else the lastFilledDate, else
                // the startDate
                Date lastReleasedDate = new Date(Long.MIN_VALUE);
                Calendar lastReleaseCal = Calendar.getInstance();
                lastReleaseCal.setTime(lastReleasedDate);
                if (medication.getPrescriptionFills() != null) {
                    for (PrescriptionFill pfill : medication.getPrescriptionFills()) {
                        if (pfill.getReleaseDate() != null) {
                            if (pfill.getReleaseDate().compareTo(lastReleaseCal) > 0) {
                                lastReleaseCal = pfill.getReleaseDate();
                                medication.setLastFilledDate(pfill.getReleaseDate());
                            }
                        }
                    }
                }
                if (medication.getLastFilledDate() == null) {
                    if (!StringUtils.isEmpty(lastFilled)) {
                        lastFilledDate = Timestamp.toDateTime(lastFilled);
                        medication.setLastFilledDate(lastFilledDate);
                    } else {
                        medication.setLastFilledDate(medication.getStartDate());
                    }
                }
                if ("V".equalsIgnoreCase(vaType)) {
                    // V does not have a sig so we'll build it from dose route schedule and rate
                    Node dosesNode = null;
                    try {
                        dosesNode = (Node) exprDoses.evaluate(medNode, XPathConstants.NODE);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }

                    String doseRoute = "";
                    String doseSchedule = "";

                    if (dosesNode != null) {
                        NodeList doseNodeList = null;
                        try {
                            doseNodeList = (NodeList) exprDose.evaluate(dosesNode, XPathConstants.NODESET);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }

                        if (doseNodeList != null) {
                            Node doseNode = doseNodeList.item(0);

                            if (doseNode != null) {
                                if (doseNode.getParentNode() != null) {
                                    doseNode.getParentNode().removeChild(doseNode);
                                }
                                try {
                                    doseRoute = (String) exprDoseRoute.evaluate(doseNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                                try {
                                    doseSchedule = (String) exprDoseSchedule.evaluate(doseNode, XPathConstants.STRING);
                                } catch (XPathExpressionException e) {
                                    logErrorMessage(e);
                                }
                            }
                        }

                    }

                    String rate = null;
                    try {
                        rate = (String) exprRate.evaluate(medNode, XPathConstants.STRING);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }

                    StringBuilder sigCodeSB = new StringBuilder();
                    if (!StringUtils.isEmpty(doseRoute)) {
                        sigCodeSB.append("Route: ");
                        sigCodeSB.append(doseRoute);
                    }
                    if (!StringUtils.isEmpty(doseSchedule)) {
                        if (sigCodeSB.length() > 0)
                            sigCodeSB.append("; ");
                        sigCodeSB.append("Schedule: ");
                        sigCodeSB.append(doseSchedule);
                    }
                    if (!StringUtils.isEmpty(rate)) {
                        if (sigCodeSB.length() > 0)
                            sigCodeSB.append("; ");
                        sigCodeSB.append("Rate: ");
                        sigCodeSB.append(rate);
                    }
                    medication.setSigCode(sigCodeSB.toString());
                }
            }

            rtc = medication;
            }
            return rtc;
        }
    }

}
