/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.AppointmentsDao;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class AppointmentsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(AppointmentsDataService.class.getName());

    public List<PatientAppointments> getPatientAppointments(QueryBean queryBean) throws VDSException
    {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> apptsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_APPOINTMENTS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_APPOINTMENTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = apptsMap.entrySet();
            List<PatientAppointments> apptsList = new ArrayList<PatientAppointments>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object appts = entry.getValue();
                if (appts instanceof Exception) {
                    appts = generateConnectionUnavailableBean(entry.getKey(), (Exception) appts);
                }
                apptsList.addAll(Arrays.asList((PatientAppointments[]) appts));
            }

            return apptsList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getEncountersReport(QueryBean queryBean) throws VDSException
    {
        validateQueryBeanDateRange(queryBean);

        RPCConnection cxn = null;

        try {
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            AppointmentsDao apptsDao = new AppointmentsDao(cxn);
            return apptsDao.getEncountersReport(queryBean);
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }


    private PatientAppointments[] generateConnectionUnavailableBean(Site site, Exception except) {
        
        PatientAppointments apt = new PatientAppointments();
        apt.setClinic(CONNECTION_UNAVAILABLE);
        apt.setApptDate(Utils.backDate().toGregorianCalendar());
        apt.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            apt.setClinic(UNABLE_TO_RETRIEVE);
        } else  {
            LOGGER.error(except.getMessage(), except);
        }
        
        return new PatientAppointments[]{apt};
    }
}
