/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.vistabeans.Encounter;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class EncountersDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(EncountersDataService.class.getName());

    public List<Encounter> getPatientEncounters(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> apptsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_ENCOUNTERS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ENCOUNTERS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = apptsMap.entrySet();
            List<Encounter> encountersList = new ArrayList<Encounter>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object encounters = entry.getValue();
                if (encounters instanceof Exception) {
                    encounters = generateConnectionUnavailableBean(entry.getKey(), (Exception) encounters);
                }
                encountersList.addAll(Arrays.asList((Encounter[]) encounters));
            }

            return encountersList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private Encounter[] generateConnectionUnavailableBean(Site site, Exception except) {
        Encounter encounter = new Encounter();
        encounter.setClinic(CONNECTION_UNAVAILABLE);
        encounter.setApptDate(Utils.backDate().toGregorianCalendar());
        encounter.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            encounter.setClinic(UNABLE_TO_RETRIEVE);
        } else {
            LOGGER.error(except.getMessage(), except);        //log if any other exception type
        }
        return new Encounter[]{encounter};
    }
}
