/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.OrdersDao;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class OrdersDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(OrdersDataService.class.getName());

    public List<Order> getPatientOrders(QueryBean queryBean) throws VDSException
    {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> ordersMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_ORDERS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ORDERS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = ordersMap.entrySet();
            List<Order> ordersList = new ArrayList<Order>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object orders = entry.getValue();
                if (orders instanceof Exception) {
                    orders = generateConnectionUnavailableBean(entry.getKey(), (Exception) orders);
                }
                ordersList.addAll(Arrays.asList((Order[]) orders));
            }

            return ordersList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getOrderDetail(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            String itemId = queryBean.getItemId();
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (itemId == null || itemId.length() <= 0) {
                throw new IllegalArgumentException("Item Id cannot be null or empty");
            } else if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            OrdersDao ordersDao = new OrdersDao(cxn);
            FreeTextReport order = ordersDao.getOrderDetail(queryBean);

            return order;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private Order[] generateConnectionUnavailableBean(Site site, Exception except) {
        
        Order order = new Order();
        order.setDescription(CONNECTION_UNAVAILABLE);
        order.setOrderDate(Utils.backDate().toGregorianCalendar());
        order.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            order.setDescription(UNABLE_TO_RETRIEVE);
        } else {
            LOGGER.error(except.getMessage(), except);        //log if any other exception type
        }
        return new Order[] {order};
    }
}
