/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao;

import com.intersys.objects.CacheException;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.provider.dao.cache.CacheSiteConnection;
import gov.va.med.vds.provider.dao.cache.AdmissionCache;
import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.QueryBean;

public class AdmissionDao extends DaoBase {
    public AdmissionDao(RPCConnection cxn) {
        super(cxn);
    }

    /**
     * Gets a list of patients who are admitted to the specified ward(s).
     * @param queryBean - the query bean requires itemId. The itemId must be
     * a string containing the ien of each ward followed by a ^. For example,
     * to get a list of all patients in wards 3B2 /24 and GTCU-B the following
     * string would be set for itemId - "24^27".
     * @return array of patients who are admitted to a given ward(s).
     * @throws CacheException
     * @throws RPCException
     * @throws UnsupportedRPCMethodException
     */
    public PatientAdmission[] getWardAdmissions(QueryBean queryBean) throws CacheException, RPCException, UnsupportedRPCMethodException {
        if (isCacheObjectSite()) {
            CacheSiteConnection cacheConnection = createCacheConnection();

            AdmissionCache cache = new AdmissionCache(cacheConnection);
            PatientAdmission[] admissions = cache.getWardAdmissions(queryBean);

            if (admissions == null) {
                admissions = new PatientAdmission[0];
            }

            return admissions;
        } else {
            throw new UnsupportedRPCMethodException("No support RPC method");
        }
    }

    /**
     * Gets a list of patients which are associated with the attending
     * user or the user.
     * @param queryBean - the bean requires no parameters.
     * @return array of patients who are admitted for given user id (ien)
     * @throws CacheException
     * @throws RPCException
     * @throws UnsupportedRPCMethodException
     */
    public PatientAdmission[] getProviderAdmissions(QueryBean queryBean) throws CacheException, RPCException, UnsupportedRPCMethodException {
        if (isCacheObjectSite()) {
            CacheSiteConnection cacheConnection = createCacheConnection();

            AdmissionCache cache = new AdmissionCache(cacheConnection);
            PatientAdmission[] admissions = cache.getProviderAdmissions(queryBean);

            if (admissions == null) {
                admissions = new PatientAdmission[0];
            }

            return admissions;
        } else {
            throw new UnsupportedRPCMethodException("No support RPC method");
        }
    }
}
