/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao;

import com.intersys.objects.CacheException;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.provider.dao.cache.CacheSiteConnection;
import gov.va.med.vds.provider.dao.cache.ConsultCache;
import gov.va.med.vds.vistabeans.Consult;
import gov.va.med.vds.vistabeans.QueryBean;

public class ConsultDao extends DaoBase {
    public ConsultDao(RPCConnection cxn) {
        super(cxn);
    }

    public Consult[] getConsultsReceived(QueryBean queryBean) throws CacheException, RPCException, UnsupportedRPCMethodException {
        if (isCacheObjectSite()) {
            CacheSiteConnection cacheConnection = createCacheConnection();

            ConsultCache cache = new ConsultCache(cacheConnection);
            Consult[] consults = cache.getConsultsReceived(queryBean);

            if (consults == null) {
                consults = new Consult[0];
            }

            return consults;
        } else {
            throw new UnsupportedRPCMethodException("No supported RPC method");
        }
    }

    public Consult[] getConsultsRequested(QueryBean queryBean) throws CacheException, RPCException, UnsupportedRPCMethodException {
        if (isCacheObjectSite()) {
            CacheSiteConnection cacheConnection = createCacheConnection();

            ConsultCache cache = new ConsultCache(cacheConnection);
            Consult[] consults = cache.getConsultsRequested(queryBean);

            if (consults == null) {
                consults = new Consult[0];
            }

            return consults;
        } else {
            throw new UnsupportedRPCMethodException("No supported RPC method");
        }
    }
}
