/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao;

import com.intersys.objects.CacheException;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.provider.dao.cache.CacheSiteConnection;
import gov.va.med.vds.provider.dao.cache.OrderCache;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.QueryBean;

public class OrderDao extends DaoBase {
    public OrderDao(RPCConnection cxn) {
        super(cxn);
    }

    public Order[] getOrdersPending(QueryBean queryBean) throws CacheException, RPCException, UnsupportedRPCMethodException {
        if (isCacheObjectSite()) {
            CacheSiteConnection cacheConnection = createCacheConnection();
            OrderCache cache = new OrderCache(cacheConnection);
            Order[] orders = cache.getOrdersPending(queryBean);

            if (orders == null) {
                orders = new Order[0];
            }

            return orders;
        } else {
            throw new UnsupportedRPCMethodException("No supported RPC method");
        }
    }

    public Order[] getOrdersResulted(QueryBean queryBean) throws CacheException, RPCException, UnsupportedRPCMethodException {
        if (isCacheObjectSite()) {
            CacheSiteConnection cacheConnection = createCacheConnection();
            OrderCache cache = new OrderCache(cacheConnection);
            Order[] orders = cache.getOrdersResulted(queryBean);

            if (orders == null) {
                orders = new Order[0];
            }

            return orders;
        } else {
            throw new UnsupportedRPCMethodException("No supported RPC method");
        }
    }

    public Order getOrderDetails(QueryBean queryBean) throws CacheException, RPCException, UnsupportedRPCMethodException {
        if (isCacheObjectSite()) {
            CacheSiteConnection cacheConnection = createCacheConnection();
            OrderCache cache = new OrderCache(cacheConnection);
            Order order = cache.getOrderDetails(queryBean);

            if (order == null) {
                order = new Order();
            }

            return order;
        } else {
            throw new UnsupportedRPCMethodException("No supported RPC method");
        }
    }
    
}
