/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache;

import com.intersys.objects.CacheException;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.handler.AppointmentHandler;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vhahon.common.Utils;

public class AppointmentCache extends CacheBase {
    public AppointmentCache(CacheSiteConnection cacheConnection) {
        super(cacheConnection);
    }

    public PatientAppointments[] getProviderAppointments(QueryBean queryBean) throws CacheException {
        PatientAppointments[] appointments = null;
        String startDate = null;
        String endDate = null;

        if (queryBean.getEndDate() != null) {
            endDate = Utils.formatDate(queryBean.getEndDate().getTime());
        }
        if (queryBean.getStartDate() != null) {
            startDate = Utils.formatDate(queryBean.getStartDate().getTime());
        }

        cacheConnection.open();

        try {
            AppointmentHandler handler = new AppointmentHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(), queryBean.getUser().getUserIen(), queryBean.getUser().getHostSite().getSiteCode());
            appointments = handler.processProviderAppts(queryBean.getUser().getUserIen(), queryBean.getUser().getNPI(), startDate, endDate);
        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }
        return appointments;
    }
}
