/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache;

import com.intersys.objects.CacheDatabase;
import com.intersys.objects.CacheException;
import com.intersys.objects.Database;
import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.rpc.site.SiteUtils;
import gov.va.med.vds.vistabeans.Endpoint;
import gov.va.med.vds.vistabeans.Site;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 */
public class CacheSiteConnection {
    private static final Logger LOGGER = LoggerFactory.getLogger(CacheSiteConnection.class);

    private Site site;
    private Endpoint cacheEndpoint;
    private String patientIen;
    private Database database;

    public CacheSiteConnection(Site site) {
        if (site == null) {
            throw new IllegalArgumentException("Site is null");
        }

        this.site = site;

        this.cacheEndpoint = SiteUtils.getCacheEndpoint(site);

        if (this.cacheEndpoint == null) {
            throw new IllegalArgumentException("Cannot find Cache endpoint within Site.");
        }

    }

    public void open() throws CacheException {
        if (database != null) {
            this.close();
        }

        database = getCacheConnection();
    }

    private Database getCacheConnection() throws CacheException {
        Database dbase = null;

        dbase = CacheDatabase.getDatabase(cacheEndpoint.getHost(),
                EncryptUtil.decrypt(cacheEndpoint.getUsername()), EncryptUtil.decrypt(cacheEndpoint.getPassword()));
        if (dbase == null) {
            throw new CacheException("CacheDatabase connection is null.");
        }
        return dbase;
    }

    public void close() {
        if (database != null) {
            try {
                database.flush();
            } catch (CacheException e) {
                LOGGER.error(e.getMessage());
            }
            try {
                database.closeAllObjects();
            } catch (CacheException e) {
                LOGGER.error(e.getMessage());
            }
            try {
                database.close();
            } catch (CacheException e) {
                LOGGER.error(e.getMessage());
            }
            database = null;
        }
    }

    public Database getDatabase() {
        return database;
    }

    public Site getSite() {
        return site;
    }

    public Endpoint getCacheEndpoint() {
        return cacheEndpoint;
    }

    public String getPatientIEN() {
        return patientIen;
    }

    public void setPatientIEN(String patientIen) {
        this.patientIen = patientIen;
    }
}
