/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.handler;

import com.intersys.objects.Database;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.model.AdmissionData;
import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.Site;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

public class AdmissionHandler extends Handler {
    private static final Logger LOGGER = LoggerFactory.getLogger(AdmissionHandler.class);

    public AdmissionHandler(Database dbase, Site site, String userId, String userSiteCode) {
        super(dbase, site, userId, userSiteCode);
    }

    public AdmissionHandler(Database dbase, Site site, String userId,
                            String username, String userSiteCode) {
        super(dbase, site, userId, username, userSiteCode);
    }

    public PatientAdmission[] processWardAdmissions(String wardIds, String userIen, String providerNpi) {
        PatientAdmission[] admissions;
        if (wardIds == null || m_db == null) {
            admissions = null;
        } else {
    
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
    
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_WARD_ADMISSIONS(?)}");
                cacheCallable.setString(1, wardIds);
                tableOfResults = cacheCallable.executeQuery();
    
                AdmissionData data = new AdmissionData(m_site, tableOfResults);
                admissions = data.getWardAdmissions();
            } catch (Exception e) {
                throw new CacheHandlerException("AdmissionHandler::processWardAdmissions(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
        return admissions;
    }

    public PatientAdmission[] processProviderAdmissions(String userIen, String providerNpi) {
        PatientAdmission[] admissions;
        if (userIen == null || m_db == null) {
            admissions = null;
        } else {
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
    
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_PROVIDER_ADMISSIONS(?,?)}");
                cacheCallable.setString(1, userIen);
                cacheCallable.setString(2, providerNpi);
                tableOfResults = cacheCallable.executeQuery();
    
                AdmissionData data = new AdmissionData(m_site, tableOfResults);
                admissions = data.getProviderAdmissions();
            } catch (Exception e) {
                throw new CacheHandlerException("AdmissionHandler::processProviderAdmissions(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
        return admissions;
    }
}