/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.handler;

import com.intersys.objects.Database;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.vistabeans.DataBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

/**
 * @author vhahongotom
 */
public class Handler {
    //SB: This max value is used when a cache query requires a max value and an empty string is not suffice.
    protected static final String DEFAULT_MAX = "10000";

    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();

    protected static final String PROTOCOL_CACHE = APP_CONFIG.getProtocolCache();

    protected transient Database m_db;
    protected String m_userId;
    protected String m_username;
    protected String m_userSiteCode;
    protected Site m_site;

    public Handler(Database dbase, Site site, String userId, String userSiteCode) {
        m_db = dbase;
        m_userId = userId;
        m_site = site;
        m_username = "";
        m_userSiteCode = userSiteCode;
    }

    public Handler(Database dbase, Site site, String userId, String username, String userSiteCode) {
        m_db = dbase;
        m_userId = userId;
        m_username = username;
        m_userSiteCode = userSiteCode;
        m_site = site;
    }

    protected void fillCommonDataBeanValues(DataBean bean) {
        bean.setSite(m_site);
        bean.setSourceProtocol(PROTOCOL_CACHE);
    }

    protected String toFilemanEndDate(String endDate) {
        return Utils.getFilemanDate(endDate) + ".99999";
    }

    /**
     * @return the m_db
     */
    protected Database getM_db() {
        return m_db;
    }

    /**
     * @param mDb the m_db to set
     */
    protected void setM_db(Database mDb) {
        m_db = mDb;
    }

    /**
     * @return the m_userId
     */
    protected String getM_userId() {
        return m_userId;
    }

    /**
     * @param mUserId the m_userId to set
     */
    protected void setM_userId(String mUserId) {
        m_userId = mUserId;
    }

    /**
     * @return the m_username
     */
    protected String getM_username() {
        return m_username;
    }

    /**
     * @param mUsername the m_username to set
     */
    protected void setM_username(String mUsername) {
        m_username = mUsername;
    }

    /**
     * @return the m_userSiteCode
     */
    protected String getM_userSiteCode() {
        return m_userSiteCode;
    }

    /**
     * @param mUserSiteCode the m_userSiteCode to set
     */
    protected void setM_userSiteCode(String mUserSiteCode) {
        m_userSiteCode = mUserSiteCode;
    }

    /**
     * @return the m_site
     */
    protected Site getM_site() {
        return m_site;
    }

    /**
     * @param mSite the m_site to set
     */
    protected void setM_site(Site mSite) {
        m_site = mSite;
    }
}
