/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.model;

import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.LoggerFactory;

import java.sql.ResultSet;
import java.text.ParseException;
import java.util.ArrayList;

public class AdmissionData extends ModelData {
    public static final int PATIENT_ID = 0;
    public static final int PATIENT_NAME = 1;
    public static final int WARD_ID = 2;
    public static final int WARD_NAME = 3;
    public static final int ROOM_BED = 4;
    public static final int USER_IEN = 5;
    public static final int PROVIDER_NAME = 6;
    public static final int ATTENDING_USER_IEN = 7;
    public static final int ATTENDING_PROVIDER_NAME = 8;
    public static final int ADMISSION_DATE = 9;
    public static final int ADMISSION_ID = 10;

    public AdmissionData(Site site, ResultSet table) {
        super(site, table);
    }

    public PatientAdmission[] getWardAdmissions() {
        PatientAdmission[] admissions = null;

        ArrayList<PatientAdmission> temp = new ArrayList<PatientAdmission>();
        DataModelIterator iter = new DataModelIterator(this);

        int totalRows = 0;
        String patientId = "";
        String patientName = "";
        String wardId = "";
        String wardName = "";
        String roomBed = "";

        while (iter.hasNext()) {
            patientId =  iter.getItem(PATIENT_ID);
            patientName =  iter.getItem(PATIENT_NAME);
            wardId =  iter.getItem(WARD_ID);
            wardName =  iter.getItem(WARD_NAME);
            roomBed =  iter.getItem(ROOM_BED);

            PatientAdmission admission = new PatientAdmission();
            admission.setPatientName(patientName);
            admission.setWardId(wardId);
            admission.setWard(wardName);
            admission.setRoomBed(roomBed);
            admission.setPatientId(patientId);

            fillCommonDataBeanValues(admission);

            temp.add(admission);
            totalRows++;
        }

        if (totalRows > 0) {
            admissions = temp.toArray(new PatientAdmission[0]);
        }

        return admissions;
    }

    public PatientAdmission[] getProviderAdmissions() {
        PatientAdmission[] admissions = null;

        ArrayList<PatientAdmission> temp = new ArrayList<PatientAdmission>();
        DataModelIterator iter = new DataModelIterator(this);

        int totalRows = 0;
        String patientId = "";
        String patientName = "";
        String wardId = "";
        String wardName = "";
        String roomBed = "";
        String userIen = "";
        String providerName = "";
        String admittingUserIen = "";
        String admittingProviderName = "";
        String admissionDate = "";
        String admissionId = "";

        while (iter.hasNext()) {
            patientId =  iter.getItem(PATIENT_ID);
            patientName =  iter.getItem(PATIENT_NAME);
            wardId =  iter.getItem(WARD_ID);
            wardName =  iter.getItem(WARD_NAME);
            roomBed =  iter.getItem(ROOM_BED);
            userIen =  iter.getItem(USER_IEN);
            providerName =  iter.getItem(PROVIDER_NAME);
            admittingUserIen =  iter.getItem(ATTENDING_USER_IEN);
            admittingProviderName =  iter.getItem(ATTENDING_PROVIDER_NAME);
            admissionDate =  iter.getItem(ADMISSION_DATE);
            admissionId =  iter.getItem(ADMISSION_ID);

            PatientAdmission admission = new PatientAdmission();
            admission.setPatientName(patientName);
            admission.setWardId(wardId);
            admission.setWard(wardName);
            admission.setRoomBed(roomBed);
            admission.setUserIen(userIen);
            admission.setProvider(providerName);
            admission.setAdmittingUserIen(admittingUserIen);
            admission.setAdmittingProvider(admittingProviderName);
            admission.setAdmissionId(admissionId);
            admission.setRegisterNumber(admissionId);
            admission.setPatientId(patientId);

            try {
                admission.setAdmissionDate(Utils.getCalFromFormat(admissionDate, "MMM dd,yyyy@HH:mm:ss"));
            } catch (ParseException e) {
                LoggerFactory.getLogger(AdmissionData.class.getName()).error(e.getMessage(), e);
            }

            fillCommonDataBeanValues(admission);

            temp.add(admission);
            totalRows++;
        }

        if (totalRows > 0) {
            admissions = temp.toArray(new PatientAdmission[0]);
        }

        return admissions;
    }

}
