/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.model;

import gov.va.med.vds.vistabeans.Consult;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.sql.ResultSet;
import java.util.ArrayList;

public class ConsultData extends ModelData {
    public static final int CONSULT_ID = 0;
    public static final int PATIENT_ID = 1;
    public static final int PATIENT_NAME = 2;
    public static final int CONSULT_DATE = 3;
    public static final int TO_SERVICE = 4;
    public static final int CONSULT_STATUS = 5;
    public static final int PATIENT_NEXT_APPT = 6;
    
    public ConsultData(Site site, ResultSet table) {
        super(site, table);
    }

    public Consult[] getConsultsRequested() {
        Consult[] consults = null;

        ArrayList<Consult> temp = new ArrayList<Consult>();
        DataModelIterator iter = new DataModelIterator(this);

        int totalRows = 0;
        String consultId = "";
        String patientId = "";
        String patientName = "";
        String consultDate = "";
        String toService = "";
        String consultStatus = "";

        while (iter.hasNext()) {
            consultId = iter.getItem(CONSULT_ID);
            patientId = iter.getItem(PATIENT_ID);
            patientName = iter.getItem(PATIENT_NAME);
            consultDate = iter.getItem(CONSULT_DATE);
            toService = iter.getItem(TO_SERVICE);
            consultStatus = iter.getItem(CONSULT_STATUS);

            Consult consult = new Consult();
            consult.setId(consultId);
            consult.setPatientName(patientName);
            consult.setRequestDate(Utils.getCalFromSQLFormat(consultDate));
            consult.setService(toService);
            consult.setStatus(consultStatus);
            consult.setPatientId(patientId);

            fillCommonDataBeanValues(consult);

            temp.add(consult);
            totalRows++;
        }

        if (totalRows > 0) {
            consults = temp.toArray(new Consult[0]);
        }

        return consults;
    }

    public Consult[] getConsultsReceived() {
        Consult[] consults = null;

        ArrayList<Consult> temp = new ArrayList<Consult>();
        DataModelIterator iter = new DataModelIterator(this);

        int totalRows = 0;
        String consultId = "";
        String patientId = "";
        String patientName = "";
        String consultDate = "";
        String toService = "";
        String consultStatus = "";
        String patientNextAppt = "";

        while (iter.hasNext()) {
            consultId = iter.getItem(CONSULT_ID);
            patientId = iter.getItem(PATIENT_ID);
            patientName = iter.getItem(PATIENT_NAME);
            consultDate = iter.getItem(CONSULT_DATE);
            toService = iter.getItem(TO_SERVICE);
            consultStatus = iter.getItem(CONSULT_STATUS);
            patientNextAppt = iter.getItem(PATIENT_NEXT_APPT);

            Consult consult = new Consult();
            consult.setId(consultId);
            consult.setPatientName(patientName);
            consult.setRequestDate(Utils.getCalFromSQLFormat(consultDate));
            consult.setService(toService);
            consult.setStatus(consultStatus);
            consult.setPatientId(patientId);
            if ("none".equalsIgnoreCase(patientNextAppt)) {
                consult.setPatientNextAppointment(null);
            } else {
                consult.setPatientNextAppointment(Utils.getCalFromSQLFormat(patientNextAppt));
            }

            fillCommonDataBeanValues(consult);

            temp.add(consult);
            totalRows++;
        }

        if (totalRows > 0) {
            consults = temp.toArray(new Consult[0]);
        }

        return consults;
    }

}