/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.provider.dao.AppointmentDao;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.User;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author vhahonuyehac
 */
public class ProviderAppointmentsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(ProviderAppointmentsDataService.class.getName());

    public PatientAppointments[] getProviderAppointments(QueryBean queryBean) throws VDSException {
        queryParamsCheckProviderWithDateRange(queryBean);
        User user = queryBean.getUser();

        if (user.getHostSite() == null) {
            throw new IllegalArgumentException("hostSite cannot be null");
        }

        checkProviderQueryHostSite(user);

        RPCConnection cxn = null;

        try {
            cxn = setupCPRSContextSingleRemoteSourceQuery(user, user.getHostSite());
            AppointmentDao appointmentsDao = new AppointmentDao(cxn);
            PatientAppointments[] appointments = appointmentsDao.getProviderAppointments(queryBean);

            return appointments;
        } catch (Exception e) {
            throw new VDSException(e);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

}
