/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

/**
 * The LabOrder class represents lab orders for a patient.
 */
public class LabOrder extends DataBean {
    /**
     * Date the sample was collected.
     */
    private Calendar collectionDate;
    /**
     * Name of test ordered.
     */
    private String testName;
    /**
     * The sample collected from the patient for this lab order. Equivalent to Topography or Specimen type.
     */
    private String collectionSample;
    /**
     * Internal Id of the order, used to retrieve results for an order.
     */
    private String internalId;
    /**
     * Order Id of the order.
     */
    private String orderId;
    /**
     * Subscript of the test associated with the lab order.  Subscripts indicate the
     * type of results to be returned.  The getLabOrderResults method is used for CH
     * Subscripts.
     * <p/>
     * Other Subscripts: MI - MicroBiology, SP - Surgical Pathology, CY - Cytology,
     * AU - Autopsy, BB-BloodBank, EM - Electron Microscopy
     */
    private String subscript;
    private Calendar orderDate;
    private Provider requestingProvider;
    private String status;

    private String facilityName;
    private String stationNumber;
    private String labTestIEN;
    private String collectionSampleIEN;
    private String urgency;
    private String requestingProviderIEN;
    private String accessionNumber;
    private Calendar availableDate;
    private String comment;

    /**
     * Lab results associated with lab order -
     * this is used if data source returns lab results along with order data (BHIE).
     */
    private LabResult[] labResults;

    /**
     * Calendar patient sample was collected.
     */
    public Calendar getCollectionDate() {
        return collectionDate;
    }

    public void setCollectionDate(Calendar newCollectionDate) {
        collectionDate = newCollectionDate;
    }

    /**
     * Lab test name for this lab order.
     */
    public String getTestName() {
        return testName;
    }

    public void setTestName(String newTestName) {
        testName = newTestName;
    }

    /**
     * Collection sample for the lab order.
     */
    public String getCollectionSample() {
        return collectionSample;
    }

    public void setCollectionSample(String newCollectionSample) {
        collectionSample = newCollectionSample;
    }

    /**
     * Internal ID of lab order.  Used to retrieve results for order.
     */
    public String getInternalId() {
        return internalId;
    }

    public void setInternalId(String newInternalId) {
        internalId = newInternalId;
    }

    /**
     * Order ID of this lab order.
     */
    public String getOrderId() {
        return orderId;
    }

    public void setOrderId(String newOrderId) {
        orderId = newOrderId;
    }

    /**
     * Subscript for the test associated with the lab order.
     * <p/>
     * Values: CH - CLinical Chemistry
     * MI - Microbiology
     * SP - Surgical Pathology
     * AU - Autopsy
     * BB - BloodBank
     * EM - Electron Microscopy
     */
    public String getSubscript() {
        return subscript;
    }

    public void setSubscript(String newSubscript) {
        subscript = newSubscript;
    }

    public Calendar getOrderDate() {
        return orderDate;
    }

    public void setOrderDate(Calendar newOrderDate) {
        orderDate = newOrderDate;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public LabResult[] getLabResults() {
        LabResult[] rtc = null;
        if (labResults != null) {
            rtc = labResults.clone();
        }
        return rtc;
    }

    public void setLabResults(LabResult[] labResults) {
        if (labResults == null) {
            this.labResults = null;
        } else {
            this.labResults = labResults.clone();
        }
    }

    public String getFacilityName() {
        return facilityName;
    }

    public void setFacilityName(String facilityName) {
        this.facilityName = facilityName;
    }

    public String getStationNumber() {
        return stationNumber;
    }

    public void setStationNumber(String stationNumber) {
        this.stationNumber = stationNumber;
    }

    public String getLabTestIEN() {
        return labTestIEN;
    }

    public void setLabTestIEN(String labTestIEN) {
        this.labTestIEN = labTestIEN;
    }

    public String getCollectionSampleIEN() {
        return collectionSampleIEN;
    }

    public void setCollectionSampleIEN(String collectionSampleIEN) {
        this.collectionSampleIEN = collectionSampleIEN;
    }

    public String getUrgency() {
        return urgency;
    }

    public void setUrgency(String urgency) {
        this.urgency = urgency;
    }

    public String getRequestingProviderIEN() {
        return requestingProviderIEN;
    }

    public void setRequestingProviderIEN(String requestingProviderIEN) {
        this.requestingProviderIEN = requestingProviderIEN;
    }

    public String getAccessionNumber() {
        return accessionNumber;
    }

    public void setAccessionNumber(String accessionNumber) {
        this.accessionNumber = accessionNumber;
    }

    public Calendar getAvailableDate() {
        return availableDate;
    }

    public void setAvailableDate(Calendar availableDate) {
        this.availableDate = availableDate;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public Provider getRequestingProvider() {
        return requestingProvider;
    }

    public void setRequestingProvider(Provider requestingProvider) {
        this.requestingProvider = requestingProvider;
    }
}
