/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

/**
 * This class support CH subscript lab results.
 */
public class LabResult extends DataBean {
    /**
     * Internal Id of the test. Used to retieve all results for a particuler test over
     * time.
     */
    private String testId;
    /**
     * Result value.
     */
    private String result;
    /**
     * accession number associated with the result.
     */
    private String accession;
    /**
     * Order Id associated with the result.
     */
    private String orderId;
    /**
     * Name of test associated with result.
     */
    private String testName;
    /**
     * Type of test associated with result.
     */
//    private String testType;
    /**
     * Print name of the test.  This is usually a shorter name than the test name.
     */
    private String printName;
    /**
     * User initials who verified the result.
     */
    private String verifiedBy;
    /**
     * Flag indicating if the result is high or low.
     * Values: L =low L* = critical low, H = high, H* = critical high
     */
    private String interpretation;
    /**
     * reference range for the test.
     */
    private String referenceRange;
    /**
     * status of result.
     */
    private String resultStatus;
    /**
     * Units associated with the result.
     */
    private String units;
    /**
     * Comment associated with the result.
     */
    private String comment;
    /**
     * Comments associated with the order
     */
    private String orderComment;
    /**
     * Comments associated with the accession
     */
    private String accessionComment;
    /**
     * Date the test was resulted. Equivalent to Collection Date.
     */
    private Calendar resultDate;
    private Calendar collectedDate;
    /**
     * Specimen associated with the result.
     */
    private String specimen;

    private Calendar orderDate;

    private String facilityName;

    /**
     * Ordering user of the medication/prescription.
     */
    private Provider provider;

    private String stationNumber;

    private String requestingLocation;

    private String performingLabName;
    private String performingLabPhone;
    private String performingLabStreet1;
    private String performingLabStreet2;
    private String performingLabStreet3;
    private String performingLabCity;
    private String performingLabStateProvince;
    private String performingLabPostalCode;


    /**
     * Quantity for Microbiology lab result
     */
    private String quantity;

    /**
     * Result sensitive
     */
    private boolean sensitive;

    /**
     * Internal Id of test.  Used to retrieve results for a test over time.
     */
    public String getTestId() {
        return testId;
    }

    public void setTestId(String newTestId) {
        testId = newTestId;
    }

    public Provider getProvider() {
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    /**
     * Value of result.
     */
    public String getResult() {
        return result;
    }

    public void setResult(String newResult) {
        result = newResult;
    }

    /**
     * Accession number for lab result.
     */
    public String getAccession() {
        return accession;
    }

    public void setAccession(String newAccession) {
        accession = newAccession;
    }

    /**
     * Order Id of result.
     */
    public String getOrderId() {
        return orderId;
    }

    public void setOrderId(String newOrderId) {
        orderId = newOrderId;
    }

    /**
     * Test name for result.
     */
    public String getTestName() {
        return testName;
    }

    public void setTestName(String newTestName) {
        testName = newTestName;
    }

    /**
     * Test Type for result.
     */
    public String getTestType() {
        return testName;
    }

    public void setTestType(String newTestType) {
        testName = newTestType;
    }

    /**
     * Print name of the test.  Print name is usually a short version of the test name.
     */
    public String getPrintName() {
        return printName;
    }

    public void setPrintName(String newPrintName) {
        printName = newPrintName;
    }

    /**
     * User initials who verified the result.
     */
    public String getVerifiedBy() {
        return verifiedBy;
    }

    public void setVerifiedBy(String newVerifiedBy) {
        verifiedBy = newVerifiedBy;
    }

    /**
     * Flag indicatinf whether result is out of range.
     * L=low, L*=critical low, H=High, H*=critical High
     */
    public String getInterpretation() {
        return this.interpretation;
    }

    public void setInterpretation(String interpretation) {
        this.interpretation = interpretation;
    }

    /**
     * Reference range for result.
     */
    public String getReferenceRange() {
        return referenceRange;
    }

    public void setReferenceRange(String newReferenceRange) {
        referenceRange = newReferenceRange;
    }

    /**
     * Status of result.
     */
    public String getResultStatus() {
        return resultStatus;
    }

    public void setResultStatus(String newResultStatus) {
        resultStatus = newResultStatus;
    }

    /**
     * Units for result.
     */
    public String getUnits() {
        return units;
    }

    public void setUnits(String newUnits) {
        units = newUnits;
    }

    /**
     * Comment associated with result.
     */
    public String getComment() {
        return comment;
    }

    public void setComment(String newComment) {
        comment = newComment;
    }

    /**
     * Calendar test was resulted. Equivalent to Collection Date.
     */
    public Calendar getResultDate() {
        return resultDate;
    }

    public void setResultDate(Calendar newResultDate) {
        resultDate = newResultDate;
    }

    /**
     * Specimen for the result. Equivalent to Topography or collectionSample.
     */
    public String getSpecimen() {
        return specimen;
    }

    public void setSpecimen(String newSpecimen) {
        specimen = newSpecimen;
    }

    public Calendar getOrderDate() {
        return orderDate;
    }

    public void setOrderDate(Calendar orderDate) {
        this.orderDate = orderDate;
    }

    public String getFacilityName() {
        return facilityName;
    }

    public void setFacilityName(String facilityName) {
        this.facilityName = facilityName;
    }

    public String getStationNumber() {
        return stationNumber;
    }

    public void setStationNumber(String stationNumber) {
        this.stationNumber = stationNumber;
    }

    public String getRequestingLocation() {
        return requestingLocation;
    }

    public void setRequestingLocation(String requestingLocation) {
        this.requestingLocation = requestingLocation;
    }

    public String getPerformingLabName() {
        return performingLabName;
    }

    public void setPerformingLabName(String performingLabName) {
        this.performingLabName = performingLabName;
    }

    public String getPerformingLabPhone() {
        return performingLabPhone;
    }

    public void setPerformingLabPhone(String performingLabPhone) {
        this.performingLabPhone = performingLabPhone;
    }

    public String getPerformingLabStreet1() {
        return performingLabStreet1;
    }

    public void setPerformingLabStreet1(String performingLabStreet1) {
        this.performingLabStreet1 = performingLabStreet1;
    }

    public String getPerformingLabStreet2() {
        return performingLabStreet2;
    }

    public void setPerformingLabStreet2(String performingLabStreet2) {
        this.performingLabStreet2 = performingLabStreet2;
    }

    public String getPerformingLabStreet3() {
        return performingLabStreet3;
    }

    public void setPerformingLabStreet3(String performingLabStreet3) {
        this.performingLabStreet3 = performingLabStreet3;
    }

    public String getPerformingLabCity() {
        return performingLabCity;
    }

    public void setPerformingLabCity(String performingLabCity) {
        this.performingLabCity = performingLabCity;
    }

    public String getPerformingLabStateProvince() {
        return performingLabStateProvince;
    }

    public void setPerformingLabStateProvince(String performingLabStateProvince) {
        this.performingLabStateProvince = performingLabStateProvince;
    }

    public String getPerformingLabPostalCode() {
        return performingLabPostalCode;
    }

    public void setPerformingLabPostalCode(String performingLabPostalCode) {
        this.performingLabPostalCode = performingLabPostalCode;
    }

    public String getOrderComment() {
        return orderComment;
    }

    public void setOrderComment(String orderComment) {
        this.orderComment = orderComment;
    }

    /**
     * Quantity for Microbiology lab result
     *
     * @return
     */
    public String getQuantity() {
        return quantity;
    }

    public void setQuantity(String quantity) {
        this.quantity = quantity;
    }

    public boolean isSensitive() {
        return sensitive;
    }

    public void setSensitive(boolean sensitive) {
        this.sensitive = sensitive;
    }

    public Calendar getCollectedDate() {
        return collectedDate;
    }

    public void setCollectedDate(Calendar collectedDate) {
        this.collectedDate = collectedDate;
    }

    public String getAccessionComment() {
        return accessionComment;
    }

    public void setAccessionComment(String accessionComment) {
        this.accessionComment = accessionComment;
    }
}