/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

public class Medication extends DataBean {
    /**
     * Date the medication was ordered.
     */
    private Calendar orderDate;

    /**
     * Date the prescription was last filled.
     */
    private Calendar lastFilledDate;

    /**
     * The stop date of an inpatient medication.
     */
    private Calendar stopDate;

    /**
     * The stop date of an inpatient medication.
     */
    private Calendar startDate;

    /**
     * Name of drug ordered.
     */
    private String drugName;
    /**
     * Number of refills for a prescription.
     */
    private String refills;
    /**
     * Sig code of a prescription, Administration instruction of an inpatient medication.
     */
    private String sigCode;
    /**
     * Ordering user of the medication/prescription.
     */
    private Provider orderingProvider;
    /**
     * Ordering user of the medication/prescription.
     */
    private Provider currentProvider;
    /**
     * Quantity ordered for an outpatient prescription or inpatient medication.
     */
    private String quantity;
    /**
     * Days supply of the outpatient prescription.
     */
    private String daysSupply;

    /**
     * Pharmacy name where the outpatient prescription was last dispensed.
     */
    private String lastDispensingPharmacy;
    /**
     * Internal ID of the medication.  Used to retrieve the detail of a medications/ prescription.
     */
    private String medId;
    /**
     * Flag to indicate if the prescription/medication is active. 1=active, 0=inactive
     */
    private String active;
    /**
     * User comments associated with the medication/prescription.
     */
    private String comment;

    private String status;

    /**
     * The type of medication (I = Inpatient, O = Outpatient)
     */
    private String medType;
    private String rxNumber;

    private String pharmacyId;
    private String units;
    private boolean childResistant;
    private String refillsRemaining;
    private Calendar fillExpirationDate;

    private PrescriptionFill[] prescriptionFills;

    /**
     * The IEN for the order pertaining to this medication
     */
    private String orderId;


    /**
     * Date the medication was ordered.
     */
    public Calendar getOrderDate() {
        return orderDate;
    }

    public void setOrderDate(Calendar orderDate) {
        this.orderDate = orderDate;
    }

    public Calendar getLastFilledDate() {
        return lastFilledDate;
    }

    public void setLastFilledDate(Calendar lastFilledDate) {
        this.lastFilledDate = lastFilledDate;
    }

    /**
     * Name of drug.
     */
    public String getDrugName() {
        return drugName;
    }

    public void setDrugName(String newDrugName) {
        drugName = newDrugName;
    }

    /**
     * Number of refills for an Outpatient prescription.
     */
    public String getRefills() {
        return refills;
    }

    public void setRefills(String newRefills) {
        refills = newRefills;
    }

    /**
     * Sig code of the prescription.
     * Administration instructions/Schedule of inpatient medication.
     */
    public String getSigCode() {
        return sigCode;
    }

    public void setSigCode(String newSigCode) {
        sigCode = newSigCode;
    }

    /**
     * User ordering the medicaiton/prescription.
     */
    public Provider getOrderingProvider() {
        return orderingProvider;
    }

    public void setOrderingProvider(Provider newOrderingProvider) {
        orderingProvider = newOrderingProvider;
    }

    /**
     * Sets the Ordering provider name. If the Provider has not yet been set, creates a new Provider.
     * Only the name will be set
     *
     * @param provider The Provider's name
     */
    public void setOrderingProvider(String provider) {
        this.orderingProvider = new Provider();
        this.orderingProvider.setName(provider);
    }

    /**
     * Quantity of the medication/prescription.
     */
    public String getQuantity() {
        return quantity;
    }

    public void setQuantity(String newQuantity) {
        quantity = newQuantity;
    }

    /**
     * Days supply of prescription.
     */
    public String getDaysSupply() {
        return daysSupply;
    }

    public void setDaysSupply(String newDaysSupply) {
        daysSupply = newDaysSupply;
    }


    public Calendar getStopDate() {
        return stopDate;
    }

    public void setStopDate(Calendar stopDate) {
        this.stopDate = stopDate;
    }

    public Calendar getStartDate() {
        return startDate;
    }

    public void setStartDate(Calendar startDate) {
        this.startDate = startDate;
    }

    /**
     * Name of pharmacy that last dispensed the outpatient prescription.
     */
    public String getLastDispensingPharmacy() {
        return lastDispensingPharmacy;
    }

    public void setLastDispensingPharmacy(String newLastDispensingPharmacy) {
        lastDispensingPharmacy = newLastDispensingPharmacy;
    }

    /**
     * Internal ID of medication/prescription.  Used to retrieve the details of the
     * medication/prescription.
     */
    public String getMedId() {
        return medId;
    }

    public void setMedId(String newRxId) {
        medId = newRxId;
    }

    /**
     * Active =1 , Inactive = 0
     */
    public String getActive() {
        return active;
    }

    public void setActive(String newRxActive) {
        active = newRxActive;
    }

    /**
     * User comments associated with the medication/prescription.
     */
    public String getComment() {
        return comment;
    }

    public void setComment(String newComment) {
        comment = newComment;
    }

    /**
     * I = Inpatient Medication, O = Outpatient Prescription
     */
    public String getMedType() {
        return medType;
    }

    /**
     * I = Inpatient Medication, O = Oupatient Prescription
     */
    public void setMedType(String newMedType) {
        medType = newMedType;
    }

    public String getRxNumber() {
        return rxNumber;
    }

    public void setRxNumber(String newRxNumber) {
        rxNumber = newRxNumber;
    }

    public String getPharmacyId() {
        return pharmacyId;
    }

    public void setPharmacyId(String pharmacyId) {
        this.pharmacyId = pharmacyId;
    }

    public PrescriptionFill[] getPrescriptionFills() {
        PrescriptionFill[] rtc = null;
        if (prescriptionFills != null) {
            rtc = prescriptionFills.clone();
        }
        return rtc;
    }

    public void setPrescriptionFills(PrescriptionFill[] prescriptionFills) {
        if (prescriptionFills == null) {
            this.prescriptionFills = null;
        } else {
            this.prescriptionFills = prescriptionFills.clone();            
        }

    }

    /**
     * The ID for the order pertaining to this medication
     */
    public String getOrderId() {
        return orderId;
    }

    /**
     * The ID for the order pertaining to this medication
     */
    public void setOrderId(String orderId) {
        this.orderId = orderId;
    }

    /**
     * The patient's current provider
     */
    public Provider getCurrentProvider() {
        return currentProvider;
    }

    /**
     * The patient's current provider
     */
    public void setCurrentProvider(Provider currentProvider) {
        this.currentProvider = currentProvider;
    }

    /**
     * Sets the Current provider name. If the Provider has not yet been set, creates a new Provider.
     * Only the name will be set
     *
     * @param provider The Provider's name
     */
    public void setCurrentProvider(String provider) {
        this.currentProvider = new Provider();
        this.currentProvider.setName(provider);
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getUnits() {
        return units;
    }

    public void setUnits(String units) {
        this.units = units;
    }

    public boolean isChildResistant() {
        return childResistant;
    }

    public void setChildResistant(boolean childResistant) {
        this.childResistant = childResistant;
    }

    public String getRefillsRemaining() {
        return refillsRemaining;
    }

    public void setRefillsRemaining(String refillsRemaining) {
        this.refillsRemaining = refillsRemaining;
    }

    public Calendar getFillExpirationDate() {
        return fillExpirationDate;
    }

    public void setFillExpirationDate(Calendar fillExpirationDate) {
        this.fillExpirationDate = fillExpirationDate;
    }
}