/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import gov.va.med.rpc.KeyValuePair;

import java.util.List;

/**
 * This is the generic patient object used for patient lookup. This class has a limited number of attributes.
 */
public class Patient {

    /**
     * Name of patient.
     */
    private String name;
    /**
     * Family Member Prefix (FMP).
     */
    private String FMP;
    /**
     * Social Security Number of patient.
     */
    private String SSN;
    /**
     * Date of  Birth of patient
     */
    private String dob;
    /**
     * Gender of patient.  F = Female, M = Male, U = Unknown
     */
    private String gender;

    /**
     * Age of patient.
     */
    private String age;
    /**
     * VA sensitive patient indicator.  True or False.
     */
    private boolean sensitive;

    /**
     * Patient vista sites list
     */
    private List<Site> vistaSites;

    /**
     * Patient iens mapped by siteCode
     */
    private List<KeyValuePair<String, String>> patientIens;

    /**
     * DOD EDIPI
     */
    private String EDIPI;

    /**
     * DOD Unit Number
     */
    private String ahltaUnitNumber;

    /**
     * VistA ICN
     */
    private String ICN;

    /**
     * Sponsor SSN
     */
    private String sponsorSSN;

    public Patient() {
        sensitive = false;
    }

    /**
     * Name of patient.
     */
    public String getName() {
        return name;
    }

    public void setName(String newName) {
        name = newName;
    }

    /**
     * Social Security Number of patient.
     */
    public String getSSN() {
        return SSN;
    }

    public void setSSN(String newSSN) {
        SSN = newSSN;
    }

    /**
     * FMP of patient.
     */
    public String getFMP() {
        return FMP;
    }

    public void setFMP(String newFMP) {
        FMP = newFMP;
    }

    /**
     * Date of  Birth of patient.
     */
    public String getDob() {
        return dob;
    }

    public void setDob(String newDob) {
        dob = newDob;
    }

    /**
     * gender of patient (F=Female, M=Male)
     */
    public String getGender() {
        return gender;
    }

    public void setGender(String newGender) {
        gender = newGender;
    }

    /**
     * Age of patient
     */
    public String getAge() {
        return age;
    }

    public void setAge(String newAge) {
        age = newAge;
    }

    /**
     * VA Sensitive indicator
     */
    public boolean isSensitive() {
        return sensitive;
    }

    public void setSensitive(boolean sensitive) {
        this.sensitive = sensitive;
    }

    public String getICN() {
        return ICN;
    }

    public void setICN(String ICN) {
        this.ICN = ICN;
    }

    public List<Site> getVistaSites() {
        return vistaSites;
    }

    public void setVistaSites(List<Site> vistaSites) {
        this.vistaSites = vistaSites;
    }

    public String getEDIPI() {
        return EDIPI;
    }

    public void setEDIPI(String EDIPI) {
        this.EDIPI = EDIPI;
    }

    public String getAhltaUnitNumber() {
        return ahltaUnitNumber;
    }

    public void setAhltaUnitNumber(String ahltaUnitNumber) {
        this.ahltaUnitNumber = ahltaUnitNumber;
    }

    public List<KeyValuePair<String, String>> getPatientIens() {
        return patientIens;
    }

    public void setPatientIens(List<KeyValuePair<String, String>> patientIens) {
        this.patientIens = patientIens;
    }

    public String getSponsorSSN() { return sponsorSSN; }

    public void setSponsorSSN(String sponsorSSN) { this.sponsorSSN = sponsorSSN; }
}
