/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

/**
 * This class represents Appointments associated with a patient;
 */
public class PatientAppointments extends DataBean {
    /**
     * Date/time of appointment.
     */
    private Calendar apptDate;
    /**
     * Name of clinic associated with appointment.
     */
    private String clinic;
    /**
     * Status of appointment.
     */
    private String status;
    /**
     * Type of appointment.
     */
    private String apptType;
    /**
     * Provider associated with appointment.
     */
    private Provider provider;
    /**
     * Reason for appointment.
     */
    private String reason;
    /**
     * Internal id of clinic for this appointment.
     */
    private String clinicId;
    /**
     * Internal ID of user associated with appointment.  Not populated for vista
     * appointments.
     */
    private String userIen;
    /**
     * User class of the user associated with appointment.  Not populated for
     * vista appointments.
     */
    private String providerClass;
    /**
     * Phone number of user associated with appointment. Not populated for vista
     * appointments.
     */
    private String providerPhone;
    /**
     * pager number of user associated with this appointment.  This property
     * is not populated for vista appointments.
     */
    private String providerPager;
    /**
     * internal id of appointment.  this is used to retieve notes, diagnoses and/or
     * procedures associated with an appointment/visit.
     */
    private String apptId;

    private String patientClass;
    private String serviceCategoryCode;
    private String serviceCategoryName;
    private String service;
    private String visitString;

    private String clinicStopCode;
    private String clinicStopName;
    private String providerCode;

    private String creditStopCode;
    private String creditStopName;

    /**
     * Date/Time of appointment.
     */
    public Calendar getApptDate() {
        return apptDate;
    }

    public void setApptDate(Calendar newApptDate) {
        apptDate = newApptDate;
    }

    /**
     * Clinic name for the appointment.
     */
    public String getClinic() {
        return clinic;
    }

    public void setClinic(String newClinic) {
        clinic = newClinic;
    }

    /**
     * Status of appointment.
     */
    public String getStatus() {
        return status;
    }

    public void setStatus(String newStatus) {
        status = newStatus;
    }

    /**
     * Type of Appointment.
     */
    public String getApptType() {
        return apptType;
    }

    public void setApptType(String newApptType) {
        apptType = newApptType;
    }

    /**
     * Name of User for the appointment. Not defined for vista data.
     */
    public Provider getProvider() {
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    /**
     * Sets the provider name. If the Provider has not yet been set, creates a new Provider.
     * Only the name will be set
     *
     * @param provider The Provider's name
     */
    public void setProvider(String provider) {
        this.provider = new Provider();
        this.provider.setName(provider);
    }

    /**
     * Reason for appointment.
     */
    public String getReason() {
        return reason;
    }

    public void setReason(String newReason) {
        reason = newReason;
    }

    /**
     * Internal id of clinic.
     */
    public String getClinicId() {
        return clinicId;
    }

    public void setClinicId(String newClinicId) {
        clinicId = newClinicId;
    }

    /**
     * Internal ID of user.  Not set for vista data.
     */
    public String getUserIen() {
        return userIen;
    }

    public void setUserIen(String newUserIen) {
        userIen = newUserIen;
    }

    /**
     * User Class of user associated with appointment. Not set for vista data.
     */
    public String getProviderClass() {
        return providerClass;
    }

    public void setProviderClass(String newProviderClass) {
        providerClass = newProviderClass;
    }

    /**
     * Phone number for clinic if vista data, otherwise phone number of user.
     */
    public String getProviderPhone() {
        return providerPhone;
    }

    public void setProviderPhone(String newProviderPhone) {
        providerPhone = newProviderPhone;
    }

    /**
     * User's pager number. Not defined for vista data.
     */
    public String getProviderPager() {
        return providerPager;
    }

    public void setProviderPager(String newProviderPager) {
        providerPager = newProviderPager;
    }

    /**
     * Internal Id of appointment.  Used to retrieve notes, procedures or diagnoses
     * associated with an appointment.
     */
    public String getApptId() {
        return apptId;
    }

    public void setApptId(String newApptId) {
        apptId = newApptId;
    }

    public String getProviderCode() {
        return providerCode;
    }

    public void setProviderCode(String providerCode) {
        this.providerCode = providerCode;
    }

    public String getPatientClass() {
        return patientClass;
    }

    public void setPatientClass(String patientClass) {
        this.patientClass = patientClass;
    }

    public String getServiceCategoryCode() {
        return serviceCategoryCode;
    }

    public void setServiceCategoryCode(String serviceCategoryCode) {
        this.serviceCategoryCode = serviceCategoryCode;
    }

    public String getServiceCategoryName() {
        return serviceCategoryName;
    }

    public void setServiceCategoryName(String serviceCategoryName) {
        this.serviceCategoryName = serviceCategoryName;
    }

    public String getVisitString() {
        return visitString;
    }

    public void setVisitString(String visitString) {
        this.visitString = visitString;
    }

    public String getClinicStopCode() {
        return clinicStopCode;
    }

    public void setClinicStopCode(String clinicStopCode) {
        this.clinicStopCode = clinicStopCode;
    }

    public String getClinicStopName() {
        return clinicStopName;
    }

    public void setClinicStopName(String clinicStopName) {
        this.clinicStopName = clinicStopName;
    }

    public String getService() {
        return service;
    }

    public void setService(String service) {
        this.service = service;
    }

    public String getCreditStopCode() {
        return creditStopCode;
    }

    public void setCreditStopCode(String creditStopCode) {
        this.creditStopCode = creditStopCode;
    }

    public String getCreditStopName() {
        return creditStopName;
    }

    public void setCreditStopName(String creditStopName) {
        this.creditStopName = creditStopName;
    }
}