/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;


/**
 * Class representing patient problems.
 */
public class Problem extends DataBean {
    /**
     * Internal ID of problem. Used to retrieve problem details.
     */
    private String id;
    /**
     * ICD code of the problem.
     */
    private String icdCode;
    /**
     * Description of problem.
     */
    private String description;
    /**
     * Acuity of problem (Acute or chronic)
     */
    private String acuity;
    /**
     * Date of  onset of the problem.
     */
    private String onsetDate;
    /**
     * Status of a problem.
     */
    private String status;
    /**
     * Date the problem was last modified.
     */
    private Calendar lastModifiedDate;
    /**
     * Indicator as to whether this is service connected. (YES if service connected)
     */
    private String serviceConnected;

    // Values from VISTA RPC: ORQQPL PROBLEM LIST
    /**
     * Any special exposures (i.e., AO for Agent Orange, IR for Ionizing Radiation, etc.)
     */
    private String specialExposure;
    /**
     * The problem condition (either T for Transcribed or P for Permanent)
     */
    private String condition;
    /**
     * Location IEN
     */
    private String locationIEN;
    /**
     * Location type (i.e., C for Clinic, W for Ward, OR for Operating Room, etc.)
     */
    private String locationType;
    /**
     * Provider IEN
     */
    private String providerIEN;
    /**
     * Service IEN
     */
    private String serviceIEN;
    /**
     * Service Name
     */
    private String serviceName;
    /**
     * Priority
     */
    private String priority;
    /**
     * Comment flag (0 or 1) The comment flag piece can be quite useful for knowing the problems with a comment and
     * triggering the ORQQPL PROB COMMENTS RPC to present those comments or using this data to activate or inactivate
     * a Comments button or action, for example.
     */
    private String hasComment;
    /**
     * Combined SC Conditions. A combination of the service connection and special exposures pieces (if the problem is
     * related to the Service Connected condition of the patient)
     */
    private String scConditions;
    /**
     *  A flag if the ICD code used for the problem is inactive in the ICD file (flagged with a # if it is inactive;
     *  otherwise, it is NULL).
     */
    private String inactiveICDCode;
    /**
     * Hospital location where the problem was first identified.
     */
    private String hospitalLocation;
    /**
     * Provider associated with the patient's problem.
     */
    private String responsibleProvider;
    /**
     * Date the problem was first recorded.  This property is only populated when
     * requesting problem details.
     */
    private Calendar recordedDate;

    /**
     * Inpatient or Outpatient Problem
     */
    private String inOutpatient;


    /**
     * Interal Id of problem. Used to request problem details.
     */
    public String getId() {
        return id;
    }

    public void setId(String newId) {
        id = newId;
    }

    /**
     * ICD code for problem.
     */
    public String getIcdCode() {
        return icdCode;
    }

    public void setIcdCode(String newIcdCode) {
        icdCode = newIcdCode;
    }

    /**
     * Problem description text.
     */
    public String getDescription() {
        return description;
    }

    public void setDescription(String newDescription) {
        description = newDescription;
    }

    /**
     * Acuity of problem (Acute or Chronic)
     */
    public String getAcuity() {
        return acuity;
    }

    public void setAcuity(String newAcuity) {
        acuity = newAcuity;
    }

    /**
     * Date of  onset for the problem.
     */
    public String getOnsetDate() {
        return onsetDate;
    }

    public void setOnsetDate(String newOnsetDate) {
        onsetDate = newOnsetDate;
    }

    /**
     * Status of problem (Active or Inactive)
     */
    public String getStatus() {
        return status;
    }

    public void setStatus(String newStatus) {
        status = newStatus;
    }

    /**
     * Calendar problem last modified.
     */
    public Calendar getLastModifiedDate() {
        return lastModifiedDate;
    }

    public void setLastModifiedDate(Calendar newLastModifiedDate) {
        lastModifiedDate = newLastModifiedDate;
    }

    /**
     * Indicates whether this problem is service connected. (YES= service connected).
     */
    public String getServiceConnected() {
        return serviceConnected;
    }

    /**
     * Indicates whether this problem is service connected. (YES = service connected)
     */
    public void setServiceConnected(String newServiceConnected) {
        serviceConnected = newServiceConnected;
    }

    // Values from VISTA RPC: ORQQPL PROBLEM LIST
    /**
     * Any special exposures (i.e., AO for Agent Orange, IR for Ionizing Radiation, etc.)
     */
    public String getSpecialExposure() {
        return specialExposure;
    }

    public void setSpecialExposure(String specialExposure) {
        this.specialExposure = specialExposure;
    }

    /**
     * The problem condition (either T for Transcribed or P for Permanent)
     */    public String getCondition() {
        return condition;
    }

    public void setCondition(String condition) {
        this.condition = condition;
    }

    /**
     * Location IEN
     */
    public String getLocationIEN() {
        return locationIEN;
    }

    public void setLocationIEN(String locationIEN) {
        this.locationIEN = locationIEN;
    }

    /**
     * Location type (i.e., C for Clinic, W for Ward, OR for Operating Room, etc.)
     */
    public String getLocationType() {
        return locationType;
    }

    public void setLocationType(String locationType) {
        this.locationType = locationType;
    }

    /**
     * Provider IEN
     */
    public String getProviderIEN() {
        return providerIEN;
    }

    public void setProviderIEN(String providerIEN) {
        this.providerIEN = providerIEN;
    }

    /**
     * Service IEN
     */
    public String getServiceIEN() {
        return serviceIEN;
    }

    public void setServiceIEN(String serviceIEN) {
        this.serviceIEN = serviceIEN;
    }

    /**
     * Service Name
     */
    public String getServiceName() {
        return serviceName;
    }

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    /**
     * Priority
     */
    public String getPriority() {
        return priority;
    }

    public void setPriority(String priority) {
        this.priority = priority;
    }

    public String getHasComment() {
        return hasComment;
    }

    /**
     * Comment flag (0 or 1) The comment flag piece can be quite useful for knowing the problems with a comment and
     * triggering the ORQQPL PROB COMMENTS RPC to present those comments or using this data to activate or inactivate
     * a Comments button or action, for example.
     */
    public void setHasComment(String hasComment) {
        this.hasComment = hasComment;
    }

    /**
     * Combined SC Conditions. A combination of the service connection and special exposures pieces (if the problem is
     * related to the Service Connected condition of the patient)
     */
    public String getScConditions() {
        return scConditions;
    }

    public void setScConditions(String scConditions) {
        this.scConditions = scConditions;
    }

    /**
     *  A flag if the ICD code used for the problem is inactive in the ICD file (flagged with a # if it is inactive;
     *  otherwise, it is NULL).
     */
    public String getInactiveICDCode() {
        return inactiveICDCode;
    }

    public void setInactiveICDCode(String inactiveICDCode) {
        this.inactiveICDCode = inactiveICDCode;
    }

    /**
     * Name of hospital location where this problem was identified.
     */
    public String getHospitalLocation() {
        return hospitalLocation;
    }

    public void setHospitalLocation(String newHospitalLocation) {
        hospitalLocation = newHospitalLocation;
    }

    /**
     * Responsible provider assigned to problem.
     */
    public String getResponsibleProvider() {
        return responsibleProvider;
    }

    public void setResponsibleProvider(String newResponsibleProvider) {
        responsibleProvider = newResponsibleProvider;
    }

    /**
     * Calendar problem was recorded.  This will only be populated when requesting problem
     * details.
     */
    public Calendar getRecordedDate() {
        return recordedDate;
    }

    public void setRecordedDate(Calendar newRecordedDate) {
        recordedDate = newRecordedDate;
    }

    public String getInOutpatient() {
        return inOutpatient;
    }

    public void setInOutpatient(String inOutpatient) {
        this.inOutpatient = inOutpatient;
    }
}