/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * User: VHAHONUYEHAC
 * Date: 8/2/11
 * Time: 10:39 AM
 */
public class TestConfig
{
    private static final Logger logger = LoggerFactory.getLogger(TestConfig.class);

    static
    {
        //point to debug properties
        System.setProperty("log4j.configuration", "log4j-debug.xml");
    }

    private static Properties testConfigProp = new Properties();

    public static final String PROVIDER_ACCESS_CODE = "testconfig.provider.access.code";
    public static final String PROVIDER_VERIFY_CODE = "testconfig.provider.verify.code";
    public static final String PROVIDER_SITE_CODE   = "testconfig.provider.sitecode";
    public static final String PROVIDER_LOGIN_SITE_CODE = "testconfig.provider.login.sitecode";
    public static final String PATIENT_NAME = "testconfig.patient.name";
    public static final String PATIENT_SSN  = "testconfig.patient.ssn";
    public static final String PATIENT_ICN  = "testconfig.patient.icn";
    public static final String EHR_PATIENT_IEN = "testconfig.ehr.patient.ien";
    public static final String EHR_SITECODE = "testconfig.ehr.datasource.sitecode";

    public static final String RPC_SITES_FILE = "testconfig.sites.rpc.file";
    public static final String CACHE_SITES_FILE = "testconfig.sites.cache.file";

    public static final String ENTERPRISE_SITE = "testconfig.enterprise.site";
    public static final String ENTERPRISE_USER_IEN = "testconfig.enterprise.user.ien";
    public static final String ENTERPRISE_USER_NAME = "testconfig.enterprise.user.name";
    public static final String ENTERPRISE_PATIENT_IEN = "testconfig.enterprise.patient.ien";

    public static String get(String property)
    {
        if (testConfigProp.isEmpty())
        {
            init();
        }

        return (String) testConfigProp.get(property);

    }

    private static void init()
    {
        ClassLoader classLoader = TestConfig.class.getClassLoader();

        if (classLoader == null) throw new RuntimeException("Classloader is null");

        InputStream inputStream = null;

        try {
            inputStream = classLoader.getResourceAsStream("testconfig.properties");

            testConfigProp.load(inputStream);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        finally {
            if (inputStream != null)
            {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    logger.error(e.getMessage());
                }
            }
        }
    }
}
