/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.ClinicalRemindersDataService;
import gov.va.med.vds.vistabeans.ClinicalReminder;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class ClinicalRemindersDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientClinicalReminders()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            ClinicalRemindersDataService remindersService = new ClinicalRemindersDataService();
            List<ClinicalReminder> reminders = remindersService.getPatientClinicalReminders(queryBean);
            assertTrue(reminders.size() > 0);
            for(ClinicalReminder reminder : reminders)
            {
                assertFalse(StringUtils.isEmpty(reminder.getSourceProtocol()));
                assertFalse(reminder.getSite() == null);
                assertFalse(reminder.getDescription().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientClinicalReminderDetail()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            ClinicalRemindersDataService remindersService = new ClinicalRemindersDataService();
            List<ClinicalReminder> reminders = remindersService.getPatientClinicalReminders(queryBean);
            assertTrue(reminders.size() > 0);

            ClinicalReminder reminderBean = reminders.get(reminders.size()-1);
            queryBean = setupQuery(user, patient, null, null);
            queryBean.setRecordSite(reminderBean.getSite());
            queryBean.setItemId(reminderBean.getId());
            FreeTextReport reminder = remindersService.getClinicalReminderDetail(queryBean);
            assertFalse(StringUtils.isEmpty(reminder.getReportText()));
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }        
    }
}
