/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.MedicationsDataService;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.Medication;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class MedicationsDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientMedicationsOutpatient()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(1980, Calendar.OCTOBER, 1);
            endDate.set(2013, Calendar.NOVEMBER, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            queryBean.setStatus("O");

            MedicationsDataService medsService = new MedicationsDataService();
            List<Medication> meds = medsService.getPatientMedications(queryBean);
            assertTrue(meds.size() > 0);
            for(Medication medInpt : meds)
            {
                assertFalse(StringUtils.isEmpty(medInpt.getSourceProtocol()));
                assertFalse(medInpt.getSite() == null);
                assertTrue(medInpt.getMedType().equals("O") || medInpt.getMedType().equals("N"));
                assertFalse(medInpt.getDrugName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientOutpatientMedDetails()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2007, 0, 1);
            endDate.set(2010, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            queryBean.setStatus("O");

            MedicationsDataService medsService = new MedicationsDataService();
            List<Medication> meds = medsService.getPatientMedications(queryBean);
            assertTrue(meds.size() > 0);

            queryBean = new QueryBean();
            queryBean.setUser(user);
            queryBean.setPatient(patient);
            queryBean.setRecordSite(meds.get(0).getSite());
            queryBean.setItemId(meds.get(0).getMedId());
            FreeTextReport rx = medsService.getMedicationDetail(queryBean);
            assertTrue(rx.getReportText().length() > 0);
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientMedicationsInpatient()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(1998, Calendar.DECEMBER, 1);
            endDate.set(1999, Calendar.DECEMBER, 5);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            queryBean.setStatus("I");

            MedicationsDataService medsService = new MedicationsDataService();
            List<Medication> meds = medsService.getPatientMedications(queryBean);
            assertTrue(meds.size() > 0);
            for(Medication medInpt : meds)
            {

                assertFalse(StringUtils.isEmpty(medInpt.getSourceProtocol()));
                assertFalse(medInpt.getSite() == null);
                assertFalse(medInpt.getMedType().equals("O") || medInpt.getMedType().equals("N"));
                assertFalse(medInpt.getDrugName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
