/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.RadiologyDataService;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.RadiologyExam;
import gov.va.med.vds.vistabeans.RadiologyReport;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class RadiologyDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientRads()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2007, 0, 1);
            endDate.set(2011, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            
            RadiologyDataService radService = new RadiologyDataService();
            List<RadiologyReport> rads = radService.getPatientRads(queryBean);
            assertTrue(rads.size() > 0);
            for(RadiologyExam rad : rads)
            {
                assertFalse(StringUtils.isEmpty(rad.getSourceProtocol()));
                assertFalse(rad.getSite() == null);
                assertFalse(rad.getStudy().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientRadDetails()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2007, 0, 1);
            endDate.set(2011, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            RadiologyDataService radService = new RadiologyDataService();
            List<RadiologyReport> rads = radService.getPatientRads(queryBean);
            assertTrue(rads.size() > 0);

            for(RadiologyExam rad : rads)
            {
                queryBean = setupQuery(user, patient, startDate, endDate);
                queryBean.setItemId(rad.getExamId());
                queryBean.setStatus(rad.getAccessionNumber());
                queryBean.setRecordSite(rad.getSite());
                RadiologyReport report = radService.getRadiologyReport(queryBean);
                assertNotNull(report.getReportText());
                break;
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
