/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.provider;

import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.provider.dataservice.ProviderAppointmentsDataService;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.*;

/**
 * Gets the list of appointments for a specific user.
 */
public class AppointmentDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetAppointments()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2010, 2, 1);
            endDate.set(2010, 3, 1);

            user = loginTestProvider("59607", site);
            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            ProviderAppointmentsDataService aptsService = new ProviderAppointmentsDataService();
            PatientAppointments[] apts = aptsService.getProviderAppointments(queryBean);
            assertTrue(apts.length > 0);
            for(PatientAppointments apt : apts)
            {
                assertFalse(StringUtils.isEmpty(apt.getSourceProtocol()));
                assertFalse(apt.getSite() == null);
                assertFalse(apt.getClinic().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }
}
