/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionStrategy;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.patient.dao.rpc.AppointmentsRPC;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.PatientAppointments;
import org.junit.Before;
import org.junit.Test;

import java.lang.reflect.Method;
import java.util.Calendar;
import java.util.List;
import java.util.logging.Logger;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.*;

/**
 *
 *
 */
public class AppointmentsRPCTest extends BaseRPCTest
{
    private static final Logger logger = Logger.getLogger(AppointmentsRPCTest.class.getName());

    @Before
    public void setUp()
    {
        vistaConnect();
        testUserLogin();

        patientId = "15610";
    }


    //CAPRI_CONTEXT needed for appointments
//    @Override
//    protected void testUserLogin()
//    {
//        UserRPC userRPC = new UserRPC(vistaCxn);
//        String accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
//        String verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);
//        try {
//            userRPC.login(accessCode, verifyCode, VistaConstants.CAPRI_CONTEXT);
//        } catch (RPCException e) {
//            logger.severe(e.getMessage());
//        }
//    }

    @Test
    public void testGetAppointments() throws RPCException
    {
//        RPCConnection mockRPCConnection = mock(RPCConnection.class);
//        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
//        when(mockRPCConnection.query(matches(".*PAST.*"))).thenReturn(PAST_APPOINTMENTS_REPORT_TEXT);
//        when(mockRPCConnection.query(matches(".*FUTURE.*"))).thenReturn(FUTURE_APPOINTMENTS_REPORT_TEXT);
//
//        AppointmentsRPC apptRPC = new AppointmentsRPC(mockRPCConnection);
//        Calendar startDate = Calendar.getInstance();
//        Calendar endDate = Calendar.getInstance();
//        startDate.set(2009, 0, 1);
//        endDate.set(2011, 0, 1);
////
//        PatientAppointments[] appts = apptRPC.getAppointments(patientId, startDate, endDate);
//        assertTrue(appts.length == 20);

        AppointmentsRPC apptRPC = new AppointmentsRPC(vistaCxn);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2000, 0, 1);
        endDate.set(2015, 0, 1);

        PatientAppointments[] appts = apptRPC.getAppointmentsVPR(patientId, startDate, endDate);
        assertTrue(appts.length > 0);

    }

    @Test
    public void testGetEncountersReport() throws RPCException
    {
        AppointmentsRPC apptRPC = new AppointmentsRPC(vistaCxn);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2009, 0, 1);
        endDate.set(2010, 0, 1);

        FreeTextReport report= apptRPC.getEncountersReport(patientId, startDate, endDate);
        assertTrue(!report.getReportText().isEmpty());
    }

    @Test
    public void testToPatientAppointments() throws Exception
    {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);

        AppointmentsRPC apptRPC = new AppointmentsRPC(mockRPCConnection);
        String methodName = "toPatientAppointments";
        Class[] parameterTypes = new Class[2];
        parameterTypes[0] = String.class;
        parameterTypes[1] = String.class;
        Object[] parameters = new Object[2];
        Method method = apptRPC.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);

        parameters[0] = APPOINTMENTS_LIST_ORQQVS_VISITS_APPTS;
        parameters[1] = "";
        PatientAppointments[] appts = (PatientAppointments []) method.invoke(apptRPC, parameters);

        assertTrue(appts.length == 11);
    }

    @Test
    public void testParsePastAppointmentsReport() throws Exception
    {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);

        AppointmentsRPC apptRPC = new AppointmentsRPC(mockRPCConnection);
        String methodName = "parsePastOrFutureAppointmentsReport";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = apptRPC.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);

        parameters[0] = PAST_APPOINTMENTS_REPORT_TEXT;
        List<PatientAppointments> appts = (List<PatientAppointments>) method.invoke(apptRPC, parameters);

        assertTrue(appts.size() == 20);
    }

    @Test
    public void testParseFutureAppointmentsReport() throws Exception
    {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);

        AppointmentsRPC apptRPC = new AppointmentsRPC(mockRPCConnection);
        String methodName = "parsePastOrFutureAppointmentsReport";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = apptRPC.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);

        parameters[0] = FUTURE_APPOINTMENTS_REPORT_TEXT;
        List<PatientAppointments> appts = (List<PatientAppointments>) method.invoke(apptRPC, parameters);

        assertTrue(appts.size() == 1);
    }

    private static final String APPOINTMENTS_LIST_ORQQVS_VISITS_APPTS = "3120924.09;s^TEST MH CLINIC^sched:^3120924.09^6879074.91" + StringUtils.CRLF +
            "3121004.113;s^TEST MH2 CLINIC^sched:^3121004.113^6878994.887" + StringUtils.CRLF +
            "3121017.0915;s^TEST NUR CLINIC^sched:^3121017.0915^6878981.9085" + StringUtils.CRLF +
            "3121108.09;s^TEST OPTOMETRY CLINIC^sched:^3121108.09^6878890.91" + StringUtils.CRLF +
            "3121109.08;s^TEST DENTAL CLINIC^sched:^3121109.08^6878889.92" + StringUtils.CRLF +
            "6247870;v^TEST MH3 CLINIC^visited:^3120921.08^6879077.92" + StringUtils.CRLF +
            "6247869;v^TEST MH3 CLINIC^visited:^3120921.08^6879077.92" + StringUtils.CRLF +
            "6244362;v^TEST MH3 CLINIC^visited:^3120920.08^6879078.92" + StringUtils.CRLF +
            "6244361;v^TEST MH3 CLINIC^visited:^3120920.08^6879078.92" + StringUtils.CRLF +
            "6241310;v^TEST MH3 CLINIC^visited:^3120919.08^6879079.92" + StringUtils.CRLF +
            "6241309;v^TEST MH3 CLINIC^visited:^3120919.08^6879079.92";

    private static final String PAST_APPOINTMENTS_REPORT_TEXT = "Printed for data from -11908991 to 08/04/2014                  08/04/2014 17:14" + StringUtils.CRLF +
            "***********************  CONFIDENTIAL  SUMMARY   pg. 1 ************************" + StringUtils.CRLF +
            "IPOBELLOWS,JEFFRY RAY    523-54-9304                            DOB: 05/03/1947" + StringUtils.CRLF +
            " " + StringUtils.CRLF +
            "-------------------------- CVP - Past Clinic Visits --------------------------" + StringUtils.CRLF +
            "                                  " + StringUtils.CRLF +
            "12/06/2010 13:00  CHY MH SONES                           " + StringUtils.CRLF +
            "12/01/2010 14:20  CHY POD SCHWEINBERGER                  " + StringUtils.CRLF +
            "12/01/2010 09:40  ZZCHY PC POLVERINO                     " + StringUtils.CRLF +
            "11/29/2010 11:15  CHY ULTRASOUND                          CANCELLED BY CLINIC" + StringUtils.CRLF +
            "11/23/2010 15:30  CHY POD SCHWEINBERGER                  " + StringUtils.CRLF +
            "11/18/2010 09:30  CHY MH SONES                            CANCELLED BY CLINIC" + StringUtils.CRLF +
            "11/16/2010 13:00  CHY POD SCHWEINBERGER                  " + StringUtils.CRLF +
            "11/12/2010 08:00  CHY MINOR PRO SCHWEINBERG              " + StringUtils.CRLF +
            "11/09/2010 09:00  CHY POD SCHWEINBERGER RET               CANCELLED BY CLINIC" + StringUtils.CRLF +
            "11/02/2010 11:20  CHY POD SCHWEINBERGER                  " + StringUtils.CRLF +
            "10/27/2010 15:00  ZZCHY PT KIRMIS                         UNSCHEDULED" + StringUtils.CRLF +
            "10/27/2010 13:00  ZZCHY PC POLVERINO                     " + StringUtils.CRLF +
            "10/27/2010 11:20  CHY POD SCHWEINBERGER RET              " + StringUtils.CRLF +
            "10/25/2010 13:51  CHY TELEPHONE NURSING                   UNSCHEDULED" + StringUtils.CRLF +
            "10/19/2010 13:00  CHY MH TAFOYA                          " + StringUtils.CRLF +
            "10/12/2010 14:00  CHY POD SCHWEINBERGER                  " + StringUtils.CRLF +
            "10/04/2010 15:30  CHY POD SCHWEINBERGER RET              " + StringUtils.CRLF +
            "10/04/2010 09:59  CHY EMERGENCY ROOM                     " + StringUtils.CRLF +
            "09/09/2010 09:04  CHY MH GARDNER                          UNSCHEDULED" + StringUtils.CRLF +
            "09/09/2010 09:00  CHY MH GARDNER                         " + StringUtils.CRLF +
            " " + StringUtils.CRLF +
            "*** END ***************  CONFIDENTIAL  SUMMARY   pg. 1 ***********************";

    private static final String FUTURE_APPOINTMENTS_REPORT_TEXT = "Printed for data from 01/01/2007 to 01/01/2020                 08/07/2014 21:03" + StringUtils.CRLF +
            "***********************  CONFIDENTIAL  SUMMARY   pg. 1 ************************" + StringUtils.CRLF +
            "IPOBELLOWS,JEFFRY RAY    523-54-9304                            DOB: 05/03/1947" + StringUtils.CRLF +
            " " + StringUtils.CRLF +
            "--------------------------- CVF - Fut Clinic Visits ---------------------------" + StringUtils.CRLF +
            "                                  " + StringUtils.CRLF +
            "08/14/2014 09:00  CHY ANTICOAG                           " + StringUtils.CRLF +
            "      No data available                           " + StringUtils.CRLF +
            " " + StringUtils.CRLF +
            "*** END ***************  CONFIDENTIAL  SUMMARY   pg. 1 ***********************";

    private static final String VPR_VISITS = "<results version='1.1' timeZone='-0500'>\n" +
            "    <visits total='6'>" +
            "        <visit>\n" +
            "            <dateTime value='3050523.1'/>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <id value='4096'/>\n" +
            "            <location value='GENERAL MEDICINE'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='301' name='GENERAL INTERNAL MEDICINE'/>\n" +
            "            <type name='GENERAL MEDICINE VISIT'/>\n" +
            "            <visitString value='23;3050523.1;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <cpts>\n" +
            "                <cpt code='99201' name='OFFICE/OUTPATIENT VISIT NEW'/>\n" +
            "            </cpts>\n" +
            "            <dateTime value='3040402.030001'/>\n" +
            "            <documents>\n" +
            "                <document id='3220' localTitle='AUDIOLOGY - HEARING LOSS CONSULT'/>\n" +
            "            </documents>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <icds>\n" +
            "                <icd code='389.9' name='UNSPECIFIED HEARING LOSS' narrative='Hearing loss' ranking='PRIMARY'/>\n" +
            "            </icds>\n" +
            "            <id value='3789'/>\n" +
            "            <location value='GENERAL MEDICINE'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <providers>\n" +
            "                <provider code='11748' name='PATHOLOGY,ONE' role='P' primary='1' taxonomyCode='203B00000N' providerType='Physicians (M.D. and D.O.)' classification='Physician/Osteopath'/>\n" +
            "            </providers>\n" +
            "            <reason code='389.9' name='UNSPECIFIED HEARING LOSS' narrative='Hearing loss'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='301' name='GENERAL INTERNAL MEDICINE'/>\n" +
            "            <type code='99201' name='OFFICE/OUTPATIENT VISIT NEW'/>\n" +
            "            <visitString value='23;3040402.030001;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <dateTime value='3040331.161625'/>\n" +
            "            <documents>\n" +
            "                <document id='3006' localTitle='GENERAL MEDICINE NOTE'/>\n" +
            "            </documents>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <id value='3574'/>\n" +
            "            <location value='GENERAL MEDICINE'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='301' name='GENERAL INTERNAL MEDICINE'/>\n" +
            "            <type name='GENERAL MEDICINE VISIT'/>\n" +
            "            <visitString value='23;3040331.161625;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <dateTime value='3040331.124716'/>\n" +
            "            <documents>\n" +
            "                <document id='2875' localTitle='DIABETES'/>\n" +
            "            </documents>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <id value='3443'/>\n" +
            "            <location value='DIABETIC'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='306' name='DIABETES'/>\n" +
            "            <type name='DIABETIC VISIT'/>\n" +
            "            <visitString value='285;3040331.124716;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <cpts>\n" +
            "                <cpt code='99201' name='OFFICE/OUTPATIENT VISIT NEW'/>\n" +
            "                <cpt code='99202' name='OFFICE/OUTPATIENT VISIT NEW2'/>" +
            "            </cpts>\n" +
            "            <arrivalDateTime value='3040325.094947'/>\n" +
            "            <dateTime value='3040325.094947'/>\n" +
            "            <departureDateTime value='3040325.193123'/>\n" +
            "            <documents>\n" +
            "                <document id='3098' localTitle='Discharge Summary' nationalTitle='DISCHARGE SUMMARY'/>\n" +
            "            </documents>\n" +
            "            <facility code='998' name='ABILENE (CAA)'/>\n" +
            "            <id value='3667'/>\n" +
            "            <location value='7A GEN MED'/>\n" +
            "            <patientClass value='IMP'/>\n" +
            "            <providers>\n" +
            "                <provider code='20349' name='VEHU,ONEHUNDREDSIXTEEN' role='A'/>\n" +
            "            </providers>\n" +
            "            <reason narrative='htn'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='H' name='HOSPITALIZATION'/>\n" +
            "            <specialty value='GENERAL MEDICINE'/>\n" +
            "            <type name='HOSPITALIZATION'/>\n" +
            "            <visitString value='158;3040325.094947;H'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <arrivalDateTime value='3030415.171245'/>\n" +
            "            <dateTime value='3030415.171245'/>\n" +
            "            <departureDateTime value='3040126.110114'/>\n" +
            "            <documents>\n" +
            "                <document id='2254' localTitle='ADMISSION REVIEW - NURSING'/>\n" +
            "            </documents>\n" +
            "            <facility code='998' name='ABILENE (CAA)'/>\n" +
            "            <id value='3158'/>\n" +
            "            <location value='3 NORTH SURG'/>\n" +
            "            <patientClass value='IMP'/>\n" +
            "            <providers>\n" +
            "                <provider code='20024' name='VEHU,TWENTYFOUR' role='P' primary='1'/>\n" +
            "                <provider code='20024' name='VEHU,TWENTYFOUR' role='A'/>\n" +
            "            </providers>\n" +
            "            <reason narrative='RIH'/>\n" +
            "            <roomBed value='AS-8'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='H' name='HOSPITALIZATION'/>\n" +
            "            <specialty value='GENERAL MEDICINE'/>\n" +
            "            <type name='HOSPITALIZATION'/>\n" +
            "            <visitString value='5;3030415.171245;H'/>\n" +
            "        </visit>" +
            "    </visits>\n" +
            "</results>";

}
