/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.NotesRPC;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.Consult;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.ProgressNote;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.assertTrue;

/**
 *
 *
 */
public class NotesRPCTest extends BaseRPCTest
{
    @Test
    public void testGetOutpatientNotes() throws RPCException
    {
        NotesRPC notesRPC = new NotesRPC(vistaCxn);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2009, 0, 1);
        endDate.set(2010, 0, 1);

        ProgressNote[] notes = notesRPC.getOutpatientNotes(patientId, startDate, endDate);
        assertTrue(notes.length > 0);
    }

    @Test
    public void testGetInpatientNotes() throws RPCException
    {
        NotesRPC notesRPC = new NotesRPC(vistaCxn);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2009, 0, 1);
        endDate.set(2010, 0, 1);

        ProgressNote[] notes = notesRPC.getInpatientNotes(patientId, startDate, endDate);
        assertTrue(notes.length > 0);
    }

    @Test
    public void testGetOutpatientConsults() throws RPCException
    {
        NotesRPC notesRPC = new NotesRPC(vistaCxn);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2009, 0, 1);
        endDate.set(2010, 0, 1);

        Consult[] consults = notesRPC.getOutpatientConsults(patientId, startDate, endDate);
        assertTrue(consults.length > 0);
    }

    @Test
    public void testGetOutpatientConsultReport() throws RPCException
    {
        NotesRPC notesRPC = new NotesRPC(vistaCxn);
        FreeTextReport consult = notesRPC.getOutpatientConsultReport("406912");
        assertTrue(consult != null);
    }

    @Test
    public void testGetProgressNotesVPR() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        NotesRPC pnVPRRPC = new NotesRPC(cxn);

        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -5);
        ProgressNote[] progressNotes = pnVPRRPC.getProgressNotesVPR(patientId, startDate, endDate);

        assertTrue(progressNotes.length > 0);
    }

    @Test
    public void testGetNotesbyId() throws RPCException
    {
        NotesRPC notesRPC = new NotesRPC(vistaCxn);
        FreeTextReport consult = notesRPC.getNotesById("6182144");
        assertTrue(consult != null);
    }

    @Test
    public void testGetConsultRequestsVPR() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        NotesRPC pnVPRRPC = new NotesRPC(cxn);

        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -15);
        Consult[]  consults = pnVPRRPC.getConsultRequestsVPR(patientId, startDate, endDate);

        assertTrue(consults.length > 0);
    }

}
