/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.PatientRPC;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.PatientDemographics;
import gov.va.med.vds.vistabeans.PatientDemographicsDetail;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

/**
 *
 *
 */
public class PatientRPCTest extends BaseRPCTest {

    public PatientRPCTest() {
    }

    @Test
    public void testMatch() throws RPCException
    {
        PatientRPC patientRPC = new PatientRPC(vistaCxn);
        Patient[] patients = patientRPC.match("ZZTEST,BBBB");
        assertEquals(patients[0].getName(), "ZZTEST,BBBB");
    }

    @Test
    public void testSelect() throws RPCException {
        PatientRPC patientRPC = new PatientRPC(vistaCxn);
        Patient patient = patientRPC.select("15610");
        assertEquals(patient.getICN(), "1011763095");
        assertEquals(patient.getName(), "ZZTEST,BBBB");
    }

//    @Test
//    public void testSelectBySSN() throws RPCException {
//        PatientRPC patientRPC = new PatientRPC(vistaCxn);
//        Patient patient = patientRPC.selectBySSN("000000006");
//        assertEquals(patient.getPatientId(), "15610");
//        assertEquals(patient.getICN(), "1011763095");
//        assertEquals(patient.getName(), "ZZTEST,BBBB");
//    }

    @Test
    public void testLocalPid() throws RPCException {
        PatientRPC patientRPC = new PatientRPC(vistaCxn);
        String localPid = patientRPC.getLocalPid("1011763095");
        assertEquals(localPid, "15610");
    }

    @Test
    public void testGetPatientDemographics() throws RPCException
    {
        PatientRPC patientRPC = new PatientRPC(vistaCxn);
        PatientDemographics patientDemo = patientRPC.getPatientDemographics("15610", this.user);
        assertEquals(patientDemo.getICN(), "1011763095");
    }

    @Test
    public void testGetPatientDemographicsDetail() throws RPCException
    {
        PatientRPC patientRPC = new PatientRPC(vistaCxn);
        PatientDemographicsDetail demoDetail = patientRPC.getPatientDemographicsDetail("15610");
        assertNotNull(demoDetail.getReport());
    }
}