package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.Form;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.hdd.ConceptCodeMapper;
import gov.va.med.jmeadows.dao.util.BHIERelayBeanFactory;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.PatientMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.smartbeans.patient.SmartForm;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

public class QuestionnaireDao extends DaoBase {

//    /**
//     * Retrieves a list of patient questionnaires
//     *
//     * @param queryBean Query parameters.
//     *
//     * {@link QueryBean} required parameters:
//     *
//     * {@link gov.va.med.vds.webservice.User}
//     * {@link gov.va.med.vds.webservice.Patient}
//     *
//     * @return A list of patient questionnaires
//     * @throws JMeadowsException
//     */
//    public List<QuestionnaireBean> getPatientQuestionnaires(JMeadowsQuery queryBean) throws JMeadowsException
//    {
//        validateQueryBean(queryBean,
//                QueryBeanParams.USER, QueryBeanParams.PATIENT);
//
//       auditQuery("Questionnaires", queryBean);
//
//        List<String> includedSources = new ArrayList<String>();
//        includedSources.add(APP_CONFIG.getBHIECustodianOidAHLTA());
//
//        gov.va.med.bhie.QueryBean bhieQueryBean =
//                BHIERelayBeanFactory.create(queryBean, includedSources, null);
//
//        List<QuestionnaireBean> questionnaires = new ArrayList<QuestionnaireBean>();
//        QuestionnaireResultCollectionV3 rc = sm_bhieRelayServiceHandler.getPatientQuestionnaires(bhieQueryBean);
//        if (rc != null) {
//            SmartQuestionnaire sq = new SmartQuestionnaire();
//            QuestionnaireBean[] qba = sq.mapQuestionnairesFromBHIE(rc);
//            questionnaires = Arrays.asList((QuestionnaireBean[])qba);
//        }
//        return questionnaires;
//    }

    /**
     * Retrieves a list of patient questionnaires
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * @return A list of patient questionnaires
     * @throws JMeadowsException
     */
    public List<Form> getPatientForms(JMeadowsQuery queryBean) throws JMeadowsException
    {

//        //TODO: melcolm - remove these dates for production
//        queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate("2000-01-01"))));
//        queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(Utils.getDate("2015-02-12"))));
        queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate("1950-01-01"))));
        queryBean.setEndDate(Calendar.getInstance());


        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("Questionnaires", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);
        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {

                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                //TODO: remove comment for included sources for production
                //includedSources.add(APP_CONFIG.getBHIECustodianOidFHIE());

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean, includedSources, null);

//                //SB: Exclude data range to retrieve entire data set
//                bhieQueryBean.setStartDate(null);
//                bhieQueryBean.setEndDate(null);

                MultiLocationQueryConfig multiLocPatQuestionnairesConfig =
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getPatientQuestionnaires")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapQuestionnairesFromBHIE")
                                .location(bhieLoc)
                                .build();


                MultiLocationQueryConfig multiLocPatDeploymentFormsConfig =
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getDeploymentForms")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapDeploymentFormsFromBHIE")
                                .location(bhieLoc)
                                .build();

                configs.add(multiLocPatQuestionnairesConfig);
                configs.add(multiLocPatDeploymentFormsConfig);


            }
        }

        //SmartHistory smartHistory = new SmartHistory();

        Object[] formObjs = MultiLocationQuery.execute(
                configs, new SmartForm(), queryBean);

        List<Form> forms = null;

        if (formObjs != null) {
            forms = Arrays.asList((Form[]) formObjs);

//            //map problem concept codes
            ConceptCodeMapper.getInstance().mapConceptCodes(hddDao, forms, CodingSystem.NOTES);
        }

        return forms;
    }
}
