package gov.va.med.jmeadows.dao.util;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import gov.va.med.jmeadows.common.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.beans.PropertyVetoException;

public class DataSourceFactory {
    private static final Logger LOGGER = LoggerFactory.getLogger(DataSourceFactory.class);

    private static final AppConfig APPCONFIG = AppConfig.getInstance();

    public static DataSource createDataSource() {
        DataSource dataSource = null;

        if (APPCONFIG.useC3PO()) {
            //setup c3po db connection pools
            try {

                LOGGER.info("jMeadows JDBC Connection pooling: using C3P0");

                ComboPooledDataSource c3poSQLConnectionPool = new ComboPooledDataSource();
                c3poSQLConnectionPool.setDriverClass(APPCONFIG.getSQLDriver());
                c3poSQLConnectionPool.setJdbcUrl(APPCONFIG.getSQLURL());
                dataSource = c3poSQLConnectionPool;
            } catch (PropertyVetoException ex) {
                LOGGER.error(ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }
        } else {
            try {
                LOGGER.info("jMeadows JDBC Connection pooling: using JNDI");
                dataSource = InitialContext.doLookup(APPCONFIG.getJDBC_JNDILookup());
            } catch (NamingException ex) {
                LOGGER.error(ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }
        }

        return dataSource;
    }

    public static DataSource createDataSourceEncrypted() {
        DataSource dataSource = null;

        if (APPCONFIG.useC3PO()) {
            //setup c3po db connection pools
            try {

                LOGGER.info("jMeadows JDBC Connection pooling: using C3P0");

                ComboPooledDataSource c3poSQLConnectionPool = new ComboPooledDataSource();
                c3poSQLConnectionPool.setDriverClass(APPCONFIG.getSQLDriver());
                c3poSQLConnectionPool.setJdbcUrl(APPCONFIG.getSQLURLEncrypted());
                dataSource = c3poSQLConnectionPool;
            } catch (PropertyVetoException ex) {
                LOGGER.error(ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }
        } else {
            try {
                LOGGER.info("jMeadows JDBC Connection pooling: using JNDI");
                dataSource = InitialContext.doLookup(APPCONFIG.getJDBC_JNDILookupEncrypted());
            } catch (NamingException ex) {
                LOGGER.error(ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }
        }

        return dataSource;
    }
}
