package gov.va.med.jmeadows.dao.util;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.site.Sites;
import gov.va.med.vds.webservice.KeyValuePair;
import gov.va.med.vds.webservice.Patient;
import gov.va.med.vds.webservice.QueryBean;
import gov.va.med.vds.webservice.Site;
import gov.va.med.vds.webservice.User;
import gov.va.med.vhahon.common.Utils;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;

public class VistaBeanFactory {

    /**
     * Helper function that maps query bean to a VistA query bean.
     *
     * @param qbean Query bean source.
     * @return VistA query bean.
     * @throws gov.va.med.jmeadows.common.JMeadowsException if a serious error occurs.
     */
    public static QueryBean create(JMeadowsQuery qbean) throws JMeadowsException {
        try {
            QueryBean vistaQb = new QueryBean();

            vistaQb.setActive(qbean.getActive());
            vistaQb.setItemId(qbean.getItemId());
            vistaQb.setMax(qbean.getMax());
            vistaQb.setRequestingApp(qbean.getRequestingApp());
            vistaQb.setSortBy(qbean.getSortBy());
            vistaQb.setStatus(qbean.getStatus());
            if (qbean.getStartDate() != null) {
                vistaQb.setStartDate(Utils.toXMLGregorianCalendar(qbean.getStartDate().getTime()));
            }
            if (qbean.getEndDate() != null) {
                vistaQb.setEndDate(Utils.toXMLGregorianCalendar(qbean.getEndDate().getTime()));
            }

            if (StringUtils.isNotBlank(qbean.getRecordSiteCode())) {
                vistaQb.setRecordSite(Sites.getInstance().getSite(qbean.getRecordSiteCode()));
            }

            User user = null;
            if (qbean.getUser() != null) {
                user = cloneProvider(qbean.getUser());
            }

            Patient patient = null;
            if (qbean.getPatient() != null) {
                patient = clonePatient(qbean.getPatient());
            }

            vistaQb.setPatient(patient);
            vistaQb.setUser(user);

            return vistaQb;
        } catch (Exception ex) {
            throw new JMeadowsException(ex);
        }
    }

    /**
     * Helper function that clones a user bean.
     *
     * @param user User bean to clone.
     * @return A new user bean.
     * @throws JMeadowsException if an error occurs.
     */
    private static User cloneProvider(User user) throws JMeadowsException {
        try {
            //BeanUtils.cloneBean() only performs a shallow copy
            User usr = (User) BeanUtils.cloneBean(user);

            usr.setHostSite(user.getHostSite());

            return usr;
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }
    }

    /**
     * Helper function that clones a patient bean.
     *
     * @param patient Patient bean to clone.
     * @return A new patient bean.
     * @throws JMeadowsException if an error occurs.
     */
    private static Patient clonePatient(Patient patient) throws JMeadowsException {
        try {
            //BeanUtils.cloneBean() only performs a shallow copy
            Patient pat = (Patient) BeanUtils.cloneBean(patient);

            if (patient.getPatientIens() != null) {
                if (pat.getPatientIens() != null) {
                    pat.getPatientIens().clear();
                }

                for (KeyValuePair patIenKV : patient.getPatientIens()) {
                    pat.getPatientIens().add(patIenKV);
                }
            }

            if (patient.getVistaSites() != null) {
                if (pat.getVistaSites() != null) {
                    pat.getVistaSites().clear();
                }

                for (Site vistaSite : patient.getVistaSites()) {
                    if (vistaSite != null) {
                        pat.getVistaSites().add(vistaSite);
                    }
                }
            }

            return pat;
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }
    }
}
