package gov.va.med.jmeadows.dataservicehandler;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.log4j.Logger;

public class PDWSBlackList {
    /**
     * Logger.
     */
    private static final Logger LOGGER = Logger.getLogger(PDWSBlackList.class);
    
    /**
     * List of edipns to blacklist.
     */
    private Properties blacklist = null;
    
    private PDWSBlackList() {
        super();
        loadProperties();
    }
    
    /**
     * Prevent cloning.
     *
     * @return nothing
     * 
     * @throws CloneNotSupportedException exception
     */
    @SuppressWarnings("all")
    public Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException();
    }
    
    private void loadProperties() {
        String propertiesFile = "pdwsblacklist.properties";

        InputStream inputStream = null;

        blacklist = new Properties();

        ClassLoader classLoader = PDWSBlackList.class.getClassLoader();
        if (classLoader != null) {
            inputStream = classLoader.getResourceAsStream(propertiesFile);
        }

        if (inputStream != null) {
            try {
                blacklist.load(inputStream);
            } catch (IOException e) {
                LOGGER.info("Failed to load blacklist properties: " + e.getMessage());
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }

        try {
            inputStream = new FileInputStream(propertiesFile);
            try {
                blacklist.load(inputStream);
            } catch (IOException e) {
                LOGGER.info("Failed load application properties: " + e.getMessage());
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }

        } catch (FileNotFoundException e) {
            LOGGER.info("Failed to load blacklist from file (pdwsblacklist.properties) -"
                + " utilizing default app properties.");
        }
    }

    public boolean isBlackListed(String edipn) {
        boolean rtc = false;
        if (blacklist != null && !blacklist.isEmpty() && (blacklist.containsKey(edipn) || blacklist.containsValue(edipn))) {
            rtc = true;
            LOGGER.debug("blacklisted patient found: " + edipn);                
        }
        return rtc;
    }
    
    /**
     * Retrieves the singleton.
     *
     * @return JXPTransactionsFlat - Singleton class
     */
    @SuppressWarnings("all")
    public static synchronized PDWSBlackList getInstance() {
        return PDWSBlackListHolder.INSTANCE;
    }

    /**
     * Singleton helper class.
     *
     */
    private static class PDWSBlackListHolder {
        /**
         * Properties object.
         */
        private static final PDWSBlackList INSTANCE = new PDWSBlackList();

    }
}
