/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.heartbeat;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import gov.va.med.jmeadows.common.AppConfig;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Ehcache;
import net.sf.ehcache.constructs.blocking.SelfPopulatingCache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.beans.PropertyVetoException;

public class ServiceErrors {
    private static final Logger LOGGER = LoggerFactory.getLogger(ServiceErrors.class);
    private static final AppConfig APP_CONFIG = AppConfig.getInstance();
    private static final ServiceErrors INSTANCE = new ServiceErrors();
    private final static String CACHE_KEY_HEARTBEAT_MAP = "heartbeatMap";
    private final static String EHCACHE_NAME_HEARTBEATMAP = "heartbeat";
    private Ehcache cache;
    private DataSource dataSource;

    private ServiceErrors() {
        if (APP_CONFIG.useC3PO()) {
            //setup c3po db connection pools
            try {

                LOGGER.info("jMeadows JDBC Connection pooling: using C3P0");

                ComboPooledDataSource c3poSQLConnectionPool = new ComboPooledDataSource();
                c3poSQLConnectionPool.setDriverClass(APP_CONFIG.getSQLDriver());
                c3poSQLConnectionPool.setJdbcUrl(APP_CONFIG.getSQLURL());
                dataSource = c3poSQLConnectionPool;
            } catch (PropertyVetoException ex) {
                LOGGER.error(ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }
        } else {
            try {
                dataSource = InitialContext.doLookup(APP_CONFIG.getJDBC_JNDILookup());
            } catch (NamingException ex) {
                LOGGER.error(ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }
        }

        // TODO: Put this in config file?
        this.cache = CacheManager.getInstance().getCache(EHCACHE_NAME_HEARTBEATMAP);

        ServiceErrorsCacheEntryFactory heartbeatCacheEntryFactory =
                new ServiceErrorsCacheEntryFactory(dataSource,
                        CACHE_KEY_HEARTBEAT_MAP);

        SelfPopulatingCache selfPopulatingCache = new SelfPopulatingCache(this.cache, heartbeatCacheEntryFactory);
        CacheManager.getInstance().replaceCacheWithDecoratedCache(this.cache, selfPopulatingCache);
        this.cache = selfPopulatingCache;
    }

    public static final ServiceErrors getInstance() {
        return INSTANCE;
    }

    public String getServiceErrors() {
        return (String) cache.get(CACHE_KEY_HEARTBEAT_MAP).getObjectValue();
    }
}
