/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.AdmissionV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.PatientAdmission;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

//import org.hui.chcs.AdmissionBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

public class SmartAdmission extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartAdmission.class);

    public static final int DATE = 1;
    public static final int SITE = 2;

    public PatientAdmission m_admission;

    public SmartAdmission() {
        super();
        m_admission = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartAdmission(PatientAdmission admission, String sort) {
        super();
        m_admission = admission;
        getSortValue(sort);
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc = null;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartAdmission[] temp = new SmartAdmission[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartAdmission newAll = new SmartAdmission((PatientAdmission) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartAdmission newAll = new SmartAdmission((PatientAdmission) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientAdmission[] ret = new PatientAdmission[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_admission;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartAdmission[] temp = new SmartAdmission[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartAdmission newAll = new SmartAdmission((PatientAdmission) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientAdmission[] ret = new PatientAdmission[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_admission;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (!(obj instanceof SmartAdmission)) {
            rtc = false;
        } else {
            rtc = compareTo(obj) == 0;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_admission);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public int compareTo(Object obj) {
        SmartAdmission tmp = (SmartAdmission) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_admission.getAdmissionDate(), tmp.m_admission.getAdmissionDate());
        }
        if (this.m_sortBy == SITE && this.m_admission.getSite() != null && tmp.m_admission.getSite() != null) {
            retVal = compareString(this.m_admission.getSite().getSiteCode(), tmp.m_admission.getSite().getSiteCode());
        }

        if (!m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("admissionDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }

    }

//    public PatientAdmission[] mapPatientAdmissionsFromCHCS(Object[] dodBean) {
//        AdmissionBean[] adBean = (AdmissionBean[]) dodBean;
//        //Mil_osd_ha_citpo_jvi_webservice_beans_AdmissionBean[] adBean = (Mil_osd_ha_citpo_jvi_webservice_beans_AdmissionBean[])dodBean;
//        int size = dodBean.length;
//        PatientAdmission[] vistaBean = new PatientAdmission[size];
//
//        for (int i = 0; i < dodBean.length; i++) {
//            PatientAdmission bean = new PatientAdmission();
//            bean.setAdmissionDate(Utils.getXMLCal(adBean[i].getAdmissionDate()));
//            bean.setDispositionDate(Utils.getXMLCal(adBean[i].getDispositionDate()));
//            bean.setWard(adBean[i].getWard());
//            bean.setRegisterNumber(adBean[i].getRegistrationNumber());
//            bean.setAdmissionId(adBean[i].getAdmissionId());
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location,
                                                      String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientAdmission adm = new PatientAdmission();
        adm.setDrg(errorMsg);
        adm.setCodingCompleteDate(Utils.backDate());
        adm.setDispositionDate(Utils.backDate());
        adm.setDrgDescription(errorMsg);
        adm.setWard(errorMsg);
        adm.setSite(getConnectionUnavailableSite(location));

        return new PatientAdmission[]{adm};
    }

    public PatientAdmission[] mapAdmissionsFromBHIE(Object resultCollection) {
        V4ResultCollection admissionResultCollection = (V4ResultCollection) resultCollection;
        List<PatientAdmission> vistaBeans = new ArrayList<PatientAdmission>();
//
        if (admissionResultCollection.getDataRecordList() != null && admissionResultCollection.getDataRecordList().getAdmissions() != null) {
            for (AdmissionV4 bhieAdmission : admissionResultCollection.getDataRecordList().getAdmissions())
            {
                PatientAdmission bean = new PatientAdmission();
                if (bhieAdmission.getArrivalDate() != null && bhieAdmission.getArrivalDate().getStart().length() > 0) {
                    bean.setAdmissionDate(bhieToXMLCalV4(bhieAdmission.getArrivalDate()));
                }
//                bean.setAdmissionId(bhieAdmission.getAdmissionId());
                if (bhieAdmission.getRecordId() != null && bhieAdmission.getRecordId().getId() != null) {
                    bean.setAdmissionId(bhieAdmission.getRecordId().getId());
                }
                if (bhieAdmission.getProvider() != null) {
                    bean.setAdmittingProvider(mapProvider(bhieAdmission.getProvider()));
                    bean.setProvider(mapProvider(bhieAdmission.getProvider()));
                }
                if (bhieAdmission.getRegistrationNumber() != null) {
                    bean.setRegisterNumber(bhieAdmission.getRegistrationNumber().getId());
                }
                bean.setDispositionDate(bhieToXMLCalV4(bhieAdmission.getDepartureDate()));
//                bean.setWard(bhieAdmission.getFacility());
//                bean.setWard(bhieAdmission.getWard());
//                bean.setDivision(bhieAdmission.getDivision());

//                addBHIESiteDataToVistaBean(bhieAdmission.getCustodianId(), bhieAdmission.getCustodianName(), bhieAdmission.getFacility(), bhieAdmission.getEventId(), bean);
                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieAdmission.getRepository() != null
                        && bhieAdmission.getRepository().getIds() != null
                        && !bhieAdmission.getRepository().getIds().isEmpty()) {
                    repositoryId = bhieAdmission.getRepository().getIds().get(0).getId();
                }
                if (bhieAdmission.getRepository() != null) {
                    repositoryName = bhieAdmission.getRepository().getName();
                }
                if (bhieAdmission.getFacility() != null) {
                    facilityName = bhieAdmission.getFacility().getName();
                }
                if (bhieAdmission.getRecordId() != null) {
                    eventId = bhieAdmission.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName, 
                        facilityName, eventId, bean);

                vistaBeans.add(bean);
            }
        }

        String siteStatusReport = generateBhieStatusReportV4(admissionResultCollection.getStatusListv4(),
                admissionResultCollection.getErrorListv4(), admissionResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (PatientAdmission[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new PatientAdmission[vistaBeans.size()]);
    }
}