/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.AllergyV4;
import gov.va.med.bhie.ReactionV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Allergy;
import gov.va.med.vds.webservice.Code;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
//import org.hui.chcs.AllergyBean;

//import mil.osd.ha.citpo.jvi.webservice.*;

public class SmartAllergy extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartAllergy.class);

    public static final int NAME = 0;
    public static final int SITE = 1;

    public Allergy m_allergy;

    public SmartAllergy() {
        super();
        m_allergy = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartAllergy(Allergy allergy, String sort) {
        super();
        m_allergy = allergy;
        getSortValue(sort);
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc = null;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartAllergy[] temp = new SmartAllergy[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartAllergy newAll = new SmartAllergy((Allergy) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartAllergy newAll = new SmartAllergy((Allergy) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Allergy[] ret = new Allergy[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_allergy;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        int size = array1.length;
        SmartAllergy[] temp = new SmartAllergy[size];
        int cnt = 0;
        for (int i = 0; i < array1.length; i++) {
            SmartAllergy newAll = new SmartAllergy((Allergy) array1[i], sort);
            temp[cnt] = newAll;
            cnt++;
        }
        Arrays.sort(temp);
        Allergy[] ret = new Allergy[temp.length];
        for (int i = 0; i < temp.length; i++) {
            ret[i] = temp[i].m_allergy;
        }
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartAllergy) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_allergy);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public int compareTo(Object obj) {
        SmartAllergy tmp = (SmartAllergy) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_allergy.getSite() != null && tmp.m_allergy.getSite() != null) {
            retVal = compareString(this.m_allergy.getSite().getSiteCode(), tmp.m_allergy.getSite().getSiteCode());
        } else {
            retVal = compareString(this.m_allergy.getAllergyName(), tmp.m_allergy.getAllergyName());
        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }
        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = NAME;
        m_bAscending = true;
        if (sort != null) {
            sort = setAscending(sort);
            if ("allergyname".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }


    }

//    public Allergy[] mapAllergiesFromCHCS(Object[] dodBean) {
//        int size = dodBean.length;
//        Allergy[] vistaBean = new Allergy[size];
//
//        for (int i = 0; i < dodBean.length; i++) {
//            Allergy bean = new Allergy();
//            bean.setAllergyId(((AllergyBean) dodBean[i]).getAllergyId());
//            bean.setAllergyName(((AllergyBean) dodBean[i]).getAllergy());
//            bean.setComment(((AllergyBean) dodBean[i]).getComment());
//
//            addCHCSSiteDataToVistaBean(bean);
////        bean.setSiteCode(((Mil_osd_ha_citpo_jvi_webservice_beans_AllergyBean)dodBean[i]).getLocationCode());
////        bean.setAllergyName(((Mil_osd_ha_citpo_jvi_webservice_beans_AllergyBean)dodBean[i]).getAllergy());
////        bean.setComment(((Mil_osd_ha_citpo_jvi_webservice_beans_AllergyBean)dodBean[i]).getComment());
////        bean.setSiteName(((Mil_osd_ha_citpo_jvi_webservice_beans_AllergyBean)dodBean[i]).getLocationName());
//
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }

//    Version 3
//    public Allergy[] mapAllergiesFromBHIE(Object resultCollection) {
//        AllergyResultCollection allergyResultCollection = (AllergyResultCollection) resultCollection;
//        List<Allergy> vistaBeans = new ArrayList<Allergy>();
//
//        if (allergyResultCollection.getDataRecordList() != null) {
//
//            for (AllergyDataRecord dataRecord : allergyResultCollection.getDataRecordList())
//            {
//                if (dataRecord == null || dataRecord.getAllergy() == null) continue;
//
//                Allergy bean = new Allergy();
//                gov.va.med.bhie.Allergy bhieAllergy = dataRecord.getAllergy();
//                bean.setAllergyName(bhieAllergy.getAllergen());
//
//                bean.setComment(bhieAllergy.getReaction());
//
//                if(StringUtils.isNotBlank(bhieAllergy.getOtherIdentifier()))
//                {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.DOD_ALLERGY_IEN.getDisplayName());
//                    code.setCode(bhieAllergy.getOtherIdentifier());
//                    bean.getCodes().add(code);
//                }
//
////                addBHIESiteDataToVistaBean(bhieAllergy.getCustodianId(),
////                        bhieAllergy.getCustodianName(), bhieAllergy.getFacility(), bhieAllergy.getEventId(), bean);
////                addBHIESiteDataToVistaBean(bhieAllergy.getRepository().getId().get(0).getId(),
////                        bhieAllergy.getRepository().getName(), bhieAllergy.getFacility().getName(), bhieAllergy.getRecordId().getId(), bean);
//                String repositoryId = "";
//                String repositoryName = "";
//                String facilityName = "";
//                String eventId = "";
//                if (bhieAllergy.getRepository() != null && bhieAllergy.getRepository().getId() != null && 
//                        !bhieAllergy.getRepository().getId().isEmpty()) {
//                    repositoryId = bhieAllergy.getRepository().getId().get(0).getId();
//                }
//                if (bhieAllergy.getRepository() != null) {
//                    repositoryName = bhieAllergy.getRepository().getName();
//                }
//                if (bhieAllergy.getFacility() != null) {
//                    facilityName = bhieAllergy.getFacility().getName();
//                }
//                if (bhieAllergy.getRecordId() != null) {
//                    eventId = bhieAllergy.getRecordId().getId();
//                }
//                if (repositoryId == null || repositoryId.length() == 0) {
//                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
//                }
//                if (repositoryName == null || repositoryName.length() == 0) {
//                    repositoryName = "ahlta";
//                }
//                addBHIESiteDataToVistaBean(repositoryId, repositoryName, 
//                        facilityName, eventId, bean);
//
//                vistaBeans.add(bean);
//            }
//
//        }
//
//        //include status report
//        String siteStatusReport = generateBhieStatusReport(allergyResultCollection.getStatusList(), allergyResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//            (Allergy[]) generateConnectionUnavailableBean(
//                    BHIE_STATUS_REPORT_LOCATION,
//                    siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Allergy[0]);
//    }

//    public Allergy[] mapAllergiesFromBHIE(Object resultCollection) {
//        AllergyResultCollection allergyResultCollection = (AllergyResultCollection) resultCollection;
//        List<Allergy> vistaBeans = new ArrayList<Allergy>();
//
//        if (allergyResultCollection.getDataRecordList() != null) {
//
//            for (AllergyDataRecord dataRecord : allergyResultCollection.getDataRecordList()) {
//                if (dataRecord == null || dataRecord.getAllergy() == null) continue;
//
//                Allergy bean = new Allergy();
//                gov.va.med.bhie.Allergy bhieAllergy = dataRecord.getAllergy();
//                bean.setAllergyName(bhieAllergy.getAllergen());
//
//                bean.setComment(bhieAllergy.getReaction());
//
//                if (StringUtils.isNotBlank(bhieAllergy.getOtherIdentifier())) {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.DOD_ALLERGY_IEN.getDisplayName());
//                    code.setCode(bhieAllergy.getOtherIdentifier());
//                    bean.getCodes().add(code);
//                }
//
//                addBHIESiteDataToVistaBean(bhieAllergy.getCustodianId(),
//                        bhieAllergy.getCustodianName(), bhieAllergy.getFacility(), bhieAllergy.getEventId(), bean);
//
//                vistaBeans.add(bean);
//            }
//
//        }
//
//        //include status report
//        String siteStatusReport = generateBhieStatusReport(allergyResultCollection.getStatusList(), allergyResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (Allergy[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Allergy[0]);
//    }

    public Allergy[] mapAllergiesFromBHIEV4(Object resultCollection) {
        V4ResultCollection allergyResultCollection = (V4ResultCollection) resultCollection;
        List<Allergy> vistaBeans = new ArrayList<Allergy>();

//        boolean vlerExists = false;

        if (allergyResultCollection.getDataRecordList() != null) {
//            ////check to see if there are zero coded VLER records, and if narratives exist
//            if(allergyResultCollection.getDataRecordList().getAllergy().size() < 1
//                    && allergyResultCollection.getNarrativeList().size() > 0){
//
//            }
            for (AllergyV4 vlerAllergy : allergyResultCollection.getDataRecordList().getAllergy()) {
                if (vlerAllergy == null || vlerAllergy.getAllergen() == null) {
                    continue;
                }

                Allergy bean = new Allergy();

                for (SpdcBeanV4 allergen : vlerAllergy.getAllergen()) {
                    if (allergen == null) {
                        continue;
                    }
                    if (allergen.isPrimary()) {
                        bean.setAllergyName(allergen.getDisplay());
                    }
                    if (allergen.getCode() != null && StringUtils.isNotEmpty(allergen.getCode())) {
                        Code code = new Code();
                        if (allergen.getSystem() != null && (allergen.isPrimary() || CodingSystem.DOD_ALLERGY_IEN.getOID().equalsIgnoreCase(allergen.getSystem()))) {
                            code.setSystem(CodingSystem.DOD_ALLERGY_IEN.getDisplayName());
                        } else {
                            code.setSystem(allergen.getSystem());
                        }
                        code.setDisplay(allergen.getDisplay());
                        code.setCode(allergen.getCode());
                        bean.getCodes().add(code);
                    }
                }

                StringBuilder commentString = new StringBuilder();
                for (ReactionV4 reaction : vlerAllergy.getReactions()) {
                    if (reaction != null && !reaction.getReaction().isEmpty()) {
                        for (SpdcBeanV4 react : reaction.getReaction()) {
                            if (react != null) {
                                Code code = new Code();
                                code.setDisplay(react.getDisplay());
                                code.setSystem(react.getSystem());
                                code.setCode(react.getCode());
                                bean.getReactions().add(code);
                                break;
                            }
                        }

                        if (reaction.getSeverity() != null && !reaction.getSeverity().isEmpty() && getPrimarySpdcV4(reaction.getSeverity()) != null) {
                            bean.setSeverity(getPrimarySpdcV4(reaction.getSeverity()).getDisplay());
                        }

                        if (reaction.getComment() != null) {
                            if (commentString.length() > 0) {
                                commentString.append("; ");
                            }
                            commentString.append(reaction.getComment());
                        }
                    }
                }

//                for (SpdcBeanV4 severity : vlerAllergy.getSeverity()) {
//                    if (severity != null && severity.isPrimary()) {
//                        bean.setSeverity(severity.getDisplay());
//                    }
//                }

                if (vlerAllergy.getEnteredBy() != null) {
                    bean.setProvider(mapProvider(vlerAllergy.getEnteredBy()));
                }

                if(vlerAllergy.getNotedDate() != null){
                    bean.setEnteredDate(bhieToXMLCalV4(vlerAllergy.getNotedDate()));
                } else if (vlerAllergy.getEnteredDate() != null) {
                    bean.setEnteredDate(bhieToXMLCalV4(vlerAllergy.getEnteredDate()));
                }

                if (vlerAllergy.getOnsetDate() != null) {
                    bean.setOnsetDate(bhieToXMLCalV4(vlerAllergy.getOnsetDate()));
                }

                if (vlerAllergy.getComment() != null) {
                    if (commentString.length() > 0) {
                        commentString.append("; ");
                    }
                    commentString.append(vlerAllergy.getComment());
                }

                if (StringUtils.isNotBlank(vlerAllergy.getTextDescription())) {
                    if (commentString.length() > 0) {
                        commentString.insert(0, "Comments: ");
                        commentString.append("; ");
                    }
                    commentString.append("Notes: ");
                    commentString.append(vlerAllergy.getTextDescription());
                }
                bean.setComment(commentString.toString());

                bean.setNarrativeId(vlerAllergy.getNarrativeId());

//                if(StringUtils.isNotBlank(bhieAllergy.getOtherIdentifier()))
//                {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.DOD_ALLERGY_IEN.getDisplayName());
//                    code.setCode(bhieAllergy.getOtherIdentifier());
//                    bean.getCodes().add(code);
//                }


                String facilityName = "";
                String repositoryName = "";
                String repositoryId = "";
                String recordId = "";

                if (vlerAllergy.getFacility() != null && vlerAllergy.getFacility().getName() != null) {
                    facilityName = vlerAllergy.getFacility().getName();
                }

                if (vlerAllergy.getRepository() != null && vlerAllergy.getRepository().getName() != null) {
                    repositoryName = vlerAllergy.getRepository().getName();
                    if (vlerAllergy.getRepository().getIds() != null && vlerAllergy.getRepository().getIds().get(0) != null) {
                        repositoryId = vlerAllergy.getRepository().getIds().get(0).getId();
                    }
                }

                if (vlerAllergy.getRecordId() != null) {
                    recordId = vlerAllergy.getRecordId().getId();
                }

//                addBHIESiteDataToVistaBean(bhieAllergy.getCustodianId(),
//                        bhieAllergy.getCustodianName(), bhieAllergy.getFacility(), bhieAllergy.getEventId(), bean);

                if (allergyResultCollection.isVler()) {
//                    vlerExists = true;
                    addBHIESiteDataToVistaBean(repositoryId,
                            "VLER", facilityName, recordId, bean);
                } else {
                    addBHIESiteDataToVistaBean(repositoryId,
                            repositoryName, facilityName, recordId, bean);
                }
                vistaBeans.add(bean);
            }

        }

//        // if there are no VLER coded items, but narratives exist,
//        // create a record so JLV can pull the narratives
//        if (!vlerExists && allergyResultCollection != null
//                && allergyResultCollection.getNarrativeList() != null
//                && allergyResultCollection.getNarrativeList().size() > 0) {
//            Allergy bean = new Allergy();
//            bean.setNarrativeId(
//
//
//        }

            //include status report
//        String siteStatusReport = generateBhieStatusReport(vlerResultCollection.getStatusList(), vlerResultCollection.getErrorList());
        String siteStatusReport = generateBhieStatusReportV4(allergyResultCollection.getStatusListv4(),
                allergyResultCollection.getErrorListv4(), allergyResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Allergy[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));


        return vistaBeans.toArray(new Allergy[vistaBeans.size()]);
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Allergy allergy = new Allergy();
        allergy.setAllergyName(errorMsg);
        allergy.setSite(getConnectionUnavailableSite(location));

        return new Allergy[]{allergy};
    }
}
