package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.Comment;
import gov.va.med.bhie.History;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.beans.Finding;
import gov.va.med.jmeadows.dao.beans.PatientHistory;
import gov.va.med.bhie.HistoryDataRecord;
import gov.va.med.bhie.HistoryResultCollection;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: yidali
 * Date: 4/7/14
 * Time: 2:01 PM
 * To change this template use File | Settings | File Templates.
 */
public class SmartHistory extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartHistory.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int RELATIONSHIP = 3;

    public PatientHistory m_Patient_history;

    public SmartHistory() {
        super();
        m_Patient_history = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartHistory(PatientHistory history, String sort) {
        super();
        m_Patient_history = history;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartHistory) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_Patient_history);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
    
            int size = array1.length;
            SmartHistory[] temp = new SmartHistory[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartHistory newAll = new SmartHistory((PatientHistory) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientHistory[] ret = new PatientHistory[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_history;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartHistory tmp = (SmartHistory) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_Patient_history.getCustodianId() != null && tmp.m_Patient_history.getCustodianId() != null) {
            retVal = compareString(this.m_Patient_history.getCustodianId(), tmp.m_Patient_history.getCustodianId());
        }
        if (this.m_sortBy == RELATIONSHIP) {
            retVal = compareString(this.m_Patient_history.getRelationship(), tmp.m_Patient_history.getRelationship());
        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("date".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = RELATIONSHIP;
            }
        }

    }



    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else  if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartHistory[] temp = new SmartHistory[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartHistory newAll = new SmartHistory((PatientHistory) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartHistory newAll = new SmartHistory((PatientHistory) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientHistory[] ret = new PatientHistory[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_history;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientHistory hist = new PatientHistory();
        hist.setType(errorMsg);
        hist.setDateOnset(Utils.backDate().toGregorianCalendar());
        hist.setSite(getConnectionUnavailableSite(location));
        return new PatientHistory[]{hist};
    }

    public PatientHistory[] mapHistoryFromBHIE(Object resultCollection) {
        HistoryResultCollection historyResultCollection = (HistoryResultCollection) resultCollection;
        List<PatientHistory> vistaBeans = new ArrayList<PatientHistory>();

        if (historyResultCollection.getDataRecordList() != null) {

            for (HistoryDataRecord dataRecord : historyResultCollection.getDataRecordList()) {
                if (dataRecord == null || dataRecord.getHistory() == null) {
                    continue;
                }

                History bhieHistory = dataRecord.getHistory();
                PatientHistory bean = new PatientHistory();


                bean.setChronicity(bhieHistory.getChronicity());
                bean.setDateOnset(Utils.getCal(bhieHistory.getDateOnset()));
                bean.setCouplerType(bhieHistory.getCouplerType());
                bean.setCustodianId(bhieHistory.getCustodianId());
                bean.setCustodianName(bhieHistory.getCustodianName());
                bean.setDateReported(Utils.getCal(bhieHistory.getDateReported()));
                bean.setEncComment(bhieHistory.getEncComment());
                bean.setEnteredBy(bhieHistory.getEnteredBy());
                bean.setEnteredByDate(Utils.getCal(bhieHistory.getEnteredByDate()));
                bean.setFacility(bhieHistory.getFacility());
                bean.setFormattedFindings(bhieHistory.getFormattedFindings());
                bean.setHistoryMedcinId(bhieHistory.getHistoryMedcinId());
                bean.setLoincCode(bhieHistory.getLoincCode());
                bean.setOtherIdentifier(bhieHistory.getOtherIdentifier());
                bean.setPeriodEndDate(Utils.getCal(bhieHistory.getPeriodEndDate()));
                bean.setPeriodStartDate(Utils.getCal(bhieHistory.getPeriodStartDate()));
                bean.setRelationship(bhieHistory.getRelationship());
                bean.setRelationshipId(bhieHistory.getRelationshipId());
                bean.setReportingClinician(bhieHistory.getReportingClinician());
                bean.setRepositoryId(bhieHistory.getRepositoryId());
                bean.setSensitivity(bhieHistory.getSensitivity());
                bean.setSource(bhieHistory.getSource());
                bean.setStatus(bhieHistory.getStatus());
                bean.setType(bhieHistory.getType());
                bean.setUnverifiedFlag(bhieHistory.getUnverifiedFlag());

                List<Finding> findings = new ArrayList<Finding>();
                for (gov.va.med.bhie.Finding finding : bhieHistory.getFindings()) {
                    Finding tempFinding = new Finding();
                    tempFinding.setAnnotation(finding.getAnnotation());
                    tempFinding.setCategory(finding.getCategory());
                    tempFinding.setFlag(finding.getFlag());
                    tempFinding.setItem(finding.getItem());
                    findings.add(tempFinding);
                }
                bean.setFindings(findings);

                List<String> comments = new ArrayList<String>();
                for (Comment comment : bhieHistory.getComments()) {
                    comments.add(comment.getComment());
                }
                bean.setComments(comments);

                if (StringUtils.isNotBlank(bhieHistory.getLoincCode())) {
                    Code code = new Code();
                    code.setSystem(CodingSystem.LOINC.getDisplayName());
                    code.setCode(bhieHistory.getLoincCode());
                    bean.getCodes().add(code);
                }

                if (StringUtils.isNotBlank(bhieHistory.getHistoryMedcinId())) {
                    Code code = new Code();
                    code.setSystem(CodingSystem.DOD_MEDCIN.getDisplayName());
                    code.setCode(bhieHistory.getHistoryMedcinId());
                    bean.getCodes().add(code);
                }

                addBHIESiteDataToVistaBean(bhieHistory.getCustodianId(),
                        bhieHistory.getCustodianName(), bhieHistory.getFacility(), bhieHistory.getEventId(), bean);

                vistaBeans.add(bean);
            }

        }

        //include status report
        String siteStatusReport = generateBhieStatusReport(historyResultCollection.getStatusList(), historyResultCollection.getErrorList());

        vistaBeans.addAll(Arrays.asList(
                (PatientHistory[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new PatientHistory[vistaBeans.size()]);
    }

}
