/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.ResultV4;
import gov.va.med.bhie.ResultsV4;
import gov.va.med.bhie.TestV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.LabOrder;
import gov.va.med.vds.webservice.LabResult;
import gov.va.med.vds.webservice.Provider;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

//import org.hui.chcs.AntibioticBean;
//import org.hui.chcs.AutopsyResultBean;
//import org.hui.chcs.BloodBankAntibodyBean;
//import org.hui.chcs.BloodBankResultBean;
//import org.hui.chcs.BloodBankUnitBean;
//import org.hui.chcs.BoneMarrowResultBean;
//import org.hui.chcs.ChemistryResultBean;
//import org.hui.chcs.CytologyGynResultBean;
//import org.hui.chcs.CytologyNonGynResultBean;
//import org.hui.chcs.EntityBean;
//import org.hui.chcs.LabOrderBean;
//import org.hui.chcs.MicrobiologyResultBean;
//import org.hui.chcs.OrganismBean;
//import org.hui.chcs.SnoMedCodeBean;
//import org.hui.chcs.SpecimenBean;
//import org.hui.chcs.SurgicalPathologyResultBean;

public class SmartLabOrder extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartLabOrder.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;
    public static final int SAMPLE = 4;

    public LabOrder m_laborder;

    public SmartLabOrder() {
        super();
        m_laborder = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartLabOrder(LabOrder lab, String sort) {
        super();
        m_laborder = lab;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartLabOrder) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_laborder);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartLabOrder[] temp = new SmartLabOrder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartLabOrder newAll = new SmartLabOrder((LabOrder) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartLabOrder newAll = new SmartLabOrder((LabOrder) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            LabOrder[] ret = new LabOrder[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_laborder;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartLabOrder[] temp = new SmartLabOrder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartLabOrder newAll = new SmartLabOrder((LabOrder) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            LabOrder[] ret = new LabOrder[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_laborder;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartLabOrder tmp = (SmartLabOrder) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_laborder.getCollectionDate(), tmp.m_laborder.getCollectionDate());
        }
        if (this.m_sortBy == SITE && this.m_laborder.getSite() != null && tmp.m_laborder.getSite() != null) {
            retVal = compareString(this.m_laborder.getSite().getSiteCode(), tmp.m_laborder.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_laborder.getTestName(), tmp.m_laborder.getTestName());
        }
        if (this.m_sortBy == SAMPLE) {
            retVal = compareString(this.m_laborder.getCollectionSample(), tmp.m_laborder.getCollectionSample());
        }

        if (!m_bAscending) {
            retVal = -retVal;
        }
        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("collectionDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("testName".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
            if ("collectionSample".equalsIgnoreCase(sort)) {
                m_sortBy = SAMPLE;
            }
        }


    }

    public LabOrder[] mapChemistryLabsFromBHIE(Object resultCollection) {
        V4ResultCollection labChemResultCollection = (V4ResultCollection) resultCollection;
        List<LabOrder> vistaBeans = new ArrayList<LabOrder>();

        if (labChemResultCollection.getDataRecordList() != null) {
            for (ResultV4 bhieBean : labChemResultCollection.getDataRecordList().getLabChemResults()) {
                if (bhieBean.getOrder() == null) {
                    continue;
                }

                LabOrder bean = new LabOrder();

                String accessionNum = "";
                bean.setCollectionDate(bhieToXMLCalV4(bhieBean.getOrder().getCollectionDate()));
//                if (bhieBean.getOrder().getSpecimen() != null && !bhieBean.getOrder().getSpecimen().isEmpty()) {
//                    bean.setCollectionSample(getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()).getDisplay());
//                }
                if (bhieBean.getOrder().getCollectionSample() != null && bhieBean.getOrder().getCollectionSample().length() > 0) {
                    bean.setCollectionSample(bhieBean.getOrder().getCollectionSample());
                }
                if (bhieBean.getOrder().getTest() != null && !bhieBean.getOrder().getTest().isEmpty()
                        && getPrimarySpdcV4(bhieBean.getOrder().getTest()) != null) {
                    bean.setTestName(getPrimarySpdcV4(bhieBean.getOrder().getTest()).getDisplay());
                }
                if (bhieBean.getResults() != null && bhieBean.getResults().size() > 0
                        && bhieBean.getResults().get(0) != null
                        && bhieBean.getResults().get(0).getStatus() != null
                        && !bhieBean.getResults().get(0).getStatus().isEmpty()) {
                    bean.setStatus(bhieBean.getResults().get(0).getStatus().get(0).getDisplay());
                }

                Provider requestingProvider = mapProvider(bhieBean.getOrder().getProvider());
                bean.setRequestingProvider(requestingProvider);
                if (bhieBean.getOrder() != null) {
                    if (bhieBean.getOrder().getAccession() != null && bhieBean.getOrder().getAccession().getId() != null) {
                        bean.setAccessionNumber(bhieBean.getOrder().getAccession().getId());
                        accessionNum = bhieBean.getOrder().getAccession().getId();
                    }
                    if (bhieBean.getOrder().getPriority() != null && !bhieBean.getOrder().getPriority().isEmpty()
                            && getPrimarySpdcV4(bhieBean.getOrder().getPriority()) != null) {
                        bean.setUrgency(getPrimarySpdcV4(bhieBean.getOrder().getPriority()).getDisplay());
                    }
                    if (bhieBean.getOrder().getOrderNumber() != null) {
                        bean.setOrderId(bhieBean.getOrder().getOrderNumber().getId());
                    }
                    bean.setOrderDate(bhieToXMLCalV4(bhieBean.getOrder().getDate()));
                    bean.setComment(bhieBean.getOrder().getComment());
                }
                if (bhieBean.getRecordId() != null) {
                    bean.setLabTestIEN(bhieBean.getRecordId().getId());
                }

                bean.setSubscript("CH");

                String facilityName = "";
                String repositoryName = "";
                String repositoryId = "";
                String recordId = "";

                if (bhieBean.getFacility() != null && bhieBean.getFacility().getName() != null) {
                    facilityName = bhieBean.getFacility().getName();
                }

                if (bhieBean.getRepository() != null && bhieBean.getRepository().getName() != null) {
                    repositoryName = bhieBean.getRepository().getName();
                    if (bhieBean.getRepository().getIds() != null && bhieBean.getRepository().getIds().get(0) != null) {
                        repositoryId = bhieBean.getRepository().getIds().get(0).getId();
                    }
                }

                if (bhieBean.getRecordId() != null) {
                    recordId = bhieBean.getRecordId().getId();
                }

                addBHIESiteDataToVistaBean(repositoryId,
                        repositoryName, facilityName, recordId, bean);


                if (bhieBean.getResults() != null) {
                    List<ResultsV4> bhieLabResults = bhieBean.getResults();
                    List<LabResult> labResults = bean.getLabResults();

                    for (ResultsV4 labChemResult : bhieLabResults) {
                        LabResult labResult = new LabResult();

                        labResult.setSensitive(bhieBean.isSensitive());

                        labResult.setAccession(accessionNum);
                        if (labChemResult.getTest() != null && !labChemResult.getTest().isEmpty()
                                && getPrimarySpdcV4(labChemResult.getTest()) != null) {
                            labResult.setTestName(getPrimarySpdcV4(labChemResult.getTest()).getDisplay());
                        }
                        labResult.setResult(labChemResult.getResult());
                        if (labChemResult.getCodedInterpretation() != null && !labChemResult.getCodedInterpretation().isEmpty()
                                && getPrimarySpdcV4(labChemResult.getCodedInterpretation()) != null) {
                            labResult.setInterpretation(getPrimarySpdcV4(labChemResult.getCodedInterpretation()).getDisplay());
                        }
                        labResult.setReferenceRange(labChemResult.getReferenceRange());
                        labResult.setResultDate(bhieToXMLCalV4(labChemResult.getCertifiedDate()));
                        labResult.setComment(labChemResult.getComment());
//                        bean.setComment(labChemResult.getComment());

                        //verified by not needed
                        //labResult.setVerifiedBy(mapProvider(bhieBean.getEnteredBy()));
                        if (labChemResult.getUnits() != null && !labChemResult.getUnits().isEmpty()
                                && getPrimarySpdcV4(labChemResult.getUnits()) != null) {
                            labResult.setUnits(getPrimarySpdcV4(labChemResult.getUnits()).getDisplay());
                        }
                        if (bhieBean.getOrder() != null
                                && bhieBean.getOrder().getSpecimen() != null
                                && !bhieBean.getOrder().getSpecimen().isEmpty()
                                && getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()) != null) {
                            labResult.setSpecimen(getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()).getDisplay());
                            labResult.setOrderComment(bhieBean.getOrder().getComment());
                        }

                        if (bhieBean.getOrder() != null && bhieBean.getOrder().getRequestingLocation() != null) {
                            labResult.setRequestingLocation(bhieBean.getOrder().getRequestingLocation().getName());
                        }

//                        if (labChemResult.getPerformingLabLocation() != null) {
//                            if (labChemResult.getPerformingLabLocation().getAddress() != null && labChemResult.getPerformingLabLocation().getAddress().getRaw() != null) {
//                                labResult.setPerformingLabStreet1(labChemResult.getPerformingLabLocation().getAddress().getRaw());
//                            }
//                            labResult.setPerformingLabName(labChemResult.getPerformingLabLocation().getName());
//                        }

                        if (labChemResult.getPerformingLabLocation() != null) {
                            labResult.setFacilityName(labChemResult.getPerformingLabLocation().getName());
                        }

                        addBHIESiteDataToVistaBean(repositoryId,
                                repositoryName, facilityName, recordId, labResult);

                        labResults.add(labResult);
                    }
                }

                vistaBeans.add(bean);

            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Lab Chemistries", labChemResultCollection.getStatusListv4(),
                labChemResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (LabOrder[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new LabOrder[vistaBeans.size()]);
    }

    public LabOrder[] mapMicrobiologyLabsFromBHIE(Object resultCollection) {
        List<LabOrder> vistaBeans = new ArrayList<LabOrder>();
        V4ResultCollection labMicrobiologyResultCollection = (V4ResultCollection) resultCollection;

        if (labMicrobiologyResultCollection.getDataRecordList() != null) {
            for (ResultV4 bhieBean : labMicrobiologyResultCollection.getDataRecordList().getLabMicroResults()) {
                if (bhieBean.getOrder() == null) {
                    continue;
                }
                LabOrder bean = new LabOrder();

                String accessionNum = "";
                bean.setCollectionDate(bhieToXMLCalV4(bhieBean.getOrder().getCollectionDate()));
//                if (bhieBean.getOrder().getSpecimen() != null && !bhieBean.getOrder().getSpecimen().isEmpty()) {
//                    bean.setCollectionSample(getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()).getDisplay());
//                }
                if (bhieBean.getOrder().getCollectionSample() != null && bhieBean.getOrder().getCollectionSample().length() > 0) {
                    bean.setCollectionSample(bhieBean.getOrder().getCollectionSample());
                }
                if (bhieBean.getOrder().getTest() != null && !bhieBean.getOrder().getTest().isEmpty()
                        && getPrimarySpdcV4(bhieBean.getOrder().getTest()) != null) {
                    bean.setTestName(getPrimarySpdcV4(bhieBean.getOrder().getTest()).getDisplay());
                }
//                if (bhieBean.getResults() != null && bhieBean.getResults().size() > 0
//                        && bhieBean.getResults().get(0) != null
//                        && bhieBean.getResults().get(0).getStatus() != null
//                        && !bhieBean.getResults().get(0).getStatus().isEmpty()) {
//                    bean.setStatus(bhieBean.getResults().get(0).getStatus().get(0).getDisplay());
//                }

                Provider requestingProvider = mapProvider(bhieBean.getOrder().getProvider());
                bean.setRequestingProvider(requestingProvider);
                if (bhieBean.getOrder() != null) {
                    if (bhieBean.getOrder().getAccession() != null) {
                        bean.setAccessionNumber(bhieBean.getOrder().getAccession().getId());
                        accessionNum = bhieBean.getOrder().getAccession().getId();
                    }
                    if (bhieBean.getOrder().getPriority() != null && !bhieBean.getOrder().getPriority().isEmpty()
                            && getPrimarySpdcV4(bhieBean.getOrder().getPriority()) != null) {
                        bean.setUrgency(getPrimarySpdcV4(bhieBean.getOrder().getPriority()).getDisplay());
                    }
                    if (bhieBean.getOrder().getOrderNumber() != null) {
                        bean.setOrderId(bhieBean.getOrder().getOrderNumber().getId());
                    }
                    bean.setOrderDate(bhieToXMLCalV4(bhieBean.getOrder().getDate()));
                }
                if (bhieBean.getRecordId() != null) {
                    bean.setLabTestIEN(bhieBean.getRecordId().getId());
                }

                bean.setSubscript("MI");

                String facilityName = "";
                String repositoryName = "";
                String repositoryId = "";
                String recordId = "";

                if (bhieBean.getFacility() != null && bhieBean.getFacility().getName() != null) {
                    facilityName = bhieBean.getFacility().getName();
                }

                if (bhieBean.getRepository() != null && bhieBean.getRepository().getName() != null) {
                    repositoryName = bhieBean.getRepository().getName();
                    if (bhieBean.getRepository().getIds() != null && bhieBean.getRepository().getIds().get(0) != null) {
                        repositoryId = bhieBean.getRepository().getIds().get(0).getId();
                    }
                }

                if (bhieBean.getRecordId() != null) {
                    recordId = bhieBean.getRecordId().getId();
                }

                addBHIESiteDataToVistaBean(repositoryId,
                        repositoryName, facilityName, recordId, bean);


                if (bhieBean.getTests() != null) {
                    List<TestV4> bhieMicrobiologyTests = bhieBean.getTests();

                    List<LabResult> labResults = bean.getLabResults();

                    for (TestV4 microbiologyTest : bhieMicrobiologyTests) {
                        if (microbiologyTest.getResults() != null) {
                            for (ResultsV4 microbiologyResult : microbiologyTest.getResults()) {
                                LabResult labResult = new LabResult();
                                if (bhieBean.getOrder() != null
                                        && bhieBean.getOrder().getSpecimen() != null
                                        && !bhieBean.getOrder().getSpecimen().isEmpty()
                                        && getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()) != null) {
                                    labResult.setSpecimen(getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()).getDisplay());
                                    labResult.setOrderComment(bhieBean.getOrder().getComment());
                                }

                                if (bhieBean.getOrder() != null && bhieBean.getOrder().getRequestingLocation() != null) {
                                    labResult.setRequestingLocation(bhieBean.getOrder().getRequestingLocation().getName());
                                }

                                labResult.setSensitive(bhieBean.isSensitive());
                                labResult.setAccession(accessionNum);
                                if (microbiologyResult.getType() != null && !microbiologyResult.getType().isEmpty()
                                        && getPrimarySpdcV4(microbiologyResult.getType()) != null) {
                                    labResult.setTestName(getPrimarySpdcV4(microbiologyResult.getType()).getDisplay());
                                }

                                StringBuilder resultString = new StringBuilder();
                                if (microbiologyResult.getContent() != null) {
                                    resultString.append(microbiologyResult.getContent());
                                    resultString.append("\n");
                                }
                                if (microbiologyResult.getResult() != null) {
                                    resultString.append(microbiologyResult.getResult());
                                }
                                labResult.setResult(resultString.toString());

                                //labResult.setResult(microbiologyResult.getResult());
                                labResult.setResultDate(bhieToXMLCalV4(bhieBean.getCertifiedDate()));
                                if (microbiologyResult.getCertifiedDate() != null) {
                                    labResult.setResultDate(bhieToXMLCalV4(microbiologyResult.getCertifiedDate()));
                                }
                                //verified field not needed
                                //abResult.setVerifiedBy(microbiologyResult.getCertifiedBy());
                                if (microbiologyResult.getStatus() != null && !microbiologyResult.getStatus().isEmpty()
                                        && getPrimarySpdcV4(microbiologyResult.getStatus()) != null) {
                                    labResult.setResultStatus(getPrimarySpdcV4(microbiologyResult.getStatus()).getDisplay());
                                    bean.setStatus(getPrimarySpdcV4(microbiologyResult.getStatus()).getDisplay());
                                }

                                addBHIESiteDataToVistaBean(repositoryId,
                                        repositoryName, facilityName, recordId, labResult);

                                labResults.add(labResult);
                            }
                        }
                    }
                }

                vistaBeans.add(bean);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Lab Microbiologies", labMicrobiologyResultCollection.getStatusListv4(),
                labMicrobiologyResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (LabOrder[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new LabOrder[vistaBeans.size()]);
    }

    private LabOrder mapAnatomicPathologyRecord(Object resultCollection) {
        LabOrder bean = new LabOrder();
        ResultV4 bhieBean = (ResultV4) resultCollection;

        String accessionNum = "";
        bean.setCollectionDate(bhieToXMLCalV4(bhieBean.getOrder().getCollectionDate()));
//        if (bhieBean.getOrder().getSpecimen() != null && !bhieBean.getOrder().getSpecimen().isEmpty()) {
//            bean.setCollectionSample(getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()).getDisplay());
//        }
        if (bhieBean.getOrder().getCollectionSample() != null && bhieBean.getOrder().getCollectionSample().length() > 0) {
            bean.setCollectionSample(bhieBean.getOrder().getCollectionSample());
        }
        if (bhieBean.getOrder().getTest() != null && !bhieBean.getOrder().getTest().isEmpty()
                && getPrimarySpdcV4(bhieBean.getOrder().getTest()) != null) {
            bean.setTestName(getPrimarySpdcV4(bhieBean.getOrder().getTest()).getDisplay());
        }
        if (bhieBean.getResults() != null && bhieBean.getResults().size() > 0
                && bhieBean.getResults().get(0) != null
                && bhieBean.getResults().get(0).getStatus() != null
                && !bhieBean.getResults().get(0).getStatus().isEmpty()
                && bhieBean.getResults().get(0).getStatus().get(0) != null) {
            bean.setStatus(bhieBean.getResults().get(0).getStatus().get(0).getDisplay());
        }

        Provider requestingProvider = mapProvider(bhieBean.getOrder().getProvider());
        bean.setRequestingProvider(requestingProvider);
        if (bhieBean.getOrder() != null) {
            if (bhieBean.getOrder().getAccession() != null) {
                bean.setAccessionNumber(bhieBean.getOrder().getAccession().getId());
                accessionNum = bhieBean.getOrder().getAccession().getId();
            }
            if (bhieBean.getOrder().getPriority() != null && !bhieBean.getOrder().getPriority().isEmpty()
                    && getPrimarySpdcV4(bhieBean.getOrder().getPriority()) != null) {
                bean.setUrgency(getPrimarySpdcV4(bhieBean.getOrder().getPriority()).getDisplay());
            }
            if (bhieBean.getOrder().getOrderNumber() != null) {
                bean.setOrderId(bhieBean.getOrder().getOrderNumber().getId());
            }
            bean.setComment(bhieBean.getOrder().getComment());
            bean.setOrderDate(bhieToXMLCalV4(bhieBean.getOrder().getDate()));
        }
        if (bhieBean.getRecordId() != null) {
            bean.setLabTestIEN(bhieBean.getRecordId().getId());
        }

        bean.setSubscript("AP");

        String facilityName = "";
        String repositoryName = "";
        String repositoryId = "";
        String recordId = "";

        if (bhieBean.getFacility() != null && bhieBean.getFacility().getName() != null) {
            facilityName = bhieBean.getFacility().getName();
        }

        if (bhieBean.getRepository() != null && bhieBean.getRepository().getName() != null) {
            repositoryName = bhieBean.getRepository().getName();
            if (bhieBean.getRepository().getIds() != null && bhieBean.getRepository().getIds().get(0) != null) {
                repositoryId = bhieBean.getRepository().getIds().get(0).getId();
            }
        }

        if (bhieBean.getRecordId() != null) {
            recordId = bhieBean.getRecordId().getId();
        }

        addBHIESiteDataToVistaBean(repositoryId,
                repositoryName, facilityName, recordId, bean);

        if (bhieBean.getResults() != null) {
            List<ResultsV4> bhieLabAnatomicPathologyResults = bhieBean.getResults();

            List<LabResult> labResults = bean.getLabResults();

            for (ResultsV4 anatomicPathologyResult : bhieLabAnatomicPathologyResults) {
                LabResult labResult = new LabResult();
//                if (bhieBean.getOrder() != null
//                        && bhieBean.getOrder().getSpecimen() != null
//                        && !bhieBean.getOrder().getSpecimen().isEmpty()) {
//                    labResult.setSpecimen(getPrimarySpdcV4(bhieBean.getOrder().getSpecimen()).getDisplay());
//                }

                if (bhieBean.getOrder() != null && bhieBean.getOrder().getRequestingLocation() != null) {
                    labResult.setRequestingLocation(bhieBean.getOrder().getRequestingLocation().getName());
                }

                StringBuilder resultString = new StringBuilder();
                if (anatomicPathologyResult.getContent() != null) {
                    resultString.append(anatomicPathologyResult.getContent());
                    resultString.append("\n");
                }
                if (anatomicPathologyResult.getResult() != null) {
                    resultString.append(anatomicPathologyResult.getResult());
                }
                labResult.setResult(resultString.toString());
                StringBuilder specimens = new StringBuilder();
                int sCount = 0;
                for (String specimen : anatomicPathologyResult.getSpecimens()) {
                    if (sCount > 0) {
                        specimens.append(", ");
                    }
                    specimens.append(specimen);
                    sCount++;
                }
                labResult.setSpecimen(specimens.toString());
//                bean.setCollectionSample(specimens.toString());

                labResult.setSensitive(bhieBean.isSensitive());

                labResult.setAccession(accessionNum);
                if (anatomicPathologyResult.getType() != null
                        && !anatomicPathologyResult.getType().isEmpty()
                        && getPrimarySpdcV4(anatomicPathologyResult.getType()) != null) {
                    labResult.setTestName(getPrimarySpdcV4(anatomicPathologyResult.getType()).getDisplay());
                } else if (anatomicPathologyResult.getTest() != null && !anatomicPathologyResult.getTest().isEmpty()
                        && getPrimarySpdcV4(anatomicPathologyResult.getTest()) != null) {
                    labResult.setTestName(getPrimarySpdcV4(anatomicPathologyResult.getTest()).getDisplay());
                }
                //labResult.setResult(anatomicPathologyResult.getResult());
                if (anatomicPathologyResult.getCertifiedDate() != null) {
                    labResult.setResultDate(bhieToXMLCalV4(anatomicPathologyResult.getCertifiedDate()));
                } else if (bhieBean.getEnteredDate() != null) {
                    labResult.setResultDate(bhieToXMLCalV4(bhieBean.getEnteredDate()));
                }
                //labResult.setResultDate(bhieToXMLCalV4(anatomicPathologyResult.getCertifiedDate()));
                labResult.setVerifiedBy(mapProvider(anatomicPathologyResult.getCertifiedBy()).getName());
                if (anatomicPathologyResult.getStatus() != null && !anatomicPathologyResult.getStatus().isEmpty()
                        && getPrimarySpdcV4(anatomicPathologyResult.getStatus()) != null) {
                    labResult.setResultStatus(getPrimarySpdcV4(anatomicPathologyResult.getStatus()).getDisplay());
                }
                labResult.setComment(anatomicPathologyResult.getComment());

                addBHIESiteDataToVistaBean(repositoryId,
                        repositoryName, facilityName, recordId, labResult);

                labResults.add(labResult);
            }
        }

        return bean;
    }

    public LabOrder[] mapAnatomicPathologyLabsFromBHIE(Object resultCollection) {
        List<LabOrder> vistaBeans = new ArrayList<LabOrder>();
        V4ResultCollection labAnatomicPathologyResultCollection = (V4ResultCollection) resultCollection;

        if (labAnatomicPathologyResultCollection.getDataRecordList() != null) {
            for (ResultV4 bhieBean : labAnatomicPathologyResultCollection.getDataRecordList().getLabApResults()) {
                if (bhieBean.getOrder() == null) {
                    continue;
                }
                LabOrder labOrder = mapAnatomicPathologyRecord(bhieBean);
                labOrder.setSubscript("AP");
                vistaBeans.add(labOrder);

            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Lab Anatomic Pathologies", labAnatomicPathologyResultCollection.getStatusListv4(),
                labAnatomicPathologyResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (LabOrder[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new LabOrder[vistaBeans.size()]);
    }

//    public LabOrder[] getLabOrdersFromCHCS(Object[] dodBean) {
//        LabOrderBean[] chcsBean = (LabOrderBean[]) dodBean;
//        int size = chcsBean.length;
//        LabOrder[] vistaBean = new LabOrder[size];
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            LabOrder bean = new LabOrder();
//            bean.setCollectionDate(Utils.getXMLCal(chcsBean[i].getCollectionDateTime()));
//            bean.setCollectionSample(chcsBean[i].getCollectionSample());
//            bean.setInternalId(chcsBean[i].getResultId());
//            bean.setOrderDate(Utils.getXMLCal(chcsBean[i].getOrderDateTime()));
//            bean.setOrderId(chcsBean[i].getOrderId());
//            Provider requestingProvider = new Provider();
//            requestingProvider.setName(chcsBean[i].getOrderingProvider());
//            bean.setRequestingProvider(requestingProvider);
//            bean.setSubscript(Utils.getPiece(chcsBean[i].getResultId(), ";", 3));
//            bean.setTestName(chcsBean[i].getTestName());
//
//            /*
//            bean.setTestName(chcsBean[i].getTestName());
//            bean.setCollectionDate(Utils.getDateFromSQLFormat(chcsBean[i].getCollectionDateTime()));
//            bean.setCollectionSample(chcsBean[i].getSpecimenName());
//            bean.setInternalId(chcsBean[i].getResultId());
//            bean.setOrderId(chcsBean[i].getResultId());
//            bean.setSubscript(Utils.getPiece(chcsBean[i].getResultId(),";",3));
//            */
//
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }
//
//    public LabResult[] getCHLabResultsFromCHCS(ChemistryResultBean[] chcsBean, String siteCode) {
//        int size = chcsBean.length;
//        LabResult[] vistaBean = new LabResult[size];
//        StringBuilder comment = new StringBuilder();
//        for (int i = 0; i < chcsBean.length; i++) {
//            LabResult bean = new LabResult();
//
//            //bean.setSiteCode(siteCode);
//            addCHCSSiteDataToVistaBean(bean);
//
//            bean.setInterpretation(chcsBean[i].getAbnormal());
//            bean.setPrintName(chcsBean[i].getLabTest());
//            bean.setReferenceRange(chcsBean[i].getReferenceRange());
//            bean.setResult(chcsBean[i].getResult());
//
//            bean.setResultDate(Utils.getXMLCal(chcsBean[i].getDateCertified()));
//            bean.setResultStatus("");
//            bean.setTestId(chcsBean[i].getLabId());
//            bean.setTestName(chcsBean[i].getLabTest());
//            bean.setUnits(chcsBean[i].getUnits());
//            bean.setVerifiedBy(chcsBean[i].getCertifyName());
//            bean.setSpecimen(chcsBean[i].getSpecimen());
//            bean.setAccession(chcsBean[i].getAccession());
//            String interp = chcsBean[i].getInterpretation();
//            String comm = chcsBean[i].getResultComment();
//            comment.setLength(0);
//            if (comm != null && (comm.length() > 0)) {
//                comment.append("Result Comment: ");
//                comment.append(comm);
//            }
//            if (interp != null && (interp.length() > 0)) {
//                comment.append("\nInterpretation: ");
//                comment.append(interp);
//            }
//            bean.setComment(comment.toString());
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }
//
//    public LabResult[] getCGLabResultsFromCHCS(CytologyGynResultBean chcsBean, String siteCode) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//
//        //bean.setSiteCode(siteCode);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setAccession(chcsBean.getAccession());
//        bean.setPrintName(chcsBean.getLabTest());
//        bean.setTestName(chcsBean.getLabTest());
//        bean.setResultDate(Utils.getXMLCal(chcsBean.getDateTimeCertified()));
//        bean.setVerifiedBy(chcsBean.getPathologist());
//        bean.setResultStatus(chcsBean.getOrderTaskStatus());
//        bean.setSpecimen("NA");
//
//        // get report
//        if (chcsBean.isIsCopathReport()) {
//            bean.setResult(chcsBean.getAtomicPathologyReport());
//        } else {
//            StringBuilder result = new StringBuilder();
//            result.append("<basefont size=\"4\"><PRE><PRE>");
//
//            result.append(addText(chcsBean.getOrderingProvider(), "Ordering User: ", ""));
//            result.append(addText(chcsBean.getRequestingLocation(), "Requesting Location: ", ""));
//            result.append(addText(chcsBean.getAccessionComment(), "Accession Comment: ", "\n"));
//            result.append(addText(bean.getResultDate().toString(), "Certified Date/Time: ", ""));
//            result.append(addText(chcsBean.getLmp(), "LMP: ", ""));
//            result.append(addText(chcsBean.getIud(), "IUD: ", ""));
//            result.append(addText(chcsBean.getBirthControlPills(), "Birth Control Pills: ", ""));
//            result.append(addText(chcsBean.getHormoneTherapy(), "Hormone Therapy: ", ""));
//            result.append(addText(chcsBean.getPostMenopausal(), "Post-Menopausal: ", ""));
//            result.append(addText(chcsBean.getHysterectomy(), "Hysterectomy: ", ""));
//            result.append(addText(chcsBean.getRadiationTherapy(), "Radiation Therapy: ", ""));
//            result.append(addText(chcsBean.getCytotoxicTherapy(), "Cytotoxic Therapy: ", ""));
//            if (chcsBean.getWeeksPregnant() > 0) {
//                result.append(addText(Integer.toString(chcsBean.getWeeksPregnant()), "Weeks Pregnant: ", ""));
//            }
//            if (chcsBean.getWeeksPostPartum() > 0) {
//                result.append(addText(Integer.toString(chcsBean.getWeeksPostPartum()), "Weeks Post Partum: ", ""));
//            }
//            result.append(addText(chcsBean.getCytoComment(), "Cytology Comment: ", ""));
//            result.append(addText(chcsBean.getPrevCytologyDiagnosis(), "Previous Cytological Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getDiagnosis(), "Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getDiagnosisComments(), "Diagnosis Comments: ", "\n"));
//            result.append(addText(chcsBean.getRecommendations(), "Recommendations: ", "\n"));
//            result.append(addText(chcsBean.getCytotechnologist(), "Cytotechnologist: ", ""));
//            result.append(addText(chcsBean.getPathologist(), "Staff Pathologist: ", ""));
//
//            if (chcsBean.getSpecimens() != null && chcsBean.getSpecimens().size() > 0) {
//                List<SpecimenBean> specBean = null;
////              java.util.List<SpecimenBean> specimenList = chcsBean.getSpecimens();
////              specBean = (SpecimenBean[])specimenList.toArray(new SpecimenBean[0]);
//                specBean = chcsBean.getSpecimens();
//                result.append(addSpecimens(specBean));
//
//            }
//
//            result.append(addText(Utils.formatText(chcsBean.getClinicalHistory()), "Clinical History: ", "\n "));
//            result.append(addText(Utils.formatText(chcsBean.getPreOperativeDiagnosis()), "Preoperative Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getOtherFindings(), "Other Findings: ", "\n"));
//            result.append(addText(chcsBean.getEstrogenEffect(), "Estrogen Effect: ", ""));
//            result.append(addText(chcsBean.getEstrogenInvalidDueTo(), "Estrogen invalid due to: ", ""));
//            result.append(addText(chcsBean.getMaturationIndex(), "Maturation Index: ", ""));
//            result.append(addText(chcsBean.getSuperficial(), "     Superficial: ", ""));
//            result.append(addText(chcsBean.getIntermediate(), "     Intermediate: ", ""));
//            result.append(addText(chcsBean.getParabasal(), "     Parabasal: ", ""));
//            result.append(addText(chcsBean.getLoginComment(), "Login Comment: ", "\n "));
//            if (chcsBean.getSnoMedCodes() != null && chcsBean.getSnoMedCodes().size() > 0) {
//                List<SnoMedCodeBean> snowBean = null;
////            java.util.List<SnoMedCodeBean> snowList = chcsBean.getSnoMedCodes();
////            snowBean = (SnoMedCodeBean[])snowList.toArray(new SnoMedCodeBean[0]);
//                snowBean = chcsBean.getSnoMedCodes();
//                result.append(addSnomedCodes(snowBean));
//            }
//
//            result.append("</PRE></basefont>");
//            bean.setResult(result.toString());
//
//        }
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//    public LabResult[] getCNLabResultsFromCHCS(CytologyNonGynResultBean chcsBean, String siteCode) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//
//        //bean.setSiteCode(siteCode);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setAccession(chcsBean.getAccession());
//        bean.setPrintName(chcsBean.getLabTest());
//        bean.setTestName(chcsBean.getLabTest());
//        bean.setResultDate(Utils.getXMLCal(chcsBean.getDateTimeCertified()));
//        bean.setVerifiedBy(chcsBean.getPathologist());
//        bean.setResultStatus(chcsBean.getOrderTaskStatus());
//        bean.setSpecimen("NA");
//
//        // get report
//        if (chcsBean.isIsCopathReport()) {
//            bean.setResult(chcsBean.getAtomicPathologyReport());
//        } else {
//            StringBuilder result = new StringBuilder();
//            result.append("<basefont size=\"4\"><PRE><PRE>");
//
//            result.append(addText(chcsBean.getCollectionDateTime(), "Collection Date: ", ""));
//            result.append(addText(chcsBean.getOrderingProvider(), "Ordering User: ", ""));
//            result.append(addText(chcsBean.getOrderId(), "Order Number: ", ""));
//            result.append(addText(chcsBean.getRequestingLocation(), "Requesting Location: ", "\n"));
//            result.append(addText(chcsBean.getResultComment(), "Result Comment: ", "\n"));
//            result.append(addText(chcsBean.getAccessionComment(), "Accession Comment: ", "\n"));
//            result.append(addText(chcsBean.getAtomicPathologyReport(), "CoPath Report: ", "\n"));
//            result.append(addText(chcsBean.getDiagnosis(), "Microscopic Diagnosis:", "\n"));
//            result.append(addText(chcsBean.getDiagnosisComments(), "Diagnosis Comments: ", "\n"));
//            result.append(addText(chcsBean.getRecommendations(), "Recommendations: ", "\n"));
//            result.append(addText(chcsBean.getOtherFindings(), "Other Findings: ", "\n"));
//            result.append(addText(chcsBean.getCytotechnologist(), "Cytotechnologist: ", ""));
//            result.append(addText(chcsBean.getPathologist(), "Staff Pathologist: ", ""));
//            if (chcsBean.getSpecimens() != null && chcsBean.getSpecimens().size() > 0) {
//                List<SpecimenBean> specBean = null;
////              java.util.List<SpecimenBean> specimenList = chcsBean.getSpecimens();
////              specBean = (SpecimenBean[])specimenList.toArray(new SpecimenBean[0]);
//                specBean = chcsBean.getSpecimens();
//                result.append(addSpecimens(specBean));
//
//            }
//            result.append(addText(Utils.formatText(chcsBean.getClinicalHistory()), "Clinical History:", "\n "));
//            result.append(addText(Utils.formatText(chcsBean.getPreOperativeDiagnosis()), "Preoperative Diagnosis: ", "\n"));
//            result.append(addText(Utils.formatText(chcsBean.getOperativeFindings()), "Operative Findings: ", "\n"));
//            result.append(addText(Utils.formatText(chcsBean.getPostOperativeDiagnosis()), "PostOperative Diagnosis:", "\n "));
//            result.append(addText(Utils.formatText(chcsBean.getLoginComment()), "Login Comment:", "\n"));
//            if (chcsBean.getSnoMedCodes() != null && chcsBean.getSnoMedCodes().size() > 0) {
//                List<SnoMedCodeBean> snowBean = null;
////            java.util.List<SnoMedCodeBean> snowList = chcsBean.getSnoMedCodes();
////            snowBean = (SnoMedCodeBean[])snowList.toArray(new SnoMedCodeBean[0]);
//                snowBean = chcsBean.getSnoMedCodes();
//                result.append(addSnomedCodes(snowBean));
//            }
//
//            result.append("</PRE></basefont>");
//            bean.setResult(result.toString());
//
//        }
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//    public LabResult[] getBMLabResultsFromCHCS(BoneMarrowResultBean chcsBean, String siteCode) {
//        LabResult[] vistaBean = new LabResult[1];
//
//
//        LabResult bean = new LabResult();
//        //bean.setSiteCode(siteCode);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setAccession(chcsBean.getAccession());
//        bean.setPrintName(chcsBean.getLabTest());
//        bean.setTestName(chcsBean.getLabTest());
//        bean.setResultDate(Utils.getXMLCal(chcsBean.getDateTimeCertified()));
//        bean.setResultStatus(chcsBean.getOrderTaskStatus());
//        bean.setSpecimen("NA");
//
//        // get report
//        if (chcsBean.isIsCopathReport()) {
//            bean.setResult(chcsBean.getCopathReport());
//        } else {
//            StringBuilder result = new StringBuilder();
//            result.append("\n<basefont size=\"4\"><PRE><PRE>");
//            result.append(addText(chcsBean.getOrderingProvider(), "Ordering User: ", ""));
//            result.append(addText(chcsBean.getRequestingLocation(), "Requesting Location: ", ""));
//            result.append(addText(chcsBean.getAccessionComment(), "Accession Comment:", "\n"));
//            result.append(addText(bean.getResultDate().toString(), "Certified Date/Time: ", ""));
//            result.append(addText(chcsBean.getMicroscopicDiagnosis(), "Microscopic Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getPathologyComment(), "Diagnosis Comments: ", "\n"));
//            result.append(addText(chcsBean.getGrossDiagnosis(), "Gross Diagnosis: ", "\n"));
//
//            if (chcsBean.getResidents() != null && chcsBean.getResidents().size() > 0) {
//                List<EntityBean> resBean = null;
////            java.util.List<EntityBean> resList = chcsBean.getResidents();
////            resBean = (EntityBean[])resList.toArray(new EntityBean[0]);
//                resBean = chcsBean.getResidents();
//                result.append(addEntity(resBean, "Residents: "));
//            }
//
//            if (chcsBean.getPathologists() != null && chcsBean.getPathologists().size() > 0) {
//                List<EntityBean> pathBean = null;
////            java.util.List<EntityBean> pathList = chcsBean.getPathologists();
////            pathBean = (EntityBean[])pathList.toArray(new EntityBean[0]);
//                pathBean = chcsBean.getPathologists();
//                result.append(addEntity(pathBean, "Staff Pathologists: "));
//            }
//
//            if (chcsBean.getSpecimens() != null && chcsBean.getSpecimens().size() > 0) {
//                List<SpecimenBean> specBean = null;
////              java.util.List<SpecimenBean> specimenList = chcsBean.getSpecimens();
////              specBean = (SpecimenBean[])specimenList.toArray(new SpecimenBean[0]);
//                specBean = chcsBean.getSpecimens();
//                result.append(addSpecimens(specBean));
//
//            }
//
//            result.append(addText(chcsBean.getMicroscopicDescription(), "Microscopic Description:", "\n "));
//            result.append(addText(chcsBean.getGrossDescription(), "Gross Description:", " \n"));
//            result.append(addText(Utils.formatText(chcsBean.getClinicalHistory()), "Clinical History: ", "\n"));
//            result.append(addText(Utils.formatText(chcsBean.getPreopDiagnosis()), "Preoperative Diagnosis: ", "\n"));
//            result.append(addText(Utils.formatText(chcsBean.getLoginComment()), "Login Comment: ", "\n"));
//
//            if (chcsBean.getSnoMedCodes() != null && chcsBean.getSnoMedCodes().size() > 0) {
//                List<SnoMedCodeBean> snowBean = null;
////            java.util.List<SnoMedCodeBean> snowList = chcsBean.getSnoMedCodes();
////            snowBean = (SnoMedCodeBean[])snowList.toArray(new SnoMedCodeBean[0]);
//                snowBean = chcsBean.getSnoMedCodes();
//                result.append(addSnomedCodes(snowBean));
//            }
//
//            result.append("</PRE></basefont>");
//            bean.setResult(result.toString());
//
//        }
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//    public LabResult[] getBBLabResultsFromCHCS(BloodBankResultBean[] chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//        bean.setAccession(chcsBean[0].getAccession());
//        bean.setPrintName(chcsBean[0].getLabTest());
//        bean.setTestName(chcsBean[0].getLabTest());
//        //bean.setResultDate(chcsBean[0].);
//        bean.setResultStatus(chcsBean[0].getStatus());
//        bean.setVerifiedBy(chcsBean[0].getCertifyInit());
//        bean.setSpecimen(chcsBean[0].getCollectionSample());
//
//        //bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        // get results
//        StringBuilder result = new StringBuilder();
//        String temp = "";
//
//        //get comments
//        result.append(addText(chcsBean[0].getOrderComment(), "Order Comment: ", ""));
//        result.append(addText(chcsBean[0].getLoginComment(), "Login Comment: ", ""));
//        if (chcsBean.length > 0) {
//            result.append("<table width='100%'><tr><td><b>Test Name</b></td><td><b>Result</b></td><td><b>CertifiedBy/Certified Date</b></td></tr>");
//
//            for (int i = 0; i < chcsBean.length; i++) {
//                // get results
//                String test = chcsBean[i].getLabTest();
//                String res = chcsBean[i].getResult();
//                String certBy = chcsBean[i].getCertifyInit();
//                String certDt = chcsBean[i].getDateCertified();
//                result.append("<tr><td>");
//                result.append(test);
//                result.append("</td><td>");
//                result.append(res);
//                result.append("</td><td>");
//                result.append(certBy);
//                result.append("/");
//                result.append(certDt);
//                result.append("</td></tr>");
//                temp = addText(chcsBean[i].getResultComment(), "Result Comment: ", "\n");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//
//                if (chcsBean[i].getHasAntibodies().length() > 0) {
//                    List<BloodBankAntibodyBean> antiBean = null;
////            java.util.List<BloodBankAntibodyBean> antiList = chcsBean[i].getAntibodies();
////            antiBean = (BloodBankAntibodyBean[])antiList.toArray(new BloodBankAntibodyBean[0]);
//                    antiBean = chcsBean[i].getAntibodies();
//                    temp = addAntibodies(antiBean /*, "Antibodies(s): "*/);
//
//                    if (temp.length() > 0) {
//                        result.append("<tr><td colspan='3'>");
//                        result.append(temp);
//                        result.append("</td></tr>");
//                    }
//                }
//                if (i == (chcsBean.length - 1)) {
//                    List<BloodBankUnitBean> unitBean = null;
////            java.util.List<BloodBankUnitBean> unitList = chcsBean[i].getUnits();
////            unitBean = (BloodBankUnitBean[])unitList.toArray(new BloodBankUnitBean[0]);
//                    unitBean = chcsBean[i].getUnits();
//                    temp = addUnits(unitBean, "Unit(s):");
//
//                    if (temp.length() > 0) {
//                        result.append("<tr><td colspan='3'>");
//                        result.append(temp);
//                        result.append("</td></tr>");
//                    }
//                }
//            }
//            result.append("</table>");
//        }
//        bean.setResult(result.toString());
//
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//    public LabResult[] getBTLabResultsFromCHCS(MicrobiologyResultBean[] chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//        bean.setAccession(chcsBean[0].getAccession());
//        bean.setPrintName(chcsBean[0].getName());
//        bean.setTestName(chcsBean[0].getName());
//        bean.setResultStatus("NA");
//        bean.setSpecimen("");
//        bean.setResultDate(Utils.getXMLCal(chcsBean[0].getCertifiedDate()));
//
//        //bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setVerifiedBy(chcsBean[0].getCertifiedBy());
//        bean.setSpecimen("NA");
//
//        StringBuilder result = new StringBuilder();
//        String temp = "";
//        if (chcsBean.length > 0) {
//            result.append("<table width='100%'>");
//            result.append("<tr><td><b>Test Name</b></td><td><b>Status</b></td><td><b>CertifiedBy/Certified Date</b></td></tr>");
//
//            for (int i = 0; i < chcsBean.length; i++) {
//                String test = chcsBean[i].getName();
//                String stat = chcsBean[i].getResultStatus();
//                String certBy = chcsBean[i].getCertifiedBy();
//                String certDt = chcsBean[i].getCertifiedDate();
//                result.append("<tr><td>");
//                result.append(test);
//                result.append("</td><td>");
//                result.append(stat);
//                result.append("</td><td>");
//                result.append(certBy);
//                result.append("/");
//                result.append(certDt);
//                result.append("</td></tr>");
//                // get results
//                temp = addText("", "Gram Stain: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                temp = addText(chcsBean[i].getResult(), "\nBacteriology Result: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                result.append("<tr><td colspan='3'>");
//
//                //clear temp for use with organisms
//                temp = "";
//
//                if (chcsBean[i].getOrganisms() != null && chcsBean[i].getOrganisms().size() > 0) {
//                    List<OrganismBean> orgBean = null;
////                        java.util.List<OrganismBean> orgList = chcsBean[i].getOrganisms();
////                        orgBean = (OrganismBean[])orgList.toArray(new OrganismBean[0]);
//                    orgBean = chcsBean[i].getOrganisms();
//                    temp = addOrganisms(orgBean, "Organism", "BT");
//                }
//
//
//                if (temp.length() > 0) {
//                    result.append(temp);
//                }
//            }
//            result.append("&nbsp;</td></tr></table>");
//        }
//        bean.setResult(result.toString());
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//    public LabResult[] getMYLabResultsFromCHCS(MicrobiologyResultBean[] chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//
//
//        bean.setAccession(chcsBean[0].getAccession());
//        bean.setPrintName(chcsBean[0].getName());
//        bean.setTestName(chcsBean[0].getName());
//        bean.setResultStatus("NA");
//        bean.setSpecimen("");
//        bean.setResultDate(Utils.getXMLCal(chcsBean[0].getCertifiedDate()));
//        //bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setVerifiedBy(chcsBean[0].getCertifiedBy());
//
//        bean.setSpecimen("NA");
//
//        StringBuilder result = new StringBuilder();
//        String temp = "";
//        if (chcsBean.length > 0) {
//            result.append("<table width='100%'><tr><td><b>Test Name</b></td><td><b>Status</b></td><td><b>CertifiedBy/Certified Date</b></td></tr>");
//            for (int i = 0; i < chcsBean.length; i++) {
//                String test = chcsBean[i].getName();
//                String stat = chcsBean[i].getResultStatus();
//                String certBy = chcsBean[i].getCertifiedBy();
//                String certDt = chcsBean[i].getCertifiedDate();
//                String gramStat = chcsBean[i].getGramStatus();
//
//                if (gramStat == null) {
//                    gramStat = "";
//                }
//
//                result.append("<tr><td>");
//                result.append(test);
//                result.append("</td><td>");
//                result.append(stat);
//                result.append("</td><td>");
//                result.append(certBy);
//                result.append("/");
//                result.append(certDt);
//                result.append("</td></tr>");
//                if (gramStat.length() > 0) {
//                    result.append("<tr><td></td><td>");
//                    result.append(gramStat);
//                    result.append("</td><td></td></tr>");
//                }
//                // get results
//
//                temp = addText("", "Smear/Prep: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                temp = addText(chcsBean[i].getResult(), "Mycology Result: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                result.append("<tr><td colspan='3'>");
//                result.append(addOrganisms(chcsBean[i].getOrganisms(), "Yeast/Fungus: ", "MY"));
//
//            }
//            result.append("&nbsp;</td></tr></table>");
//        }
//        bean.setResult(result.toString());
//
//
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//
//    public LabResult[] getTBLabResultsFromCHCS(MicrobiologyResultBean[] chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//        bean.setAccession(chcsBean[0].getAccession());
//        bean.setPrintName(chcsBean[0].getName());
//        bean.setTestName(chcsBean[0].getName());
//        bean.setResultStatus("NA");
//        bean.setSpecimen("");
//        bean.setResultDate(Utils.getXMLCal(chcsBean[0].getCertifiedDate()));
//
//        //bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setVerifiedBy(chcsBean[0].getCertifiedBy());
//
//        bean.setSpecimen("NA");
//
//        StringBuilder result = new StringBuilder();
//        String temp = "";
//        if (chcsBean.length > 0) {
//
//            result.append("<table width='100%'><tr><td><b>Test Name</b></td><td><b>Status</b></td><td><b>CertifiedBy/Certified Date</b></td></tr>");
//            for (int i = 0; i < chcsBean.length; i++) {
//
//                String test = chcsBean[i].getName();
//                String stat = chcsBean[i].getResultStatus();
//                String certBy = chcsBean[i].getCertifiedBy();
//                String certDt = chcsBean[i].getCertifiedDate();
//                String gramStat = chcsBean[i].getGramStatus();
//                result.append("<tr><td>");
//                result.append(test);
//                result.append("</td><td>");
//                result.append(stat);
//                result.append("</td><td>");
//                result.append(certBy);
//                result.append("/");
//                result.append(certDt);
//                result.append("</td></tr>");
//                if (gramStat != null && gramStat.length() > 0) {
//                    result.append("<tr><td></td><td>");
//                    result.append(gramStat);
//                    result.append("</td><td></td></tr>");
//                }
//                // get results
//                temp = addText("", "Acidfast Stain: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                temp = addText(chcsBean[i].getResult(), "TB Result: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                result.append("<tr><td colspan='3'>");
//                result.append(addOrganisms(chcsBean[i].getOrganisms(), "Organism ", "TB"));
//            }
//            result.append("&nbsp;</td></tr></table>");
//        }
//        bean.setResult(result.toString());
//
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//
//    public LabResult[] getOPLabResultsFromCHCS(MicrobiologyResultBean[] chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//        bean.setAccession(chcsBean[0].getAccession());
//        bean.setPrintName(chcsBean[0].getName());
//        bean.setTestName(chcsBean[0].getName());
//        bean.setResultStatus("NA");
//        bean.setSpecimen("");
//        bean.setResultDate(Utils.getXMLCal(chcsBean[0].getCertifiedDate()));
//        // bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setVerifiedBy(chcsBean[0].getCertifiedBy());
//
//        bean.setSpecimen("NA");
//
//        StringBuilder result = new StringBuilder();
//        String temp = "";
//        if (chcsBean.length > 0) {
//
//            result.append("<table width='100%'><tr><td><b>Test Name</b></td><td><b>Status</b></td><td><b>CertifiedBy/Certified Date</b></td></tr>");
//            for (int i = 0; i < chcsBean.length; i++) {
//
//                String test = chcsBean[i].getName();
//                String stat = chcsBean[i].getResultStatus();
//                String certBy = chcsBean[i].getCertifiedBy();
//                String certDt = chcsBean[i].getCertifiedDate();
//                result.append("<tr><td>");
//                result.append(test);
//                result.append("</td><td>");
//                result.append(stat);
//                result.append("</td><td>");
//                result.append(certBy);
//                result.append("/");
//                result.append(certDt);
//                result.append("</td></tr>");
//                // get results
//                temp = addText("", "Parasitology Result: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                temp = addText(chcsBean[i].getResult(), "Stool Result: ", "");
//                if (temp.length() > 0) {
//                    result.append("<tr><td colspan='3'>");
//                    result.append(temp);
//                    result.append("</td></tr>");
//                }
//                result.append("<tr><td colspna='3'>");
//
//                if (chcsBean[i].getOrganisms() != null && chcsBean[i].getOrganisms().size() > 0) {
//                    List<OrganismBean> orgBean = null;
////            java.util.List<OrganismBean> orgList = chcsBean[i].getOrganisms();
////            orgBean = (OrganismBean[])orgList.toArray(new OrganismBean[0]);
//                    orgBean = chcsBean[i].getOrganisms();
//                    result.append(addOrganisms(orgBean, "Parasite ", "OP"));
//                }
//
//            }
//            result.append("&nbsp;</td></tr></table>");
//        }
//        bean.setResult(result.toString());
//
//        vistaBean[0] = bean;
//
//        return vistaBean;
//    }
//
//    public LabResult[] getSPLabResultsFromCHCS(SurgicalPathologyResultBean chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//        bean.setAccession(chcsBean.getAccession());
//        bean.setPrintName(chcsBean.getLabTest());
//        bean.setTestName(chcsBean.getLabTest());
//        bean.setResultStatus("NA");
//        bean.setSpecimen("");
//        bean.setResultDate(Utils.getXMLCal(chcsBean.getDateTimeCertified()));
//        //bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setVerifiedBy(chcsBean.getCertifiedBy());
//
//        bean.setSpecimen("NA");
//
//        // get report
//        if (chcsBean.isIsCopathReport()) {
//            bean.setResult(chcsBean.getCopathReport());
//        } else {
//            StringBuilder result = new StringBuilder();
//            result.append("<basefont size=\"4\"><PRE><PRE>");
//
//            result.append(addText(chcsBean.getOrderingProvider(), "Ordering User: ", ""));
//            result.append(addText(chcsBean.getRequestingLocation(), "Requesting Location: ", ""));
//            result.append(addText(chcsBean.getAccessionComment(), "Accession Comment:", "\n"));
//            result.append(addText(bean.getResultDate().toString(), "Certified Date/Time: ", ""));
//            result.append(addText(chcsBean.getMicroscopicDiagnosis(), "Microscopic Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getPathologyComment(), "Diagnosis Comments: ", "\n"));
//            result.append(addText(chcsBean.getGrossDiagnosis(), "Gross Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getFrozenDiagnosis(), "Frozen Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getFrozenComment(), "Frozen Comment: ", "\n"));
//
//            if (chcsBean.getResidents() != null && chcsBean.getResidents().size() > 0) {
//                List<EntityBean> resBean = null;
////            java.util.List<EntityBean> resList = chcsBean.getResidents();
////            resBean = (EntityBean[])resList.toArray(new EntityBean[0]);
//                resBean = chcsBean.getResidents();
//                result.append(addEntity(resBean, "Residents: "));
//            }
//
//            if (chcsBean.getPathologists() != null && chcsBean.getPathologists().size() > 0) {
//                List<EntityBean> pathBean = null;
////            java.util.List<EntityBean> pathList = chcsBean.getPathologists();
////            pathBean = (EntityBean[])pathList.toArray(new EntityBean[0]);
//                pathBean = chcsBean.getPathologists();
//                result.append(addEntity(pathBean, "Staff Pathologists: "));
//            }
//
//            if (chcsBean.getSpecimens() != null && chcsBean.getSpecimens().size() > 0) {
//                List<SpecimenBean> specBean = null;
////              java.util.List<SpecimenBean> specimenList = chcsBean.getSpecimens();
////              specBean = (SpecimenBean[])specimenList.toArray(new SpecimenBean[0]);
//                specBean = chcsBean.getSpecimens();
//                result.append(addSpecimens(specBean));
//
//            }
//
//            result.append(addText(chcsBean.getMicroscopicDescription(), "Microscopic Description: ", "\n"));
//            result.append(addText(chcsBean.getGrossDescription(), "Gross Description: ", "\n"));
//            result.append(addText(chcsBean.getFrozenDesc(), "Frozen Description: ", "\n"));
//            result.append(addText(Utils.formatText(chcsBean.getClinicalHistory()), "Clinical History:", "\n "));
//            result.append(addText(Utils.formatText(chcsBean.getPreopDiagnosis()), "Preoperative Diagnosis: ", "\n"));
//            result.append(addText(Utils.formatText(chcsBean.getOtherFindings()), "Operative Findings:", "\n "));
//            result.append(addText(Utils.formatText(chcsBean.getPostopDiagnosis()), "Postoperative Diagnosis:", "\n "));
//            result.append(addText(Utils.formatText(chcsBean.getLoginComment()), "Login Comment:", "\n "));
//
//            if (chcsBean.getSnoMedCodes() != null && chcsBean.getSnoMedCodes().size() > 0) {
//                List<SnoMedCodeBean> snowBean = null;
////            java.util.List<SnoMedCodeBean> snowList = chcsBean.getSnoMedCodes();
////            snowBean = (SnoMedCodeBean[])snowList.toArray(new SnoMedCodeBean[0]);
//                snowBean = chcsBean.getSnoMedCodes();
//                result.append(addSnomedCodes(snowBean));
//            }
//
//            result.append("</PRE></basefont>");
//            bean.setResult(result.toString());
//        }
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//    public LabResult[] getVILabResultsFromCHCS(MicrobiologyResultBean[] chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//        bean.setAccession(chcsBean[0].getAccession());
//        bean.setPrintName(chcsBean[0].getName());
//        bean.setTestName(chcsBean[0].getName());
//        bean.setResultStatus("NA");
//        bean.setSpecimen("");
//        bean.setResultDate(Utils.getXMLCal(chcsBean[0].getCertifiedDate()));
//
//        bean.setVerifiedBy(chcsBean[0].getCertifiedBy());
//        //bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        bean.setSpecimen("NA");
//
//        StringBuilder resultSB = new StringBuilder("");
//        String temp = "";
//        if (chcsBean.length > 0) {
//
//            resultSB.append("<table width='100%'><tr><td><b>Test Name</b></td><td><b>Status</b></td><td><b>CertifiedBy/Certified Date</b></td></tr>");
//            for (int i = 0; i < chcsBean.length; i++) {
//
//                String test = chcsBean[i].getName();
//                String stat = chcsBean[i].getResultStatus();
//                String certBy = chcsBean[i].getCertifiedBy();
//                String certDt = chcsBean[i].getCertifiedDate();
//                resultSB.append("<tr><td>");
//                resultSB.append(test);
//                resultSB.append("</td><td>");
//                resultSB.append(stat);
//                resultSB.append("</td><td>");
//                resultSB.append(certBy);
//                resultSB.append("/");
//                resultSB.append(certDt);
//                resultSB.append("</td></tr>");
//                // get results
//                temp = addText(chcsBean[i].getResult(), "Virology Result: ", "");
//                if (temp.length() > 0) {
//                    resultSB.append("<tr><td colspan='3'>");
//                    resultSB.append(temp);
//                    resultSB.append("</td></tr>");
//                }
//                resultSB.append("<tr><td colspan='3'>");
//
//                if (chcsBean[i].getOrganisms() != null && chcsBean[i].getOrganisms().size() > 0) {
//                    List<OrganismBean> orgBean = null;
////                java.util.List<OrganismBean> orgList = chcsBean[i].getOrganisms();
////                orgBean = (OrganismBean[])orgList.toArray(new OrganismBean[0]);
//                    orgBean = chcsBean[i].getOrganisms();
//                    resultSB.append(addOrganisms(orgBean, "Virus", "VI"));
//                }
//            }
//
//            resultSB.append("&nbsp;</td></tr></table>");
//        }
//        bean.setResult(resultSB.toString());
//
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }
//
//    public LabResult[] getAULabResultsFromCHCS(AutopsyResultBean chcsBean, String siteId) {
//        LabResult[] vistaBean = new LabResult[1];
//
//        LabResult bean = new LabResult();
//        bean.setAccession(chcsBean.getAccession());
//        bean.setPrintName(chcsBean.getLabTest());
//        bean.setTestName(chcsBean.getLabTest());
//        bean.setResultDate(Utils.getXMLCal(chcsBean.getDateTimeCertified()));
//        bean.setSpecimen(chcsBean.getAutopsyType());
//        bean.setResultStatus(chcsBean.getOrderTaskStatus());
//        //bean.setSiteCode(siteId);
//        addCHCSSiteDataToVistaBean(bean);
//
//        // get report
//        if (chcsBean.getCopathReport() != null && chcsBean.getCopathReport().length() > 0) {
//            bean.setResult(chcsBean.getCopathReport());
//        } else {
//            StringBuilder result = new StringBuilder();
//            result.append("<basefont size=\"4\"><PRE><PRE>");
//            result.append(addText(chcsBean.getOrderingProvider(), "Ordering User: ", ""));
//            result.append(addText(chcsBean.getRequestingLocation(), "Requesting Location: ", ""));
//            result.append(addText(chcsBean.getOrderComment(), "Order Comment:", "\n"));
//            result.append(addText(chcsBean.getAccessionComment(), "Accession Comment:", "\n"));
//            result.append(addText(Utils.formatDisplayDateTime(Utils.getDate(chcsBean.getDateOfDeath())), "Date/Time of Death: ", ""));
//            result.append(addText(Utils.formatDisplayDateTime(Utils.getDate(chcsBean.getDateOfAutopsy())), "Date/Time of Autopsy: ", ""));
//            result.append(addText(chcsBean.getCauseOfDeath(), "Cause of Death: ", ""));
//            result.append(addText(chcsBean.getMannerOfDeath(), "Manner of Death: ", ""));
//            result.append(addText(chcsBean.getFinalSummary(), "Final Summary: ", "\n"));
//            result.append(addText(chcsBean.getPathologyComment(), "Comments: ", "\n"));
//
//            if (chcsBean.getPathologists() != null && chcsBean.getPathologists().size() > 0) {
//                List<EntityBean> pathBean = null;
////            java.util.List<EntityBean> pathList = chcsBean.getPathologists();
////            pathBean = (EntityBean[])pathList.toArray(new EntityBean[0]);
//                pathBean = chcsBean.getPathologists();
//                result.append(addEntity(pathBean, "Staff Pathologists: "));
//            }
//            if (chcsBean.getProsectors() != null && chcsBean.getProsectors().size() > 0) {
//                List<EntityBean> prosBean = null;
////            java.util.List<EntityBean> prosList = chcsBean.getProsectors();
////            prosBean = (EntityBean[])prosList.toArray(new EntityBean[0]);
//                prosBean = chcsBean.getProsectors();
//                result.append(addEntity(prosBean, "Prosector(s): "));
//            }
//            result.append(addText(chcsBean.getPathologyDiagnosis(), "Microscopic Diagnosis: ", "\n"));
//            result.append(addText(chcsBean.getMicroExam(), "Micro Examination: ", "\n"));
//            result.append(addText(chcsBean.getPostmortumInterval(), "Postmortem Interval: ", ""));
//            result.append(addText(chcsBean.getGrossExam(), "Gross Examination: ", "\n"));
//            result.append(addText(chcsBean.getClinicalDiagnosis(), "Clinical Diagnosis: ", "\n"));
//            result.append(addText(Utils.formatText(chcsBean.getClinicalHistory()), "Clinical History:", "\n "));
//            result.append(addText(chcsBean.getSpecialStudies(), "Special Lab Studies: ", "\n"));
//            result.append(addText(chcsBean.getPreviousDiagnosis(), "Log-in Comment: ", "\n"));
//
//            if (chcsBean.getSpecimens() != null && chcsBean.getSpecimens().size() > 0) {
//                List<SpecimenBean> specBean = null;
////              java.util.List<SpecimenBean> specimenList = chcsBean.getSpecimens();
////              specBean = (SpecimenBean[])specimenList.toArray(new SpecimenBean[0]);
//                specBean = chcsBean.getSpecimens();
//                result.append(addSpecimens(specBean));
//
//            }
//
//            if (chcsBean.getSnoMedCodes() != null && chcsBean.getSnoMedCodes().size() > 0) {
//                List<SnoMedCodeBean> snowBean = null;
////            java.util.List<SnoMedCodeBean> snowList = chcsBean.getSnoMedCodes();
////            snowBean = (SnoMedCodeBean[])snowList.toArray(new SnoMedCodeBean[0]);
//                snowBean = chcsBean.getSnoMedCodes();
//                result.append(addSnomedCodes(snowBean));
//            }
//
//            result.append("</PRE></basefont>");
//            bean.setResult(result.toString());
//        }
//        vistaBean[0] = bean;
//
//
//        return vistaBean;
//    }


//    private static String addText(String text, String title, String sNewLine) {
//        StringBuilder result = new StringBuilder();
//
//        if (text != null && text.length() > 0) {
//            result.append("\n<b>");
//            result.append(title);
//            result.append("</b>");
//            result.append(sNewLine);
//            result.append(text);
//            result.append(sNewLine);
//        }
//
//        return result.toString();
//    }

//    private static String addSpecimens(List<SpecimenBean> spec) {
//        StringBuilder resultSB = new StringBuilder("");
//        int specSize = spec.size();
//        if (specSize > 0) {
//            resultSB.append("\n<b>Specimen(s):</b>\n");
//
//            for (int j = 0; j < specSize; j++) {
//
//                String container = spec.get(j).getContainer();
//                String frozen = spec.get(j).getFrozen();
//                String topo = spec.get(j).getTopography();
//                String description = spec.get(j).getDescription();
//
//                if (container != null) {
//                    resultSB.append(container);
//                }
//                if (frozen != null && "1".equals(frozen)) {
//                    resultSB.append("[Frozen] : ");
//                } else {
//                    resultSB.append(" : ");
//                }
//                if (description != null && description.length() > 0) {
//                    resultSB.append(description);
//                    resultSB.append("\n");
//                }
//                if (topo != null && topo.length() > 0) {
//                    resultSB.append(topo);
//                    resultSB.append("\n");
//                }
//            }
//        }
//        return resultSB.toString();
//    }
//
//    private static String addSnomedCodes(List<SnoMedCodeBean> snomed) {
//        StringBuilder resultSB = new StringBuilder("");
//        int snomedSize = snomed.size();
//        if (snomedSize > 0) {
//            resultSB.append("\n<b>Snomed Code(s):</b>\n");
//
//            for (int j = 0; j < snomedSize; j++) {
//                String container = snomed.get(j).getContainer();
//                String code = snomed.get(j).getCode();
//                String topo = snomed.get(j).getDescription();
//
//                if (code != null) {
//                    resultSB.append(code);
//                }
//                if (container != null) {
//                    resultSB.append("  ");
//                    resultSB.append(container);
//                } else {
//                    resultSB.append(" : ");
//                }
//                if (topo != null && topo.length() > 0) {
//                    resultSB.append(topo);
//                    resultSB.append("\n");
//                } else {
//                    resultSB.append("\n");
//                }
//            }
//        }
//        return resultSB.toString();
//    }
//
//    private static String addOrganisms(List<OrganismBean> organism, String title, String subscript) {
//        StringBuilder resultSB = new StringBuilder("");
//        int organismSize = organism.size();
//        if (organismSize > 0) {
//            resultSB.append("<table><tr><td>&nbsp;</td></tr>");
//            if ("OP".equals(subscript)) {
//                resultSB.append("<tr><td><b>");
//                resultSB.append(title);
//                resultSB.append("</b></td><td><b>Stage</b></td></tr>");
//            } else {
//                resultSB.append("<tr><td><b>");
//                resultSB.append(title);
//                resultSB.append("</b></td><td><b>Quantity</b></td></tr>");
//            }
//            int oCnt = 0;
//            int[] org = new int[organismSize];
//            StringBuilder name = new StringBuilder();
//            for (int j = 0; j < organismSize; j++) {
//                String number = organism.get(j).getOrganismNumber();
//                name.setLength(0);
//                String tempName = organism.get(j).getOrganism();
//                String quantity = organism.get(j).getQuantity();
//                String stage = organism.get(j).getStage();
//                org[j] = 0;
//                if (tempName != null) {
//                    name.append("(");
//                    name.append(number);
//                    name.append(")  ");
//                    name.append(tempName);
//                }
//
//                if (stage != null && stage.length() > 0) {
//                    quantity = stage;
//                }
//
//                if (quantity == null) {
//                    quantity = "";
//                }
//
//                resultSB.append("<tr><td align='center'>");
//                resultSB.append(name.toString());
//                resultSB.append("</td><td align='center'>");
//                resultSB.append(quantity);
//                resultSB.append("</td></tr>");
//
//                List<AntibioticBean> sBean = null;
//
//                if (organism.get(j).getAntibioticSensitivities() != null && organism.get(j).getAntibioticSensitivities().size() > 0) {
////                java.util.List<AntibioticBean> antiList = organism[j].getAntibioticSensitivities();
////                sBean = (AntibioticBean[]) antiList.toArray(new AntibioticBean[0]);
//                    sBean = organism.get(j).getAntibioticSensitivities();
//                }
//
//
//                resultSB.append("</table>");
//                if (sBean != null && (sBean.size() > 0)) {
//                    AntibioticBean[] sens = (AntibioticBean[]) SmartSensitivity.sort(sBean.toArray(new AntibioticBean[1]));
//                    resultSB.append("<table><tr><td align='top'><b>Antibiotic</b></td>");
//                    for (int j2 = 0; j2 < oCnt; j2++) {
//                        int jNum = j2 + 1;
//                        if (org[j2] == 1) {
//                            resultSB.append("<td align='center'><b>(");
//                            resultSB.append(jNum);
//                            resultSB.append(")</b></td>");
//                        }
//                    }
//                    resultSB.append("</tr><tr><td align='top'>&nbsp;</td>");
//                    for (int j3 = 0; j3 < oCnt; j3++) {
//
//                        if (org[j3] == 1) {
//                            resultSB.append("<td align='right'><b>mcg/ml</b></td><td align='top'><b>Interp</b></td>");
//                        }
//                    }
//                    resultSB.append("</tr>");
//                    String lastAnti = "";
////                    int nCnt = 0;
//                    for (int k = 0; k < sens.length; k++) {
//                        String antibiotic = sens[k].getAntibioticName();
//                        String res = sens[k].getAntibioticResult();
//                        String interp = sens[k].getInterpretation();
//
//                        if (interp == null) {
//                            interp = "";
//                        }
//
//                        if (!lastAnti.equals(antibiotic)) {
//                            if (k > 0) {
//                                resultSB.append("</tr>");
//                            }
//                            resultSB.append("<tr><td>");
//                            resultSB.append(antibiotic);
//                            resultSB.append("</td>");
//                            lastAnti = antibiotic;
////                            nCnt = 1;
//                        }
//
//                        resultSB.append("<td align='right'>");
//                        resultSB.append(res);
//                        resultSB.append("/");
//                        resultSB.append("</td><td>");
//                        resultSB.append(interp);
//                        resultSB.append("</td>");
////                        nCnt++;
//                    }
//                    resultSB.append("</tr></table>");
//                }
//            }
//        }
//        return resultSB.toString();
//
//    }
//
//    private static String addEntity(List<EntityBean> entity, String title) {
//        StringBuilder resultSB = new StringBuilder("");
//        int entitySize = entity.size();
//        if (entitySize > 0) {
//            resultSB.append("\n<b>");
//            resultSB.append(title);
//            resultSB.append("</b>\n");
//
//            for (int j = 0; j < entitySize; j++) {
//                String name = entity.get(j).getName();
//                resultSB.append(name);
//                resultSB.append("\n");
//            }
//        }
//        return resultSB.toString();
//
//    }
//
//    private static String addAntibodies(List<BloodBankAntibodyBean> antibody /*, String title*/) {
//        StringBuilder resultSB = new StringBuilder("");
//        if (antibody != null && !antibody.isEmpty()) {
//            resultSB.append("<table><tr><td><b>Antibody</b></td><td><b>Titre</b></td></tr>");
//
//            int antibodySize = antibody.size();
//            for (int j = 0; j < antibodySize; j++) {
//                String name = antibody.get(j).getAntibody();
//                String titre = antibody.get(j).getTitre();
//                if (titre == null) {
//                    titre = "";
//                }
//                resultSB.append("<tr><td>");
//                resultSB.append(name);
//                resultSB.append("</td><td>");
//                resultSB.append(titre);
//                resultSB.append("</td></tr>");
//            }
//            resultSB.append("</table></n>");
//        }
//        return resultSB.toString();
//    }
//
//    private static String addUnits(List<BloodBankUnitBean> unit, String title) {
//        StringBuilder resultSB = new StringBuilder("");
//        if (unit != null && !unit.isEmpty()) {
//            resultSB.append("<b>");
//            resultSB.append(title);
//            resultSB.append("</b>");
//            resultSB.append("\n<table><tr><td><b>Product</b></td><td><b>Unit Date</b></td><td><b># of Units</b></td><td><b># Available</b></td><td><b># Issued</b></td><td><b># Transfused</b></td></tr>");
//
//            int unitSize = unit.size();
//            for (int j = 0; j < unitSize; j++) {
//                String name = unit.get(j).getProduct();
//                String udate = Utils.formatDisplayDateTime(Utils.getDate(unit.get(j).getUnitDate()));
//                String numUnits = unit.get(j).getNumberOfUnits();
//                String numAvail = unit.get(j).getNumberAvailable();
//                String numIssued = unit.get(j).getNumberIssued();
//                String numTrans = unit.get(j).getNumberTransfused();
//
//                resultSB.append("<tr><td>");
//                resultSB.append(name);
//                resultSB.append("</td><td>");
//                resultSB.append(udate);
//                resultSB.append("</td><td align='right'>");
//                resultSB.append(numUnits);
//                resultSB.append("</td><td align='right'>");
//                resultSB.append(numAvail);
//                resultSB.append("</td><td align='right'>");
//                resultSB.append(numIssued);
//                resultSB.append("</td><td align='right'>");
//                resultSB.append(numTrans);
//                resultSB.append("</td></tr>");
//            }
//            resultSB.append("</table>\n");
//        }
//        return resultSB.toString();
//
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        LabOrder lab = new LabOrder();

        lab.setTestName(errorMsg);
        lab.setCollectionDate(Utils.backDate());
        lab.setSite(getConnectionUnavailableSite(location));

        return new LabOrder[]{lab};
    }
}
