/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.*;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.LabResult;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

//import org.hui.chcs.LabResultBean;

public class SmartLabResult extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartLabResult.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;

    public LabResult m_result;

    public SmartLabResult() {
        super();
        m_result = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartLabResult(LabResult result, String sort) {
        super();
        m_result = result;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartLabResult) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_result);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartLabResult[] temp = new SmartLabResult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartLabResult newAll = new SmartLabResult((LabResult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartLabResult newAll = new SmartLabResult((LabResult) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            LabResult[] ret = new LabResult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_result;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartLabResult[] temp = new SmartLabResult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartLabResult newAll = new SmartLabResult((LabResult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            LabResult[] ret = new LabResult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_result;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartLabResult tmp = (SmartLabResult) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_result.getResultDate(), tmp.m_result.getResultDate());
        }
        if (this.m_sortBy == SITE && this.m_result.getSite() != null && tmp.m_result.getSite() != null) {
            retVal = compareString(this.m_result.getSite().getSiteCode(), tmp.m_result.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_result.getPrintName(), tmp.m_result.getPrintName());
        }
        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("resultDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
        }
    }

//    public LabResult[] getLabTestResultsFromCHCS(Object[] dodBeans) {
//        LabResultBean[] dodLabResults = (LabResultBean[]) dodBeans;
//        LabResult[] labResults = new LabResult[dodBeans.length];
//
//        for (int i = 0; i < dodBeans.length; i++) {
//            LabResult lresult = new LabResult();
//            lresult.setTestId(dodLabResults[i].getTestId());
//            lresult.setTestName(dodLabResults[i].getTestName());
//            lresult.setResult(dodLabResults[i].getResult());
//            lresult.setResultDate(Utils.getXMLCal(dodLabResults[i].getResultedDate()));
//            lresult.setAccession(dodLabResults[i].getAccession());
//            lresult.setUnits(dodLabResults[i].getUnits());
//            lresult.setReferenceRange(dodLabResults[i].getReferenceRange());
//            lresult.setSpecimen(dodLabResults[i].getSpecimen());
//            lresult.setInterpretation(dodLabResults[i].getAlertFlag());
//            addCHCSSiteDataToVistaBean(lresult);
//            labResults[i] = lresult;
//        }
//
//        return labResults;
//    }

    /**
     * Map labs from BHIE to Vista Beans.
     *
     * @param resultCollection BHIE Result collection.
     * @return List of BHIE lab results that are no longer grouped by Order.
     */
    public LabResult[] mapLabChemResultsFromBHIE(Object resultCollection) {
        V4ResultCollection labResultCollection = (V4ResultCollection) resultCollection;
        List<LabResult> vistaBeans = new ArrayList<LabResult>();
        String testType = "";

        if (labResultCollection != null && labResultCollection.getDataRecordList() != null) {
            List<ResultV4> bhieBean = new ArrayList<ResultV4>();

            if (labResultCollection.getDataRecordList().getLabChemResults() != null
                    && labResultCollection.getDataRecordList().getLabChemResults().size() > 0) {
                bhieBean.addAll(labResultCollection.getDataRecordList().getLabChemResults());
            }

            if (bhieBean != null) {
                List<LabResult> labResults = new ArrayList<LabResult>();

                for (ResultV4 bhieLabResult : bhieBean) {
                    for (ResultsV4 bhieResult : bhieLabResult.getResults()) {
                        LabResult labResult = new LabResult();

                        labResult.setTestType("CHEM");

                        labResult.setSensitive(bhieLabResult.isSensitive());
                        //labResult.setOrderDate(bhieToXMLCalV4(bhieLabResult.getEnteredDate()));

                        if (bhieLabResult.getOrder() != null && bhieLabResult.getOrder().getOrderNumber() != null) {
                            labResult.setOrderId(bhieLabResult.getOrder().getOrderNumber().getId());
                            if (bhieLabResult.getOrder().getDate() != null) {
                                labResult.setOrderDate(bhieToXMLCalV4(bhieLabResult.getOrder().getDate()));
                            }
                            if (bhieLabResult.getOrder().getAccession() != null) {
                                StringBuilder accession = new StringBuilder();
                                accession.append(bhieLabResult.getOrder().getAccession().getId());
                                accession.append("^CH");
                                labResult.setAccession(accession.toString());
                            }
                            if (bhieLabResult.getOrder().getProvider() != null) {
                                labResult.setProvider(mapProvider(bhieLabResult.getOrder().getProvider()));
                            }
                            if (bhieLabResult.getOrder().getSpecimen() != null && !bhieLabResult.getOrder().getSpecimen().isEmpty()
                                    && getPrimarySpdcV4(bhieLabResult.getOrder().getSpecimen()) != null) {
                                labResult.setSpecimen(getPrimarySpdcV4(bhieLabResult.getOrder().getSpecimen()).getDisplay());
                            }
                            if (bhieLabResult.getOrder().getRequestingLocation() != null && bhieLabResult.getOrder().getRequestingLocation().getName() != null) {
                                labResult.setRequestingLocation(bhieLabResult.getOrder().getRequestingLocation().getName());
                            }
                            labResult.setOrderComment(bhieLabResult.getOrder().getComment());
                            labResult.setCollectedDate(bhieToXMLCalV4(bhieLabResult.getOrder().getCollectionDate()));

                        }

                        if (bhieResult.getUnits() != null && !bhieResult.getUnits().isEmpty()
                                && getPrimarySpdcV4(bhieResult.getUnits()) != null) {
                            labResult.setUnits(getPrimarySpdcV4(bhieResult.getUnits()).getDisplay());
                        }

                        if (bhieResult.getCodedInterpretation() != null && !bhieResult.getCodedInterpretation().isEmpty()
                                && getPrimarySpdcV4(bhieResult.getCodedInterpretation()) != null) {
                            labResult.setInterpretation(getPrimarySpdcV4(bhieResult.getCodedInterpretation()).getDisplay());
                        }

//                        if (bhieResult.getPerformingLabLocation() != null) {
//                            if (bhieResult.getPerformingLabLocation().getAddress() != null && bhieResult.getPerformingLabLocation().getAddress().getRaw() != null) {
//                                labResult.setPerformingLabStreet1(bhieResult.getPerformingLabLocation().getAddress().getRaw());
//                            }
//                            labResult.setPerformingLabName(bhieResult.getPerformingLabLocation().getName());
//                        }

                        if (bhieResult.getTest() != null) {
                            for (SpdcBeanV4 test : bhieResult.getTest()) {
                                if (CodingSystem.DOD_NCID.getOID().equalsIgnoreCase(test.getSystem())) {
                                    Code code = new Code();
                                    code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                                    code.setCode(test.getCode());
                                    labResult.getCodes().add(code);
                                }
                            }

                        }

                        if (bhieResult.getStatus() != null && !bhieResult.getStatus().isEmpty()
                                && getPrimarySpdcV4(bhieResult.getStatus()) != null) {
                            labResult.setResultStatus(getPrimarySpdcV4(bhieResult.getStatus()).getDisplay());
                        }

                        if (bhieResult.getCertifiedDate() != null) {
                            labResult.setResultDate(bhieToXMLCalV4(bhieResult.getCertifiedDate()));
                        } else if (bhieLabResult.getEnteredDate() != null) {
                            labResult.setResultDate(bhieToXMLCalV4(bhieLabResult.getEnteredDate()));
                        }
                        labResult.setComment(bhieResult.getComment());
                        labResult.setReferenceRange(bhieResult.getReferenceRange());
                        labResult.setResult(bhieResult.getResult());
                        if (getPrimarySpdcV4(bhieResult.getTest()) != null) {
                            labResult.setTestName(getPrimarySpdcV4(bhieResult.getTest()).getDisplay());
                        }

                        String facilityName = "";
                        String repositoryName = "";
                        String repositoryId = "";
                        String recordId = "";

                        if (bhieLabResult.getFacility() != null && bhieLabResult.getFacility().getName() != null) {
                            facilityName = bhieLabResult.getFacility().getName();
                        }

                        if (bhieLabResult.getRepository() != null && bhieLabResult.getRepository().getName() != null) {
                            repositoryName = bhieLabResult.getRepository().getName();
                            if (bhieLabResult.getRepository().getIds() != null && bhieLabResult.getRepository().getIds().get(0) != null) {
                                repositoryId = bhieLabResult.getRepository().getIds().get(0).getId();
                            }
                        }

                        if (bhieLabResult.getRecordId() != null) {
                            recordId = bhieLabResult.getRecordId().getId();
                        }

                        addBHIESiteDataToVistaBean(repositoryId,
                                repositoryName, facilityName, recordId, labResult);

                        labResults.add(labResult);

                    }
                }
                vistaBeans.addAll(labResults);
            }

        }
        String siteStatusReport = "";
        if (labResultCollection != null) {
            siteStatusReport = generateBhieStatusReportV4("Lab Chemistries", labResultCollection.getStatusListv4(),
                    labResultCollection.getErrorListv4());
        }

        vistaBeans.addAll(Arrays.asList(
                (LabResult[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new LabResult[0]);
    }

    /**
     * Map labs from BHIE to Vista Beans.
     *
     * @param resultCollection BHIE Result collection.
     * @return List of BHIE lab results that are no longer grouped by Order.
     */
    public LabResult[] mapLabMicroResultsFromBHIE(Object resultCollection) {
        V4ResultCollection labResultCollection = (V4ResultCollection) resultCollection;
        List<LabResult> vistaBeans = new ArrayList<LabResult>();

        if (labResultCollection != null && labResultCollection.getDataRecordList() != null) {
            List<ResultV4> bhieBean = new ArrayList<ResultV4>();

            if (labResultCollection.getDataRecordList().getLabMicroResults() != null
                    && labResultCollection.getDataRecordList().getLabMicroResults().size() > 0) {
                bhieBean.addAll(labResultCollection.getDataRecordList().getLabMicroResults());
            }

            if (bhieBean != null) {
                List<LabResult> labResults = new ArrayList<LabResult>();

                for (ResultV4 bhieLabResult : bhieBean) {
                    if (bhieLabResult == null) {
                        continue;
                    }
                    for (TestV4 bhieTest : bhieLabResult.getTests()) {
                        if (bhieTest == null) {
                            continue;
                        }
                        for (ResultsV4 bhieResult : bhieTest.getResults()) {
                            if (bhieResult == null) {
                                continue;
                            }
                            LabResult labResult = new LabResult();
                            labResult.setTestType("MICRO");
                            labResult.setSensitive(bhieLabResult.isSensitive());
                            labResult.setOrderDate(bhieToXMLCalV4(bhieLabResult.getEnteredDate()));

                            if (bhieLabResult.getOrder() != null && bhieLabResult.getOrder().getOrderNumber() != null) {
                                labResult.setOrderId(bhieLabResult.getOrder().getOrderNumber().getId());
                                if (bhieLabResult.getOrder().getAccession() != null) {
                                    StringBuilder accession = new StringBuilder();
                                    accession.append(bhieLabResult.getOrder().getAccession().getId());
                                    accession.append("^MI");


                                    labResult.setAccession(accession.toString());
                                }
                                if (bhieLabResult.getOrder().getProvider() != null) {
                                    labResult.setProvider(mapProvider(bhieLabResult.getOrder().getProvider()));
                                }
                                if (bhieLabResult.getOrder().getSpecimen() != null && !bhieLabResult.getOrder().getSpecimen().isEmpty()
                                        && getPrimarySpdcV4(bhieLabResult.getOrder().getSpecimen()) != null) {
                                    labResult.setSpecimen(getPrimarySpdcV4(bhieLabResult.getOrder().getSpecimen()).getDisplay());
                                }
                                if (bhieLabResult.getOrder().getRequestingLocation() != null && bhieLabResult.getOrder().getRequestingLocation().getName() != null) {
                                    labResult.setRequestingLocation(bhieLabResult.getOrder().getRequestingLocation().getName());
                                }

                                labResult.setOrderComment(bhieLabResult.getOrder().getComment());
                                //labResult.setAccessionComment
                                if (bhieLabResult.getOrder().getDate() != null) {
                                    labResult.setOrderDate(bhieToXMLCalV4(bhieLabResult.getOrder().getDate()));
                                }
                                labResult.setCollectedDate(bhieToXMLCalV4(bhieLabResult.getOrder().getCollectionDate()));

                            }

                            labResult.setResultDate(bhieToXMLCalV4(bhieLabResult.getCertifiedDate()));


                            StringBuilder resultString = new StringBuilder();
                            resultString.append(bhieResult.getContent());
                            resultString.append("\n");
                            resultString.append(bhieResult.getResult());
                            labResult.setResult(resultString.toString());

                            if (bhieResult.getStatus() != null && !bhieResult.getStatus().isEmpty() && getPrimarySpdcV4(bhieResult.getStatus()) != null) {
                                labResult.setResultStatus(getPrimarySpdcV4(bhieResult.getStatus()).getDisplay());
                            }
                            if (bhieResult.getCodedInterpretation() != null
                                    && !bhieResult.getCodedInterpretation().isEmpty()
                                    && getPrimarySpdcV4(bhieResult.getCodedInterpretation()) != null) {
                                labResult.setInterpretation(getPrimarySpdcV4(bhieResult.getCodedInterpretation()).getDisplay());
                            }

//                            if (bhieResult.getPerformingLabLocation() != null) {
//                                if (bhieResult.getPerformingLabLocation().getAddress() != null && bhieResult.getPerformingLabLocation().getAddress().getRaw() != null) {
//                                    labResult.setPerformingLabStreet1(bhieResult.getPerformingLabLocation().getAddress().getRaw());
//                                }
//                                labResult.setPerformingLabName(bhieResult.getPerformingLabLocation().getName());
//                            }

                            labResult.setComment(bhieResult.getComment());
                            if (bhieResult.getCertifiedDate() != null) {
                                labResult.setResultDate(bhieToXMLCalV4(bhieResult.getCertifiedDate()));
                            }


                            if (bhieTest.getType() != null
                                    && !bhieTest.getType().isEmpty()) {
                                for (SpdcBeanV4 typeBean : bhieTest.getType()) {
                                    if (typeBean.isPrimary()) {
                                        labResult.setTestName(typeBean.getDisplay());
                                    }
                                    if (CodingSystem.DOD_NCID.getOID().equalsIgnoreCase(typeBean.getSystem())) {
                                        Code code = new Code();
                                        code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                                        code.setCode(typeBean.getCode());
                                        labResult.getCodes().add(code);
                                    }
                                }

                            }


                            String facilityName = "";
                            String repositoryName = "";
                            String repositoryId = "";
                            String recordId = "";

                            if (bhieLabResult.getFacility() != null && bhieLabResult.getFacility().getName() != null) {
                                facilityName = bhieLabResult.getFacility().getName();
                            }

                            if (bhieLabResult.getRepository() != null && bhieLabResult.getRepository().getName() != null) {
                                repositoryName = bhieLabResult.getRepository().getName();
                                if (bhieLabResult.getRepository().getIds() != null && bhieLabResult.getRepository().getIds().get(0) != null) {
                                    repositoryId = bhieLabResult.getRepository().getIds().get(0).getId();
                                }
                            }

                            if (bhieLabResult.getRecordId() != null) {
                                recordId = bhieLabResult.getRecordId().getId();
                            }

                            addBHIESiteDataToVistaBean(repositoryId,
                                    repositoryName, facilityName, recordId, labResult);

                            labResults.add(labResult);

                        }
                    }
                }

                vistaBeans.addAll(labResults);
            }

        }
        String siteStatusReport = "";
        if (labResultCollection != null) {
            siteStatusReport = generateBhieStatusReportV4("Lab Microbiologies", labResultCollection.getStatusListv4(),
                    labResultCollection.getErrorListv4());
        }
        vistaBeans.addAll(Arrays.asList(
                (LabResult[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new LabResult[vistaBeans.size()]);
    }

    /**
     * Map labs from BHIE to Vista Beans.
     *
     * @param resultCollection BHIE Result collection.
     * @return List of BHIE lab results that are no longer grouped by Order.
     */
    public LabResult[] mapLabResultsFromBHIE(Object resultCollection) {
        V4ResultCollection labResultCollection = (V4ResultCollection) resultCollection;
        List<LabResult> vistaBeans = new ArrayList<LabResult>();
        String testType = "";

        if (labResultCollection != null && labResultCollection.getDataRecordList() != null) {
            List<ResultV4> bhieBean = new ArrayList<ResultV4>();

            if (labResultCollection.getDataRecordList().getLabApResults() != null
                    && labResultCollection.getDataRecordList().getLabApResults().size() > 0) {
                bhieBean.addAll(labResultCollection.getDataRecordList().getLabApResults());
                testType = "AP";
            }
            if (labResultCollection.getDataRecordList().getExternalPartnerResults() != null
                    && labResultCollection.getDataRecordList().getExternalPartnerResults().size() > 0) {
                bhieBean.addAll(labResultCollection.getDataRecordList().getExternalPartnerResults());
                testType = "EXTERNAL";
            }

            if (bhieBean != null) {
                List<LabResult> labResults = new ArrayList<LabResult>();

                for (ResultV4 bhieLabResult : bhieBean) {
                    if ("EXTERNAL".equalsIgnoreCase(testType)) {
                        LabResult labResult = new LabResult();
                        labResult.setTestType("EXTERNAL");
                        labResult.setSensitive(bhieLabResult.isSensitive());

                        String facilityName = "";
                        String repositoryName = "";
                        String repositoryId = "";
                        String recordId = "";

                        if (bhieLabResult.getFacility() != null && bhieLabResult.getFacility().getName() != null) {
                            facilityName = bhieLabResult.getFacility().getName();
                        }

                        if (bhieLabResult.getRepository() != null && bhieLabResult.getRepository().getName() != null) {
                            repositoryName = bhieLabResult.getRepository().getName();
                            if (bhieLabResult.getRepository().getIds() != null && bhieLabResult.getRepository().getIds().get(0) != null) {
                                repositoryId = bhieLabResult.getRepository().getIds().get(0).getId();
                            }
                        }

                        if (bhieLabResult.getRecordId() != null) {
                            recordId = bhieLabResult.getRecordId().getId();
                        }

                        addBHIESiteDataToVistaBean(repositoryId,
                                repositoryName, facilityName, recordId, labResult);

                        labResults.add(labResult);
                        continue;
                    }
                    if (bhieLabResult == null) {
                        continue;
                    }
                    for (ResultsV4 bhieResult : bhieLabResult.getResults()) {
                        if (bhieResult == null) {
                            continue;
                        }
                        LabResult labResult = new LabResult();

                        if ("AP".equalsIgnoreCase(testType)) {
                            labResult.setTestType("AP");
                        } else {
                            continue;
                        }

                        labResult.setSensitive(bhieLabResult.isSensitive());
                        //labResult.setOrderDate(bhieToXMLCalV4(bhieLabResult.getEnteredDate()));

                        if (bhieLabResult.getOrder() != null && bhieLabResult.getOrder().getOrderNumber() != null) {
                            labResult.setOrderId(bhieLabResult.getOrder().getOrderNumber().getId());
                            if (bhieLabResult.getOrder().getAccession() != null) {
                                StringBuilder accession = new StringBuilder();
                                accession.append(bhieLabResult.getOrder().getAccession().getId());
                                if ("AP".equalsIgnoreCase(labResult.getTestType())) {
                                    accession.append("^");
                                }

                                labResult.setAccession(accession.toString());
                            }
                            if (bhieLabResult.getOrder().getProvider() != null) {
                                labResult.setProvider(mapProvider(bhieLabResult.getOrder().getProvider()));
                            }
                            if (bhieLabResult.getOrder().getSpecimen() != null && !bhieLabResult.getOrder().getSpecimen().isEmpty()
                                    && getPrimarySpdcV4(bhieLabResult.getOrder().getSpecimen()) != null) {
                                labResult.setSpecimen(getPrimarySpdcV4(bhieLabResult.getOrder().getSpecimen()).getDisplay());
                            }
                            if (bhieLabResult.getOrder().getRequestingLocation() != null && bhieLabResult.getOrder().getRequestingLocation().getName() != null) {
                                labResult.setRequestingLocation(bhieLabResult.getOrder().getRequestingLocation().getName());
                            }

                            labResult.setOrderComment(bhieLabResult.getOrder().getComment());
                            //labResult.setAccessionComment
                            labResult.setOrderDate(bhieToXMLCalV4(bhieLabResult.getOrder().getDate()));
                            labResult.setCollectedDate(bhieToXMLCalV4(bhieLabResult.getOrder().getCollectionDate()));

                        }

//                        if (bhieResult.getPerformingLabLocation() != null) {
//                            if (bhieResult.getPerformingLabLocation().getAddress() != null && bhieResult.getPerformingLabLocation().getAddress().getRaw() != null) {
//                                labResult.setPerformingLabStreet1(bhieResult.getPerformingLabLocation().getAddress().getRaw());
//                            }
//                            labResult.setPerformingLabName(bhieResult.getPerformingLabLocation().getName());
//                        }

                        if (bhieResult.getTest() != null
                                && !bhieResult.getTest().isEmpty()) {

                            if (bhieResult.getUnits() != null && !bhieResult.getUnits().isEmpty() && getPrimarySpdcV4(bhieResult.getUnits()) != null) {
                                labResult.setUnits(getPrimarySpdcV4(bhieResult.getUnits()).getDisplay());
                            }

                            if (bhieResult.getCodedInterpretation() != null && !bhieResult.getCodedInterpretation().isEmpty()
                                    && getPrimarySpdcV4(bhieResult.getCodedInterpretation()) != null) {
                                labResult.setInterpretation(getPrimarySpdcV4(bhieResult.getCodedInterpretation()).getDisplay());
                            }

                            for (SpdcBeanV4 test : bhieResult.getTest()) {
                                if (CodingSystem.DOD_NCID.getOID().equalsIgnoreCase(test.getSystem())) {
                                    Code code = new Code();
                                    code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                                    code.setCode(test.getCode());
                                    labResult.getCodes().add(code);
                                }
                            }

                            if (bhieResult.getStatus() != null && !bhieResult.getStatus().isEmpty()
                                    && getPrimarySpdcV4(bhieResult.getStatus()) != null) {
                                labResult.setResultStatus(getPrimarySpdcV4(bhieResult.getStatus()).getDisplay());
                            }

                            if (bhieResult.getCertifiedDate() != null) {
                                labResult.setResultDate(bhieToXMLCalV4(bhieResult.getCertifiedDate()));
                            } else if (bhieLabResult.getEnteredDate() != null) {
                                labResult.setResultDate(bhieToXMLCalV4(bhieLabResult.getEnteredDate()));
                            }
                            labResult.setComment(bhieResult.getComment());
                            labResult.setReferenceRange(bhieResult.getReferenceRange());
                            if ("AP".equalsIgnoreCase(labResult.getTestType())) {
                                labResult.setResult(bhieResult.getContent());
                                StringBuilder specimens = new StringBuilder();
                                int sCount = 0;
                                for (String specimen : bhieResult.getSpecimens()) {
                                    if (sCount > 0) {
                                        specimens.append(", ");
                                    }
                                    specimens.append(specimen);
                                    sCount++;
                                }
                                labResult.setSpecimen(specimens.toString());
                            } else {
                                labResult.setResult(bhieResult.getResult());
                            }

                            if (getPrimarySpdcV4(bhieResult.getTest()) != null) {
                                labResult.setTestName(getPrimarySpdcV4(bhieResult.getTest()).getDisplay());
                            }


                        }

                        String facilityName = "";
                        String repositoryName = "";
                        String repositoryId = "";
                        String recordId = "";

                        if (bhieLabResult.getFacility() != null && bhieLabResult.getFacility().getName() != null) {
                            facilityName = bhieLabResult.getFacility().getName();
                        }

                        if (bhieLabResult.getRepository() != null && bhieLabResult.getRepository().getName() != null) {
                            repositoryName = bhieLabResult.getRepository().getName();
                            if (bhieLabResult.getRepository().getIds() != null && bhieLabResult.getRepository().getIds().get(0) != null) {
                                repositoryId = bhieLabResult.getRepository().getIds().get(0).getId();
                            }
                        }

                        if (bhieLabResult.getRecordId() != null) {
                            recordId = bhieLabResult.getRecordId().getId();
                        }

                        addBHIESiteDataToVistaBean(repositoryId,
                                repositoryName, facilityName, recordId, labResult);

                        labResults.add(labResult);

                    }
                }

                vistaBeans.addAll(labResults);
            }

        }
        String siteStatusReport = "";
        if ("EXTERNAL".equalsIgnoreCase(testType)) {
            siteStatusReport = generateBhieStatusReportV4("External Partner Results", labResultCollection.getStatusListv4(),
                    labResultCollection.getErrorListv4());
        } else {
            siteStatusReport = generateBhieStatusReportV4("Lab Anatomic Pathologies", labResultCollection.getStatusListv4(),
                    labResultCollection.getErrorListv4());
        }

        vistaBeans.addAll(Arrays.asList(
                (LabResult[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new LabResult[0]);
    }

//    /**
//     * Map Chemistry labs from BHIE to Vista Beans.
//     *
//     * @param resultCollection BHIE Result collection.
//     * @return List of BHIE lab results that are no longer grouped by Order.
//     */
//    public LabResult[] mapChemistryLabsFromBHIE(Object resultCollection) {
//        LabChemistryResultCollection labChemResultCollection = (LabChemistryResultCollection) resultCollection;
//        List<LabResult> vistaBeans = new ArrayList<LabResult>();
//
//        if (labChemResultCollection.getDataRecordList() != null) {
//            for (LabChemistryDataRecord dataRecord : labChemResultCollection.getDataRecordList()) {
//                if (dataRecord == null || dataRecord.getLabChemistry() == null) continue;
//
//                LabChemistry bhieBean = dataRecord.getLabChemistry();
//
//                //include
//                if (bhieBean.getResults() != null) {
//                    List<LabChemistryResult> bhieLabResults = bhieBean.getResults();
//                    List<LabResult> labResults = new ArrayList<LabResult>();
//
//                    for (LabChemistryResult labChemResult : bhieLabResults) {
//                        LabResult labResult = new LabResult();
//
//                        labResult.setTestType("CHEM");
//                        labResult.setOrderDate(bhieToXMLCal(bhieBean.getOrderDateTime()));
//                        labResult.setOrderId(bhieBean.getOrderId());
//
//                        String accession = "";
//
//                        if (bhieBean.getAccession() != null)
//                            accession = bhieBean.getAccession();
//
//                        labResult.setAccession(accession + "^CH");
//
//                        labResult.setSpecimen(bhieBean.getSpecimenName());
//
//                        labResult.setTestName(labChemResult.getLabTest());
//                        labResult.setResult(labChemResult.getResult());
//                        labResult.setInterpretation(labChemResult.getAbnormal());
//                        labResult.setReferenceRange(labChemResult.getReferenceRange());
//                        labResult.setResultDate(bhieToXMLCal(bhieBean.getCollectionDateTime()));
//                        labResult.setComment(labChemResult.getResultComment());
//                        labResult.setVerifiedBy(labChemResult.getCertifyName());
//                        labResult.setUnits(labChemResult.getUnits());
//                        labResult.setFacilityName(labChemResult.getFacility());
//
//                        if (StringUtils.isNotBlank(labChemResult.getLabId())) {
//                            Code code = new Code();
//                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                            code.setCode(labChemResult.getLabId());
//                            labResult.getCodes().add(code);
//                        }
//
//                        addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                                bhieBean.getCustodianName(), bhieBean.getFacility(), labChemResult.getEventId(), labResult);
//
//                        labResults.add(labResult);
//                    }
//
//                    vistaBeans.addAll(labResults);
//                }
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Lab Chemistries", labChemResultCollection.getStatusList(),
//                labChemResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (LabResult[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new LabResult[0]);
//    }
//
//    public LabResult[] mapMicrobiologyLabsFromBHIE(Object resultCollection) {
//        LabMicrobiologyResultCollection labMicrobiologyResultCollection = (LabMicrobiologyResultCollection) resultCollection;
//        List<LabResult> vistaBeans = new ArrayList<LabResult>();
//
//        if (labMicrobiologyResultCollection.getDataRecordList() != null) {
//            for (LabMicrobiologyDataRecord dataRecord : labMicrobiologyResultCollection.getDataRecordList()) {
//                if (dataRecord == null || dataRecord.getLabMicrobiology() == null) continue;
//
//                LabMicrobiology bhieBean = dataRecord.getLabMicrobiology();
//
//                if (bhieBean.getTests() != null) {
//                    List<LabMicrobiologyTest> bhieMicrobiologyTests = bhieBean.getTests();
//
//                    for (LabMicrobiologyTest microbiologyTest : bhieMicrobiologyTests) {
//                        if (microbiologyTest.getResults() != null) {
//                            for (LabMicrobiologyResult microbiologyResult : microbiologyTest.getResults()) {
//                                LabResult labResult = new LabResult();
//                                labResult.setTestType("MICRO");
//                                labResult.setResultDate(bhieToXMLCal(bhieBean.getCollectionDateTime()));
//                                labResult.setOrderDate(bhieToXMLCal(bhieBean.getOrderDateTime()));
//                                labResult.setSpecimen(bhieBean.getSpecimenName());
//
//                                String accession = "";
//
//                                if (bhieBean.getAccession() != null)
//                                    accession = bhieBean.getAccession();
//
//                                labResult.setAccession(accession + "^MI");
//
//                                labResult.setTestName(microbiologyResult.getName());
//                                labResult.setResult(microbiologyResult.getFullReport());
//                                labResult.setVerifiedBy(microbiologyResult.getCertifiedBy());
//                                labResult.setResultStatus(microbiologyResult.getResultStatus());
//
//                                if (StringUtils.isNotBlank(bhieBean.getTestId())) {
//                                    Code code = new Code();
//                                    code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                                    code.setCode(bhieBean.getTestId());
//                                    labResult.getCodes().add(code);
//                                }
//
//                                addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                                        bhieBean.getCustodianName(), bhieBean.getFacility(), microbiologyResult.getEventId(), labResult);
//
//                                vistaBeans.add(labResult);
//                            }
//                        }
//                    }
//                }
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Lab Microbiologies", labMicrobiologyResultCollection.getStatusList(),
//                labMicrobiologyResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (LabResult[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new LabResult[0]);
//    }

//    private List<LabResult> mapAnatomicPathologyRecord(LabAnatomicPathologyDataRecord dataRecord, String subscript) {
//
//        LabAnatomicPathology bhieBean = dataRecord.getLabAnatomicPathology();
//
//        List<LabResult> labResults = new ArrayList<LabResult>();
//
//        if (bhieBean.getResults() != null) {
//            List<LabAnatomicPathologyResult> bhieLabAnatomicPathologyResults = bhieBean.getResults();
//
//            StringBuilder accession = new StringBuilder();
//            for (LabAnatomicPathologyResult anatomicPathologyResult : bhieLabAnatomicPathologyResults) {
//                LabResult labResult = new LabResult();
//                labResult.setTestType("AP");
//                labResult.setResultDate(bhieToXMLCal(bhieBean.getCollectionDateTime()));
//                labResult.setOrderDate(bhieToXMLCal(bhieBean.getOrderDateTime()));
//                labResult.setSpecimen(bhieBean.getSpecimenName());
//
//                accession.setLength(0);
//
//                if (bhieBean.getAccession() != null) {
//                    accession.append(bhieBean.getAccession());
//                }
//                accession.append("^");
//                accession.append(subscript);
//                labResult.setAccession(accession.toString());
//
//                labResult.setTestName(anatomicPathologyResult.getLabTest());
//                labResult.setResult(anatomicPathologyResult.getFullReport());
//                labResult.setVerifiedBy(anatomicPathologyResult.getCertifiedBy());
//                labResult.setResultStatus(anatomicPathologyResult.getResultStatus());
//                labResult.setComment(anatomicPathologyResult.getResultComment());
//
//                if (StringUtils.isNotBlank(bhieBean.getTestId())) {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                    code.setCode(bhieBean.getTestId());
//                    labResult.getCodes().add(code);
//                }
//
//                addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                        bhieBean.getCustodianName(), bhieBean.getFacility(), anatomicPathologyResult.getEventId(), labResult);
//
//                labResults.add(labResult);
//            }
//        }
//
//        return labResults;
//    }
//
//    public LabResult[] mapAnatomicPathologyLabsFromBHIE(Object resultCollection) {
//
//        LabAnatomicPathologyResultCollection labAnatomicPathologyResultCollection = (LabAnatomicPathologyResultCollection) resultCollection;
//        List<LabResult> vistaBeans = new ArrayList<LabResult>();
//
//        if (labAnatomicPathologyResultCollection.getDataRecordList() != null) {
//            for (LabAnatomicPathologyDataRecord dataRecord : labAnatomicPathologyResultCollection.getDataRecordList()) {
//                if (dataRecord == null || dataRecord.getLabAnatomicPathology() == null) continue;
//                List<LabResult> labResults = mapAnatomicPathologyRecord(dataRecord, "AP");
//                vistaBeans.addAll(labResults);
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Lab Anatomic Pathologies", labAnatomicPathologyResultCollection.getStatusList(),
//                labAnatomicPathologyResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (LabResult[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new LabResult[0]);
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        LabResult lresult = new LabResult();
        lresult.setTestName(errorMsg);
        lresult.setResultDate(Utils.backDate());
        lresult.setSite(getConnectionUnavailableSite(location));
        return new LabResult[]{lresult};
    }
}