/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.MedicationFillInfoV4;
import gov.va.med.bhie.MedicationV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.Medication;
import gov.va.med.vds.webservice.PrescriptionFill;
import gov.va.med.vds.webservice.Provider;
import gov.va.med.vhahon.common.Utils;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartMedication extends SmartBean implements Comparable {
//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartMedication.class);

//    public Prescription m_prescription;

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;
    public static final int ACTIVE = 4;

    public Medication m_medication;
    
    public SmartMedication() {
        super();
        m_medication = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartMedication(Medication med, String sort) {
        super();
        m_medication = med;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartMedication) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_medication);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = array1;
        } else if (array1 == null) {
            rtc = array2;
        } else {
            int size = array1.length + array2.length;
            SmartMedication[] temp = new SmartMedication[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartMedication newAll = new SmartMedication((Medication) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartMedication newAll = new SmartMedication((Medication) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Medication[] ret = new Medication[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_medication;
            }
            rtc = ret;
        }
        return rtc;
    }

//    public static Prescription[] combinePrescription(Prescription array1[], Prescription array2[], String sort) {
//        if (array2 == null) return sort(array1, sort);
//        if (array1 == null) return sort(array2, sort);
//        int size = array1.length + array2.length;
//        SmartMedication temp[] = new SmartMedication[size];
//        int cnt = 0;
//        for (int i = 0; i < array1.length; i++) {
//            SmartMedication newAll = new SmartMedication(array1[i], sort);
//            temp[cnt] = newAll;
//            cnt++;
//        }
//        for (int i = 0; i < array2.length; i++) {
//            SmartMedication newAll = new SmartMedication(array2[i], sort);
//            temp[cnt] = newAll;
//            cnt++;
//        }
//        Arrays.sort(temp);
//        Prescription ret[] = new Prescription[temp.length];
//        for (int i = 0; i < temp.length; i++) {
//            ret[i] = temp[i].m_prescription;
//        }
//        return ret;
//    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartMedication[] temp = new SmartMedication[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartMedication newAll = new SmartMedication((Medication) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Medication[] ret = new Medication[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_medication;
            }
            rtc = ret;
        }
        return rtc;
    }

//    public static Prescription[] sort(Prescription array1[], String sort) {
//
//        if (array1 == null) return array1;
//        int size = array1.length;
//        SmartMedication temp[] = new SmartMedication[size];
//        int cnt = 0;
//        for (int i = 0; i < array1.length; i++) {
//            SmartMedication newAll = new SmartMedication(array1[i], sort);
//            temp[cnt] = newAll;
//            cnt++;
//        }
//        Arrays.sort(temp);
//        Prescription ret[] = new Prescription[temp.length];
//        for (int i = 0; i < temp.length; i++) {
//            ret[i] = temp[i].m_prescription;
//        }
//        return ret;
//    }

    public int compareTo(Object obj) {
        SmartMedication tmp = (SmartMedication) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_medication.getLastFilledDate(), tmp.m_medication.getLastFilledDate());
        }
        if (this.m_sortBy == SITE && this.m_medication.getSite() != null && tmp.m_medication.getSite() != null) {
            retVal = compareString(this.m_medication.getSite().getSiteCode(), tmp.m_medication.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_medication.getDrugName(), tmp.m_medication.getDrugName());
        }
        if (this.m_sortBy == ACTIVE) {
            retVal = compareString(this.m_medication.getActive(), tmp.m_medication.getActive());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }


        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("fillOrderDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("drugName".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
            if ("active".equalsIgnoreCase(sort)) {
                m_sortBy = ACTIVE;
            }
        }
    }

//    public Medication[] getPrescriptionsFromCHCS(Object[] dodBean) {
//        PrescriptionBean chcsBean[] = (PrescriptionBean[]) dodBean;
//        int size = chcsBean.length;
//        Medication vistaBean[] = new Medication[size];
//
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            Medication bean = new Medication();
//            bean.setDrugName(chcsBean[i].getDrugName());
//            bean.setFillOrderDate(Utils.getXMLCal(chcsBean[i].getLastFillDate()));
//            bean.setDaysSupply(Integer.toString(chcsBean[i].getDaysSupply()));
//            bean.setComment(chcsBean[i].getComments());
//            bean.setStopDate(Utils.toXMLGregorianCalendar(new java.util.Date()));
//            bean.setLastDispensingPharmacy(chcsBean[i].getDispensingPharmacy());
//            bean.setQuantity(Integer.toString(chcsBean[i].getQuantity()));
//            bean.setOrderingProvider(chcsBean[i].getOrderingProvider());
//            bean.setRefills(Integer.toString(chcsBean[i].getNumberOfFills()));
//            bean.setSigCode(chcsBean[i].getSig());
//            bean.setMedId(chcsBean[i].getRxId());
//
//            if (chcsBean[i].isActive())
//                bean.setActive("ACTIVE");
//            else
//                bean.setActive("INACTIVE");
//            bean.setMedType("O");
//            bean.setRXNumber(chcsBean[i].getRxId());
//
//            addCHCSSiteDataToVistaBean(bean);
//
//            vistaBean[i] = bean;
//
//        }
//
//        return vistaBean;
//    }
//
//    public Medication[] getInpatientMedicationsFromCHCS(Object[] dodBean) {
//        MedicationBean chcsBean[] = (MedicationBean[]) dodBean;
//        int size = chcsBean.length;
//        Medication vistaBean[] = new Medication[size];
//
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            Medication bean = new Medication();
//            bean.setDrugName(chcsBean[i].getDrugName());
//            bean.setFillOrderDate(Utils.getXMLCal(chcsBean[i].getLastFillDate()));
//            bean.setDaysSupply(Integer.toString(chcsBean[i].getDaysSupply()));
//            bean.setComment(chcsBean[i].getComments());
//            bean.setStopDate(Utils.getXMLCal(chcsBean[i].getFillExpirationDate()));
//            bean.setLastDispensingPharmacy(chcsBean[i].getDispensingPharmacy());
//            bean.setQuantity(Integer.toString(chcsBean[i].getQuantity()));
//            bean.setOrderingProvider(chcsBean[i].getOrderingProvider());
//            bean.setRefills(Integer.toString(chcsBean[i].getNumberOfFills()));
//            bean.setSigCode(chcsBean[i].getSig());
//            bean.setMedId(chcsBean[i].getRxNumber());
//
//            if (chcsBean[i].isActive())
//                bean.setActive("ACTIVE");
//            else
//                bean.setActive("INACTIVE");
//            bean.setMedType("I");
//            bean.setRXNumber(chcsBean[i].getRxId());
//
//            addCHCSSiteDataToVistaBean(bean);
//
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }
//
//    public Medication[] getPDTSMedicationsFromCHCS(Object[] dodBean) {
//        MedicationBean chcsBean[] = (MedicationBean[]) dodBean;
//        int size = chcsBean.length;
//        Medication vistaBean[] = new Medication[size];
//
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            Medication bean = new Medication();
//            bean.setDrugName(chcsBean[i].getDrugName());
//            bean.setFillOrderDate(Utils.getXMLCal(chcsBean[i].getLastFillDate()));
//            bean.setDaysSupply(Integer.toString(chcsBean[i].getDaysSupply()));
//            bean.setComment("");
//            bean.setStopDate(Utils.getXMLCal(chcsBean[i].getFillExpirationDate()));
//            bean.setLastDispensingPharmacy(chcsBean[i].getLocationName());
//            bean.setQuantity(Integer.toString(chcsBean[i].getQuantity()));
//            bean.setOrderingProvider(chcsBean[i].getLocationName());
//            bean.setRefills(Integer.toString(chcsBean[i].getRefillsRemaining()));
//            bean.setSigCode("");
//            bean.setMedId("");
//            bean.setActive("");
//            bean.setMedType("O");
//            bean.setRXNumber(chcsBean[i].getRxId());
//
//            addCHCSSiteDataToVistaBean(bean);
//
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }

//    public static Prescription getPrescriptionFillsFromCHCS(PrescriptionBean chcsBean, String siteId, String status, String rxId) {
//
//        Prescription bean = null;
//
//        if (chcsBean.getRxId().equals(rxId)) {
//            bean = new Prescription();
//            bean.setSigCode(siteId);
//            bean.setDrugName(chcsBean.getDrugName());
//            bean.setFillOrderDate(Utils.getXMLCal(chcsBean.getLastFillDate()));
//            bean.setDaysSupply(Integer.toString(chcsBean.getDaysSupply()));
//            ;
//            bean.setComment(chcsBean.getComments());
//            bean.setStopDate(Utils.getXMLCal(chcsBean.getFillExpirationDate()));
//            bean.setLastDispensingPharmacy(chcsBean.getDispensingPharmacy());
//            bean.setQuantity(Integer.toString(chcsBean.getQuantity()));
//            bean.setOrderingProvider(chcsBean.getOrderingProvider());
//            bean.setRefills(Integer.toString(chcsBean.getNumberOfFills()));
//            bean.setSigCode(chcsBean.getSig());
//            bean.setMedId(chcsBean.getRxNumber());
//            bean.setActive(status);
//            bean.setMedType("O");
//
//            addCHCSSiteDataToVistaBean(bean);
//
//            int size = chcsBean.getFillDates().size();
//            if (size > 0) {
//                ArrayList<PrescriptionFill> fills = new ArrayList<PrescriptionFill>();
//
//                PrescriptionFillDateBean chcsFills[] = chcsBean.getFillDates().toArray(new PrescriptionFillDateBean[0]);
//                for (int j = 0; j < size; j++) {
//
//                    PrescriptionFill fill = new PrescriptionFill();
//                    fill.setSiteCode(siteId);
//
//                    fill.setDispenseDate(Utils.getXMLCal(chcsFills[j].getFillDate()));
//                    fill.setDispensingPharmacy(chcsFills[j].getDispensingPharmacy());
//                    fill.setDispensingQuantity(Integer.toString(chcsFills[j].getQuantity()));
//
//                    addCHCSSiteDataToVistaBean(fill);
//
//                    fills.add(fill);
//                }
//                //                        for (int j=0;j<fills.length;j++)
//                //                        {
//                //                            bean.getPrescriptionFills().add(fills[j]);
//                //                        }
//                bean.getPrescriptionFills().addAll(fills);
//
//            }
//        }
//
//        return bean;
//    }

    /**
     * Map outpatient medications from BHIE.
     * <p/>
     * This method implements the following business rules, as outlined in the BHIE 5 ICD:
     * <p/>
     * 1. If the bhie medication comment field contains the string "^I^" then the medication type is "inpatient"
     * otherwise it is considered "outpatient".
     * 2. Any string beginning and ending with a caret ("^") should be removed from the comment text before
     * displaying to an end user (this mapping function will remove the caret from the comment field when
     * mapping)
     *
     * @param resultCollection
     * @return
     */
    public Medication[] getOutpatientMedicationsFromBHIE(Object resultCollection) {
        V4ResultCollection medicationResultCollection = (V4ResultCollection) resultCollection;
        List<Medication> vistaBeans = new ArrayList<Medication>();

        if (medicationResultCollection.getDataRecordList() != null && medicationResultCollection.getDataRecordList().getMedications() != null) {
//            StringBuilder sb = new StringBuilder();
            for (MedicationV4 bhieMed : medicationResultCollection.getDataRecordList().getMedications()) {
                if (bhieMed == null) {
                    continue;
                }
                if ("I".equals(bhieMed.getPrescription())) {
                    continue;
                }
                //only map outpatient medications, check for "^I^".
//                String comments = bhieMed.getComment();

                vistaBeans.add(mapMedicationFromBHIE(bhieMed, "O"));
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(medicationResultCollection.getStatusListv4(),
                medicationResultCollection.getErrorListv4(), medicationResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Medication[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Medication[vistaBeans.size()]);
    }

    /**
     * Map inpatient medications from BHIE.
     * <p/>
     * This method implements the following business rules, as outlined in the BHIE 5 ICD:
     * <p/>
     * 1. If the bhie medication comment field contains the string "^I^" then the medication type is "inpatient"
     * otherwise it is considered "outpatient".
     * 2. Any string beginning and ending with a caret ("^") should be removed from the comment text before
     * displaying to an end user (this mapping function will remove the caret from the comment field when
     * mapping)
     *
     * @param resultCollection
     * @return
     */
    public Medication[] getInpatientMedicationsFromBHIE(Object resultCollection) {
        V4ResultCollection medicationResultCollection = (V4ResultCollection) resultCollection;
        List<Medication> vistaBeans = new ArrayList<Medication>();

        if (medicationResultCollection.getDataRecordList() != null && medicationResultCollection.getDataRecordList().getMedications() != null) {
//            StringBuilder sb = new StringBuilder();
            for (MedicationV4 bhieMed : medicationResultCollection.getDataRecordList().getMedications()) {
                if (bhieMed == null) {
                    continue;
                }
                if (!"I".equalsIgnoreCase(bhieMed.getPrescription())) {
                    continue;
                }
//                String comments = bhieMed.getComment();

                vistaBeans.add(mapMedicationFromBHIE(bhieMed, "I"));
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(medicationResultCollection.getStatusListv4(),
                medicationResultCollection.getErrorListv4(), medicationResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Medication[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Medication[vistaBeans.size()]);
    }

    // Version 4
    private Medication mapMedicationFromBHIE(gov.va.med.bhie.MedicationV4 bhieMed, String medType) {

        Medication bean = new Medication();
        if (bhieMed.getComment() != null) {
            bean.setComment(bhieMed.getComment());
        }
        if (bhieMed.getDrug() != null && !bhieMed.getDrug().isEmpty() && bhieMed.getDrug().get(0) != null) {
            bean.setDrugName(bhieMed.getDrug().get(0).getDisplay());
        }

        if (bhieMed.getOrderDate() != null) {
            bean.setOrderDate(bhieToXMLCalV4(bhieMed.getOrderDate()));
        }
        if (bhieMed.getLastFillDate() != null) {
            bean.setLastFilledDate(bhieToXMLCalV4(bhieMed.getLastFillDate()));
        }

//        if ("I".equalsIgnoreCase(medType) && bhieMed.getLastFillDate() == null) {
//            bean.setLastFilledDate(bhieToXMLCalV4(bhieMed.getOrderDate()));
//        }  // Commented out DTC-226 Do not put order date in last filled date for Inpt Meds

        if (bhieMed.getStartDate() != null) {
            bean.setStartDate(bhieToXMLCalV4(bhieMed.getStartDate()));
        }
        if (bhieMed.getEndDate() != null) {
            bean.setStopDate(bhieToXMLCalV4(bhieMed.getEndDate()));
        }
        if (bhieMed.getOrderNumber() != null) {
            bean.setOrderId(bhieMed.getOrderNumber().getId());
        }

        Provider orderingProvider = new Provider();

        if (bhieMed.getProvider() != null) {
            orderingProvider = mapProvider(bhieMed.getProvider());
//            orderingProvider.setName(bhieMed.getProvider().getName());
//            if (bhieMed.getProvider().getTaxonomy() != null && !bhieMed.getProvider().getTaxonomy().isEmpty()) {
//            orderingProvider.setTaxonomyCode(bhieMed.getProvider().getTaxonomy().get(0).getCode());
//            }
        }
//        List<gov.va.med.bhie.Provider> providerList.add(bhieMed.getProvider());
//        if(!providerList.isEmpty()){
//            orderingProvider.setName(providerList.get(0).getName());
//
//        }
//
//        List<SpdcBean> taxonomyCodeList = bhieMed.getTaxonomyCode();
//        if(!taxonomyCodeList.isEmpty()){
//            orderingProvider.setTaxonomyCode(bhieMed.getTaxonomyCode().get(0).getCode());
//            //orderingProvider.set
//        }


        //orderingProvider.setName(bhieMed.getOrderingProvider());
        bean.setOrderingProvider(orderingProvider);
        bean.setCurrentProvider(orderingProvider);
        bean.setDaysSupply(bhieMed.getDaysSupply());
        if (bhieMed.getDispensingPharmacy() != null) {
            bean.setLastDispensingPharmacy(bhieMed.getDispensingPharmacy().getName());
        }
        bean.setQuantity(bhieMed.getQuantity());
        bean.setRefills(bhieMed.getNumberOfRefills());
        bean.setSigCode(bhieMed.getSig());
        if (bhieMed.getRecordId() != null) {
            bean.setMedId(bhieMed.getRecordId().getId());
        }
        if (bhieMed.getStatus() != null
                && !bhieMed.getStatus().isEmpty()
                && bhieMed.getStatus().get(0) != null) {
            bean.setStatus(bhieMed.getStatus().get(0).getDisplay());
        }
        bean.setMedType(medType);
        if (bhieMed.getRxNumber() != null) {
            bean.setRxNumber(bhieMed.getRxNumber().getId());
        }

//        if(StringUtils.isNotBlank(bhieMed.getOtherIdentifier()))
//        {
//            Code code = new Code();
//            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//            code.setCode(bhieMed.getOtherIdentifier());
//            bean.getCodes().add(code);
//        }

        if (bhieMed.getDrug() != null && !bhieMed.getDrug().isEmpty()) {
            for (SpdcBeanV4 drug : bhieMed.getDrug()) {
                if (drug != null) {
                    Code code = new Code();
                    if (CodingSystem.DOD_IEN.getOID().equalsIgnoreCase(drug.getSystem())) {
                        code.setSystem(CodingSystem.DOD_NCID.getDisplayName());

                    } else if (CodingSystem.NDC.getOID().equalsIgnoreCase(drug.getSystem())) {
                        code.setSystem(CodingSystem.NDC.getDisplayName());
                    }
                    code.setCode(drug.getCode());
                    bean.getCodes().add(code);
                }
            }
        }
        if (bhieMed.getUnits() != null && getPrimarySpdcV4(bhieMed.getUnits()) != null) {
            bean.setUnits(getPrimarySpdcV4(bhieMed.getUnits()).getDisplay());
        }
        bean.setChildResistant(bhieMed.isChildResistant());
        bean.setFillExpirationDate(bhieToXMLCalV4(bhieMed.getEndDate()));
        bean.setRefillsRemaining(bhieMed.getRefillsRemaining());


//        addBHIESiteDataToVistaBean(bhieMed.getCustodianId(),
//                bhieMed.getCustodianName(), bhieMed.getFacility(), bhieMed.getEventId(), bean);
//        addBHIESiteDataToVistaBean(bhieMed.getRepository().getId().get(0).getId(),
//                bhieMed.getRepository().getName(), bhieMed.getFacility().getName(), bhieMed.getRecordId().getId(), bean);
        String repositoryId = "";
        String repositoryName = "";
        String facilityName = "";
        String eventId = "";
        if (bhieMed.getRepository() != null && bhieMed.getRepository().getIds() != null && !bhieMed.getRepository().getIds().isEmpty()) {
            repositoryId = bhieMed.getRepository().getIds().get(0).getId();
        }
        if (bhieMed.getRepository() != null) {
            repositoryName = bhieMed.getRepository().getName();
        }
        if (bhieMed.getFacility() != null) {
            facilityName = bhieMed.getFacility().getName();
        }
        if (bhieMed.getRecordId() != null) {
            eventId = bhieMed.getRecordId().getId();
        }
        if (repositoryId == null || repositoryId.length() == 0) {
            repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
        }
        if (repositoryName == null || repositoryName.length() == 0) {
            repositoryName = "ahlta";
        }
        addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                facilityName, eventId, bean);


        //add fill dates
        List<MedicationFillInfoV4> medFillDates = bhieMed.getFillInfo();
        for (MedicationFillInfoV4 medFillDate : medFillDates) {
            PrescriptionFill prescriptionFill = new PrescriptionFill();
            prescriptionFill.setDispenseDate(bhieToXMLCalV4(medFillDate.getFillDate()));
            if (medFillDate.getDispensingPharmacy() != null) {
                prescriptionFill.setDispensingPharmacy(medFillDate.getDispensingPharmacy().getName());
            }
            prescriptionFill.setDispensingQuantity(medFillDate.getQuantity());

//            addBHIESiteDataToVistaBean(bhieMed.getCustodianId(),
//                    bhieMed.getCustodianName(), bhieMed.getFacility(), bhieMed.getEventId(), prescriptionFill);
//            addBHIESiteDataToVistaBean(bhieMed.getRepository().getId().get(0).getId(),
//                    bhieMed.getRepository().getName(), bhieMed.getFacility().getName(), bhieMed.getRecordId().getId(), prescriptionFill);
            repositoryId = "";
            repositoryName = "";
            facilityName = "";
            eventId = "";
            if (bhieMed.getRepository() != null && bhieMed.getRepository().getIds() != null && !bhieMed.getRepository().getIds().isEmpty()) {
                repositoryId = bhieMed.getRepository().getIds().get(0).getId();
            }
            if (bhieMed.getRepository() != null) {
                repositoryName = bhieMed.getRepository().getName();
            }
            if (bhieMed.getFacility() != null) {
                facilityName = bhieMed.getFacility().getName();
            }
            if (bhieMed.getRecordId() != null) {
                eventId = bhieMed.getRecordId().getId();
            }
            addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                    facilityName, eventId, prescriptionFill);

            bean.getPrescriptionFills().add(prescriptionFill);
        }

        return bean;
    }

    // Version 2
//    private Medication mapMedicationFromBHIE(gov.va.med.bhie.Medication bhieMed, String medType, String medComment)
//    {
//        Medication bean = new Medication();
//        bean.setComment(medComment);
//        bean.setDrugName(bhieMed.getMedicationName());
//
//        if ("I".equalsIgnoreCase(medType) && StringUtils.isBlank(bhieMed.getLastFilledDate()))
//        {
//            bean.setFillOrderDate(bhieToXMLCal(bhieMed.getIssueDate()));
//        }
//        else bean.setFillOrderDate(bhieToXMLCal(bhieMed.getLastFilledDate()));
//
//        Provider orderingProvider = new Provider();
//        Provider currentProvider = new Provider();
//
//        orderingProvider.setName(bhieMed.getOrderingProvider());
////        List<gov.va.med.bhie.Provider> providerList = bhieMed.getOrderingProvider();
////        if(!providerList.isEmpty()){
////            orderingProvider.setName(providerList.get(0).getName());
////
////        }
//
//        orderingProvider.setTaxonomyCode("");
////        List<SpdcBean> taxonomyCodeList = bhieMed.getTaxonomyCode();
////        if(!taxonomyCodeList.isEmpty()){
////            orderingProvider.setTaxonomyCode(bhieMed.getTaxonomyCode().get(0).getCode());
////            //orderingProvider.set
////        }
//
//
//        //orderingProvider.setName(bhieMed.getOrderingProvider());
//        bean.setOrderingProvider(orderingProvider);
//        bean.setCurrentProvider(currentProvider);
//        bean.setDaysSupply(bhieMed.getSupplyDays());
//        bean.setStopDate(bhieToXMLCal(bhieMed.getFillExpirationDate()));
//        bean.setLastDispensingPharmacy(bhieMed.getDispensingPharmacy());
//        bean.setQuantity(bhieMed.getQuantity());
//        bean.setRefills(bhieMed.getRefillsRemaining());
//        bean.setSigCode(bhieMed.getSig());
//        bean.setMedId(bhieMed.getRxId());
//        bean.setActive(bhieMed.getStatus());
//        bean.setMedType(medType);
//        bean.setRXNumber(bhieMed.getRxNumber());
//
//        if(StringUtils.isNotBlank(bhieMed.getOtherIdentifier()))
//        {
//            Code code = new Code();
//            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//            code.setCode(bhieMed.getOtherIdentifier());
//            bean.getCodes().add(code);
//        }
//
//        addBHIESiteDataToVistaBean(bhieMed.getCustodianId(),
//                bhieMed.getCustodianName(), bhieMed.getFacility(), bhieMed.getEventId(), bean);
//
//
//        //add fill dates
//        List<MedicationFillDate> medFillDates = bhieMed.getMedicationFillDates();
//        for(MedicationFillDate medFillDate : medFillDates)
//        {
//            PrescriptionFill prescriptionFill = new PrescriptionFill();
//            prescriptionFill.setDispenseDate(bhieToXMLCal(medFillDate.getFillDate()));
//            prescriptionFill.setDispensingPharmacy(medFillDate.getDispensingPharmacy());
//            prescriptionFill.setDispensingQuantity(medFillDate.getQuantity());
//
//            addBHIESiteDataToVistaBean(bhieMed.getCustodianId(),
//                    bhieMed.getCustodianName(), bhieMed.getFacility(), bhieMed.getEventId(), prescriptionFill);
//
//            bean.getPrescriptionFills().add(prescriptionFill);
//        }
//
//        return bean;
//    }


    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Medication med = new Medication();

        med.setDrugName(errorMsg);
        med.setLastFilledDate(Utils.backDate());
        med.setOrderDate(Utils.backDate());
        med.setSite(getConnectionUnavailableSite(location));

        return new Medication[]{med};
    }
}