/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.ConsultOrderV4;
import gov.va.med.bhie.LabOrderV4;
import gov.va.med.bhie.MedicationOrderV4;
import gov.va.med.bhie.RadiologyOrderV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsConstants;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.Order;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartOrder extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartOrder.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int TYPE = 3;
    public static final int STATUS = 4;

    public Order m_order;

    public SmartOrder() {
        super();
        m_order = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartOrder(Order order, String sort) {
        super();
        m_order = order;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartOrder) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_order);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartOrder[] temp = new SmartOrder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartOrder newAll = new SmartOrder((Order) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartOrder newAll = new SmartOrder((Order) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Order[] ret = new Order[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_order;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartOrder[] temp = new SmartOrder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartOrder newAll = new SmartOrder((Order) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Order[] ret = new Order[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_order;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        int retVal = 0;
        if (obj != null) {
            SmartOrder tmp = (SmartOrder) obj;
            if (this.m_sortBy == DATE) {
                retVal = compareDate(this.m_order.getOrderDate(), tmp.m_order.getOrderDate());
            }
            if (this.m_sortBy == SITE && this.m_order.getSite() != null && tmp.m_order.getSite() != null) {
                retVal = compareString(this.m_order.getSite().getSiteCode(), tmp.m_order.getSite().getSiteCode());
            }
            if (this.m_sortBy == TYPE) {
                retVal = compareString(this.m_order.getType(), tmp.m_order.getType());
            }
            if (this.m_sortBy == STATUS) {
                retVal = compareString(this.m_order.getStatus(), tmp.m_order.getStatus());
            }

            if (!this.m_bAscending) {
                retVal = -retVal;
            }
        }
        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("StartDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("type".equalsIgnoreCase(sort)) {
                m_sortBy = TYPE;
            }
            if ("status".equalsIgnoreCase(sort)) {
                m_sortBy = STATUS;
            }
        }


    }

//    public Order[] mapOrdersFromCHCS(Object dodBean[]) {
//
//        int size = dodBean.length;
//        Order vistaBean[] = new Order[size];
//
//
//        for (int i = 0; i < dodBean.length; i++) {
//            Order bean = new Order();
//            bean.setDescription(((OrderBean) dodBean[i]).getDescription());
//            bean.setOrderid(((OrderBean) dodBean[i]).getOrderid());
//            bean.setOrderingProvider(((OrderBean) dodBean[i]).getOrderingProvider());
//            bean.setStartDate(Utils.getXMLCal(((OrderBean) dodBean[i]).getStartDate()));
//            bean.setStatus(((OrderBean) dodBean[i]).getStatus());
//            bean.setType(((OrderBean) dodBean[i]).getType());
//            addCHCSSiteDataToVistaBean(bean);
//
//            vistaBean[i] = bean;
//
//        }
//
//        return vistaBean;
//    }

//    private static String CONSULT = "11487-6";
//    private static String LAB = "26436-6";
//    private static String MED = "29305-0";
//    private static String RAD = "18726-0";

    // Version 4
    public Order[] mapOrdersFromBHIE(Object resultCollection) {
        V4ResultCollection ordersResultCollection = (V4ResultCollection) resultCollection;
        List<Order> vistaBeans = new ArrayList<Order>();

        if (ordersResultCollection.getDataRecordList() != null) {
            Order[] labOrders = mapLabOrdersFromBHIE(ordersResultCollection);
            Order[] consultOrders = mapConsultOrdersFromBHIE(ordersResultCollection);
            Order[] radiologyOrders = mapRadiologyOrdersFromBHIE(ordersResultCollection);
            Order[] medicationOrders = mapMedicationOrdersFromBHIE(ordersResultCollection);

            if (labOrders != null) {
                vistaBeans.addAll(Arrays.asList(labOrders));
            }
            if (consultOrders != null) {
                vistaBeans.addAll(Arrays.asList(consultOrders));
            }
            if (radiologyOrders != null) {
                vistaBeans.addAll(Arrays.asList(radiologyOrders));
            }
            if (medicationOrders != null) {
                vistaBeans.addAll(Arrays.asList(medicationOrders));
            }

            //include site status report
            String siteStatusReport = generateBhieStatusReportV4("Patient Orders", ordersResultCollection.getStatusListv4(),
                    ordersResultCollection.getErrorListv4());

            vistaBeans.addAll(Arrays.asList(
                    (Order[]) generateConnectionUnavailableBean(
                            BHIE_STATUS_REPORT_LOCATION,
                            siteStatusReport)
            ));
        }

        return vistaBeans.toArray(new Order[vistaBeans.size()]);
    }

    public Order[] mapConsultOrdersFromBHIE(Object resultCollection) {

        V4ResultCollection consultOrderResultCollection = (V4ResultCollection) resultCollection;
        List<Order> vistaBeans = new ArrayList<Order>();

        if (consultOrderResultCollection.getDataRecordList() != null && consultOrderResultCollection.getDataRecordList().getConsultOrders() != null) {
            for (ConsultOrderV4 bhieBean : consultOrderResultCollection.getDataRecordList().getConsultOrders()) {
                if (bhieBean == null) {
                    continue;
                }

                Order bean = new Order();

                bean.setOrderDate(bhieToXMLCalV4(bhieBean.getDate()));
                bean.setType(JMeadowsConstants.CONSULT);
                bean.setDescription(bhieBean.getDescription());
//                bean.setOrderid(bhieBean.getOrderId());
                if (bhieBean.getStatus() != null && !bhieBean.getStatus().isEmpty()
                        && bhieBean.getStatus().get(0) != null) {
                    bean.setStatus(bhieBean.getStatus().get(0).getDisplay());
                }
                if (bhieBean.getReferringProvider() != null) {
                    bean.setOrderingProvider(mapProvider(bhieBean.getReferringProvider()));
                }

                bean.setOrderDetail(bhieBean.getReason());

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieBean.getRepository() != null && bhieBean.getRepository().getIds() != null && !bhieBean.getRepository().getIds().isEmpty()) {
                    repositoryId = bhieBean.getRepository().getIds().get(0).getId();
                }
                if (bhieBean.getRepository() != null) {
                    repositoryName = bhieBean.getRepository().getName();
                }
                if (bhieBean.getFacility() != null) {
                    facilityName = bhieBean.getFacility().getName();
                }
                if (bhieBean.getRecordId() != null) {
                    eventId = bhieBean.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);
//use recordtype
                //removed because we do not do terminology mapping for Consults
//                if (bhieBean.getDataDomain() != null && !bhieBean.getDataDomain().isEmpty() && StringUtils.isNotBlank(bhieBean.getDataDomain().get(0).getCode())) {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.LOINC.getDisplayName());
//                    code.setCode(bhieBean.getDataDomain().get(0).getCode());
//                    bean.getCodes().add(code);
//                }

                vistaBeans.add(bean);

            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Consult Orders", consultOrderResultCollection.getStatusListv4(),
                consultOrderResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (Order[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Order[vistaBeans.size()]);
    }

    public Order[] mapLabOrdersFromBHIE(Object resultCollection) {
        V4ResultCollection labOrderResultCollection = (V4ResultCollection) resultCollection;
        List<Order> vistaBeans = new ArrayList<Order>();

        if (labOrderResultCollection.getDataRecordList() != null && labOrderResultCollection.getDataRecordList().getLabOrders() != null) {
            for (LabOrderV4 bhieBean : labOrderResultCollection.getDataRecordList().getLabOrders()) {
                if (bhieBean == null) {
                    continue;
                }

//                List<LabOrder> orders = bhieBean.getLabOrderList();
                Order bean = new Order();

                bean.setOrderDate(bhieToXMLCalV4(bhieBean.getDate()));
                bean.setType(JMeadowsConstants.LAB);
                bean.setPriority(bhieBean.getPriority());
                //              bean.setOrderid(bhieBean.getOrderId());
                if (bhieBean.getStatus() != null && !bhieBean.getStatus().isEmpty()
                        && bhieBean.getStatus().get(0) != null) {
                    bean.setStatus(bhieBean.getStatus().get(0).getDisplay());
                }
                if (bhieBean.getOrderingProvider() != null) {
                    bean.setOrderingProvider(mapProvider(bhieBean.getOrderingProvider()));
                }
                if (bhieBean.getOrderable() != null && getPrimarySpdcV4(bhieBean.getOrderable()) != null) {
                    bean.setDescription(getPrimarySpdcV4(bhieBean.getOrderable()).getDisplay());
                    bean.setOrderDetail(getPrimarySpdcV4(bhieBean.getOrderable()).getDisplay());

                    for (SpdcBeanV4 orderable : bhieBean.getOrderable()) {
                        if (orderable != null && orderable.getSystem().equalsIgnoreCase(CodingSystem.DOD_NCID.getOID())) {
                            Code code = new Code();
                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                            code.setCode(orderable.getCode());
                            bean.getCodes().add(code);
                        }
                    }
                }

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieBean.getRepository() != null && bhieBean.getRepository().getIds() != null && !bhieBean.getRepository().getIds().isEmpty()) {
                    repositoryId = bhieBean.getRepository().getIds().get(0).getId();
                }
                if (bhieBean.getRepository() != null) {
                    repositoryName = bhieBean.getRepository().getName();
                }
                if (bhieBean.getFacility() != null) {
                    facilityName = bhieBean.getFacility().getName();
                }
                if (bhieBean.getRecordId() != null) {
                    eventId = bhieBean.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);

                vistaBeans.add(bean);

            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Lab Orders", labOrderResultCollection.getStatusListv4(),
                labOrderResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (Order[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Order[vistaBeans.size()]);
    }

    public Order[] mapMedicationOrdersFromBHIE(Object resultCollection) {

        V4ResultCollection medicationOrderResultCollection = (V4ResultCollection) resultCollection;
        List<Order> vistaBeans = new ArrayList<Order>();

        if (medicationOrderResultCollection.getDataRecordList() != null && medicationOrderResultCollection.getDataRecordList().getMedicationOrders() != null) {
            for (MedicationOrderV4 bhieBean : medicationOrderResultCollection.getDataRecordList().getMedicationOrders()) {
                if (bhieBean == null) {
                    continue;
                }

//                List<MedicationOrder> orders = bhieBean.getMedicationOrderList();
                Order bean = new Order();

                bean.setOrderDate(bhieToXMLCalV4(bhieBean.getDate()));
                bean.setType(JMeadowsConstants.MEDICATION);
                bean.setPriority(bhieBean.getPriority());
//                bean.setOrderid(bhieBean.getOrderId());
                if (bhieBean.getStatus() != null && !bhieBean.getStatus().isEmpty()
                        && bhieBean.getStatus().get(0) != null) {
                    bean.setStatus(bhieBean.getStatus().get(0).getDisplay());
                }
                if (bhieBean.getOrderingProvider() != null) {
                    bean.setOrderingProvider(mapProvider(bhieBean.getOrderingProvider()));
                }
                if (getPrimarySpdcV4(bhieBean.getOrderable()) != null) {
                    bean.setOrderDetail(getPrimarySpdcV4(bhieBean.getOrderable()).getDisplay());
                    bean.setDescription(getPrimarySpdcV4(bhieBean.getOrderable()).getDisplay());
                    for (SpdcBeanV4 orderable : bhieBean.getOrderable()) {
                        if (orderable != null && orderable.getSystem().equalsIgnoreCase(CodingSystem.DOD_NCID.getOID())) {
                            Code code = new Code();
                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                            code.setCode(orderable.getCode());
                            bean.getCodes().add(code);
                        }
                    }
                }

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieBean.getRepository() != null && bhieBean.getRepository().getIds() != null && !bhieBean.getRepository().getIds().isEmpty()) {
                    repositoryId = bhieBean.getRepository().getIds().get(0).getId();
                }
                if (bhieBean.getRepository() != null) {
                    repositoryName = bhieBean.getRepository().getName();
                }
                if (bhieBean.getFacility() != null) {
                    facilityName = bhieBean.getFacility().getName();
                }
                if (bhieBean.getRecordId() != null) {
                    eventId = bhieBean.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);

                vistaBeans.add(bean);


            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Medication Orders", medicationOrderResultCollection.getStatusListv4(),
                medicationOrderResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (Order[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Order[vistaBeans.size()]);
    }

    public Order[] mapRadiologyOrdersFromBHIE(Object resultCollection) {

        V4ResultCollection radiologyOrderResultCollection = (V4ResultCollection) resultCollection;
        List<Order> vistaBeans = new ArrayList<Order>();

        if (radiologyOrderResultCollection.getDataRecordList() != null && radiologyOrderResultCollection.getDataRecordList().getRadiologyOrders() != null) {
            for (RadiologyOrderV4 bhieBean : radiologyOrderResultCollection.getDataRecordList().getRadiologyOrders()) {
                if (bhieBean == null) {
                    continue;
                }


//                List<RadiologyOrder> orders = bhieBean.getRadiologyOrderList();
                Order bean = new Order();

                bean.setOrderDate(bhieToXMLCalV4(bhieBean.getDate()));
                bean.setType(JMeadowsConstants.RADIOLOGY);
                bean.setPriority(bhieBean.getPriority());
//                bean.setOrderid(bhieBean.getOrderId());
                if (bhieBean.getStatus() != null && !bhieBean.getStatus().isEmpty()
                        && bhieBean.getStatus().get(0) != null) {
                    bean.setStatus(bhieBean.getStatus().get(0).getDisplay());
                }
                if (bhieBean.getOrderingProvider() != null) {
                    bean.setOrderingProvider(mapProvider(bhieBean.getOrderingProvider()));
                }

                if (bhieBean.getOrderable() != null && getPrimarySpdcV4(bhieBean.getOrderable()) != null) {
                    bean.setDescription(getPrimarySpdcV4(bhieBean.getOrderable()).getDisplay());
                    bean.setOrderDetail(getPrimarySpdcV4(bhieBean.getOrderable()).getDisplay());

                    for (SpdcBeanV4 orderable : bhieBean.getOrderable()) {
                        if (orderable.getSystem().equalsIgnoreCase(CodingSystem.DOD_NCID.getOID())) {
                            Code code = new Code();
                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                            code.setCode(orderable.getCode());
                            bean.getCodes().add(code);
                        }
                    }
                }

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieBean.getRepository() != null && bhieBean.getRepository().getIds() != null && !bhieBean.getRepository().getIds().isEmpty()) {
                    repositoryId = bhieBean.getRepository().getIds().get(0).getId();
                }
                if (bhieBean.getRepository() != null) {
                    repositoryName = bhieBean.getRepository().getName();
                }
                if (bhieBean.getFacility() != null) {
                    facilityName = bhieBean.getFacility().getName();
                }
                if (bhieBean.getRecordId() != null) {
                    eventId = bhieBean.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);

                vistaBeans.add(bean);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Radiology Orders", radiologyOrderResultCollection.getStatusListv4(),
                radiologyOrderResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (Order[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Order[vistaBeans.size()]);
    }

    // Version 2
//    public Order[] mapOrdersFromBHIE (Object resultCollection)
//    {
//        OrdersResultCollection ordersResultCollection = (OrdersResultCollection) resultCollection;
//        List<Order> vistaBeans = new ArrayList<Order>();
//
//        if (ordersResultCollection.getDataRecordList() != null)
//        {
//            for(OrderDataRecord dataRecord : ordersResultCollection.getDataRecordList())
//            {
//                Order bean = new Order();
//
//                if (dataRecord == null || dataRecord.getOrderRecord() == null) continue;
//
//                OrderRecord record = dataRecord.getOrderRecord();
//
//                if(record instanceof ConsultOrderRecord)
//                {
//                    ConsultOrderRecord bhieBean = (ConsultOrderRecord) record;
//                    bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                    bean.setType(JMeadowsConstants.CONSULT);
//                    bean.setDescription(bhieBean.getDescription());
//                    bean.setOrderid(bhieBean.getOrderId());
//                    bean.setStatus(bhieBean.getStatus());
//                    bean.setOrderingProvider(bhieBean.getOrderingProvider());
//                    if(bhieBean.getConsultOrderList() != null)
//                    {
//                        bean.setOrderDetail(bhieBean.getConsultOrderList().get(0).getReasonForRequest());
//                    }
//
//                    addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                            bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//                    if(StringUtils.isNotBlank(bhieBean.getLoincCode()))
//                    {
//                        Code code = new Code();
//                        code.setSystem(CodingSystem.LOINC.getDisplayName());
//                        code.setCode(bhieBean.getLoincCode());
//                        bean.getCodes().add(code);
//                    }
//                }
//                else if(record instanceof LabOrderRecord)
//                {
//                    LabOrderRecord bhieBean = (LabOrderRecord) record;
//                    List<LabOrder> orders = bhieBean.getLabOrderList();
//
//                    bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                    bean.setType(JMeadowsConstants.LAB);
//                    bean.setOrderid(bhieBean.getOrderId());
//                    bean.setStatus(bhieBean.getStatus());
//                    bean.setOrderingProvider(bhieBean.getOrderingProvider());
//
//                    addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                            bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//                    if(!orders.isEmpty())
//                    {
//                        bean.setDescription(orders.get(0).getUniversalServiceText());
//                        bean.setOrderDetail(orders.get(0).getUniversalServiceText());
//
//                        if(StringUtils.isNotBlank(orders.get(0).getNcid()))
//                        {
//                            Code code = new Code();
//                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                            code.setCode(orders.get(0).getNcid());
//                            bean.getCodes().add(code);
//                        }
//                    }
//                }
//                else if(record instanceof MedicationOrderRecord)
//                {
//                    MedicationOrderRecord  bhieBean = (MedicationOrderRecord) record;
//                    List<MedicationOrder> orders = bhieBean.getMedicationOrderList();
//
//                    bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                    bean.setType(JMeadowsConstants.MEDICATION);
//                    bean.setOrderid(bhieBean.getOrderId());
//                    bean.setStatus(bhieBean.getStatus());
//                    bean.setOrderingProvider(bhieBean.getOrderingProvider());
//
//                    addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                            bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//                    if(!orders.isEmpty())
//                    {
//                        bean.setOrderDetail(orders.get(0).getDrugText());
//                        bean.setDescription(orders.get(0).getDrugText());
//
//                        if(StringUtils.isNotBlank(orders.get(0).getNcid()))
//                        {
//                            Code code = new Code();
//                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                            code.setCode(orders.get(0).getNcid());
//                            bean.getCodes().add(code);
//                        }
//                    }
//                }
//                else if(record instanceof RadiologyOrderRecord)
//                {
//                    RadiologyOrderRecord  bhieBean = (RadiologyOrderRecord) record;
//                    List<RadiologyOrder> orders = bhieBean.getRadiologyOrderList();
//
//                    bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                    bean.setType(JMeadowsConstants.RADIOLOGY);
//                    bean.setOrderid(bhieBean.getOrderId());
//                    bean.setStatus(bhieBean.getStatus());
//                    bean.setOrderingProvider(bhieBean.getOrderingProvider());
//
//                    addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                            bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//                    if(!orders.isEmpty())
//                    {
//                        bean.setDescription(orders.get(0).getUniversalServiceText());
//                        bean.setOrderDetail(orders.get(0).getUniversalServiceText());
//
//                        if(StringUtils.isNotBlank(orders.get(0).getNcid()))
//                        {
//                            Code code = new Code();
//                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                            code.setCode(orders.get(0).getNcid());
//                            bean.getCodes().add(code);
//                        }
//                    }
//                }
//
//                vistaBeans.add(bean);
//
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Patient Orders", ordersResultCollection.getStatusList(),
//                ordersResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (Order[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Order[0]);
//    }
//
//    public Order[] mapConsultOrdersFromBHIE(Object resultCollection)
//    {
//        ConsultOrderResultCollection consultOrderResultCollection = (ConsultOrderResultCollection) resultCollection;
//        List<Order> vistaBeans = new ArrayList<Order>();
//
//        if (consultOrderResultCollection.getDataRecordList() != null)
//        {
//            for(ConsultOrderDataRecord dataRecord : consultOrderResultCollection.getDataRecordList())
//            {
//                if (dataRecord == null || dataRecord.getOrderRecord() == null) continue;
//
//                ConsultOrderRecord bhieBean = dataRecord.getOrderRecord();
//                Order bean = new Order();
//
//                bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                bean.setType(JMeadowsConstants.CONSULT);
//                bean.setDescription(bhieBean.getDescription());
//                bean.setOrderid(bhieBean.getOrderId());
//                bean.setStatus(bhieBean.getStatus());
//                bean.setOrderingProvider(bhieBean.getOrderingProvider());
//                if(bhieBean.getConsultOrderList() != null)
//                {
//                    bean.setOrderDetail(bhieBean.getConsultOrderList().get(0).getReasonForRequest());
//                }
//
//                addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                        bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//                if(StringUtils.isNotBlank(bhieBean.getLoincCode()))
//                {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.LOINC.getDisplayName());
//                    code.setCode(bhieBean.getLoincCode());
//                    bean.getCodes().add(code);
//                }
//
//                vistaBeans.add(bean);
//
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Consult Orders", consultOrderResultCollection.getStatusList(),
//                consultOrderResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (Order[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Order[0]);
//    }
//
//    public Order[] mapLabOrdersFromBHIE(Object resultCollection)
//    {
//        LabOrderResultCollection labOrderResultCollection = (LabOrderResultCollection) resultCollection;
//        List<Order> vistaBeans = new ArrayList<Order>();
//
//        if (labOrderResultCollection.getDataRecordList() != null)
//        {
//            for(LabOrderDataRecord dataRecord : labOrderResultCollection.getDataRecordList())
//            {
//                if (dataRecord == null || dataRecord.getOrderRecord() == null) continue;
//
//                LabOrderRecord bhieBean = dataRecord.getOrderRecord();
//                List<LabOrder> orders = bhieBean.getLabOrderList();
//                Order bean = new Order();
//
//                bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                bean.setType(JMeadowsConstants.LAB);
//                bean.setOrderid(bhieBean.getOrderId());
//                bean.setStatus(bhieBean.getStatus());
//                bean.setOrderingProvider(bhieBean.getOrderingProvider());
//
//                if(!orders.isEmpty())
//                {
//                    bean.setDescription(orders.get(0).getUniversalServiceText());
//                    bean.setOrderDetail(orders.get(0).getUniversalServiceText());
//
//                    if(StringUtils.isNotBlank(orders.get(0).getNcid()))
//                    {
//                        Code code = new Code();
//                        code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                        code.setCode(orders.get(0).getNcid());
//                        bean.getCodes().add(code);
//                    }
//                }
//
//                addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                        bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//                vistaBeans.add(bean);
//
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Lab Orders", labOrderResultCollection.getStatusList(),
//                labOrderResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (Order[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Order[0]);
//    }
//
//    public Order[] mapMedicationOrdersFromBHIE(Object resultCollection)
//    {
//        MedicationOrderResultCollection medicationOrderResultCollection = (MedicationOrderResultCollection) resultCollection;
//        List<Order> vistaBeans = new ArrayList<Order>();
//
//        if (medicationOrderResultCollection.getDataRecordList() != null)
//        {
//            for(MedicationOrderDataRecord dataRecord : medicationOrderResultCollection.getDataRecordList())
//            {
//                if (dataRecord == null || dataRecord.getOrderRecord() == null) continue;
//
//                MedicationOrderRecord bhieBean = dataRecord.getOrderRecord();
//                List<MedicationOrder> orders = bhieBean.getMedicationOrderList();
//                Order bean = new Order();
//
//                bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                bean.setType(JMeadowsConstants.MEDICATION);
//                bean.setOrderid(bhieBean.getOrderId());
//                bean.setStatus(bhieBean.getStatus());
//                bean.setOrderingProvider(bhieBean.getOrderingProvider());
//
//                if(!orders.isEmpty())
//                {
//                    bean.setOrderDetail(orders.get(0).getDrugText());
//                    bean.setDescription(orders.get(0).getDrugText());
//
//                    if(StringUtils.isNotBlank(orders.get(0).getNcid()))
//                    {
//                        Code code = new Code();
//                        code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                        code.setCode(orders.get(0).getNcid());
//                        bean.getCodes().add(code);
//                    }
//                }
//
//
//                addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                        bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//
//                vistaBeans.add(bean);
//
//
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Medication Orders", medicationOrderResultCollection.getStatusList(),
//                medicationOrderResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (Order[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Order[0]);
//    }
//
//    public Order[] mapRadiologyOrdersFromBHIE(Object resultCollection)
//    {
//        RadiologyOrderResultCollection radiologyOrderResultCollection = (RadiologyOrderResultCollection) resultCollection;
//        List<Order> vistaBeans = new ArrayList<Order>();
//
//        if (radiologyOrderResultCollection.getDataRecordList() != null)
//        {
//            for(RadiologyOrderDataRecord dataRecord : radiologyOrderResultCollection.getDataRecordList())
//            {
//                if (dataRecord == null || dataRecord.getOrderRecord() == null) continue;
//
//                RadiologyOrderRecord bhieBean = dataRecord.getOrderRecord();
//                List<RadiologyOrder> orders = bhieBean.getRadiologyOrderList();
//                Order bean = new Order();
//
//                bean.setOrderDate(bhieToXMLCal(bhieBean.getOrderDate()));
//                bean.setType(JMeadowsConstants.RADIOLOGY);
//                bean.setOrderid(bhieBean.getOrderId());
//                bean.setStatus(bhieBean.getStatus());
//                bean.setOrderingProvider(bhieBean.getOrderingProvider());
//
//                if(!orders.isEmpty())
//                {
//                    bean.setDescription(orders.get(0).getUniversalServiceText());
//                    bean.setOrderDetail(orders.get(0).getUniversalServiceText());
//
//                    if(StringUtils.isNotBlank(orders.get(0).getNcid()))
//                    {
//                        Code code = new Code();
//                        code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                        code.setCode(orders.get(0).getNcid());
//                        bean.getCodes().add(code);
//                    }
//                }
//
//                addBHIESiteDataToVistaBean(bhieBean.getCustodianId(),
//                        bhieBean.getCustodianName(), bhieBean.getFacility(), bhieBean.getEventId(), bean);
//
//                vistaBeans.add(bean);
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport("Radiology Orders", radiologyOrderResultCollection.getStatusList(),
//                radiologyOrderResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (Order[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Order[0]);
//    }


    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Order order = new Order();

        order.setDescription(errorMsg);
        order.setOrderDate(Utils.backDate());
        order.setSite(getConnectionUnavailableSite(location));

        return new Order[]{order};
    }

}