/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */

package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.ProcedureV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.SpdcWrapperV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.Procedure;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartOutpatientProcedure extends SmartBean implements Comparable {
//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartOutpatientProcedure.class);

    public static final int DATE = 0;
    public static final int SITE = 1;
    
    public Procedure m_procedure;
    
    public SmartOutpatientProcedure() {
        super();
        m_procedure = null;
        m_sortBy = 1;
        m_bAscending = false;
    }
    
    public SmartOutpatientProcedure(Procedure procedure, String sort) {
        super();
        m_procedure = procedure;
        getSortValue(sort);
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartOutpatientProcedure[] temp = new SmartOutpatientProcedure[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartOutpatientProcedure newAll = new SmartOutpatientProcedure((Procedure) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartOutpatientProcedure newAll = new SmartOutpatientProcedure((Procedure) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Procedure[] ret = new Procedure[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_procedure;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartOutpatientProcedure[] temp = new SmartOutpatientProcedure[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartOutpatientProcedure newAll = new SmartOutpatientProcedure((Procedure) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Procedure[] ret = new Procedure[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_procedure;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartOutpatientProcedure) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_procedure);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public int compareTo(Object obj) {
        SmartOutpatientProcedure tmp = (SmartOutpatientProcedure) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_procedure.getDate(), tmp.m_procedure.getDate());
        }
        if (this.m_sortBy == SITE && this.m_procedure.getSite() != null && tmp.m_procedure.getSite() != null) {
            retVal = compareString(this.m_procedure.getSite().getSiteCode(), tmp.m_procedure.getSite().getSiteCode());
        }

        if (!m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("date".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }

    }

    public Procedure[] mapOutpatientProceduresFromBHIE(Object resultCollection) {

        V4ResultCollection outpatientProcedureResultCollection = (V4ResultCollection) resultCollection;
        List<Procedure> vistaBeans = new ArrayList<Procedure>();

        if (outpatientProcedureResultCollection.getDataRecordList() != null && outpatientProcedureResultCollection.getDataRecordList().getProcedures() != null) {
            for (ProcedureV4 bhieOutpatientProcedure : outpatientProcedureResultCollection.getDataRecordList().getProcedures()) {
                if (bhieOutpatientProcedure == null) { continue; }

                String facilityName = "";

                Procedure bean = new Procedure();

                // Date/Time
                bean.setDate(bhieToXMLCalV4(bhieOutpatientProcedure.getDate()));
                // Location
                if (bhieOutpatientProcedure.getFacility() != null) {
                    facilityName = bhieOutpatientProcedure.getFacility().getName();
                    bean.setLocation(facilityName);

                }
                // Provider / Provider Specialty
                if (bhieOutpatientProcedure.getProvider() != null) {
//                    bean.setProvider(mapProvider(bhieOutpatientProcedure.getEnteredBy()));
                    bean.setProvider(mapProvider(bhieOutpatientProcedure.getProvider()));
                }

                if (bhieOutpatientProcedure.getStatus() != null && !bhieOutpatientProcedure.getStatus().isEmpty()
                        && getPrimarySpdcV4(bhieOutpatientProcedure.getStatus()) != null) {
                    bean.setCodingStatus(getPrimarySpdcV4(bhieOutpatientProcedure.getStatus()).getDisplay());
                }

                if (bhieOutpatientProcedure.getProcedure() != null && bhieOutpatientProcedure.getProcedure().size() > 0) {
                    // Procedure Description


                    // CPT Code
                    Code code = new Code();

                    for (SpdcBeanV4 proc : bhieOutpatientProcedure.getProcedure()) {
                        if (proc == null) {
                            continue;
                        }
                        if (proc.isPrimary()) {
                            bean.setDescription(proc.getDisplay());
                        }
                        //check if CPT OID (description will be blank)
                        if ("2.16.840.1.113883.6.12".equalsIgnoreCase(proc.getSystem())) {
                            code.setSystem(CodingSystem.CPT.getDisplayName());
                            code.setCode(proc.getCode());
                        }
//                        //check if medcin OID
//                        if("2.16.840.1.113883.6.26".equalsIgnoreCase(proc.getSystem())){
//                            bean.setDescription(proc.getDisplay());
//                        }
                        //check if NCID OID  - sometimes this will not be present
                        if ("2.16.840.1.113883.3.42.126.100001.16".equalsIgnoreCase(proc.getSystem())) {
                            code.setDisplay(proc.getDisplay());
                        }
                    }
                    if (code.getCode() != null && code.getDisplay() != null) {
                        bean.getCodes().add(code);
                    }


                }
                bean.setNarrativeId(bhieOutpatientProcedure.getNarrativeId());
                if (bhieOutpatientProcedure.getRepository() != null) {
                    // Site

                    String repositoryName = "";
                    String repositoryId = "";
                    String recordId = "";

                    if (bhieOutpatientProcedure.getFacility() != null && bhieOutpatientProcedure.getFacility().getName() != null) {
                        facilityName = bhieOutpatientProcedure.getFacility().getName();
                    }

                    if (bhieOutpatientProcedure.getRepository() != null && bhieOutpatientProcedure.getRepository().getName() != null) {
                        repositoryName =  bhieOutpatientProcedure.getRepository().getName();
                        if (bhieOutpatientProcedure.getRepository().getIds() != null
                                && bhieOutpatientProcedure.getRepository().getIds().size() > 0
                                && bhieOutpatientProcedure.getRepository().getIds().get(0) != null) {
                            repositoryId = bhieOutpatientProcedure.getRepository().getIds().get(0).getId();
                        }
                    }

                    if (bhieOutpatientProcedure.getRecordId() != null) {
                        recordId = bhieOutpatientProcedure.getRecordId().getId();
                    }

                    if (bhieOutpatientProcedure.getQualifiers() != null && bhieOutpatientProcedure.getQualifiers().size() > 0) {
                        StringBuilder sbuff = new StringBuilder();
                        for (SpdcWrapperV4 qualifiers : bhieOutpatientProcedure.getQualifiers()) {
                            if (getPrimarySpdcV4(qualifiers.getSpdcList()) != null) {
                                SpdcBeanV4 qualifier = getPrimarySpdcV4(qualifiers.getSpdcList());
                                if (qualifier != null && qualifier.getDisplay() != null) {
                                    sbuff.append(qualifier.getDisplay());
                                    sbuff.append("; ");
                                }
                            }

                        }
                        if (sbuff.length() > 0) {
                            bean.setQualifiers(sbuff.toString());
                        }
                    }

                    bean.setComment(bhieOutpatientProcedure.getComment());
                    bean.setNotes(bhieOutpatientProcedure.getTextDescription());
                    if (bhieOutpatientProcedure.getBodySite() != null) {
                        if (getPrimarySpdcV4(bhieOutpatientProcedure.getBodySite()) != null) {
                            SpdcBeanV4 bodySite = getPrimarySpdcV4(bhieOutpatientProcedure.getBodySite());
                            if (bodySite != null) {
                                bean.setBodySite(bodySite.getDisplay());
                            }
                        }
                    }

//                addBHIESiteDataToVistaBean(bhieAllergy.getCustodianId(),
//                        bhieAllergy.getCustodianName(), bhieAllergy.getFacility(), bhieAllergy.getEventId(), bean);

                    if (outpatientProcedureResultCollection.isVler()) {
                        addBHIESiteDataToVistaBean(repositoryId,
                                "VLER",
                                facilityName,
                                recordId, bean);
                    } else {
                        addBHIESiteDataToVistaBean(repositoryId,
                                repositoryName,
                                facilityName,
                                recordId, bean);
                    }
                }

                vistaBeans.add(bean);
            }
        }

        String siteStatusReport = generateBhieStatusReportV4(outpatientProcedureResultCollection.getStatusListv4(),
                outpatientProcedureResultCollection.getErrorListv4(), outpatientProcedureResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Procedure[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION, siteStatusReport)
        ));

        return vistaBeans.toArray(new Procedure[vistaBeans.size()]);
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Procedure proc = new Procedure();
        proc.setLocation(errorMsg);
        proc.setSite(getConnectionUnavailableSite(location));

        return new Procedure[]{proc};
    }
}
