/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.DischargeNote;
import gov.va.med.bhie.DischargeNoteDataRecord;
import gov.va.med.bhie.DischargeNoteResultCollection;
import gov.va.med.bhie.InpatientNote;
import gov.va.med.bhie.NoteDataRecord;
import gov.va.med.bhie.NoteResultCollection;
import gov.va.med.bhie.NoteV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.ProgressNote;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.StringUtils;
//import org.hui.chcs.NoteBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import web.webservice.jvi.citpo.ha.osd.mil.Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean;

public class SmartProgressNote extends SmartBean implements Comparable {

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int TYPE = 3;
    public static final int HCP = 4;
    public static final int LOC = 5;

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartProgressNote.class);

    public ProgressNote m_note;

    public SmartProgressNote() {
        super();
        m_note = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartProgressNote(ProgressNote note, String sort) {
        super();
        m_note = note;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartProgressNote) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_note);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartProgressNote[] temp = new SmartProgressNote[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProgressNote newAll = new SmartProgressNote((ProgressNote) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartProgressNote newAll = new SmartProgressNote((ProgressNote) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            ProgressNote[] ret = new ProgressNote[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_note;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartProgressNote[] temp = new SmartProgressNote[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProgressNote newAll = new SmartProgressNote((ProgressNote) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            ProgressNote[] ret = new ProgressNote[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_note;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartProgressNote tmp = (SmartProgressNote) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_note.getNoteDate(), tmp.m_note.getNoteDate());
        }
        if (this.m_sortBy == SITE && this.m_note.getSite() != null && tmp.m_note.getSite() != null) {
            retVal = compareString(this.m_note.getSite().getSiteCode(), tmp.m_note.getSite().getSiteCode());
        }
        if (this.m_sortBy == TYPE) {
            retVal = compareString(this.m_note.getNoteType(), tmp.m_note.getNoteType());
        }
        if (this.m_sortBy == HCP) {
            retVal = compareString(this.m_note.getProvider().getName(), tmp.m_note.getProvider().getName());
        }
        if (this.m_sortBy == LOC) {
            retVal = compareString(this.m_note.getLocation(), tmp.m_note.getLocation());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("notedate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("notetype".equalsIgnoreCase(sort)) {
                m_sortBy = TYPE;
            }
            if ("user".equalsIgnoreCase(sort)) {
                m_sortBy = HCP;
            }
            if ("location".equalsIgnoreCase(sort)) {
                m_sortBy = LOC;
            }
        }

    }

    public static ProgressNote[] getDischargeNotesFromSHARE(Object[] dodBean) {
        Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[] shareBean =
                (Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[]) dodBean;
        int size = shareBean.length;
        int cnt = 0;
        ProgressNote[] temp = new ProgressNote[size];
        ProgressNote[] retBean = null;
        ProgressNote bean = null;
        for (int i = 0; i < size; i++) {
            bean = new ProgressNote();
            bean.setNoteDate(Utils.getXMLCal(shareBean[i].getVerifiedDate()));
            bean.setNoteId(shareBean[i].getItemId());
            bean.setLocation(shareBean[i].getNoteTitle());
            bean.setNoteType(shareBean[i].getNoteTypeName());
            //TODO:populate provider from BHIERelay
            //bean.setProvider(shareBean[i].getVerifiedBy());
            bean.setImages(shareBean[i].getCompleteNote());
            bean.setStatus(shareBean[i].getNoteFormat());
            bean.setVisitDate("");

            addSHARESiteDataToVistaBean(bean);

            temp[cnt] = bean;
            cnt = cnt + 1;
        }

        if (cnt > 0) {
            retBean = new ProgressNote[cnt];
            for (int i = 0; i < cnt; i++) {
                retBean[i] = temp[i];
            }
        }


        return retBean;

    }

    public static ProgressNote[] getProgressNotesFromSHARE(Object[] dodBean) {
        Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[] shareBean =
                (Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[]) dodBean;
        int size = shareBean.length;
        int cnt = 0;
        ProgressNote[] temp = new ProgressNote[size];
        ProgressNote[] retBean = null;
        ProgressNote bean = null;
        String itemid = null;

        for (int i = 0; i < size; i++) {
            itemid = shareBean[i].getItemId();
            if (itemid == null) {
                itemid = "";
            }


            if (!shareBean[i].getNoteTitle().contains("CONSULT") 
                    && !shareBean[i].getNoteTitle().contains("Consult")
                    && !shareBean[i].getNoteTitle().contains("consult")
                    && !shareBean[i].getNoteTitle().contains("DISCHARGE")
                    && !shareBean[i].getNoteTitle().contains("Discharge")
                    && !shareBean[i].getNoteTitle().contains("discharge")
                    && !shareBean[i].getNoteTypeName().contains("CONSULT")
                    && !itemid.contains("CIS")) {
                bean = new ProgressNote();
                bean.setNoteDate(Utils.getXMLCal(shareBean[i].getVerifiedDate()));
                bean.setNoteId(shareBean[i].getItemId());
                bean.setLocation(shareBean[i].getNoteTitle());
                bean.setNoteType(shareBean[i].getNoteTypeName());
                //TODO:populate provider from BHIERelay
                //bean.setProvider(shareBean[i].getVerifiedBy());
                bean.setImages(shareBean[i].getCompleteNote());
                bean.setStatus(shareBean[i].getNoteFormat());
                bean.setVisitDate("");
                addSHARESiteDataToVistaBean(bean);
                temp[cnt] = bean;
                cnt = cnt + 1;
            }

        }

        if (cnt > 0) {
            retBean = new ProgressNote[cnt];
            for (int i = 0; i < cnt; i++) {
                retBean[i] = temp[i];
            }
        }


        return retBean;

    }

//    public static ProgressNote[] getProgressNotesFromCHCS(Object[] dodBean) {
//        NoteBean[] shareBean = (NoteBean[]) dodBean;
//        int size = shareBean.length;
//        int cnt = 0;
//        ProgressNote[] temp = new ProgressNote[size];
//        ProgressNote[] retBean = null;
//        ProgressNote bean = null;
//        String itemid = null;
//        String noteTitle = "";
//        String noteTypeName = "";
//        for (int i = 0; i < size; i++) {
//            itemid = shareBean[i].getItemId();
//            if (itemid == null) {
//                itemid = "";
//            }
//
//            noteTitle = "";
//            if (shareBean[i].getNoteTitle() != null) {
//                noteTitle = shareBean[i].getNoteTitle();
//            }
//            noteTypeName = "";
//            if (shareBean[i].getNoteTypeName() != null) {
//                noteTypeName = shareBean[i].getNoteTypeName();
//            }
//
//            if (!noteTitle.contains("CONSULT")
//                    && !noteTitle.contains("Consult")
//                    && !noteTitle.contains("consult")
//                    && !noteTitle.contains("DISCHARGE")
//                    && !noteTitle.contains("Discharge")
//                    && !noteTitle.contains("discharge")
//                    && !noteTypeName.contains("CONSULT")
//                    && !noteTypeName.contains("ENCOUNTER")
//                    && !itemid.contains("CIS")) {
//                bean = new ProgressNote();
//                bean.setNoteDate(Utils.getXMLCal(shareBean[i].getVerifiedDate()));
//                bean.setNoteId(shareBean[i].getItemId());
//                bean.setLocation(shareBean[i].getNoteTitle());
//                bean.setNoteType(shareBean[i].getNoteTypeName());
//                //TODO:populate provider from BHIERelay
//                //bean.setProvider(shareBean[i].getVerifiedBy());
//                bean.setImages(shareBean[i].getCompleteNote());
//                bean.setStatus(shareBean[i].getNoteFormat());
//                if (bean.getStatus() == null) {
//                    bean.setStatus("Text");
//                }
//                bean.setVisitDate("");
//                addCHCSSiteDataToVistaBean(bean);
//                temp[cnt] = bean;
//                cnt = cnt + 1;
//            }
//
//        }
//
//        if (cnt > 0) {
//            retBean = new ProgressNote[cnt];
//            for (int i = 0; i < cnt; i++) {
//                retBean[i] = temp[i];
//            }
//        }
//
//
//        return retBean;
//
//    }

    // Version 4
    public ProgressNote[] getProgressNotesFromBHIE(Object resultCollection) throws JMeadowsException {
        V4ResultCollection noteResultCollection = (V4ResultCollection) resultCollection;
        List<ProgressNote> vistaBeans = new ArrayList<ProgressNote>();
        try {

            if (noteResultCollection.getDataRecordList() != null) {
                List<NoteV4> noteList = new ArrayList<NoteV4>();
                if (noteResultCollection.getDataRecordList().getNotes() != null
                        && noteResultCollection.getDataRecordList().getNotes().size() > 0) {
                    noteList = noteResultCollection.getDataRecordList().getNotes();
                } else if (noteResultCollection.getDataRecordList().getEncounterNotes() != null
                        && noteResultCollection.getDataRecordList().getEncounterNotes().size() > 0) {
                    noteList = noteResultCollection.getDataRecordList().getEncounterNotes();
                }

                if (noteResultCollection.getDataRecordList().getInpatientNotes() != null
                        && noteResultCollection.getDataRecordList().getInpatientNotes().size() > 0) {
                    noteList.addAll(noteResultCollection.getDataRecordList().getInpatientNotes());
                }

                for (NoteV4 bhieNote : noteList) {
                    if (bhieNote == null) {
                        continue;
                    }

                    //filter out consult note
                    String loinc = "";
                    if (bhieNote.getDataDomain() != null && !bhieNote.getDataDomain().isEmpty()) {
                        loinc = bhieNote.getDataDomain().get(0).getCode();
                    }
                    if (LOINC_CODE_NOTES.equals(loinc) && NOTE_TYPE_CODE_CONSULT.contains(bhieNote.getType().get(0).getCode())) {
                        continue;
                    }
                    ProgressNote bean;

                    bean = new ProgressNote();
                    if (bhieNote.getTitle() != null && !bhieNote.getTitle().isEmpty()) {
                        bean.setNoteTitle(bhieNote.getTitle());
                    }
                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty()
                            && bhieNote.getType().get(0) != null) {
                        bean.setNoteType(bhieNote.getType().get(0).getDisplay());
                    }
                    bean.setSensitive(bhieNote.getSensitive());
                    if (bhieNote.getContentType() != null && !bhieNote.getContentType().isEmpty()) {
                        bean.setStatus(bhieNote.getContentType().get(0).getCode());
                    }
                    bean.setComplexDataUrl(bhieNote.getComplexDataUrl());
                    String custodian = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        custodian = bhieNote.getRepository().getIds().get(0).getId();
                    }
                    if (bhieNote.getEnteredDate() != null
                            && StringUtils.isNotBlank(bhieNote.getEnteredDate().getStart())) {
                        bean.setNoteDate(bhieToXMLCal(bhieNote.getEnteredDate().getStart()));
                    } else if (APP_CONFIG.getBHIECustodianOidSHARE().equalsIgnoreCase(custodian)
                            && bhieNote.getVerifiedDate() != null
                            && bhieNote.getVerifiedDate().getStart() != null) {
                        bean.setNoteDate(bhieToXMLCal(bhieNote.getVerifiedDate().getStart()));
                    } else if (bhieNote.getVerifiedDate() != null
                            && bhieNote.getVerifiedDate().getStart() != null) {
                        bean.setNoteDate(bhieToXMLCal(bhieNote.getVerifiedDate().getStart()));
                    }
                    String clinic = "";
                    if (bhieNote.getLocation() != null && bhieNote.getLocation().getName() != null) {
                        clinic = bhieNote.getLocation().getName();
                    } else if (bhieNote.getFacility() != null) {
                        clinic = bhieNote.getFacility().getName();
                    }

                    //dod adapter facility format: location^clinic
                    if (StringUtils.isNotBlank(clinic) && clinic.contains("^")) {
                        clinic = clinic.substring(clinic.indexOf('^') + 1);
                    }

                    bean.setLocation(clinic);

                    //                 ALTHA currrently does not send CDA and uses complexURL
                    //                if ("CDA".equalsIgnoreCase(bhieNote.getNoteFormat())) {
                    //                    //may complete complete note and event id
                    //                    mapCDAToProgressNote(bhieNote, bean);
                    //                } else
                    if (bhieNote.getComplexDataUrl() == null && bhieNote.getContent() != null) {
                        bean.setNoteText(bhieNote.getContent());
                    } else if (bhieNote.getCompleteNote() != null) {
                        bean.setNoteText(bhieNote.getCompleteNote());
                    }
//                    if(bhieNote.getRepository() != null
//                            && bhieNote.getRepository().getId() != null
//                            && !bhieNote.getRepository().getId().isEmpty()
//                            && bhieNote.getRepository().getId().get(0).getAssigningAuthority() != null
//                            && APP_CONFIG.getBHIECustodianOidSHARE().matches(bhieNote.getRepository().getId().get(0).getAssigningAuthority())
//                            && bhieNote.getContentType() != null
//                            && !bhieNote.getContentType().isEmpty()
//                            && bhieNote.getContentType().get(0).getCode() != null
//                            && "CDA".equalsIgnoreCase(bhieNote.getContentType().get(0).getCode()))
//                        //TODO:populate provider from BHIERelay
//                        bean.setProvider(mapProvider(bhieNote.getEnteredBy()));
                    if (bhieNote.getVerifiedBy() != null) {
                        bean.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                    }

                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty() && StringUtils.isNotBlank(bhieNote.getType().get(0).getCode())) {
                        Code code = new Code();
                        code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                        code.setCode(bhieNote.getType().get(0).getCode());
                        bean.getCodes().add(code);
                    }

                    if (bhieNote.getDataDomain() != null && !bhieNote.getDataDomain().isEmpty() && StringUtils.isNotBlank(bhieNote.getDataDomain().get(0).getCode())) {
                        Code code = new Code();
                        code.setSystem(CodingSystem.DOD_NOTES.getDisplayName());
                        code.setCode(bhieNote.getDataDomain().get(0).getCode());
                        bean.getCodes().add(code);
                    }

                    //                addBHIESiteDataToVistaBean(bhieNote.getCustodianId(),
                    //                        bhieNote.getCustodianName(), bhieNote.getFacility(), bhieNote.getEventId(), bean);
                    //                addBHIESiteDataToVistaBean(bhieNote.getRepository().getId().get(0).getId(),
                    //                        bhieNote.getRepository().getName(), bhieNote.getFacility().getName(), bhieNote.getRecordId().getId(), bean);
                    String repositoryId = "";
                    String repositoryName = "";
                    String facilityName = "";
                    String eventId = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        repositoryId = bhieNote.getRepository().getIds().get(0).getId();
                    }
                    if (bhieNote.getRepository() != null) {
                        repositoryName = bhieNote.getRepository().getName();
                    }
                    if (bhieNote.getFacility() != null) {
                        facilityName = bhieNote.getFacility().getName();
                    }
                    if (bhieNote.getRecordId() != null) {
                        eventId = bhieNote.getRecordId().getId();
                    }
                    if (repositoryId == null || repositoryId.length() == 0) {
                        repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                    }
                    if (repositoryName == null || repositoryName.length() == 0) {
                        repositoryName = "ahlta";
                    }
                    addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                            facilityName, eventId, bean);

                    vistaBeans.add(bean);
                }
            }
        } catch (Exception ex) {

            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }
        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(noteResultCollection.getStatusListv4(),
                noteResultCollection.getErrorListv4(), noteResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (ProgressNote[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));
        return vistaBeans.toArray(new ProgressNote[vistaBeans.size()]);
    }

    //Version 2
    public ProgressNote[] getProgressNotesFromBHIEV2(Object resultCollection) {
        NoteResultCollection noteResultCollection = (NoteResultCollection) resultCollection;
        List<ProgressNote> vistaBeans = new ArrayList<ProgressNote>();

        if (noteResultCollection.getDataRecordList() != null) {
            for (NoteDataRecord noteDataRecord : noteResultCollection.getDataRecordList()) {
                if (noteDataRecord == null || noteDataRecord.getNote() == null) {
                    continue;
                }

                InpatientNote bhieNote = noteDataRecord.getNote();

                //filter out consult note
                if (LOINC_CODE_NOTES.equals(bhieNote.getLoincCode()) && NOTE_TYPE_CODE_CONSULT.contains(bhieNote.getNoteTypeCode())) {
                    continue;
                }
                ProgressNote bean;

                bean = new ProgressNote();
                bean.setNoteTitle(bhieNote.getNoteTitle());
                bean.setNoteType(bhieNote.getNoteTypeName());
                //TODO:populate provider from BHIERelay
                //bean.setProvider(bhieNote.getEnteredBy());
                bean.setStatus(bhieNote.getNoteFormat());
                bean.setComplexDataUrl(bhieNote.getComplexDataUrl());

                if (StringUtils.isNotBlank(bhieNote.getEnteredByDate())) {
                    bean.setNoteDate(bhieToXMLCal(bhieNote.getEnteredByDate()));
                } else if (APP_CONFIG.getBHIECustodianOidSHARE().equalsIgnoreCase(bhieNote.getCustodianId())) {
                    bean.setNoteDate(bhieToXMLCal(bhieNote.getVerifiedDate()));
                }

                String clinic = bhieNote.getFacility();

                //dod adapter facility format: location^clinic
                if (StringUtils.isNotBlank(clinic) && clinic.contains("^")) {
                    clinic = clinic.substring(clinic.indexOf('^') + 1);
                }

                bean.setLocation(clinic);

//                 ALTHA currrently does not send CDA and uses complexURL
//                if ("CDA".equalsIgnoreCase(bhieNote.getNoteFormat())) {
//                    //may complete complete note and event id
//                    mapCDAToProgressNote(bhieNote, bean);
//                } else

                bean.setNoteText(bhieNote.getCompleteNote());
                //TODO:populate provider from BHIERelay
                //bean.setProvider(getProvider(bhieNote));
                if (APP_CONFIG.getBHIECustodianOidSHARE().matches(bhieNote.getCustodianId()) && "CDA".equalsIgnoreCase(bhieNote.getNoteFormat()) 
                        && StringUtils.isNotBlank(bhieNote.getNoteTypeCode())) {
                    Code code = new Code();
                    code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                    code.setCode(bhieNote.getNoteTypeCode());
                    bean.getCodes().add(code);
                }
                if (StringUtils.isNotBlank(bhieNote.getLoincCode())) {
                    Code code = new Code();
                    code.setSystem(CodingSystem.DOD_NOTES.getDisplayName());
                    code.setCode(bhieNote.getLoincCode());
                    bean.getCodes().add(code);
                }

                addBHIESiteDataToVistaBean(bhieNote.getCustodianId(),
                        bhieNote.getCustodianName(), bhieNote.getFacility(), bhieNote.getEventId(), bean);

                vistaBeans.add(bean);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReport(noteResultCollection.getStatusList(),
                noteResultCollection.getErrorList());

        vistaBeans.addAll(Arrays.asList(
                (ProgressNote[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));
        return vistaBeans.toArray(new ProgressNote[vistaBeans.size()]);
    }


    //commented out because no longer used
//    /**
//     * Map CDA complete note and eventId fields to ProgressNote images and noteId fields respectively.
//     *
//     * @param bhieNote BHIE DoD adapter note to be mapped.
//     * @param note     Target progress note.
//     */
//    private void mapCDAToProgressNote(Note bhieNote, ProgressNote note) {
////        final String XPATH_CDA_NOTE_TITLE =
////                "/ClinicalDocument/component/structuredBody/component/section/entry/act/entryRelationship/observation/text";
////
////        final String XPATH_CDA_VERIFIED_DATE =
////                "/ClinicalDocument/component/structuredBody/component/section/entry/act/entryRelationship/observation/author/time/@value";
////
////        final String XPATH_CDA_VERIFIED_BY =
////                "/ClinicalDocument/component/structuredBody/component/section/entry/act/entryRelationship/observation/author/assignedAuthor/assignedPerson/name/family";
////
////        final String XPATH_CDA_NOTE_TYPE_NAME =
////                "/ClinicalDocument/component/structuredBody/component/section/entry/act/entryRelationship/observation/entryRelationship/observation/code/@displayName";
//
//        final String XPATH_CDA_COMPLETE_NOTE =
//                "/ClinicalDocument/component/structuredBody/component/section/entry/act/entryRelationship/observation/entryRelationship/observation/text";
//
//        final String XPATH_CDA_EVENT_ID =
//                "/ClinicalDocument/component/structuredBody/component/section/entry/act/id/@extension";
//
//        final String XPATH_CDA_UNIT_NUMBER =
//                "/ClinicalDocument/component/structuredBody/component/section/entry/act/entryRelationship/observation/id/@extension";
//
////        final String XPATH_CDA_LOCATION_NAME =
////                "/ClinicalDocument/component/structuredBody/component/section/entry/act/entryRelationship/observation/author/assignedAuthor/representedOrganization/name";
//        try {
//            String cdaXML = bhieNote.getCompleteNote();
//
//            //use UTF-8 char encoding
//            byte[] cdaXMLBytes = cdaXML.getBytes(Charset.forName("UTF-8"));
//
//            ByteArrayInputStream inputStream = new ByteArrayInputStream(cdaXMLBytes);
//            DocumentBuilderFactory dbf = null;
//
//            dbf = DocumentBuilderFactory.newInstance();
//            dbf.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", true);
//            dbf.setFeature("http://xml.org/sax/features/external-general-entities", false);
//            dbf.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
//            dbf.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
//            dbf.setValidating(true);
//
//            Document xmlDoc = dbf.newDocumentBuilder().parse(inputStream);
//
//            XPath xpath = XPathFactory.newInstance().newXPath();
////            XPathExpression exprNoteTitle = xpath.compile(XPATH_CDA_NOTE_TITLE);
////            XPathExpression exprVerifiedDate = xpath.compile(XPATH_CDA_VERIFIED_DATE);
////            XPathExpression exprVerifiedBy = xpath.compile(XPATH_CDA_VERIFIED_BY);
////            XPathExpression exprNoteTypeName = xpath.compile(XPATH_CDA_NOTE_TYPE_NAME);
//            XPathExpression exprCompleteNote = xpath.compile(XPATH_CDA_COMPLETE_NOTE);
//            XPathExpression exprEventId = xpath.compile(XPATH_CDA_EVENT_ID);
//            XPathExpression exprUnitNumber = xpath.compile(XPATH_CDA_UNIT_NUMBER);
////            XPathExpression exprLocationName = xpath.compile(XPATH_CDA_LOCATION_NAME);
//
//
////            String noteTitle = (String) exprNoteTitle.evaluate(xmlDoc, XPathConstants.STRING);
////            String verifiedDate = (String) exprVerifiedDate.evaluate(xmlDoc, XPathConstants.STRING);
////            String verifiedBy = (String) exprVerifiedBy.evaluate(xmlDoc, XPathConstants.STRING);
////            String noteTypeName = (String) exprNoteTypeName.evaluate(xmlDoc, XPathConstants.STRING);
//            String completeNote = (String) exprCompleteNote.evaluate(xmlDoc, XPathConstants.STRING);
//            String eventId = (String) exprEventId.evaluate(xmlDoc, XPathConstants.STRING);
//            String unitNumber = (String) exprUnitNumber.evaluate(xmlDoc, XPathConstants.STRING);
//
////            String locationName = (String) exprLocationName.evaluate(xmlDoc, XPathConstants.STRING);
//
//            //If completed note field is of length one and consists of non-visual char - make it an empty string
//            if (completeNote.length() == 1 && (completeNote.charAt(0) < 33 || completeNote.charAt(0) > 126)) {
//                completeNote = "";
//            }
//
//            note.setNoteText(completeNote);
//
//            //set eventId to compressed data id when unitNumber equals "COMPRESSED_DATA_ID=<ID_VALUE>"
//            if (StringUtils.isNotBlank(unitNumber)) {
//                if (unitNumber.contains("COMPRESSED_DATA_ID=")) {
//                    int valStartIdx = unitNumber.indexOf("COMPRESSED_DATA_ID=") + "COMPRESSED_DATA_ID=".length();
//                    if (valStartIdx < unitNumber.length()) {
//                        eventId = unitNumber.substring(valStartIdx);
//                    }
//                }
//            }
//
//            note.setNoteId(eventId);
//        } catch (Exception e) {
//            throw new RuntimeException(e);
//        }
//    }

    public ProgressNote[] getDischargeNoteFromBHIE(Object resultCollection) {
        DischargeNoteResultCollection dischargeNoteResultCollection = (DischargeNoteResultCollection) resultCollection;
        List<ProgressNote> vistaBeans = new ArrayList<ProgressNote>();

        if (dischargeNoteResultCollection.getDataRecordList() != null) {
            for (DischargeNoteDataRecord noteDataRecord : dischargeNoteResultCollection.getDataRecordList()) {
                if (noteDataRecord == null || noteDataRecord.getDischargeNote() == null) {
                    continue;
                }

                DischargeNote bhieDischargeNote = noteDataRecord.getDischargeNote();

                ProgressNote bean = new ProgressNote();

                bean.setNoteDate(Utils.getXMLCal(bhieDischargeNote.getEnteredByDate()));
                bean.setLocation(bhieDischargeNote.getFacility());
                bean.setNoteTitle(bhieDischargeNote.getNoteTitle());

                String provider = bhieDischargeNote.getEnteredBy();

                //TODO:populate provider from BHIERelay
                // bean.setProvider(provider);

                //user data might be represented by delimited string: <identifier> ^ <provider_name> ^^^^^^^
                if (provider != null && provider.length() > 0 && provider.indexOf('^') >= 0) {
                    int firstCarrotIdx = provider.indexOf('^');
                    provider = provider.substring(firstCarrotIdx + 1, provider.indexOf("^", firstCarrotIdx + 1));
                    //TODO:populate provider from BHIERelay
                    // bean.setProvider(provider);
                }

                bean.setNoteText(bhieDischargeNote.getCompleteNote());
                bean.setStatus(bhieDischargeNote.getStatus());
                XMLGregorianCalendar admCal = bhieToXMLCal(bhieDischargeNote.getAdmissionDate());
                bean.setVisitDate(Utils.formatDisplayDateTime(admCal));

                addBHIESiteDataToVistaBean(bhieDischargeNote.getCustodianId(),
                        bhieDischargeNote.getCustodianName(), bhieDischargeNote.getFacility(), bhieDischargeNote.getEventId(), bean);

                vistaBeans.add(bean);
            }
        }
        String siteStatusReport = generateBhieStatusReport(dischargeNoteResultCollection.getStatusList(),
                dischargeNoteResultCollection.getErrorList());

        vistaBeans.addAll(Arrays.asList(
                (ProgressNote[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));
        return vistaBeans.toArray(new ProgressNote[vistaBeans.size()]);
    }


    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        ProgressNote note = new ProgressNote();
        note.setNoteId("");
        note.setNoteType(errorMsg);
        note.setNoteTitle(errorMsg);
        note.setNoteDate(Utils.backDate());
        note.setSite(getConnectionUnavailableSite(location));

        return new ProgressNote[]{note};
    }

//    /**
//     * Retries the OID from SHARE which can be either ESSENTRIS or CHCS
//     *
//     * @param note
//     * @return shareOID
//     */
//    public String getProvider(Note note){
//        final String XPATH_CDA_PROVIDER_NAME_GIVEN =
//                "/ClinicalDocument/author/assignedAuthor/assignedPerson/name/given";
//        final String XPATH_CDA_PROVIDER_NAME_FAMILY =
//                "/ClinicalDocument/author/assignedAuthor/assignedPerson/name/family";
//        try{
//            String cdaXML = note.getCompleteNote();
//
//            //use UTF-8 char encoding
//            byte[] cdaXMLBytes = cdaXML.getBytes(Charset.forName("UTF-8"));
//
//            ByteArrayInputStream inputStream = new ByteArrayInputStream(cdaXMLBytes);
//            DocumentBuilderFactory dbf = null;
//
//            dbf = DocumentBuilderFactory.newInstance();
//            dbf.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", true);
//            dbf.setFeature("http://xml.org/sax/features/external-general-entities", false);
//            dbf.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
//            dbf.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
//            dbf.setValidating(true);
//            XPath xpath = XPathFactory.newInstance().newXPath();
//            Document xmlDoc = dbf.newDocumentBuilder().parse(inputStream);
//            XPathExpression exprProviderGiven = xpath.compile(XPATH_CDA_PROVIDER_NAME_GIVEN);
//            XPathExpression exprProviderFamily = xpath.compile(XPATH_CDA_PROVIDER_NAME_FAMILY);
//            StringBuilder providerName = new StringBuilder();
//            providerName.append((String) exprProviderFamily.evaluate(xmlDoc,XPathConstants.STRING));
//            providerName.append(",");
//            providerName.append((String) exprProviderGiven.evaluate(xmlDoc, XPathConstants.STRING));
//            return providerName.toString();
//        }catch(Exception e){
//            throw new RuntimeException(e);
//        }
//    }
}