/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.provider;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.smartbeans.patient.SmartBean;
import gov.va.med.vds.webservice.PatientAdmission;
import gov.va.med.vhahon.common.Utils;

import java.util.Arrays;

public class SmartAdmission extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartAdmission.class);

    public static final int DATE = 1;
    public static final int SITE = 2;

    public PatientAdmission m_admission;

    public SmartAdmission() {
        super();
        m_admission = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartAdmission(PatientAdmission admission, String sort) {
        super();
        m_admission = admission;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartAdmission) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_admission);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartAdmission[] temp = new SmartAdmission[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartAdmission newAll = new SmartAdmission((PatientAdmission) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartAdmission newAll = new SmartAdmission((PatientAdmission) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientAdmission[] ret = new PatientAdmission[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_admission;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartAdmission[] temp = new SmartAdmission[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartAdmission newAll = new SmartAdmission((PatientAdmission) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientAdmission[] ret = new PatientAdmission[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_admission;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartAdmission tmp = (SmartAdmission) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_admission.getAdmissionDate(), tmp.m_admission.getAdmissionDate());
        }
        if (this.m_sortBy == SITE && this.m_admission.getSite() != null && tmp.m_admission.getSite() != null) {
            retVal = compareString(this.m_admission.getSite().getSiteCode(), tmp.m_admission.getSite().getSiteCode());
        }

        if (!m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("admissionDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }

    }
//
//    public PatientAdmission[] mapPatientAdmissionFromCHCS(Object dodBean[]) {
//        AdmissionDetailBean[] adBean = (AdmissionDetailBean[]) dodBean;
//        int size = dodBean.length;
//        PatientAdmission vistaBean[] = new PatientAdmission[size];
//
//        for (int i = 0; i < dodBean.length; i++) {
//            PatientAdmission bean = new PatientAdmission();
//            bean.setAdmissionDate(Utils.getXMLCal(adBean[i].getAdmissionDate()));
//            bean.setDispositionDate(Utils.getXMLCal(adBean[i].getDispositionDate()));
//            bean.setWard(adBean[i].getWard());
//
//            //admission id is synonymous with registration number
//            if (adBean[i].getRegistrationNumber() == null || adBean[i].getRegistrationNumber().isEmpty())
//            {
//                 bean.setRegisterNumber(adBean[i].getAdmissionId());
//            }
//            else bean.setRegisterNumber(adBean[i].getRegistrationNumber());
//
//            bean.setAdmissionId(adBean[i].getAdmissionId());
//            bean.setPatientId(adBean[i].getPatientId());
//            bean.setPatientName(adBean[i].getPatientName());
//            Provider provider = new Provider();
//            provider.setName(adBean[i].getAdmittingProvider());
//            bean.setAdmittingProvider(provider);
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//        }
//        return vistaBean;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientAdmission adm = new PatientAdmission();
        adm.setDrg(errorMsg);
        adm.setWard(errorMsg);
        adm.setCodingCompleteDate(Utils.backDate());
        adm.setDispositionDate(Utils.backDate());
        adm.setAdmissionDate(Utils.backDate());
        adm.setDrgDescription(errorMsg);
        adm.setSite(getConnectionUnavailableSite(location));

        return new PatientAdmission[]{adm};
    }

//    public static AdmissionDetail getAdmissionDetailFromCHCS(AdmissionDetailBean chcsBean, String siteId) {
//        AdmissionDetail vistaBean = new AdmissionDetail();
//        ProcedureBean chcsProc[] = null;
//        IcdBean chcsIcd[] = null;
//        ArrayList<Procedure> vistaProc = null;
//        ArrayList<Diagnosis> vistaIcd = null;
//
//        vistaBean.setAdmissionDate(Utils.getXMLCal(chcsBean.getAdmissionDate()));
//        vistaBean.setAdmissionId(chcsBean.getAdmissionId());
//        Provider provider = new Provider();
//        provider.setName(chcsBean.getAdmittingProvider());
//        vistaBean.setAdmittingProvider(provider);
//        vistaBean.setDispositionDate(Utils.getXMLCal(chcsBean.getDispositionDate()));
//        vistaBean.setDivision(chcsBean.getDivision());
//        vistaBean.setRegisterNumber(chcsBean.getRegistrationNumber());
//        vistaBean.setWard(chcsBean.getWard());
//        vistaBean.setDrgDescription(chcsBean.getLocationCode()); // this is the Health summary report
//        addCHCSSiteDataToVistaBean(vistaBean);
//        chcsProc = chcsBean.getProcedures().toArray(new ProcedureBean[0]);
//        chcsIcd = chcsBean.getDiagnosis().toArray(new IcdBean[0]);
//
//        if (chcsProc != null) {
//            vistaProc = new ArrayList<Procedure>();
//            for (int i = 0; i < chcsProc.length; i++) {
//                vistaProc.get(i).setCode(chcsProc[i].getCode());
//                vistaProc.get(i).setDate(Utils.getXMLCal(chcsProc[i].getProcedureDate()));
//                vistaProc.get(i).setDescription(chcsProc[i].getProcedure());
//                vistaProc.get(i).setStatus(chcsProc[i].getStatus());
//                addCHCSSiteDataToVistaBean(vistaProc.get(i));
//            }
//
//            vistaBean.getProcedures().addAll(vistaProc);
//        }
//
//        if (chcsIcd != null) {
//            vistaIcd = new ArrayList<Diagnosis>();
//            for (int i = 0; i < chcsIcd.length; i++) {
//                vistaIcd.get(i).setCode(chcsIcd[i].getCode());
//                vistaIcd.get(i).setDescription(chcsIcd[i].getDescription());
//                vistaIcd.get(i).setStatus(chcsIcd[i].getStatus());
//                addCHCSSiteDataToVistaBean(vistaIcd.get(i));
//            }
//
//            vistaBean.getDiagnoses().addAll(vistaIcd);
//        }
//
//        return vistaBean;
//    }


}