/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.beans.PDWSQueryBean;
import gov.va.med.jmeadows.dao.beans.ResponsePatientQuery;
import gov.va.med.jmeadows.dao.beans.ResponsePatientSelect;
import gov.va.med.vds.webservice.Patient;
import gov.va.med.vds.webservice.PatientDemographics;
import gov.va.med.vds.webservice.PatientDemographicsDetail;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author vhahonbrunos
 */
public class PatientDaoTest extends DaoBaseTest
{
    private PatientDao patientDao;

    public PatientDaoTest() throws JMeadowsException
    {
        patientDao = new PatientDao();
    }

    @Test
    public void testGetPatientDemographics()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            qb.setIncludeVLER(true);
            List<PatientDemographics> demographics = patientDao.getPatientDemographics(qb);
            assertTrue(demographics.size() > 0);
            for(PatientDemographics demo : demographics)
            {
                assertTrue(demo.getSite() != null);
            }
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientDemographicsDetail()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            List<PatientDemographicsDetail> demographicsDetail = patientDao.getPatientDemographicsDetail(qb);
            assertTrue(demographicsDetail.size() > 0);
            testForBeanMeta(demographicsDetail);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testLookupPatient()
    {
        try {
            JMeadowsQuery qb = new JMeadowsQuery();
            qb.setItemId("B0008");
            qb.setRecordSiteCode("200");
            qb.setUser(user);

            PatientDao patientDao = new PatientDao();
            List<Patient> patientList = patientDao.lookupPatient(qb);

            assert (patientList != null);
        }
        catch (JMeadowsException e) {
            fail (e.getMessage());
        }
    }

    @Test
    public void testPatientSearch()
    {
        try
        {
            String key = ""; // Path to keystore
            String trust = ""; // Path to truststore
            String pass = ""; // Password for keystore and truststore

            // Sets up keystore for 2 way SSL handshake
            //System.setProperty("javax.net.debug", "ssl,handshake,verbose");
            System.setProperty("javax.net.ssl.keyStore", key);
            System.setProperty("javax.net.ssl.trustStore", trust);
            System.setProperty("javax.net.ssl.keyStorePassword", pass);
            System.setProperty("javax.net.ssl.trustStorePassword", pass);

            PDWSQueryBean qb = new PDWSQueryBean();
            qb.setUser(user);
            qb.setRequestingApp("JLV");

            // InquiryType valid values:
            //     "PS" for person search. Set SSN/last name pair. Gets patient info
            //     "PR" for person retrieve. Set EDIPI. Gets patient info along with demographics
            //     "FS" for family search. Set SSN/last name pair. Gets sponsor patient info
            //     "FR" for family retrieve. Set EDIPI. Gets patient info including family members (dependents)
            qb.setInquiryType("PS"); // <-- Set PDWS inquiry type here
            if ("PS".equalsIgnoreCase(qb.getInquiryType()) || "FS".equalsIgnoreCase(qb.getInquiryType())) {
                qb.setPatientIdType("SSN");
                qb.setPatientId(""); // <-- Enter patient's SSN here
                qb.setLastName("a");
            }
            else if ("PR".equalsIgnoreCase(qb.getInquiryType()) || "FR".equalsIgnoreCase(qb.getInquiryType())) {
                qb.setPatientIdType("EDIPI");
                qb.setPatientId(""); // <-- Enter patient's EDIPI here
            }
            else {
                logger.error("InquiryType is invalid: " + qb.getInquiryType());
                fail("InquiryType is invalid: " + qb.getInquiryType());
            }

            // Patient lookup using PDWS
            ResponsePatientQuery response = patientDao.lookupPatientPDWS(qb);

            assert (response != null);
        } catch (JMeadowsException e) {
            logger.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    @Test
    public void testPatientSelect()
    {
//        String key = ""; // Path to keystore
//        String trust = ""; // Path to truststore
//        String pass = "xxx"; // Password for keystore and truststore
//
//        // Sets up keystore for 2 way SSL handshake
//        //System.setProperty("javax.net.debug", "ssl,handshake,verbose");
//        System.setProperty("javax.net.ssl.keyStore", key);
//        System.setProperty("javax.net.ssl.trustStore", trust);
//        System.setProperty("javax.net.ssl.keyStorePassword", pass);
//        System.setProperty("javax.net.ssl.trustStorePassword", pass);

        try {
            //JMeadowsQuery queryBean = new JMeadowsQuery();
            JMeadowsQuery queryBean = setupQuery(user, patient, startDate, endDate);
            //queryBean.setUser(user);

//            queryBean.setStatus("EDIPI"); // Type of identifier number to query with ("EDIPI", "ICN", or "VAIEN")
            queryBean.setItemId("0000000001"); // Identifier number
//            queryBean.setRecordSiteCode(""); // Site code for VA IEN

            ResponsePatientSelect response = patientDao.selectPatientMVI(queryBean);

            assert (response != null);
        }
        catch( JMeadowsException e ) {
            logger.error(e.getMessage(), e);
            fail( e.getMessage() );
        }
    }
}
