/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.DaoBaseTest;
import gov.va.med.vds.webservice.PatientAdmission;
import gov.va.med.vds.webservice.User;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author vhahonbrunos
 */
public class ProviderAdmissionDaoTest extends DaoBaseTest
{
    private ProviderAdmissionsDao admDao;
    
    public ProviderAdmissionDaoTest() throws JMeadowsException
    {
        super();
        admDao = new ProviderAdmissionsDao();
    }

    @Test
    public void testGetProviderAdmissions()
    {
        try
        {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            User p = qb.getUser();
            
            mockProviderBasedQuery(p, "3126", "22610");

            List<PatientAdmission> admissions = admDao.getProviderAdmissions(qb);
            assertTrue(admissions.size() > 0);
            logger.debug("# of User Admissions: " + admissions.size());
            for(PatientAdmission admission : admissions)
            {
                logger.debug("PatientName: " + admission.getPatientName());
                logger.debug("PatientId: " + admission.getPatientId());
                logger.debug("PatientLocation: " + admission.getSite().getSiteCode());
            }
            testForProviderBeanMeta(admissions);
        } 
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }

    /**
     * Gets the local vista site's current ward admissions
     */
    @Test
    public void testGetWardAdmissions()
    {
        try
        {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            qb.setItemId("24^27");  // Setup the query
            List<PatientAdmission> admissions = admDao.getWardAdmissions(qb);
            assertTrue(admissions.size() > 0);
            logger.debug("# of Inpatients: " + admissions.size());
            testForBeanMeta(admissions,false);
        }
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }
}
