package gov.va.med.jmeadows.dao.hdd;


import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.vds.webservice.DataBean;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.Immunization;
import gov.va.med.vds.webservice.Site;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class ConceptCodeMapperTest
{
    private HDDDao hddDao;

    @Before
    public void setUp()
    {
        hddDao = mock(HDDDao.class);
    }

    @Test
    public void testConceptCodeMapper()
    {
        ConceptCodeMapper conceptCodeMapper = ConceptCodeMapper.getInstance();

        List<DataBean> dataBeanList = new ArrayList<DataBean>();

        List<Code> codes = new ArrayList<Code>();

        Code codeAA = new Code();
        codeAA.setCode("FLU,NASAL");
        codeAA.setSystem("TEXT");
        codeAA.setDisplay("FLU,NASAL");
        codes.add(codeAA);

//        Code codeAMI = new Code();
//        codeAMI.setCode("AMI");
//        codeAMI.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
//        codeAMI.setDisplay("Myocardial Infarction");
//        codes.add(codeAMI);
//
//        Code codeCHF = new Code();
//        codeCHF.setCode("CHF");
//        codeCHF.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
//        codeCHF.setDisplay("Congestive heart failure");
//        codes.add(codeCHF);

        for(Code code : codes)
        {
            Immunization dataBean = new Immunization();
            dataBean.getCodes().add(code);
            dataBean.setName("FLU,NASAL");
            Site site = new Site();
            site.setAgency("VA");

            dataBean.setSite(site);
            dataBeanList.add(dataBean);

        }

        Map<String, List<Code>> codeMap = new LinkedHashMap<String, List<Code>>();

        Code AA2ICDCode = new Code();
        AA2ICDCode.setCode("sdfsdf");
        AA2ICDCode.setDisplay("FLU NASAL SPRAY");
        AA2ICDCode.setSystem("CVX");

//        Code AMI2ICDCode = new Code();
//        AMI2ICDCode.setCode("410.90");
//        AMI2ICDCode.setDisplay("Myocardial Infarction");
//        AMI2ICDCode.setSystem("ICD-9-CM Diseases");
//
//        Code CHF2ICDCode = new Code();
//        CHF2ICDCode.setCode("428.0");
//        CHF2ICDCode.setDisplay("Congestive heart failure");
//        CHF2ICDCode.setSystem("ICD-9-CM Diseases");

        List<Code> AA2ICDCodes = new ArrayList<Code>();
        AA2ICDCodes.add(AA2ICDCode);
        codeMap.put(CodingSystem.TEXT.getOID() + "^" + codeAA.getCode(), AA2ICDCodes);

//        List<Code> AMI2ICDCodes = new ArrayList<Code>();
//        AMI2ICDCodes.add(AMI2ICDCode);
//        codeMap.put(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID() + "^" + codeAMI.getCode(), AMI2ICDCodes);
//
//        List<Code> CHF2ICDCodes = new ArrayList<Code>();
//        CHF2ICDCodes.add(CHF2ICDCode);
//        codeMap.put(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID() + "^" + codeCHF.getCode(), CHF2ICDCodes);

        try {
            when(hddDao.mapConceptCodes(any(CodeMappingList.class), any(String.class))).thenReturn(codeMap);
        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }

        try {

            conceptCodeMapper.mapConceptCodes(hddDao, dataBeanList, CodingSystem.CVX);

            //assertTrue(dataBeanList.size() == 3);

            DataBean aaBean = dataBeanList.get(0);
            //assertTrue(aaBean.getCodes().size() == 2);
//            assertTrue(aaBean.getCodes().get(0).getCode().equals(codeAA.getCode()));
//            assertTrue(aaBean.getCodes().get(1).getCode().equals(AA2ICDCode.getCode()));
//
//            DataBean amiBean = dataBeanList.get(1);
//            assertTrue(amiBean.getCodes().size() == 2);
//            assertTrue(amiBean.getCodes().get(0).getCode().equals(codeAMI.getCode()));
//            assertTrue(amiBean.getCodes().get(1).getCode().equals(AMI2ICDCode.getCode()));
//
//            DataBean chfBean = dataBeanList.get(2);
//            assertTrue(chfBean.getCodes().size() == 2);
//            assertTrue(chfBean.getCodes().get(0).getCode().equals(codeCHF.getCode()));
//            assertTrue(chfBean.getCodes().get(1).getCode().equals(CHF2ICDCode.getCode()));

        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }

    public void testConceptCodeMapperFailure()
    {
        ConceptCodeMapper conceptCodeMapper = ConceptCodeMapper.getInstance();

        List<DataBean> dataBeanList = new ArrayList<DataBean>();

        List<Code> codes = new ArrayList<Code>();

        Code codeAA = new Code();
        codeAA.setCode("AA");
        codeAA.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAA.setDisplay("Aortic aneurysm");
        codes.add(codeAA);

        Code codeAMI = new Code();
        codeAMI.setCode("AMI");
        codeAMI.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAMI.setDisplay("Myocardial Infarction");
        codes.add(codeAMI);

        Code codeCHF = new Code();
        codeCHF.setCode("CHF");
        codeCHF.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeCHF.setDisplay("Congestive heart failure");
        codes.add(codeCHF);

        for(Code code : codes)
        {
            DataBean dataBean = new DataBean();
            dataBean.getCodes().add(code);
            dataBeanList.add(dataBean);
        }

        Map<String, List<Code>> codeMap = new LinkedHashMap<String, List<Code>>();

        Code AA2ICDCode = new Code();
        AA2ICDCode.setCode("441.9");
        AA2ICDCode.setDisplay("Aortic aneurysm");
        AA2ICDCode.setSystem("ICD-9-CM Diseases");

        Code AMI2ICDCode = new Code();
        AMI2ICDCode.setCode("410.90");
        AMI2ICDCode.setDisplay("Myocardial Infarction");
        AMI2ICDCode.setSystem("ICD-9-CM Diseases");

        Code CHF2ICDCode = new Code();
        CHF2ICDCode.setCode("428.0");
        CHF2ICDCode.setDisplay("Congestive heart failure");
        CHF2ICDCode.setSystem("ICD-9-CM Diseases");

        List<Code> AA2ICDCodes = new ArrayList<Code>();
        AA2ICDCodes.add(AA2ICDCode);
        codeMap.put(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID() + "^" + codeAA.getCode(), AA2ICDCodes);

        List<Code> AMI2ICDCodes = new ArrayList<Code>();
        AMI2ICDCodes.add(AMI2ICDCode);
        codeMap.put(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID() + "^" + codeAMI.getCode(), AMI2ICDCodes);

        //exclude CHF ICD mapping (simulate missing CHF mapping)

        try {
            when(hddDao.mapConceptCodes(any(CodeMappingList.class), any(String.class))).thenReturn(codeMap);
        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }

        try {

            conceptCodeMapper.mapConceptCodes(hddDao, dataBeanList, CodingSystem.ICD9_CM_DISEASES);

            assertTrue(dataBeanList.size() == 3);

            DataBean aaBean = dataBeanList.get(0);
            assertTrue(aaBean.getCodes().size() == 2);
            assertTrue(aaBean.getCodes().get(0).getCode().equals(codeAA.getCode()));
            assertTrue(aaBean.getCodes().get(1).getCode().equals(AA2ICDCode.getCode()));

            DataBean amiBean = dataBeanList.get(1);
            assertTrue(amiBean.getCodes().size() == 2);
            assertTrue(amiBean.getCodes().get(0).getCode().equals(codeAMI.getCode()));
            assertTrue(amiBean.getCodes().get(1).getCode().equals(AMI2ICDCode.getCode()));

            //CHF ICD mapping missing
            DataBean chfBean = dataBeanList.get(2);
            assertTrue(chfBean.getCodes().size() == 1);
            assertTrue(chfBean.getCodes().get(0).getCode().equals(codeCHF.getCode()));

        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }
}
